# -*- coding: utf-8 -*-

from __future__ import absolute_import

import autocomplete_light
from django.conf import settings
from django import forms
from django.conf.urls import patterns, url
from django.contrib import admin, messages
from django.contrib.admin import SimpleListFilter
from django.core.urlresolvers import reverse
from django.db import connection, DatabaseError
from django.db.models import Q
from django.template.defaultfilters import date as _date
from django.utils.translation import ugettext_lazy as _
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.contrib.auth.models import User
from django.shortcuts import redirect
from django.utils.html import strip_tags

from jmb.core.admin import AjaxInline, register_inline, ConstrainedModelForm
from jmb.core.admin.options import ExtendibleModelAdmin
from jmb.filters import CharFilter
from jmb.filters.admin import AdvancedSearchChangeList
from jmb.i18n.models import Language
from jmb.mailer.models import Attachment, MailingStatus
from jmb.mailer.admin import MailingStatusAdmin

from publieditor.event.admin_widgets import AdminImageWidget, AdminProportionImageWidget
from publieditor.event.forms import (
    EventTurnProxyAdminForm, EventStartPlaceAdminForm, FlightAdminForm, TrainAdminForm, FlightDetailAdminForm,
    TrainDetailAdminForm, TransportAnswerAdminForm, EventTravelOptionAdminForm, EventAdminForm, InviteAdminForm,
    CustomTransportDetailAdminForm,
    EventPlaceAdminForm)
from publieditor.event.admin_forms import (
    ContactActionForm, ContactGroupActionForm, InviteActionForm, EventActionForm,
    MembershipForm, DelegatedContactActionForm
)
from publieditor.event.models import (
    Event, EventTranslation,
    Invite, TravelAgencyInvite, LogisticAgencyInvite,
    ScheduleTranslation, PrivacyTranslation, ActivityIntroductionTranslation,
    Transport, TransportTranslation,
    Flight, Airport, Airline, FlightDetail,
    Train, Station, TrainType, TrainDetail,
    Turn, EventTurnProxy, EventStartPlace,
    Stage, AdhesionTranslation,
    Signature, GreetingsTranslation,
    EventQuestion, Phone, Role,
    ActivityEventQuestionProxy, RegisterEventQuestionProxy, DeclineEventQuestionProxy, PollEventQuestionProxy,
    EventDocument, Document, RegisterAnswer, TransportAnswer,
    FreeAnswer, OwnAnswer, ActivityAnswer, EventTemplate,
    DeclineAnswer, DocumentCategory, FollowUpTranslation, PollAnswer, EventDocumentEmailTranslation,
    Organization, Contact, ContactAddress, ContactGroup, Agent, Membership,
    Newspaper, NewspaperCategory, NewspaperType, NewspaperSector,
    EventSummaryDocumentTemplate,
    Card, License, ContactEmailCc, InviteEmailObject, DelegatedContact, DocumentsEmailObject,
    EventTravelOption, InviteDecisionAnswer, AreaCode, EventImage, MailingStatusProxy, InviteDocument,
    ContactSocialMedia, PasswordRecoverySettings, ContactEditDiff, ProfileDoneEmailSettings,
    InviteDocumentTravelAgencyProxy, InviteDocumentLogisticAgencyProxy,
    EventTravelAgencyProxy, EventLogisticAgencyProxy, NewsletterDocument,
    CustomTransportDetail, EventDissuasionMessageTranslation, EmailForwardedTranslation, Vehicle, EventPlace, Brand,
    VehicleModel, InviteDocumentHostessProxy, EventHostessProxy, HostessInvite, CDU, NoteFromPhoneContact)
from publieditor.event.admin_actions import(
    contact_add_to_group, contact_add_to_event_invited_list, contact_export_csv_async, contact_export_xls_async,
    contact_group_add_to_event_invited_list,
    invite_send, invite_resend, invite_send_followup, invite_send_documents, change_invites_status,
    event_send_invites, event_resend_invites, event_send_followups, event_send_documents,
    delegated_contact_invite_send, delegated_contact_invite_create, change_invite_call_status
)
from publieditor.event.admin_views import (
    event_statistics_update, event_statistics, export_contact_vcard,
    initialize_event, delegatedcontact_create_invite, delegatedcontact_send_invite,
    event_landing_page_presentation_preview,
    event_email_invite_preview, event_landing_page_schedule_preview, event_landing_page_adhesion_preview,
    invite_set_delegated, ContactImportView, EventTravelAgencyView, EventTravelAgencyExportView, EventPollAnswersView,
    EventLogisticAgencyView, EventLogisticAgencyExportView, EventLogisticAgencyTriumphExportView,
    EventHostessView)
from publieditor.survey.models import Question


########################################################################################################################
# CONTACTS #############################################################################################################
########################################################################################################################
LIST_DISPLAY_DEFAULT = ('get_edit_icon', 'get_delete_icon')
FIELDSETS_DEFAULT = (
    _('logs admin'), {
        'fields': (
            'creator', 'date_create',
            'last_modifier', 'date_last_modify',
        ),
    })
FIELDSETS_DEFAULT_COLLAPSED = (
    _('logs admin'), {
        'fields': (
            'creator', 'date_create',
            'last_modifier', 'date_last_modify',
        ),
        'classes': ('collapse',)
    })


class QSearchFilter(CharFilter):
    def filter(self, qs, value):
        if value:
            email_q_filter = value.replace("('", "Q(email__icontains='")
            qs = qs.filter(eval(email_q_filter))
        return qs


#######################
# AJAX INLINES        #
#######################
class MembershipAjaxInline(AjaxInline):
    model = Membership
    list_display = (
        'newspaper', 'role', 'get_link_default',
        'get_edit_icon_iframe', 'get_delete_icon_iframe'
    )

    def get_queryset(self, request):
        qs = super(MembershipAjaxInline, self).get_queryset(request).order_by('date_create')
        return qs


class ContactEmailCcAjaxInline(AjaxInline):
    model = ContactEmailCc
    fk_name = 'contact'
    list_display = (
        'email_details', 'get_link_default', 'note',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class PhoneAjaxInline(AjaxInline):
    model = Phone
    list_display = (
        'ordering', 'get_link_default', 'phone_type', 'area_code', 'number', 'note',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class AddressAjaxInline(AjaxInline):
    model = ContactAddress
    list_display = (
        'ordering', 'get_link_default', 'get_scopes', 'address_type', 'name', 'number',
        'zip_code', 'locality', 'province', 'country', 'note',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class CardAjaxInline(AjaxInline):
    model = Card
    list_display = (
        'ordering', 'get_link_default', 'airline', 'number', 'expiration_date',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class SocialMediaAjaxInline(AjaxInline):
    model = ContactSocialMedia
    list_display = (
        'social', 'link', 'link_open', 'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class NoteFromPhoneContactInline(AjaxInline):
    model = NoteFromPhoneContact
    width = 850
    list_display = (
        'call_result', 'text', 'last_modifier', 'date_last_modify', 'creator', 'date_create',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class NoteFromPhoneContactInlineReadOnly(AjaxInline):
    model = NoteFromPhoneContact
    width = 850
    can_delete = False
    list_display = (
        'call_result', 'text', 'last_modifier', 'date_last_modify', 'creator', 'date_create',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class ContactInviteAjaxInline(AjaxInline):
    model = Invite
    list_display = (
        'get_event_cl',
        'get_status_cl', 'get_step_cl',
        'get_transport_cl', 'get_activities_cl',
        'get_link_managed_cl', 'get_link_no_show_cl',
    )


########################################################################################################################
# MODEL ADMIN MIXINS ###################################################################################################
########################################################################################################################

class InviteAdminCSVMixin(object):

    ## metodi per recupero info contatto e invito per xls/csv

    ### DATI INVITO

    def get_turn(self, instance):
        if instance.turn_preference and instance.turn_preference.title:
            return instance.turn_preference.title
        return ""
    get_turn.short_description = _('Turno')

    def get_escaped_answers_cl(self, instance):
        return strip_tags(instance.get_answers_cl() or "" )
    get_escaped_answers_cl.short_description = _("Answers")

    def get_escaped_contact_newspaper_cl(self, instance):
        return strip_tags(instance.get_contact_newspaper_cl() or "")
    get_escaped_contact_newspaper_cl.short_description = _("Newspapers")

    def get_delegated_contact(self, instance):
        if instance.status == settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED and instance.delegated_invites.all().count():
            contact = instance.delegated_invites.all()[0].invited
            return "%s %s" % (contact.first_name, contact.last_name)
        return ""
    get_delegated_contact.short_description = _('Delegato a')

    def get_last_modified(self, instance):
        return instance.date_last_modify
    get_last_modified.short_description = _('Log ultima modifica')

    def get_transport(self, instance):
        if instance.own_answers.all().count():
            return _("I'll come by myself")
        elif instance.free_answers.all().count():
            free_answer = instance.free_answers.all()[0]
            return free_answer.answer
        elif instance.travel_option_answers.all().count():
            invite_travel_option = instance.travel_option_answers.all()[0]
            return invite_travel_option.event_travel_option.name
        return ""
    get_transport.short_description = _('Transport')

    def get_departure_transport(self, instance):
        if instance.travel_option_answers.all().count():
            invite_travel_option = instance.travel_option_answers.all()[0]
            event_travel_option = invite_travel_option.event_travel_option
            transport_details = event_travel_option.transport_details.filter(direction="d")
            details_strings = []
            for transport_detail in transport_details:
                detail_string = strip_tags(self.get_formatted_transport_detail(transport_detail))
                details_strings.append(detail_string)
            return ";".join(details_strings)
        return ""
    get_departure_transport.short_description = _('Departure')

    def get_return_transport(self, instance):
        if instance.travel_option_answers.all().count():
            invite_travel_option = instance.travel_option_answers.all()[0]
            event_travel_option = invite_travel_option.event_travel_option
            transport_details = event_travel_option.transport_details.filter(direction="r")
            details_strings = []
            for transport_detail in transport_details:
                detail_string = strip_tags(self.get_formatted_transport_detail(transport_detail))
                details_strings.append(detail_string)
            return ";".join(details_strings)
        return ""
    get_return_transport.short_description = _('Return')

    def get_formatted_transport_detail(self, transport_detail):
        transport_str = ""
        if transport_detail.category == "flightdetail":
            transport_str = "%s %s %s%s %s %s (%s) - %s %s %s (%s)" % (
                _date(transport_detail.departure, "D d/m/Y").title(),
                _("Flight"),
                transport_detail.flightdetail.flight.airline.iata_code,
                transport_detail.flightdetail.flight.number,
                transport_detail.flightdetail.flight.departure.strftime("%H:%M"),
                transport_detail.flightdetail.flight.dep_airport.place,
                transport_detail.flightdetail.flight.dep_airport.iata_code,
                transport_detail.flightdetail.flight.arrival.strftime("%H:%M"),
                "+1" if transport_detail.flightdetail.day_after() else "",
                transport_detail.flightdetail.flight.arr_airport.place,
                transport_detail.flightdetail.flight.arr_airport.iata_code,
            )
        # Print detail in transport is a train
        elif transport_detail.category == "traindetail":
            transport_str = "%s <strong>%s %s %s</strong> %s (%s) - <strong>%s %s</strong> %s (%s)" % (
                _date(transport_detail.departure, "D d/m/Y").title(),
                _("Train"),
                transport_detail.traindetail.train.code,
                transport_detail.traindetail.train.departure.strftime("%H:%M"),
                transport_detail.traindetail.train.dep_station.place,
                transport_detail.traindetail.train.dep_station.name,
                transport_detail.traindetail.train.arrival.strftime("%H:%M"),
                "+1" if transport_detail.traindetail.day_after() else "",
                transport_detail.traindetail.train.arr_station.place,
                transport_detail.traindetail.train.arr_station.name,
            )
        return transport_str

    def get_contact_first_name(self, instance):
        return instance.contact.first_name
    get_contact_first_name.short_description = _('first name')

    def get_contact_last_name(self, instance):
        return instance.contact.last_name
    get_contact_last_name.short_description = _('last name')

    def get_contact_company(self, instance):
        return instance.contact.company
    get_contact_company.short_description = _('company')

    def get_contact_default_email(self, instance):
        return instance.contact.get_default_email() or ""
    get_contact_default_email.short_description = _('email')

    def get_contact_all_email_cc(self, instance):
        emails = ""
        for email in instance.contact.get_all_email_cc():
            emails += "%s, " % email
        return emails
    get_contact_all_email_cc.short_description = _('email cc')
    get_contact_all_email_cc.allow_tags = False

    def get_contact_birthday(self, instance):
        return instance.contact.birthday or ""
    get_contact_birthday.short_description = _('birthday')

    def get_contact_default_phone(self, instance):
        types_map = {"P": "T", "M": "M", "F": "F", "T": "T"}
        if instance.contact.default_phone:
            return "{} {}".format(
                types_map[instance.contact.default_phone.phone_type],
                instance.contact.default_phone.number
            )
        return ""
    get_contact_default_phone.short_description = _('default phone')

    def get_contact_all_phones(self, instance):
        types_map = {"P": "T", "M": "M", "F": "F", "T": "T"}
        return ",".join([
            "%s %s " % (types_map[phone.phone_type], phone.number)
            for phone in instance.contact.phones.all() if phone != instance.contact.default_phone
        ])
    get_contact_all_phones.short_description = _('other phones')

    def get_contact_internal_note(self, instance):
        return instance.contact.internal_note or ""
    get_contact_internal_note.short_description = _('internal note')

    def get_contact_note(self, instance):
        return instance.contact.note or ""
    get_contact_note.short_description = _('note')

    def get_contact_frequent_flyers_cards(self, instance):
        return ",".join(["%s%s" % (card.airline.iata_code, card.number)
                         if card.airline else "%s" % card.number
                         for card in instance.contact.cards.all()]
                        )
    get_contact_frequent_flyers_cards.short_description = _('frequent flyer')

    def get_contact_passport(self, instance):
        return instance.contact.passport_number or ""
    get_contact_passport.short_description = _('passport number')

    def get_contact_passport_expiration_date(self, instance):
        return instance.contact.passport_expiration_date or ""
    get_contact_passport_expiration_date.short_description = _('passport expiration date')

    def get_contact_id_card_number(self, instance):
        return instance.contact.id_card_number or ""
    get_contact_id_card_number.short_description = _('id card number')

    def get_contact_id_card_expiration_date(self, instance):
        return instance.contact.id_card_expiration_date or ""
    get_contact_id_card_expiration_date.short_description = _('id card expiration date')

    def get_contact_license_number(self, instance):
        return instance.contact.license_number or ""
    get_contact_license_number.short_description = _('license number')

    def get_contact_license_expiration_date(self, instance):
        return instance.contact.license_expiration_date or ""
    get_contact_license_expiration_date.short_description = _('license expiration date')

    def get_contact_smoking(self, instance):
        if instance.contact.smoking and instance.contact.smoking != "none":
            return instance.contact.smoking
        return ""
    get_contact_smoking.short_description = _('smoking')

    def get_contact_food(self, instance):
        if instance.contact.food and instance.contact.food != "none":
            return instance.contact.food or ""
        return ""
    get_contact_food.short_description = _('food')

    def get_contact_food_note(self, instance):
        return instance.contact.food_note or ""
    get_contact_food_note.short_description = _('food note')

    def get_contact_shirt_size(self, instance):
        return instance.contact.shirt_size or ""
    get_contact_shirt_size.short_description = _('shirt size')

    def get_contact_height(self, instance):
        return instance.contact.height or ""
    get_contact_height.short_description = _('height')

    def get_contact_weight(self, instance):
        return instance.contact.weight or ""
    get_contact_weight.short_description = _('weight')

    def get_contact_sweater_size(self, instance):
        return instance.contact.sweater_size or ""
    get_contact_sweater_size.short_description = _('sweater size')

    def get_contact_shoes_size(self, instance):
        return instance.contact.shoes_size or ""
    get_contact_shoes_size.short_description = _('shoes size')


########################################################################################################################
# MODEL ADMIN ##########################################################################################################
########################################################################################################################

class LocalExtededModelAdmin(ExtendibleModelAdmin):

    class Media:
        css = {"all": ("contact/css/admin.css",)}

    save_on_top = False


class OrganizationAdmin(LocalExtededModelAdmin):
    list_display = LIST_DISPLAY_DEFAULT + ('ordering', 'name',)
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {'fields': ('ordering', 'name',)})
    )


class DelegatedContactAdmin(LocalExtededModelAdmin):
    readonly_fields = ('managed', 'invited', 'invite',)
    list_display = LIST_DISPLAY_DEFAULT + (
        'managed', 'get_event_name', 'get_delegator_link', 'get_delegated_full_name',
    )
    advanced_search_fields = (
        ('managed',),
        ('invite__event:Evento', 'invited:Invitato'),
    )
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': (
                'managed', 'invited', 'invite',
            )
        })
    )
    action_form = DelegatedContactActionForm
    action_form_fields = {
        'delegated_contact_invite_create': [],
        'delegated_contact_invite_send': ['test'],
    }
    actions = [
        delegated_contact_invite_create,
        delegated_contact_invite_send,
    ]

    def has_add_permission(self, request):
       return False

    def get_urls(self):
        urls = super(DelegatedContactAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^delegatedcontact_create_invite/(?P<delegatedcontact_id>\d+)/$',
                self.admin_site.admin_view(delegatedcontact_create_invite),
                name='delegatedcontact_create_invite'
            ),
            url(
                r'^delegatedcontact_send_invite/(?P<delegatedcontact_id>\d+)/$',
                self.admin_site.admin_view(delegatedcontact_send_invite),
                name='delegatedcontact_send_invite'
            ),
        )
        return my_urls + urls


class MainAddressContactListFilter(SimpleListFilter):
    title = _('main address')
    parameter_name = 'address'

    def lookups(self, request, model_admin):
        return [(address.id, address.locality) for address in ContactAddress.objects.filter(default=True).distinct()]

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(addresses__id=self.value())


class ContactAdminCSVMixin(object):

    def get_id_csv(self, contact):
        return u"{}".format(contact.id)
    get_id_csv.short_description = "id"

    def get_title_csv(self, contact):
        return u"{}".format(contact.title)
    get_title_csv.short_description = "title"

    def get_language_csv(self, contact):
        return u"{}".format(contact.language.code)
    get_language_csv.short_description = "language"

    def get_first_name_csv(self, contact):
        if contact.first_name:
            return u"{}".format(contact.first_name)
        return u""
    get_first_name_csv.short_description = "first_name"

    def get_last_name_csv(self, contact):
        if contact.last_name:
            return u"{}".format(contact.last_name)
        return u""
    get_last_name_csv.short_description = "last_name"

    def get_company_csv(self, contact):
        if contact.company:
            return u"{}".format(contact.company)
        return u""
    get_company_csv.short_description = "company"

    def get_birth_place_csv(self, contact):
        if contact.birth_place:
            return u"{}".format(contact.birth_place)
        return u""
    get_birth_place_csv.short_description = "birth_place"

    def get_birthday_csv(self, contact):
        if contact.birthday:
            return u"{}".format(contact.birthday.strftime("%d/%m/%Y"))
        return u""
    get_birthday_csv.short_description = "birthday"

    def get_email_csv(self, contact):
        if contact.email:
            return u"{}".format(contact.email)
        return u""
    get_email_csv.short_description = "email"

    def get_email_cc_csv(self, contact):
        email_ccs = ",".join([
            email_cc.contact_cc.email if email_cc.contact_cc else email_cc.email for
            email_cc in contact.cc_emails.all()
        ])
        return u"{}".format(email_ccs)
    get_email_cc_csv.short_description = "email_cc"

    def get_groups_csv(self, contact):
        groups = ",".join([group.name for group in contact.groups.filter(status=1)])
        return u"{}".format(groups)
    get_groups_csv.short_description = "groups"

    def get_phone_area_code_csv(self, contact):
        phones = contact.phones.filter(phone_type="P", status=1).order_by('ordering')
        if phones:
            phone = phones[0]
            if phone.area_code:
                return u"{}".format(phone.area_code.code)
        return u""
    get_phone_area_code_csv.short_description = "phone_area_code"

    def get_phone_number_csv(self, contact):
        phones = contact.phones.filter(phone_type="P", status=1).order_by('ordering')
        if phones:
            phone = phones[0]
            return u"{}".format(phone.number)
        return u""
    get_phone_number_csv.short_description = "phone_number"

    def get_mobile_area_code_csv(self, contact):
        phones = contact.phones.filter(phone_type="M", status=1).order_by('ordering')
        if phones:
            phone = phones[0]
            if phone.area_code:
                return u"{}".format(phone.area_code.code)
        return u""
    get_mobile_area_code_csv.short_description = "mobile_area_code"

    def get_mobile_number_csv(self, contact):
        phones = contact.phones.filter(phone_type="M", status=1).order_by('ordering')
        if phones:
            phone = phones[0]
            return u"{}".format(phone.number)
        return u""
    get_mobile_number_csv.short_description = "mobile_number"

    def get_fax_area_code_csv(self, contact):
        phones = contact.phones.filter(phone_type="F", status=1).order_by('ordering')
        if phones:
            phone = phones[0]
            if phone.area_code:
                return u"{}".format(phone.area_code.code)
        return u""
    get_fax_area_code_csv.short_description = "fax_area_code"

    def get_fax_number_csv(self, contact):
        phones = contact.phones.filter(phone_type="F", status=1).order_by('ordering')
        if phones:
            phone = phones[0]
            return u"{}".format(phone.number)
        return u""
    get_fax_number_csv.short_description = "fax_number"

    def get_other_phones_csv(self, contact):
        exclude_phones_id = []
        for phone_type in ["P", "M", "F"]:
            phones = contact.phones.filter(phone_type=phone_type, status=1).order_by('ordering')
            if phones:
                phone = phones[0]
                exclude_phones_id.append(phone.id)
        phones = contact.phones.exclude(id__in=exclude_phones_id).filter(status=1).order_by('ordering')
        phones_str = ""
        for phone in phones:
            phones_str += "[{}:{}:{}],".format(
                phone.phone_type,
                phone.area_code.code if phone.area_code else '',
                phone.number,
            )
        phones_str = phones_str.rstrip(",")
        return u"{}".format(phones_str)
    get_other_phones_csv.short_description = "other_phones"

    def get_social_media_links_csv(self, contact):
        social_media_links_str = ""
        for social_media_link in contact.social_media_links.all():
            social_media_links_str += "[{}:{}],".format(
                social_media_link.social,
                social_media_link.link,
            )
        social_media_links_str = social_media_links_str.rstrip(",")
        return u"{}".format(social_media_links_str)
    get_social_media_links_csv.short_description = "social_media"

    def get_default_address_type_csv(self, contact):
        default_address = contact.default_address
        if default_address and default_address.address_type:
            return u"{}".format(default_address.get_address_type_display())
        return u""
    get_default_address_type_csv.short_description = "default_address_type"

    def get_default_address_name_csv(self, contact):
        default_address = contact.default_address
        if default_address and default_address.name:
            return u"{}".format(default_address.name)
        return u""
    get_default_address_name_csv.short_description = "default_address_name"

    def get_default_address_number_csv(self, contact):
        default_address = contact.default_address
        if default_address and default_address.number:
            return u"{}".format(default_address.number)
        return u""
    get_default_address_number_csv.short_description = "default_address_number"

    def get_default_address_locality_csv(self, contact):
        default_address = contact.default_address
        if default_address and default_address.locality:
            return u"{}".format(default_address.locality)
        return u""
    get_default_address_locality_csv.short_description = "default_address_locality"

    def get_default_address_zip_code_csv(self, contact):
        default_address = contact.default_address
        if default_address and default_address.zip_code:
            return u"{}".format(default_address.zip_code)
        return u""
    get_default_address_zip_code_csv.short_description = "default_address_zip_code"

    def get_default_address_province_csv(self, contact):
        default_address = contact.default_address
        if default_address and default_address.province:
            return u"{}".format(default_address.province)
        return u""
    get_default_address_province_csv.short_description = "default_address_province"

    def get_default_address_country_csv(self, contact):
        default_address = contact.default_address
        if default_address:
            return u"{}".format(default_address.country.iso2_code)
        return u""
    get_default_address_country_csv.short_description = "default_address_country"

    def get_default_address_scopes_csv(self, contact):
        default_address = contact.default_address
        if default_address:
            scopes = ",".join(
                [scope.name for scope in default_address.address_scopes.filter(status=1).order_by('ordering')]
            )
            return u"{}".format(scopes)
        return u""
    get_default_address_scopes_csv.short_description = "default_address_scopes"

    def get_other_addresses_csv(self, contact):
        # type, name, number, locality, zip_code, province,country,scopes
        qs = contact.addresses.exclude(id=contact.default_address.id) if contact.default_address \
            else contact.addresses.all()
        addresses_str = u""
        for address in qs:
            address_str = u"["
            if address.address_type:
                address_str += u"{}".format(address.get_address_type_display())
            if address.name:
                address_str += u" {}".format(address.name)
            if address.number:
                address_str += u" {},".format(address.number)
            if address.zip_code:
                address_str += u" {}".format(address.zip_code)
            if address.locality:
                address_str += u" {}".format(address.locality)
            if address.province:
                address_str += u" ({})".format(address.province)
            if address.country:
                address_str += u" {}".format(address.country.name)
            address_str += u"],"
            addresses_str += address_str
        addresses_str = addresses_str.rstrip(",")
        return u"{}".format(addresses_str)
    get_other_addresses_csv.short_description = "other_addresses"

    def get_memberships_csv(self, contact):
        memberships = contact.memberships.all()
        memberships_str = ""
        for membership in memberships:
            memberships_str += \
                u"[{}:{}],".format(membership.newspaper.name, membership.role.name) \
                if membership.role and membership.role.name.strip() else \
                u"[{}],".format(membership.newspaper.name)
        memberships_str = memberships_str.rstrip(",")
        return u"{}".format(memberships_str)
    get_memberships_csv.short_description = "memberships"

    def get_cards_csv(self, contact):
        cards = contact.cards.all().order_by('ordering')
        cards_str = ""
        for card in cards:
            cards_str += \
                u"{}{},".format(card.airline.iata_code, card.number) if card.airline else \
                u"{},".format(card.number)
        cards_str = cards_str.rstrip(",")
        return u"{}".format(cards_str)
    get_cards_csv.short_description = "frequent_flyers"

    def get_privacy_acceptance_date_csv(self, contact):
        privacy_acceptance_date = getattr(contact, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD)
        if privacy_acceptance_date:
            return u"{}".format(privacy_acceptance_date.strftime("%d/%m/%Y"))
        return u""
    get_privacy_acceptance_date_csv.short_description = settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD

    def get_smoking_csv(self, contact):
        if contact.smoking:
            return u"{}".format(contact.smoking)
        return u""
    get_smoking_csv.short_description = "smoking"

    def get_food_csv(self, contact):
        if contact.food:
            return u"{}".format(contact.food)
        return u""
    get_food_csv.short_description = "food"

    def get_shirt_size_csv(self, contact):
        if contact.shirt_size:
            return u"{}".format(contact.shirt_size)
        return u""
    get_shirt_size_csv.short_description = "shirt_size"

    def get_height_csv(self, contact):
        if contact.height:
            return u"{}".format(contact.height)
        return u""
    get_height_csv.short_description = "height"

    def get_weight_csv(self, contact):
        if contact.weight:
            return u"{}".format(contact.weight)
        return u""
    get_weight_csv.short_description = "weight"

    def get_sweater_size_csv(self, contact):
        if contact.sweater_size:
            return u"{}".format(contact.sweater_size)
        return u""
    get_sweater_size_csv.short_description = "sweater_size"

    def get_shoes_size_csv(self, contact):
        if contact.shoes_size:
            return u"{}".format(contact.shoes_size)
        return u""
    get_shoes_size_csv.short_description = "shoes_size"

    def get_ski_csv(self, contact):
        if contact.ski:
            return u"{}".format(contact.ski)
        return u""
    get_ski_csv.short_description = "ski"

    def get_ski_level_csv(self, contact):
        if contact.ski_level:
            return u"{}".format(contact.ski_level)
        return u""
    get_ski_level_csv.short_description = "ski_level"

    def get_ski_equipment_csv(self, contact):
        if contact.ski:
            return u"{}".format(contact.ski_equipment)
        return u""
    get_ski_equipment_csv.short_description = "ski_equipment"

    def get_ski_boots_size_csv(self, contact):
        if contact.ski_boots_size:
            return u"{}".format(contact.ski_boots_size)
        return u""
    get_ski_boots_size_csv.short_description = "ski_boots_size"

    def get_id_card_number_csv(self, contact):
        if contact.id_card_number:
            return u"{}".format(contact.id_card_number)
        return u""
    get_id_card_number_csv.short_description = "id_card_number"

    def get_id_card_issue_date_csv(self, contact):
        if contact.id_card_issue_date:
            return u"{}".format(contact.id_card_issue_date.strftime("%d/%m/%Y"))
        return u""
    get_id_card_issue_date_csv.short_description = "id_card_issue_date"

    def get_id_card_expiration_date_csv(self, contact):
        if contact.id_card_expiration_date:
            return u"{}".format(contact.id_card_expiration_date.strftime("%d/%m/%Y"))
        return u""
    get_id_card_expiration_date_csv.short_description = "id_card_expiration_date"

    def get_passport_number_csv(self, contact):
        if contact.passport_number:
            return u"{}".format(contact.passport_number)
        return u""
    get_passport_number_csv.short_description = "passport_number"

    def get_passport_issue_date_csv(self, contact):
        if contact.passport_issue_date:
            return u"{}".format(contact.passport_issue_date.strftime("%d/%m/%Y"))
        return u""
    get_passport_issue_date_csv.short_description = "passport_issue_date"

    def get_passport_expiration_date_csv(self, contact):
        if contact.passport_expiration_date:
            return u"{}".format(contact.passport_expiration_date.strftime("%d/%m/%Y"))
        return u""
    get_passport_expiration_date_csv.short_description = "passport_expiration_date"

    def get_license_number_csv(self, contact):
        if contact.license_number:
            return u"{}".format(contact.license_number)
        return u""
    get_license_number_csv.short_description = "license_number"

    def get_license_issue_date_csv(self, contact):
        if contact.license_issue_date:
            return u"{}".format(contact.license_issue_date.strftime("%d/%m/%Y"))
        return u""
    get_license_issue_date_csv.short_description = "license_issue_date"

    def get_license_expiration_date_csv(self, contact):
        if contact.license_expiration_date:
            return u"{}".format(contact.license_expiration_date.strftime("%d/%m/%Y"))
        return u""
    get_license_expiration_date_csv.short_description = "license_expiration_date"

    def get_license_emitted_by_csv(self, contact):
        if contact.license_emitted_by:
            return u"{}".format(contact.license_emitted_by)
        return u""
    get_license_emitted_by_csv.short_description = "license_emitted_by"

    def get_license_types_csv(self, contact):
        license_types = ",".join(type.name for type in contact.license_types.all())
        return u"{}".format(license_types)
    get_license_types_csv.short_description = "license_types"

    def get_press_card_number_csv(self, contact):
        if contact.press_card_number:
            return u"{}".format(contact.press_card_number)
        return u""
    get_press_card_number_csv.short_description = "press_card_number"

    def get_elenco_csv(self, contact):
        if contact.elenco:
            return u"{}".format(contact.elenco)
        return u""
    get_elenco_csv.short_description = "elenco"

    def get_food_note_csv(self, contact):
        if contact.food_note:
            return u"{}".format(contact.food_note)
        return u""
    get_food_note_csv.short_description = "food_note"

    def get_note_csv(self, contact):
        if contact.note:
            return u"{}".format(contact.note)
        return u""
    get_note_csv.short_description = "note"

    def get_internal_note_csv(self, contact):
        if contact.internal_note:
            return u"{}".format(contact.internal_note)
        return u""
    get_internal_note_csv.short_description = "internal_note"


class ContactAdmin(LocalExtededModelAdmin, ContactAdminCSVMixin):

    class Media:
        js = (
            "event/js/admin_contact.js",
            "event/js/admin_dajaxice.js",
        )
        css = {
            "all": (
                "event/css/admin_contact.css",
            )
        }

    save_on_top = True
    search_fields = (
        'last_name', 'first_name', 'email', 'memberships__newspaper__name',
    )
    list_filter = ('language', 'groups', 'memberships__newspaper__periodicity', MainAddressContactListFilter)
    list_display = (
        'get_edit_icon', 'get_link_status', 'get_privacy_status',
        'last_name', 'first_name', 'get_company',
        'newspapers_display', 'groups_display', 'email', 'admin_thumbnail', 'get_language_code', 'title',
        'get_default_phone', 'invited_admin_url', 'export_vcard_link', 'get_delete_icon',
    )
    list_display_csv = (
        'get_id_csv', 'get_title_csv', 'get_language_csv', 'get_first_name_csv', 'get_last_name_csv', 'get_company_csv',
        'get_birth_place_csv', 'get_birthday_csv', 'get_email_csv', 'get_email_cc_csv', 'get_groups_csv',
        # Phones
        'get_phone_area_code_csv', 'get_phone_number_csv',
        'get_mobile_area_code_csv', 'get_mobile_number_csv',
        'get_fax_area_code_csv', 'get_fax_number_csv', 'get_other_phones_csv',
        # Addresses
        'get_default_address_type_csv', 'get_default_address_name_csv', 'get_default_address_number_csv',
        'get_default_address_locality_csv', 'get_default_address_zip_code_csv', 'get_default_address_province_csv',
        'get_default_address_country_csv', 'get_default_address_scopes_csv', 'get_other_addresses_csv',
        # Memberships & Frequent flyers
        'get_memberships_csv', 'get_cards_csv',
        # Social media links
        'get_social_media_links_csv',
        # generics
        'get_privacy_acceptance_date_csv', 'get_smoking_csv', 'get_food_csv', 'get_shirt_size_csv',
        'get_height_csv', 'get_weight_csv', 'get_sweater_size_csv', 'get_shoes_size_csv',
        'get_ski_csv', 'get_ski_level_csv', 'get_ski_equipment_csv', 'get_ski_boots_size_csv',
        'get_id_card_number_csv', 'get_id_card_issue_date_csv', 'get_id_card_expiration_date_csv',
        'get_passport_number_csv', 'get_passport_issue_date_csv', 'get_passport_expiration_date_csv',
        'get_license_number_csv', 'get_license_issue_date_csv', 'get_license_expiration_date_csv',
        'get_license_emitted_by_csv', 'get_license_types_csv', 'get_press_card_number_csv', 'get_elenco_csv',
        'get_food_note_csv', 'get_note_csv', 'get_internal_note_csv',
    )
    advanced_search_fields = [
        ('first_name__icontains:Nome', 'last_name__icontains:Cognome',),
        ('email__icontains:Email', 'privacy_acceptance_date__isnull:Privacy mancante',),
        ('birthday__gte:Data nascita dal', 'birthday__lte:Data nascita fino al'),
        ("addresses__locality__icontains:Citta'", 'addresses__province__icontains:Prov',),
        ('addresses__country:Stato',),
        ('phones__number__icontains:Telefono',),
        ('status',),
        ('language',),
        ('company__icontains:Company',),
        ('memberships__newspaper__name__icontains:Testata', "memberships__newspaper__periodicity:Periodicita' testata", ),
        ('memberships__newspaper__categories:Categoria testata', "memberships__newspaper__sectors:Settore testata",),
        ("memberships__newspaper__types:Tipo testata",),
        ('groups:Gruppo',),
        ('invites__event__name__icontains:Nome evento', 'invites__event:Evento',),
        ('invites__event__event_date_start__lte:Eventi fino al', 'invites__event__event_date_start__gte:Eventi dal'),
        ('invites__event__place__icontains:Luogo evento', ),
        ('email:Q search email',),
    ]
    filter_horizontal = ('groups',)
    fieldsets = (
        (_('Logs admin'), {
            'fields': (
                'date_last_modify', 'last_modifier', 'status',
            ),
            'classes': ('collapse',),
        }),
        (_('User'), {
            'fields': ('user',),
            'classes': ('collapse',)
        }),
        (_('Contact'), {
            'fields': (
                ('language', 'is_test_contact',),
                'photo',
                ('title',),
                'last_name',
                'first_name',
                'company',
                'ssn',
                ('email', 'readonly_default_phone',),
                ('birth_place', 'birthday'),
                settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD,
            )
        }),
        (_('Accommodation and food preferences'), {
            'fields': (
                'smoking', 'food', 'food_note'
            )
        }),
        (_('Groups'), {
            'fields': (
                'groups',
            ),
            'classes': ('collapse',)
        }),
        (_('Personal info'), {
            'fields': (
                'shirt_size', 'height', 'weight',
                'sweater_size', 'shoes_size',
                'ski', 'ski_level', 'ski_equipment', 'ski_boots_size',
            ),
            'classes': ('collapse',)
        }),
        (_('Documents'), {
            'fields': (
                'id_card_number', 'id_card_issue_date', 'id_card_expiration_date', 'id_card_scan',
                'passport_number', 'passport_issue_date', 'passport_expiration_date', 'passport_scan',
                'license_number', 'license_issue_date', 'license_expiration_date',
                'license_emitted_by', 'license_types', 'license_scan',
                'press_card_number', 'elenco',
            ),
            'classes': ('collapse',)
        }),
        (_('Note'), {
            'fields': (
                ('note', 'internal_note'),
            ),
        }),
    )
    tabs = (
        (_('properties'), {
            'items': [
                _('Logs admin'),
                _('User'),
                _('Groups'),
                _('Contact'),
                ContactEmailCcAjaxInline,
                PhoneAjaxInline,
                AddressAjaxInline,
                MembershipAjaxInline,
                _('Accommodation and food preferences'),
                CardAjaxInline,
                _('Personal info'),
                SocialMediaAjaxInline,
                _('Documents'),
                _('Note'),
                ContactInviteAjaxInline,
            ],
            'active': True
        }),
    )
    action_form = ContactActionForm
    action_form_fields = {
        'contact_add_to_group': ['contact_group:required'],
        'contact_add_to_event_invited_list': ['event:required'],
    }
    actions = [
        contact_export_csv_async,
        contact_export_xls_async,
        contact_add_to_group,
        contact_add_to_event_invited_list,
    ]
    readonly_fields = ("readonly_default_phone", "user")

    def get_filterset_class(self, request):

        TkFilterset = super(ContactAdmin, self).get_filterset_class(request)

        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.form.fields['invites__event'].widget = autocomplete_light.ChoiceWidget('EventAutocomplete')

                self.form.fields['email'].widget = forms.TextInput(attrs={
                    'placeholder': "Es: ('@gmail')|(('john')&~('doe'))"
                })
                self.filters['email'] = QSearchFilter()

        return TkFilters

    def get_queryset(self, request):
        """
        Check "email_q_filter" in request.GET, if exist the value is manipulated like follow:

        "('publieditor')|('thundersystems')" ==> "Q(email__icontains='publieditor')|Q(email__icontains='thundersystems')"

        and the filter is applied like follow:

        qs.filter(Q(email__icontains='publieditor')|Q(email__icontains='thundersystems'))

        :param request:
        :return:
        """
        qs = super(ContactAdmin, self).get_queryset(request)
        email_q_filter = request.GET.get('q_search_email', None)
        if email_q_filter:
            email_q_filter = email_q_filter.replace("('", "Q(email__icontains='")
            qs = qs.filter(eval(email_q_filter))
        return qs

    def invited_admin_url(self, instance):
        redirect_url = reverse("admin:event_invite_changelist")+"?contact=%s" % instance.id
        return '<a href="%s"><img src="/static/event/img/invites.png" alt="True"></a>' % redirect_url
    invited_admin_url.short_description = _('Invited')
    invited_admin_url.allow_tags = True

    def formfield_for_dbfield(self, db_field, **kwargs):
        image_fields = [
            "photo", "id_card_scan", "passport_scan", "license_scan"
        ]
        if db_field.name in image_fields:
            request = kwargs.pop("request", None)
            kwargs['widget'] = AdminImageWidget
            return db_field.formfield(**kwargs)
        return super(ContactAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    def export_vcard_link(self, contact):
        return u"""
            <a href="%(href)s">
                <img
                    src="/static/img/vcard.png"
                    alt="%(title)s" title="%(title)s"
                />
            </a>""" % {
                'href': reverse(
                    'admin:event_contact_export_vcard',
                    kwargs={'contact_id': contact.id},
                ),
                'title': _('Export vcard'),
            }
    export_vcard_link.allow_tags = True
    export_vcard_link.short_description = _('Vcard')

    def get_urls(self):
        urls = super(ContactAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^export_contact_vcard/(?P<contact_id>\d+)/$',
                self.admin_site.admin_view(export_contact_vcard),
                name='event_contact_export_vcard'
            ),
            url(
                r'^file_import/$',
                self.admin_site.admin_view(ContactImportView.as_view()),
                name='event_contact_import'
            ),
        )
        return my_urls + urls

    def set_test_contact(self, instance):
        toggle_link = reverse(
            'event:publieditor_event_contact_set_test', kwargs={'contact_id': instance.id}, current_app='event'
        )
        if hasattr(instance, 'is_test_contact'):
            if instance.is_test_contact:
                return '<a href="%s"><img src="/static/admin/img/icon-yes.gif" alt="True"></a>' % toggle_link
            else:
                return '<a href="%s"><img src="/static/admin/img/icon-no.gif" alt="False"></a>' % toggle_link
        return "Undefined"
    set_test_contact.short_description = _('Test')
    set_test_contact.allow_tags = True

    def get_default_phone(self, instance):
        return instance.default_phone or ""
    get_default_phone.short_description = _('default phone')

    def readonly_default_phone(self, instance):
        default_phone = instance.default_phone
        if default_phone:
            return u"{}{} ({})".format(
                default_phone.area_code.code if default_phone.area_code else "",
                default_phone.number,
                default_phone.phone_type,
            )
        return u""
    readonly_default_phone.short_description = _('default phone')


class ContactGroupAdmin(LocalExtededModelAdmin):
    search_fields = ('name',)
    list_display = ('get_edit_icon', 'get_status_display', 'name', 'contacts_list', 'get_delete_icon')
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': (
                'ordering', 'name',
            )
        })
    )
    advanced_search_fields = (
        ('status',),
        ('name__icontains:Nome',),
        ('agent__name__icontains:Nome agente',),
        ('contacts:Contatto',),
        ('contacts__first_name__icontains:Nome contatto', 'contacts__last_name__icontains:Cognome contatto'),
        ('contacts__email__icontains:Email contatto',),
    )
    action_form = ContactGroupActionForm
    action_form_fields = {
        'contact_group_add_to_event_invited_list': ['event:required'],
    }
    actions = [
        contact_group_add_to_event_invited_list,
    ]

    def contacts_list(self, instance):
        return '<a href="%s?groups=%s"><img src="/static/event/img/contacts.png" alt="True"></a>' % (
            reverse('admin:event_contact_changelist'), instance.id
        )
    contacts_list.short_description = _('Contacts')
    contacts_list.allow_tags = True


class AgentAdmin(LocalExtededModelAdmin):
    list_display = LIST_DISPLAY_DEFAULT + ('ordering', 'name',)
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {'fields': ('ordering', 'name',)})
    )


class MembershipAdmin(LocalExtededModelAdmin):
    form = MembershipForm
    list_display = LIST_DISPLAY_DEFAULT + ('newspaper', 'role', 'default')
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': ('newspaper', 'role', 'default', 'contact')
        })
    )


class PhoneForm(ConstrainedModelForm):
    hidden_fields = ('contact',)
    area_code = forms.ModelChoiceField(
        queryset=AreaCode.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('AreaCodeAutocomplete'),
        label=_("Area Code"),
    )

    def __init__(self, *args, **kwargs):
        super(PhoneForm, self).__init__(*args, **kwargs)
        self.fields["area_code"].required = True

    def clean_number(self):
        number = self.cleaned_data.get('number')
        if not number.isdigit():
            raise forms.ValidationError(_("Number can't contains letters"))
        return number

    class Meta:
        model = Phone


class PhoneAdmin(LocalExtededModelAdmin):
    form = PhoneForm
    search_fields = ('number',)
    list_display = LIST_DISPLAY_DEFAULT + ('ordering', 'phone_type', 'area_code', 'number', 'default', 'note')
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': (
                'ordering', 'phone_type', 'area_code', 'number', 'default', 'note', 'contact',
            )
        })
    )


class ContactEmailCcForm(ConstrainedModelForm):
    hidden_fields = ('contact',)
    contact_cc = forms.ModelChoiceField(
        queryset=Contact.objects.all(), required=False,
        widget=autocomplete_light.ChoiceWidget('ContactAutocomplete'),
        label=_("Contact"),
    )

    def clean(self):
        cleaned_data = self.cleaned_data
        if not cleaned_data.get('email', None) and not cleaned_data.get('contact_cc', None):
            raise forms.ValidationError(_("Please select a contact or fill the free email field"))
        if cleaned_data.get('email', None) and cleaned_data.get('contact_cc', None):
            raise forms.ValidationError(_("You can't select both contact and email"))
        return cleaned_data

    class Meta:
        model = ContactEmailCc


class ContactEmailCcAdmin(LocalExtededModelAdmin):
    form = ContactEmailCcForm
    list_display = LIST_DISPLAY_DEFAULT + ('email', 'contact_cc', 'note')
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': (
                'default',
                ('contact_cc', 'email', ),
                'note', 'contact',
            )
        })
    )


class ContactAddressForm(ConstrainedModelForm):
    hidden_fields = ('contact',)

    class Meta:
        model = ContactAddress


class ContactAddressAdmin(LocalExtededModelAdmin):
    form = ContactAddressForm
    list_display = LIST_DISPLAY_DEFAULT + (
        'default', 'ordering', 'address_type', 'name',
        'number', 'zip_code', 'locality', 'province', 'country',
    )
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': (
                'default', 'ordering', 'address_scopes', 'address_type', 'name', 'number',
                'zip_code', 'locality', 'province', 'country', 'note', 'contact',
            )
        })
    )


class CardForm(ConstrainedModelForm):
    hidden_fields = ('contact',)
    airline = forms.ModelChoiceField(
        queryset=Airline.objects.all(), required=False,
        widget=autocomplete_light.ChoiceWidget('AirlineAutocomplete'),
        label=_("Airline"),
    )

    def __init__(self, *args, **kwargs):
        super(CardForm, self).__init__(*args, **kwargs)

    def clean_number(self):
        number = self.cleaned_data.get('number')
        if not number.isdigit():
            raise forms.ValidationError(_("Number can't contains letters"))
        return number

    class Meta:
        model = Card


class CardAdmin(LocalExtededModelAdmin):
    form = CardForm
    list_display = LIST_DISPLAY_DEFAULT + (
        'default', 'ordering', 'airline', 'number', 'expiration_date',
    )
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': (
                'default', 'ordering', 'airline', 'number', 'expiration_date', 'contact',
            )
        })
    )


class ContactSocialMediaForm(ConstrainedModelForm):
    hidden_fields = ('contact',)

    class Meta:
        model = Card


class ContactSocialMediaAdmin(LocalExtededModelAdmin):
    form = ContactSocialMediaForm
    list_display = LIST_DISPLAY_DEFAULT + ('social', 'link')
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': ('social', 'link', 'contact')
        })
    )


class LicenseAdmin(LocalExtededModelAdmin):
    list_display = ('get_edit_icon', 'name', 'get_delete_icon',)
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': (
                'name',
            )
        })
    )


class RoleAdmin(LocalExtededModelAdmin):
    list_display = LIST_DISPLAY_DEFAULT + ('ordering', 'name',)
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {'fields': ('ordering', 'name',)})
    )


class NewspaperCategoryAdmin(LocalExtededModelAdmin):
    list_display = LIST_DISPLAY_DEFAULT + ('name',)
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': ('name',)
        })
    )


class NewspaperSectorAdmin(LocalExtededModelAdmin):
    list_display = LIST_DISPLAY_DEFAULT + ('name',)
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': ('name',)
        })
    )


class NewspaperTypeAdmin(LocalExtededModelAdmin):
    list_display = LIST_DISPLAY_DEFAULT + ('name',)
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': ('name',)
        })
    )


class NewspaperAdmin(LocalExtededModelAdmin):
    list_display = LIST_DISPLAY_DEFAULT + ('name', 'periodicity', 'get_link_url')
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': ('ordering', 'name', 'periodicity', 'categories', 'types', 'sectors', 'url')
        })
    )
    search_fields = ("name",)


register_inline(MembershipAjaxInline)
register_inline(ContactEmailCcAjaxInline)
register_inline(PhoneAjaxInline)
register_inline(AddressAjaxInline)
register_inline(CardAjaxInline)
register_inline(SocialMediaAjaxInline)
register_inline(ContactInviteAjaxInline)
register_inline(NoteFromPhoneContactInline)
register_inline(NoteFromPhoneContactInlineReadOnly)
admin.site.register(Organization, OrganizationAdmin)
admin.site.register(Contact, ContactAdmin)
admin.site.register(ContactAddress, ContactAddressAdmin)
admin.site.register(Card, CardAdmin)
admin.site.register(ContactSocialMedia, ContactSocialMediaAdmin)
admin.site.register(ContactGroup, ContactGroupAdmin)
admin.site.register(Agent, AgentAdmin)
admin.site.register(Membership, MembershipAdmin)
admin.site.register(Newspaper, NewspaperAdmin)
admin.site.register(NewspaperCategory, NewspaperCategoryAdmin)
admin.site.register(NewspaperSector, NewspaperSectorAdmin)
admin.site.register(NewspaperType, NewspaperTypeAdmin)


admin.site.register(Phone, PhoneAdmin)
admin.site.register(ContactEmailCc, ContactEmailCcAdmin)
admin.site.register(Role, RoleAdmin)


##############################
# FORMSETS                   #
##############################
class TranslationInlineBaseFormset(forms.models.BaseInlineFormSet):
    def __init__(self, *args, **kwargs):
        super(TranslationInlineBaseFormset, self).__init__(*args, **kwargs)
        instance = kwargs.pop('instance', None)
        if instance:
            try:
                self.language = instance.default_language
            except Language.DoesNotExist:
                self.language = None
            except DatabaseError:
                connection._rollback()

    def get_queryset(self):
        qs = super(TranslationInlineBaseFormset, self).get_queryset()
        # try:
        #     default_language = self.instance.default_language
        #     def compare(obj):
        #             return 1 if obj.language != default_language else -1
        #     qs = sorted(qs, key=compare)
        # except Exception, e:
        #     pass
        return qs

    def clean(self):
        # if len(self.forms) < 1:
        #     raise forms.ValidationError(
        #         _('at least one translation must be added')
        #     )
        if self.language:
            for inline_form in self.forms:
                trans_added = [
                    v for k, v in inline_form.data.iteritems() if
                    k.startswith('translations-')
                    and k.endswith('-language')
                    and not v == ""
                ]
                if not unicode(self.language.id) in trans_added:
                    raise forms.ValidationError(
                        _('the default language (%(language)s) translation must be added') % {
                            'language': self.instance.default_language
                        }
                    )


##############################
# INLINE TRANSLATION CLASSES #
##############################
class EventTranslationInline(admin.StackedInline):

    class Media:
        js = (
            "event/js/admin_event.js",
        )

    extra = 0
    model = EventTranslation
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (_('Main Configuration'), {
            'fields': (
                'language', 'title',
            ),
        }),
        (_('Header and Footer'), {
            'fields': (
                ('header_image', 'footer_image'),
            ),
        }),
        (_('Email'), {
            'fields': (
                ('email_from', 'email_from_name'),
                'header_reply',
                'email_body',
            ),
        }),
        (_('Landing Page'), {
            'fields': (
                'web_body',
            ),
        }),
        ('Disclaimer', {
            'classes': ('collapse',),
            'fields': (
                'disclaimer_text',
                'disclaimer_image',
                ('disclaimer_in_email', 'disclaimer_in_personalized_program'),
                ('disclaimer_in_presentation_step', 'disclaimer_in_schedule_step'),
                ('disclaimer_in_adhesion_step', 'disclaimer_in_register_step'),
                ('disclaimer_in_transport_step', 'disclaimer_in_activity_step'),
                ('disclaimer_in_summary_step', 'disclaimer_in_decline_step'),
                ('disclaimer_in_delegate_step', 'disclaimer_in_documents_step'),
                ('disclaimer_in_greetings_step', 'disclaimer_in_poll_step'),
            ),
        }),
        (_('Signature'), {
            'fields': (
                ('left_signature', 'right_signature'),
            ),
        }),
        (_('External image'), {
            'fields': (
                'bmw_external_images',
            ),
        }),
    )

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name in ["header_image", "footer_image", "disclaimer_image"]:
            request = kwargs.pop("request", None)
            kwargs['widget'] = AdminProportionImageWidget
            return db_field.formfield(**kwargs)
        return super(EventTranslationInline, self).formfield_for_dbfield(db_field, **kwargs)


class ScheduleTranslationInline(admin.StackedInline):
    model = ScheduleTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': ('language', 'body',),
            'classes': ('',)
        }),
    )


class ActivityIntroductionTranslationInline(admin.StackedInline):
    model = ActivityIntroductionTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': ('language', 'title', 'body',),
            'classes': ('',)
        }),
    )


class PrivacyTranslationInline(admin.StackedInline):
    model = PrivacyTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': ('language', 'body',),
            'classes': ('',)
        }),
    )


class TransportTranslationInline(admin.StackedInline):
    model = TransportTranslation
    extra = 0
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'language', 'description',
        )}),
    )


class GreetingsAjaxInline(AjaxInline):
    model = GreetingsTranslation
    fk_name = 'event'
    list_display = (
        'invite_status', 'language', 'get_body',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class AdhesionTranslationInline(admin.StackedInline):
    model = AdhesionTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': ('language', 'body',),
            'classes': ('',)
        }),
    )


class EmailForwardedTranslationInline(admin.StackedInline):
    model = EmailForwardedTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': ('language', 'body',),
            'classes': ('',)
        }),
    )


class FollowupTranslationInline(admin.StackedInline):
    model = FollowUpTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': (
                'title', 'language', 'body',
                ('left_signature', 'right_signature',),
            ),
            'classes': ('',)
        }),
    )


class EventDocumentTranslationInline(admin.StackedInline):
    model = EventDocumentEmailTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': ('title', 'language', 'body', ('left_signature', 'right_signature')),
            'classes': ('',)
        }),
    )


class EventDissuasionMessageTranslationInline(admin.StackedInline):
    model = EventDissuasionMessageTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': ('language', 'deadline_expired_message'),
            'classes': ('',)
        }),
    )


##################
# INLINE CLASSES #
##################
class FlightDetailAjaxInline(AjaxInline):
    model = FlightDetail
    fk_name = "event_travel_option"
    extra = 0
    list_display = (
        'direction', 'flight',
        'get_formatted_departure_date',
        'get_formatted_arrival_date',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class TrainDetailAjaxInline(AjaxInline):
    model = TrainDetail
    fk_name = "event_travel_option"
    extra = 0
    list_display = (
        'direction', 'train',
        'get_formatted_departure_date', 'get_formatted_arrival_date',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class CustomTransportDetailAjaxInline(AjaxInline):
    model = CustomTransportDetail
    fk_name = "event_travel_option"
    extra = 0
    list_display = (
        'direction', 'customtransport',
        'get_formatted_departure_date', 'get_formatted_arrival_date',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class VehicleModelsInlineAdmin(admin.StackedInline):
    fk_name = 'brand'
    model = VehicleModel
    extra = 1
    fieldsets = (
        ('vehicle models', {
            'fields': ([
                ('name', 'year',),
                ('descr', 'note',),

            ])
        }),
    )


class VehicleInlineAdmin(admin.StackedInline):
    fk_name = 'vehicle_model'
    model = Vehicle
    extra = 1
    fieldsets = (
        ('vehicle', {
            'fields': ([
                ('licence_plate', 'vin',),
                ('color',),
            ])
        }),
    )


class VehicleModelsThroughInlineAdmin(admin.StackedInline):
    model = Event.vehicle_models.through
    extra = 1
    fieldsets = (
        ('vehicle models', {
            'fields': (['vehicle_model'])
        }),
    )


class TurnInline(admin.StackedInline):
    model = Turn
    extra = 0
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'status', 'name',
            )
        }),
    )


class GreetingsTranslationForm(ConstrainedModelForm):
    model = GreetingsTranslation
    hidden_fields = ('event',)


class GreetingsTranslationModelAdmin(ExtendibleModelAdmin):
    model = GreetingsTranslation
    form = GreetingsTranslationForm
    list_display = (
        'get_edit_icon',
        'invite_status', 'language', 'body',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'event', 'invite_status', 'language', 'body',
        )}),
    )


class DocumentModelAdmin(ExtendibleModelAdmin):
    model = Document
    list_display = (
        'get_edit_icon',
        'category',
        'name',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (_('main'), {
            'fields': (
                'category', 'name', 'document',
            )
        }),
    )


class EventQuestionForm(ConstrainedModelForm):
    model = EventQuestion
    hidden_fields = ('event',)


class RegisterEventQuestionProxyForm(ConstrainedModelForm):
    model = RegisterEventQuestionProxy
    hidden_fields = ('event',)

    def __init__(self, *args, **kwargs):
        super(RegisterEventQuestionProxyForm, self).__init__(*args, **kwargs)
        self.fields["question"].queryset = Question.objects.filter(typology="R")


class ActivityEventQuestionProxyForm(ConstrainedModelForm):
    model = ActivityEventQuestionProxy
    hidden_fields = ('event',)

    def __init__(self, *args, **kwargs):
        super(ActivityEventQuestionProxyForm, self).__init__(*args, **kwargs)
        self.fields["question"].queryset = Question.objects.filter(typology="A")


class DeclineEventQuestionProxyForm(ConstrainedModelForm):
    model = DeclineEventQuestionProxy
    hidden_fields = ('event',)

    def __init__(self, *args, **kwargs):
        super(DeclineEventQuestionProxyForm, self).__init__(*args, **kwargs)
        self.fields["question"].queryset = Question.objects.filter(typology="D")


class PollEventQuestionProxyForm(ConstrainedModelForm):
    model = DeclineEventQuestionProxy
    hidden_fields = ('event',)

    def __init__(self, *args, **kwargs):
        super(PollEventQuestionProxyForm, self).__init__(*args, **kwargs)
        self.fields["question"].queryset = Question.objects.filter(typology="P")


class EventQuestionModelAdmin(ExtendibleModelAdmin):
    model = EventQuestion
    form = EventQuestionForm
    advanced_search_fields = [
        ['event', 'question__question__icontains'],
        ['enabled', 'question__typology:Tipologia domanda']
    ]

    list_display = (
        'get_edit_icon',
        'get_delete_icon',
        'event', 'question',
    )
    fieldsets = (
        ('Additional information', {
            'classes': ('collapse',),
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',)
        }),
        (None, {'fields': (
            'event', 'question',
            'ordering',
            ('enabled', 'required'),
        )}),
    )

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return ('event',)
        return super(EventQuestionModelAdmin, self).get_readonly_fields(request, obj)

    def get_filterset_class(self, request):
        TkFilterset = super(EventQuestionModelAdmin, self).get_filterset_class(request)
        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.form.fields['event'].widget = autocomplete_light.ChoiceWidget('EventAutocomplete')
        return TkFilters


class RegisterEventQuestionProxyModelAdmin(EventQuestionModelAdmin):

    class Media:
        js = (
            "event/js/change_add_question_url_register_event_question.js",
        )

    model = RegisterEventQuestionProxy
    form = RegisterEventQuestionProxyForm


class ActivityEventQuestionProxyModelAdmin(EventQuestionModelAdmin):

    class Media:
        js = (
            "event/js/change_add_question_url_activity_event_question.js",
        )

    model = ActivityEventQuestionProxy
    form = ActivityEventQuestionProxyForm


class DeclineEventQuestionProxyModelAdmin(EventQuestionModelAdmin):

    class Media:
        js = (
            "event/js/change_add_question_url_decline_event_question.js",
        )

    model = DeclineEventQuestionProxy
    form = DeclineEventQuestionProxyForm


class PollEventQuestionProxyModelAdmin(EventQuestionModelAdmin):

    class Media:
        js = (
            "event/js/change_add_question_url_poll_event_question.js",
        )

    model = PollEventQuestionProxy
    form = PollEventQuestionProxyForm


class EventDocumentForm(ConstrainedModelForm):
    model = EventDocument
    hidden_fields = ('event',)


class EventDocumentModelAdmin(ExtendibleModelAdmin):
    model = EventDocument
    form = EventDocumentForm
    list_display = ('event', 'document', 'attach_in_mail')
    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (_('main'), {'fields': (
            'event', 'document', 'attach_in_mail'
        )}),
    )
    tabs = [
        (_('main'), {'items': [_('main')], 'active': True}),
        (_('logs'), {'items': [_('logs admin')]}),
    ]


class EventDocumentAjaxInline(AjaxInline):
    model = EventDocument
    fk_name = "event"
    list_display = ('document', 'attach_in_mail', 'get_edit_icon_iframe', 'get_delete_icon_iframe',)
    width = 850
    height = 650


class NewsletterDocumentForm(ConstrainedModelForm):
    model = NewsletterDocument
    hidden_fields = ('event',)


class NewsletterDocumentModelAdmin(ExtendibleModelAdmin):
    model = NewsletterDocument
    form = NewsletterDocumentForm
    list_display = ('event', 'title', 'language', 'use_always')
    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (_('main'), {'fields': (
            'event', 'document', 'title', 'language', 'use_always'
        )}),
    )
    tabs = [
        (_('main'), {'items': [_('main')], 'active': True}),
        (_('logs'), {'items': [_('logs admin')]}),
    ]


class NewsletterDocumentAjaxInline(AjaxInline):
    model = NewsletterDocument
    fk_name = "event"
    list_display = (
        'title', 'language', 'use_always',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )
    width = 850
    height = 650


class InviteDocumentForm(ConstrainedModelForm):
    model = InviteDocument
    hidden_fields = ('invite',)


class InviteDocumentModelAdmin(ExtendibleModelAdmin):
    model = InviteDocument
    form = InviteDocumentForm
    list_display = ('invite', 'name', 'document')
    fieldsets = (
        ('Additional information', {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (None, {'fields': (
            'invite', 'name', 'document',
        )}),
        (_('Creator'), {
            'fields': ('created_from_travel_agency', 'created_from_logistic_agency', 'created_from_hostess')
        })
    )
    readonly_fields = ('created_from_travel_agency', 'created_from_logistic_agency', 'created_from_hostess')


class InviteDocumentTravelAgencyProxyForm(ConstrainedModelForm):
    model = InviteDocumentTravelAgencyProxy
    hidden_fields = ('invite',)

    def __init__(self, *args, **kwargs):
        if not self.invite_id:
            # no ajax inline: no invite field hidden
            self.hidden_fields = tuple()

            # no ajax inline: custom queryset is created
            super(InviteDocumentTravelAgencyProxyForm, self).__init__(*args, **kwargs)
            self.fields['invite'].queryset = TravelAgencyInvite.objects.filter(
                event__travel_agencies=self.user
            )
        else:
            super(InviteDocumentTravelAgencyProxyForm, self).__init__(*args, **kwargs)


class InviteDocumentTravelAgencyProxyAdmin(ExtendibleModelAdmin):
    model = InviteDocumentTravelAgencyProxy
    list_display = ('get_event', 'invite', 'name', 'document', 'get_edit_icon', 'get_delete_icon',)
    fieldsets = (
        ('Additional information', {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (None, {'fields': (
            'invite', 'name', 'document',
        )}),
    )

    def get_event(self, obj):
        return obj.invite.event
    get_event.short_description = _('event')
    get_event.admin_order_field = 'invite__event__name'

    def get_invited(self, obj):
        return obj.invite.contact
    get_invited.short_description = _('invited')

    def get_queryset(self, request):
        qs = super(InviteDocumentTravelAgencyProxyAdmin, self).get_queryset(request)
        if not request.user.is_superuser:
            qs = qs.filter(invite__event__travel_agencies=request.user)
        return qs

    def save_model(self, request, obj, form, change):
        obj.created_from_travel_agency = True
        obj.save()

    advanced_search_fields = (
        ('invite__contact__first_name__icontains', 'invite__contact__last_name__icontains', 'invite__contact__email__icontains',),
        ('invite__event__event_date_start', 'invite__event__event_date_end'),
        ('invite__event__name__icontains',),
    )

    def get_filterset_class(self, request):
        Filterset = super(InviteDocumentTravelAgencyProxyAdmin, self).get_filterset_class(request)

        class TkFilters(Filterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.filters['invite__contact__first_name__icontains'].label = _('Contact first name (contains)')
                self.filters['invite__contact__last_name__icontains'].label = _('Contact last name (contains)')
                self.filters['invite__contact__email__icontains'].label = _('Contact email (contains)')
                self.filters['invite__event__event_date_start'].label = _('Event start date')
                self.filters['invite__event__event_date_end'].label = _('Event end date')
                self.filters['invite__event__name__icontains'].label = _('Event name (contains)')
        return TkFilters

    def get_form(self, request, obj=None, **kwargs):
        InviteDocumentTravelAgencyProxyForm.invite_id = request.GET.get('invite', None)
        InviteDocumentTravelAgencyProxyForm.user = request.user
        kwargs['form'] = InviteDocumentTravelAgencyProxyForm
        return super(InviteDocumentTravelAgencyProxyAdmin, self).get_form(request, obj, **kwargs)


class InviteDocumentLogisticAgencyProxyForm(ConstrainedModelForm):
    model = InviteDocumentLogisticAgencyProxy
    hidden_fields = ('invite',)


class InviteDocumentLogisticAgencyProxyAdmin(ExtendibleModelAdmin):
    model = InviteDocumentLogisticAgencyProxy
    form = InviteDocumentLogisticAgencyProxyForm
    list_display = ('invite', 'name', 'document')
    fieldsets = (
        ('Additional information', {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (None, {'fields': (
            'invite', 'name', 'document',
        )}),
    )

    def save_model(self, request, obj, form, change):
        obj.created_from_logistic_agency = True
        obj.save()


class InviteDocumentHostessProxyForm(ConstrainedModelForm):
    model = InviteDocumentHostessProxy
    hidden_fields = ('invite',)


class InviteDocumentHostessProxyAdmin(ExtendibleModelAdmin):
    model = InviteDocumentHostessProxy
    form = InviteDocumentHostessProxyForm
    list_display = ('invite', 'name', 'document')
    fieldsets = (
        ('Additional information', {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (None, {'fields': (
            'invite', 'name', 'document',
        )}),
    )

    def save_model(self, request, obj, form, change):
        obj.created_from_hostess = True
        obj.save()


class InviteDocumentAjaxInline(AjaxInline):
    model = InviteDocument
    fk_name = "invite"
    list_display = (
        'name', 'document',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )
    width = 850
    height = 650


class InviteTravelAgencyDocumentAjaxInline(InviteDocumentAjaxInline):
    model = InviteDocumentTravelAgencyProxy
    queryset_kwargs = {'created_from_travel_agency': True}

    def get_queryset(self, request):
        qs = super(InviteTravelAgencyDocumentAjaxInline, self).get_queryset(request)
        qs = qs.filter(**self.queryset_kwargs)
        return qs


class InviteLogisticAgencyDocumentAjaxInline(InviteDocumentAjaxInline):
    model = InviteDocumentLogisticAgencyProxy
    queryset_kwargs = {'created_from_logistic_agency': True}

    def get_queryset(self, request):
        qs = super(InviteLogisticAgencyDocumentAjaxInline, self).get_queryset(request)
        qs = qs.filter(**self.queryset_kwargs)
        return qs


class InviteHostessDocumentAjaxInline(InviteDocumentAjaxInline):
    model = InviteDocumentHostessProxy
    queryset_kwargs = {'created_from_hostess': True}

    def get_queryset(self, request):
        qs = super(InviteHostessDocumentAjaxInline, self).get_queryset(request)
        qs = qs.filter(**self.queryset_kwargs)
        return qs


class EventPlaceAjaxInline(AjaxInline):
    model = EventPlace
    fk_name = "event"
    list_display = (
        'get_date', 'location', 'name', 'number', 'zip_code', 'locality', 'province', 'country',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class EventTurnProxyAjaxInline(AjaxInline):
    model = EventTurnProxy
    fk_name = "event"
    list_display = (
        'get_status_display', 'code', 'title', 'maximum_participants', 'get_available_seats', 'turn_start', 'turn_end',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class EventStartPlaceAjaxInline(AjaxInline):
    model = EventStartPlace
    fk_name = "event"
    list_display = (
        'name',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )


class EventTravelOptionAjaxInline(AjaxInline):
    model = EventTravelOption
    fk_name = "event"
    list_display = (
        'ordering', 'name', 'turn', 'start_place', 'get_render_transport_group',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )
    width = 800
    height = 800


class RegisterQuestionAjaxInline(AjaxInline):
    model = RegisterEventQuestionProxy
    fk_name = 'event'
    list_display = (
        'ordering', 'get_link_enabled', 'get_link_required', 'get_category',
        'question', 'get_question_qtype', 'get_choices',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )

    def get_queryset(self, request):
        if self.single_child:
            return super(RegisterQuestionAjaxInline, self).get_queryset(request)
        return super(RegisterQuestionAjaxInline, self).get_queryset(request).filter(question__typology="R")


class ActivityQuestionAjaxInline(AjaxInline):
    model = ActivityEventQuestionProxy
    fk_name = 'event'
    list_display = (
        'ordering', 'get_link_enabled', 'get_link_required', 'get_category',
        'question', 'get_question_qtype', 'get_choices',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )

    def get_queryset(self, request):
        if self.single_child:
            return super(ActivityQuestionAjaxInline, self).get_queryset(request)
        return super(ActivityQuestionAjaxInline, self).get_queryset(request).filter(question__typology="A")


class DeclineQuestionAjaxInline(AjaxInline):
    model = DeclineEventQuestionProxy
    fk_name = 'event'
    list_display = (
        'ordering', 'get_link_enabled', 'get_link_required', 'get_category',
        'question', 'get_question_qtype', 'get_choices',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )

    def get_queryset(self, request):
        if self.single_child:
            return super(DeclineQuestionAjaxInline, self).get_queryset(request)
        return super(DeclineQuestionAjaxInline, self).get_queryset(request).filter(question__typology="D")


class PollQuestionAjaxInline(AjaxInline):
    model = PollEventQuestionProxy
    fk_name = 'event'
    list_display = (
        'ordering', 'get_link_enabled', 'get_link_required', 'get_category',
        'question', 'get_question_qtype', 'get_choices',
        'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )

    def get_queryset(self, request):
        if self.single_child:
            return super(PollQuestionAjaxInline, self).get_queryset(request)
        return super(PollQuestionAjaxInline, self).get_queryset(request).filter(question__typology="P")


#################
# ADMIN CLASSES #
#################
class EventAdmin(ExtendibleModelAdmin):

    class Media:
        js = (
            "event/js/admin_dajaxice.js",
            "event/js/event_hide_m2m_add_links.js",
            "event/js/collapsed_stacked_inlines.js",
            "event/js/prevent_close.js",
            "event/js/event_use_type.js",
            "event/js/admin_event_privacy.js",
        )
        css = {
            "all": (
                "event/css/admin_event.css",
            )
        }

    form = EventAdminForm
    ordering = ('-event_date_start',)
    save_on_top = True
    search_fields = ('name', 'place')
    advanced_search_fields = (
        ('status', 'name__icontains',),
        ('event_date_start__gte:Data inizio', 'event_date_start__lte:Data inizio'),
        ('event_date_end__gte:Data fine', 'event_date_end__lte:Data fine'),
        ('event_adhesion_deadline__gte:Data deadline adesione', 'event_adhesion_deadline__lte:Data deadline adesione'),
        ('place__icontains',),
        ('invites__contact:',),
        (
            'invites__contact__first_name__icontains:Nome contatto',
            'invites__contact__last_name__icontains:Cognome contatto',
        ),
        ('invites__contact__email__icontains:Email contatto',),
    )
    list_display = (
        'get_edit_icon',
        'invited_admin_url',
        'name',
        'get_date_start_cl',
        'get_date_end_cl',
        'get_adhesion_deadline_cl',
        'get_event_views',
        'statistics_link',
        'invite_emails_admin_url',
        'invite_emails_mailing_status_admin_url',
        'documents_emails_admin_url',
        # 'travel_agency_admin_url',
        # 'logistic_agency_admin_url',
        'agency_admin_urls',
        'poll_answers_admin_url',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (_('main'), {
            'fields': (
                'status',
                'name',
                'place',
                'default_language',
                'landing_page_template',
                ('event_date_start', 'event_date_end',),
                'event_adhesion_deadline',
                'slug',
            ),
        }),
        (_('privacy'), {
            'fields': (
                'is_privacy', 'adhesion_in_email', 'has_manual_adhesion',
            ),
        }),
        (_('settings'), {
            'fields': (
                # ('invites_send_from', 'invites_send_to'),
                'start_step',
                'show_breadcrumbs',
                'breadcrumbs_position',
                'is_newsletter',
                'header_color',
                'show_presentation', 'show_schedule', 'show_adhesion', 'show_register',
                'show_transport', 'show_activity', 'show_summary', 'show_decline',
                'show_delegate', 'show_forwarding', 'show_documents', 'show_greetings',
            ),
        }),
        (_('phone'), {
            'fields': (
                ('ask_phone', 'is_phone_required'),
                'phone_question_type',
            ),
        }),
        (_('address'), {
            'fields': (
                ('ask_address', 'is_address_required'),
                'address_question_type',
            ),
        }),
        (_('birthday'), {
            'fields': (
                ('ask_birthday', 'is_birthday_required'),
                ('ask_birth_place', 'is_birth_place_required'),
            ),
        }),
        (_('ssn'), {
            'fields': (
                ('ask_ssn', 'is_ssn_required'),
            ),
        }),
        (_('frequent flyer'), {
            'fields': (
                ('ask_frequent_flyer', 'is_frequent_flyer_required'),
                'frequent_flyer_question_type',
            ),
        }),
        (_('official personal documents'), {
            'fields': (
                ('is_id_card_or_passport_required',),
                ('ask_id_card', 'is_id_card_required'),
                ('ask_passport', 'is_passport_required'),
            ),
        }),
        (_('documents'), {
            'fields': (
                ('ask_license', 'is_license_required'),
                ('ask_press_card_number', 'is_press_card_number_required'),
            ),
        }),
        (_('additional questions'), {
            'fields': (
                ('ask_food_hotel_preferences', 'is_food_hotel_preferences_required'),
                ('ask_clothing', 'is_clothing_required'),
                ('ask_body_measurements', 'is_body_measurements_required'),
                ('ask_ski', 'is_ski_required')
            ),
        }),
        (_('Transport'), {
            'fields': (
                ('enable_own', 'enable_free'),
            ),
        }),
        (_('Action'), {
            'fields': (
                'enable_delegate',
                ('enable_forwarding', 'max_limit_forwarders'),
            ),
        }),
        (_('Documents'), {
            'fields': (
                ('enable_personalized_program', 'enable_ics',),
            ),
        }),
        (_('followup settings'), {
            'fields': (
                ('ask_poll_on_followup',),
            ),
        }),
        (_('agencies'), {
            'fields': (
                'travel_agencies', 'logistic_agencies', 'hostess'
            ),
        }),
    )
    inlines = [
        EventTranslationInline,
        ScheduleTranslationInline,
        ActivityIntroductionTranslationInline,
        PrivacyTranslationInline,
        AdhesionTranslationInline,
        EmailForwardedTranslationInline,
        FollowupTranslationInline,
        EventDocumentTranslationInline,
        EventDissuasionMessageTranslationInline,
        VehicleModelsThroughInlineAdmin,
    ]
    agencies_tab = _('agencies'), {'items': [_('agencies')]}
    tabs = [
        (_('main'), {'items': [_('main')], 'active': True}),
        (_('settings'), {'items': [_('privacy'), _('settings')]}),
        (_('presentation'), {'items': [EventTranslation]}),
        (_('schedule'), {'items': [ScheduleTranslation]}),
        (_('privacy'), {'items': [PrivacyTranslation]}),
        (_('adhesion'), {'items': [AdhesionTranslation, _('Action'), EmailForwardedTranslation]}),
        (_('register'),
            {
                'items': [
                    _('phone'),
                    _('address'),
                    _('birthday'),
                    _('ssn'),
                    _('official personal documents'),
                    _('documents'),
                    _('frequent flyer'),
                    _('additional questions'),
                ]
            }
         ),
        (_('vehicle models'), {'items': [VehicleModelsThroughInlineAdmin]}),
        (_('places'), {'items': [EventPlaceAjaxInline]}),
        (_('turns'), {'items': [EventTurnProxyAjaxInline]}),
        (_('start places'), {'items': [EventStartPlaceAjaxInline]}),
        (_('travel options'), {'items': [_('Transport'), EventTravelOptionAjaxInline]}),
        (_('activity'), {'items': [ActivityIntroductionTranslationInline, ActivityQuestionAjaxInline]}),
        (_('newsletter doc'), {'items': [NewsletterDocumentAjaxInline]}),
        (_('documents'), {'items': [EventDocumentEmailTranslation, EventDocumentAjaxInline, _('Documents')]}),
        (_('decline'), {'items': [DeclineQuestionAjaxInline]}),
        (_('greetings'), {'items': [GreetingsAjaxInline]}),
        (_('followup'), {'items': [_('followup settings'), FollowUpTranslation]}),
        (_('poll'), {'items': [PollQuestionAjaxInline]}),
        (agencies_tab),
        (_('dissuasion messages'), {'items': [EventDissuasionMessageTranslation]}),
        (_('logs'), {'items': [_('logs admin')]}),
    ]
    filter_horizontal = ('travel_agencies', 'logistic_agencies', 'hostess')
    exclude = ('documents',)
    action_form = EventActionForm
    action_form_fields = {
        # 'event_duplicate_event': [],
        'event_send_invites': ['test'],
        'event_resend_invites': ['test'],
        'event_send_followups': ['test'],
        'event_send_documents': ['test'],
    }
    actions = [
        'event_duplicate_event',
        event_send_invites,
        event_resend_invites,
        event_send_followups,
        event_send_documents,
    ]

    def formfield_for_manytomany(self, db_field, request=None, **kwargs):
        travel_agencies_group, logistic_agencies_group = 'Agenzia Viaggi', 'Agenzia Logistica'
        hostess_group = 'Hostess'
        if db_field.name == "travel_agencies":
            kwargs["queryset"] = User.objects.filter(groups__name__icontains=travel_agencies_group)
        elif db_field.name == "logistic_agencies":
            kwargs["queryset"] = User.objects.filter(groups__name__icontains=logistic_agencies_group)
        elif db_field.name == "hostess":
            kwargs["queryset"] = User.objects.filter(groups__name__icontains=hostess_group)
        return super(EventAdmin, self).formfield_for_manytomany(db_field, request, **kwargs)

    def poll_answers_admin_url(self, instance):
        if PollAnswer.objects.filter(invite__event=instance).count():
            return u"<a href='{}'><img src='{}' alt='True'></a>".format(
                reverse('admin:event_poll_answers', kwargs={'event_id': instance.id}),
                static('event/img/poll.png')
            )
        return ""
    poll_answers_admin_url.short_description = 'Poll'
    poll_answers_admin_url.allow_tags = True

    def get_filterset_class(self, request):
        TkFilterset = super(EventAdmin, self).get_filterset_class(request)
        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.form.fields['invites__contact'].widget = autocomplete_light.ChoiceWidget('ContactAutocomplete')
        return TkFilters

    def invite_emails_admin_url(self, instance):
        redirect_url = reverse("admin:event_inviteemailobject_changelist")+"?invite__event=%s" % instance.id
        return '<a href="%s"><img src="/static/event/img/mail.png" alt="True"></a>' % redirect_url
    invite_emails_admin_url.short_description = _('Emails')
    invite_emails_admin_url.allow_tags = True

    def invite_emails_mailing_status_admin_url(self, instance):
        redirect_url = reverse(
            "admin:event_mailingstatusproxy_changelist"
        )+"?email_object__inviteemailobject__invite__event=%s" % instance.id
        return '<a href="%s"><img src="/static/event/img/mail.png" alt="True"></a>' % redirect_url
    invite_emails_mailing_status_admin_url.short_description = _('Email St.')
    invite_emails_mailing_status_admin_url.allow_tags = True

    def documents_emails_admin_url(self, instance):
        redirect_url = "{}?invite__event={}".format(
            reverse("admin:event_documentsemailobject_changelist"),
            instance.id
        )
        return '<a href="%s"><img src="/static/event/img/mail.png" alt="True"></a>' % redirect_url
    documents_emails_admin_url.short_description = _('D. Emails')
    documents_emails_admin_url.allow_tags = True

    def travel_agency_admin_url(self, instance):
        return u"<a href='{}'><img src='{}' alt='True'></a>".format(
            reverse('admin:event_travel_agency', kwargs={'event_id': instance.id}),
            static('event/img/plane.png')
        )
    travel_agency_admin_url.short_description = _('Travel Agency')
    travel_agency_admin_url.allow_tags = True

    def logistic_agency_admin_url(self, instance):
        return u"<a href='{}'><img src='{}' alt='True'></a>".format(
            reverse('admin:event_logistic_agency', kwargs={'event_id': instance.id}),
            static('event/img/car.png')
        )
    logistic_agency_admin_url.short_description = _('Logistic Agency')
    logistic_agency_admin_url.allow_tags = True

    def hostess_admin_url(self, instance):
        return u"<a href='{}'><img src='{}' alt='True'></a>".format(
            reverse('admin:event_hostess', kwargs={'event_id': instance.id}),
            static('event/img/car.png')
        )
    hostess_admin_url.short_description = _('Hostess')
    hostess_admin_url.allow_tags = True

    def agency_admin_urls(self, instance):
        res = self.travel_agency_admin_url(instance) + self.logistic_agency_admin_url(instance)
        # res = res + self.hostess_admin_url(instance)
        return res
    agency_admin_urls.short_description = _('Agency')
    agency_admin_urls.allow_tags = True

    def event_duplicate_event(self, request, queryset):
        from jmb.core.utils import models_utils
        if queryset.count() > 1:
            messages.info(request, _("You must select just one event."))
            return redirect(reverse("admin:event_event_changelist"))
        event = queryset.all()[0]

        event.name = event.name[:90]
        done, msg, new_event = models_utils.ModelObjectCopier(
            event,
            related_parent_name='event',
            related_managers_excludes=[
                'invites', 'travel_options', 'event_statistic', 'event_status_data_email_statistic'
            ],
        ).bulk_copy()
        url = reverse("admin:event_event_change", args=[new_event.id])
        return redirect(url)
    event_duplicate_event.short_description = _("Duplicate event")

    def changelist_view(self, request, extra_context=None):
        queryset_active = Event.objects.filter(status=1).order_by('-event_date_start')
        extra_context = extra_context or {}
        extra_context["active_events"] = queryset_active
        return super(EventAdmin, self).changelist_view(request, extra_context)

    def statistics_link(self, event):
        if InviteEmailObject.objects.filter(invite__event=event).count():
            return u"""
                <a href="%(href)s">
                    <img
                        src="/static/img/statistics.png"
                        alt="%(title)s" title="%(title)s"
                    />
                </a>""" % {
                'href': reverse(
                    'admin:event_statistics',
                    args=[event.id, '99', '9999', '99', '99']
                ),
                'title': _('View statistics')
            }
        return _(' ')
    statistics_link.allow_tags = True
    statistics_link.short_description = _('St')

    def get_urls(self):
        urls = super(EventAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^statistics/(?P<event_id>\d+)/(?P<status>[0-9]+)/(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})/$',
                self.admin_site.admin_view(event_statistics),
                name='event_statistics'
            ),
            url(
                r'^statistics/update$',
                self.admin_site.admin_view(event_statistics_update),
                name='event_statistics_update'
            ),
            url(
                r'^initialize_event/$',
                self.admin_site.admin_view(initialize_event),
                name='event_initialize_event'
            ),
            # viste per visualizzazione generico invito evento
            url(
                r'^event_invite_email/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(event_email_invite_preview),
                name='show_event_invite_email'
            ),
            url(
                r'^event_landing_page_presentation/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(event_landing_page_presentation_preview),
                name='show_event_presentation_preview'
            ),
            url(
                r'^event_landing_page_schedule/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(event_landing_page_schedule_preview),
                name='show_event_schedule_preview'
            ),
            url(
                r'^event_landing_page_adhesion/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(event_landing_page_adhesion_preview),
                name='show_event_adhesion_preview'
            ),
            url(
                r'^event_poll_answers/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(EventPollAnswersView.as_view()),
                name='event_poll_answers'
            ),
            url(
                r'^event_travel_agency/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(EventTravelAgencyView.as_view()),
                name='event_travel_agency'
            ),
            url(
                r'^event_travel_agency_export/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(EventTravelAgencyExportView.as_view()),
                name='event_travel_agency_export'
            ),
            url(
                r'^event_logistic_agency/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(EventLogisticAgencyView.as_view()),
                name='event_logistic_agency'
            ),
            url(
                r'^event_logistic_agency_export/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(EventLogisticAgencyExportView.as_view()),
                name='event_logistic_agency_export'
            ),
            url(
                r'^event_logistic_agency_export_triumph/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(EventLogisticAgencyTriumphExportView.as_view()),
                name='event_logistic_agency_export_triumph'
            ),
            # TODO
            url(
                r'^event_hostess/(?P<event_id>\d+)/$',
                self.admin_site.admin_view(EventHostessView.as_view()),
                name='event_hostess'
            ),
            # url(
            #     r'^event_hostess_export/(?P<event_id>\d+)/$',
            #     self.admin_site.admin_view(EventHostessExportView.as_view()),
            #     name='event_hostess_export'
            # ),
        )
        return my_urls + urls

    def get_actions(self, request, *args, **kw):
        actions = super(EventAdmin, self).get_actions(request, *args, **kw)
        try:
            del actions['set_disactive']
            del actions['set_active']
            del actions['export_csv']
        except Exception, e:
            pass
        return actions

    def invited_admin_url(self, instance):
        redirect_url = reverse("admin:event_invite_changelist")+"?event=%s" % instance.id
        return '<a href="%s"><img src="/static/event/img/invites.png" alt="True"></a>' % redirect_url
    invited_admin_url.short_description = _('Invited')
    invited_admin_url.allow_tags = True

    def get_event_views(self, instance):
        link_preview = "#"
        if instance.start_step == "presentation":
            link_preview = reverse('admin:show_event_presentation_preview', args=(instance.id,))
        elif instance.start_step == "schedule":
            link_preview = reverse('admin:show_event_schedule_preview', args=(instance.id,))
        elif instance.start_step == "adhesion":
            link_preview = reverse('admin:show_event_adhesion_preview', args=(instance.id,))

        preview_button = ""
        if link_preview != "#" and not instance.is_privacy and not instance.is_newsletter:
            preview_button = "<a href='%s' title='landing page'>\
            <img alt='Landing' src='/static/event/img/landing.png'>\
            </a>&nbsp;&nbsp;" % link_preview

        views_links = "%s\
        <a href='%s' title='html email'>\
            <img alt='HTML invite' src='/static/event/img/mail.png'>\
        </a>" % (
            preview_button,
            reverse('admin:show_event_invite_email', args=(instance.id,)),
        )
        return views_links
    get_event_views.short_description = _('view')
    get_event_views.allow_tags = True


class BaseEventAgencyProxyAdmin(ExtendibleModelAdmin):
    ordering = ('-event_date_start',)
    search_fields = ('name',)
    actions = None
    advanced_search_fields = (
        ('event_date_start', 'event_date_end'),
        ('name__icontains',),
    )

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def travel_agency_admin_url(self, instance):
        return u"<a href='{}'><img src='{}' alt='True'></a>".format(
            reverse('admin:event_travel_agency', kwargs={'event_id': instance.id}),
            static('event/img/plane.png')
        )

    travel_agency_admin_url.short_description = _('Agency')
    travel_agency_admin_url.allow_tags = True


class EventTravelAgencyProxyAdmin(BaseEventAgencyProxyAdmin):
    list_display = (
        'invited_admin_url',
        'get_date_start_cl',
        'get_date_end_cl',
        'get_adhesion_deadline_cl',
        # 'travel_agency_admin_url',
    )

    def get_queryset(self, request):
        qs = super(EventTravelAgencyProxyAdmin, self).get_queryset(request)
        qs = qs.filter(travel_agencies=request.user)
        return qs

    def invited_admin_url(self, instance):
        return u"<a href='{redirect_url}'>{event_name}</a>".format(
            redirect_url=reverse('admin:event_travel_agency', kwargs={'event_id': instance.id}),
            event_name=instance.name,
        )
    invited_admin_url.short_description = _('Invited')
    invited_admin_url.allow_tags = True


class EventLogisticAgencyProxyAdmin(BaseEventAgencyProxyAdmin):
    list_display = (
        'invited_admin_url',
        'get_date_start_cl',
        'get_date_end_cl',
        'get_adhesion_deadline_cl',
        # 'travel_agency_admin_url',
    )

    def get_queryset(self, request):
        qs = super(EventLogisticAgencyProxyAdmin, self).get_queryset(request)
        qs = qs.filter(logistic_agencies=request.user)
        return qs

    def invited_admin_url(self, instance):
        if instance.slug == settings.EVENT_EVENT_TRIUMPH_SLUG:
            redirect_url = reverse('admin:event_logisticagencyinvite_changelist')
        else:
            redirect_url = reverse('admin:event_logistic_agency', kwargs={'event_id': instance.id})
        return u"<a href='{redirect_url}'>{event_name}</a>".format(
            redirect_url=redirect_url,
            event_name=instance.name,
        )
    invited_admin_url.short_description = _('Invited')
    invited_admin_url.allow_tags = True


class EventHostessProxyAdmin(BaseEventAgencyProxyAdmin):
    list_display = (
        'invited_admin_url',
        'get_date_start_cl',
        'get_date_end_cl',
        'get_adhesion_deadline_cl',
        # 'travel_agency_admin_url',
    )

    def get_queryset(self, request):
        qs = super(EventHostessProxyAdmin, self).get_queryset(request)
        qs = qs.filter(hostess=request.user)
        return qs

    def invited_admin_url(self, instance):
        return u"<a href='{redirect_url}'>{event_name}</a>".format(
            redirect_url=reverse('admin:event_hostessinvite_changelist'),
            event_name=instance.name,
        )
    invited_admin_url.short_description = _('Invited')
    invited_admin_url.allow_tags = True


class TransportAdmin(ExtendibleModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                ('status',),
                ('name',),
            )
        }),
    )


class EventTurnProxyAdmin(ExtendibleModelAdmin):
    model = EventTurnProxy
    form = EventTurnProxyAdminForm
    list_display = (
        'get_edit_icon',
        'get_status_display',
        'code',
        'title',
        'turn_start',
        'turn_end',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (_('main'), {
            'fields': (
                'event',
                'status',
                'code',
                'title',
                ('turn_start', 'turn_end'),
                'maximum_participants',
            )
        }),
    )
    tabs = (
        (_('main'), {'items': [_('main')], 'active': True}),
        (_('logs'), {'items': [_('logs admin')]}),
    )


class EventStartPlaceAdmin(ExtendibleModelAdmin):
    model = EventStartPlace
    form = EventStartPlaceAdminForm
    list_display = (
        'get_edit_icon',
        'name',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (_('main'), {
            'fields': (
                'event',
                'name',
            )
        }),
    )
    tabs = (
        (_('main'), {'items': [_('main')], 'active': True}),
        (_('logs'), {'items': [_('logs admin')]}),
    )


class EventTravelOptionAdmin(ExtendibleModelAdmin):

    class Media:
        js = (
            "event/js/admin_dajaxice.js",
        )

    model = EventTravelOption
    form = EventTravelOptionAdminForm
    list_display = (
        'get_edit_icon',
        'turn',
        'start_place',
        'name',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (_('main'), {
            'fields': (
                'event',
                ('turn', 'start_place',),
                ('name', 'ordering',),
            )
        }),
    )
    tabs = (
        (_('main'), {
            'items': [
                _('main'),
                FlightDetailAjaxInline,
                TrainDetailAjaxInline,
                CustomTransportDetailAjaxInline,
            ], 'active': True}),
        (_('logs'), {'items': [_('logs admin')]}),
    )

    def render_change_form(self, request, context, *args, **kwargs):
        if "event" in request.GET and request.GET.get("event", None):
            event = Event.objects.get(id=request.GET["event"])
            context['adminform'].form.fields['turn'].queryset = event.turns.all()
            context['adminform'].form.fields['start_place'].queryset = event.start_places.all()
        return super(EventTravelOptionAdmin, self).render_change_form(request, context, args, kwargs)


class SignatureAdmin(ExtendibleModelAdmin):
    save_on_top = True
    list_display = (
        'get_edit_icon',
        'language', 'name', 'title', 'company', 'email', 'signature', 'note',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'language',
                'name',
                'title',
                'company',
                'email',
                'signature',
                'note',
            )
        }),
    )

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'signature':
            request = kwargs.pop("request", None)
            kwargs['widget'] = AdminImageWidget
            return db_field.formfield(**kwargs)
        return super(SignatureAdmin, self).formfield_for_dbfield(db_field, **kwargs)


class StageAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'get_status_display', 'name',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': ('status', 'name',)
        }),
    )


class FlightAdmin(ExtendibleModelAdmin):
    form = FlightAdminForm
    list_display = (
        'get_edit_icon',
        'airline',
        'get_code',
        'dep_airport', 'arr_airport',
        'departure', 'arrival',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'airline',
                'number',
                'dep_airport',
                'arr_airport',
                ('departure', 'arrival'),
                'duration',
                'day_after',
            )
        }),
    )
    search_fields = ("code", "number")

    class Media:
        css = {"all": ("event/css/admin_transport.css",)}
        js = (
            "event/js/admin_transport.js",
            "event/js/autocomplete_add_other_fix.js",
        )


class TrainAdmin(ExtendibleModelAdmin):
    form = TrainAdminForm
    list_display = (
        'get_edit_icon',
        'number',
        'dep_station', 'arr_station',
        'departure', 'arrival',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'train_type',
                'number',
                'dep_station',
                'arr_station',
                ('departure', 'arrival'),
                ('duration', 'day_after'),
            )
        }),
    )
    search_fields = ("code", "number")

    class Media:
        css = {"all": ("event/css/admin_transport.css",)}
        js = (
            "event/js/admin_transport.js",
            "event/js/autocomplete_add_other_fix.js",
        )


class InviteEventListFilter(SimpleListFilter):
    title = _('last events')
    parameter_name = 'event'

    def lookups(self, request, model_admin):
        return [(event.id, event.name) for event in Event.objects.all().order_by('-date_create')[:10]]

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(event=self.value())


class DelegatedInline(admin.TabularInline):
    extra = 0
    model = DelegatedContact
    readonly_fields = (
        "invited_full_name", "invited_email", "managed",
        "create_invite", "send_invite",
    )
    fields = (
        "invited_full_name", "invited_email", "managed",
        "create_invite", "send_invite",
    )
    delete = False

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def invited_full_name(self, obj):
        return obj.invited.full_name
    invited_full_name.short_description = _("Contact")

    def invited_email(self, obj):
        return obj.invited.email
    invited_email.short_description = _("Email")

    def create_invite(self, obj):
        if obj.id:
            return "<a href='%s'>X</a>" % (
                reverse('admin:delegatedcontact_create_invite', args=[obj.id])
            )
        return ""
    create_invite.short_description = _("Create")
    create_invite.allow_tags = True

    def send_invite(self, obj):
        if obj.id:
            return "<a href='%s'>X</a>" % (
                reverse('admin:delegatedcontact_send_invite', args=[obj.id])
            )
        return ""
    send_invite.short_description = _("Send")
    send_invite.allow_tags = True


class DecisionsInline(admin.TabularInline):
    extra = 0
    model = InviteDecisionAnswer
    readonly_fields = ("status", "date_create")
    fields = ("status", "date_create")
    delete = False

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class CustomAdvancedSearchChangeList(AdvancedSearchChangeList):

    def get_queryset(self, request):
        qs = super(CustomAdvancedSearchChangeList, self).get_queryset(request)
        ordering = self.get_ordering(request, qs)
        if 'contact__memberships__newspaper__name' in ordering or '-contact__memberships__newspaper__name' in ordering:
            # in caso di ordinamento per testata filtro testata di default (problema duplicati)
            default_membership = Q(contact__memberships__default=True)
            without_membership = Q(contact__memberships__isnull=True)
            qs = qs.filter(default_membership | without_membership)
        return qs


class InviteAdmin(ExtendibleModelAdmin, InviteAdminCSVMixin):

    class Media:
        js = (
            "event/js/admin_dajaxice.js",
            'event/js/list_filter_toggle.js',
        )

    form = InviteAdminForm
    save_on_top = True
    search_fields = ('event__name', 'contact__first_name', 'contact__last_name', 'contact__email')
    advanced_search_fields = [
        ('contact',),
        ('contact__first_name__icontains:Nome contatto', 'contact__last_name__icontains:Cognome contatto',),
        ('contact__email__icontains:Email contatto', 'contact__phones__number__icontains:Telefono'),
        ('status', 'contact__privacy_acceptance_date__isnull: Privacy mancante',),
        ('contact__memberships__newspaper:Testata',),
        ('event', 'step',),
        ('has_partecipated:has partecipated',),
        ('event__name__icontains:Nome evento',),
        ('event__place__icontains:Luogo evento',),
        ('event__event_date_start__gte:Data inizio evento', 'event__event_date_start__lte:Data inizio evento'),
        ('event__event_date_end__gte:Data fine evento', 'event__event_date_end__lte:Data fine evento'),
        (
            'event__event_adhesion_deadline__gte:Data deadline adesione evento',
            'event__event_adhesion_deadline__lte:Data deadline adesione evento'
        )
    ]
    list_display = [
        'get_edit_icon',
        'get_send_status_and_count',
        'get_contact_data_cl',
        'get_contact_privacy_status_cl',
        'get_contact_newspaper_cl',
        'get_status_cl', 'get_step_cl',
        'get_contact_phone_cl',
        'get_answers_cl',
        'get_turn_cl',
        'get_transport_cl',
        'get_activities_cl_short_name',
        'get_documents_cl',
        'get_note_cl',
        'get_link_webpage_cl', 'get_link_managed_cl', 'get_link_has_partecipated_cl', 'get_link_no_show_cl',
        'invite_emails_admin_url', 'invite_emails_mailig_status_admin_url',
        'get_link_set_delegated', 'poll_answers_admin_url'
    ]
    list_display_csv = (
        'get_turn', 'get_escaped_answers_cl', 'get_escaped_contact_newspaper_cl',
        'get_contact_last_name', 'get_contact_first_name', 'get_contact_company',
        'get_contact_default_email', 'get_contact_all_email_cc', 'get_contact_birthday',
        'get_contact_default_phone', 'get_contact_all_phones', 'internal_note', 'note',
        'get_delegated_contact', 'get_last_modified', 'get_transport',
        'get_departure_transport', 'get_return_transport', 'get_contact_frequent_flyers_cards',
        'get_contact_passport', 'get_contact_passport_expiration_date', 'get_contact_id_card_number',
        'get_contact_id_card_expiration_date', 'get_contact_license_number', 'get_contact_license_expiration_date',
        'get_contact_smoking', 'get_contact_food', 'get_contact_food_note', 'get_contact_shirt_size',
        'get_contact_height', 'get_contact_weight', 'get_contact_sweater_size', 'get_contact_shoes_size',
    )
    list_filter = ('status', 'step')
    inlines = (
        DelegatedInline,
        DecisionsInline,
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (_('main'), {
            'fields': (
                'status', 'event', 'get_contact_id', 'contact', 'step',
                'sent', 'has_partecipated',
                'managed', 'no_show', 'forwarded_from'
            )
        }),
        (_('triumph'), {
            'fields': (
                'favourite_vehicle_model', 'event_place', 'assigned_vehicle', 'assigned_place'
            )
        }),
        (_('notes'), {
            'fields': (
                ('note', 'internal_note'),
            )
        }),
    )
    readonly_fields = ('get_contact_id', 'forwarded_from') # , 'assigned_vehicle', 'assigned_place')
    tabs = (
        (_('main'), {'items': [_('main'), _('notes'),  DelegatedInline, DecisionsInline], 'active': True}),
        (_('documents'), {'items': [InviteDocumentAjaxInline]}),
        (_('triumph'), {'items': [_('triumph')]}),
        (_('note from contact'), {'items': [NoteFromPhoneContactInline]}),
        (_('logs'), {'items': [_('logs admin')]}),
    )
    action_form = InviteActionForm
    action_form_fields = {
        'invite_send': ['test'],
        'invite_resend': ['test'],
        'invite_send_followup': ['test'],
        'invite_send_documents': ['test'],
        'change_invites_status': ['status'],
        'change_invite_call_status': ['call_status']
    }
    actions = [
        invite_send,
        invite_resend,
        invite_send_followup,
        invite_send_documents,
        change_invites_status,
        change_invite_call_status
    ]

    def poll_answers_admin_url(self, instance):
        if instance.poll_answers.all().count():
            reverse_url = "{}?invite__event={}&invite__contact={}".format(
                reverse('admin:event_pollanswer_changelist'),
                instance.event.id,
                instance.contact.id,
            )
            return u"<a href='{}'><img src='{}' alt='True'></a>".format(
                reverse_url,
                static('event/img/poll.png')
            )
        return ""
    poll_answers_admin_url.short_description = _('Poll')
    poll_answers_admin_url.allow_tags = True

    def get_contact_id(self, instance):
        return instance.contact.id
    get_contact_id.short_description = _("Contact id")

    def get_urls(self):
        urls = super(InviteAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^set-delegate/(?P<invite_id>\d+)/$',
                self.admin_site.admin_view(invite_set_delegated),
                name='invite_set_delegated'
            ),
        )
        return my_urls + urls

    def get_filterset_class(self, request):
        TkFilterset = super(InviteAdmin, self).get_filterset_class(request)
        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.form.fields['contact'].widget = autocomplete_light.ChoiceWidget('ContactAutocomplete')
                self.form.fields['event'].widget = autocomplete_light.ChoiceWidget('EventAutocomplete')
        return TkFilters

    def invite_emails_admin_url(self, instance):
        redirect_url = reverse("admin:event_inviteemailobject_changelist")+"?invite__event=%s&invite__contact=%s" % (
          instance.event.id, instance.contact.id
        )
        return '<a href="%s"><img src="/static/event/img/mail.png" alt="True"></a>' % redirect_url
    invite_emails_admin_url.short_description = _('Emails')
    invite_emails_admin_url.allow_tags = True

    def invite_emails_mailig_status_admin_url(self, instance):
        redirect_url = reverse(
            "admin:event_mailingstatusproxy_changelist"
        )+"?email_object__inviteemailobject__invite__event=%s&email_object__inviteemailobject__invite__contact=%s" % (
          instance.event.id, instance.contact.id
        )
        return '<a href="%s"><img src="/static/event/img/mail.png" alt="True"></a>' % redirect_url
    invite_emails_mailig_status_admin_url.short_description = _('Emails st')
    invite_emails_mailig_status_admin_url.allow_tags = True

    def documents_emails_admin_url(self, instance):
        redirect_url = "{}?invite__event={}&invite__contact={}".format(
            reverse("admin:event_documentsemailobject_changelist"),
            instance.event.id, instance.contact.id
        )
        return u'<a href="{}"><img src="/static/event/img/mail.png" alt="True"></a>'.format(redirect_url)
    documents_emails_admin_url.short_description = _('D. Emails')
    documents_emails_admin_url.allow_tags = True

    def event_link(self, obj):
        return obj.event.get_admin_change_url()
    event_link.allow_tags = True
    event_link.short_description = _("Event")

    def contact_link(self, obj):
        return obj.contact.get_admin_change_url()
    contact_link.allow_tags = True
    contact_link.short_description = _("Contact")

    def get_changelist(self, request):
        return CustomAdvancedSearchChangeList

    def get_queryset(self, request):
        """"
        Returns a QuerySet of Invite model instance filtered by event_id.
        """
        path = request.path
        last_path = path.rstrip("/").split("/")[-1]
        queryset = super(InviteAdmin, self).get_queryset(request)
        if last_path == "invite" and ("event" not in request.GET and "contact" not in request.GET):
            return queryset.none()
        return queryset.distinct()

    def get_list_display(self, request):
        list_display = [
            'get_edit_icon',
            'get_send_status_and_count',
            'event',
            'get_contact_data_cl',
            'get_contact_newspaper_cl',
            'get_status_cl', 'get_step_cl',
            'get_contact_phone_cl',
            'get_answers_cl', 'get_transport_cl', 'get_activities_cl', 'get_documents_cl',
            'get_note_cl',
            'get_link_managed_cl', 'get_link_has_partecipated_cl', 'get_link_no_show_cl',
            'get_link_webpage_cl', 'invite_emails_admin_url', 'invite_emails_mailig_status_admin_url',
            'get_link_set_delegated', 'poll_answers_admin_url'
        ]

        if request and "event" not in request.GET:
            return list_display
        elif request.GET['event'] == '1098':
            # only for triumph event
            list_display.append('get_cdu_cl')
            list_display.append('call_status')
            return list_display
        return super(InviteAdmin, self).get_list_display(request)

    def get_form(self, request, obj=None, **kwargs):
        form = super(InviteAdmin, self).get_form(request, obj, **kwargs)
        event_id = request.GET.get('event', None)
        if event_id:
            event = Event.objects.get(pk=event_id)
            form.base_fields['event'].initial = event
        return form

    def changelist_view(self, request, extra_context=None, *args, **kwargs):
        """
        The 'change list' admin view override for Invite model.
        Get event_id from url
        """
        queryset_active = Event.objects.filter(status=1).order_by('-event_date_start')
        extra_context = extra_context or {}
        extra_context["active_events"] = queryset_active
        if request.GET.get("event", None):
            extra_context["event_id"] = int(request.GET["event"])
            extra_context["event"] = Event.objects.get(id=extra_context["event_id"])
        return super(InviteAdmin, self).changelist_view(request, extra_context, *args, **kwargs)

    def get_activities_cl_short_name(self, obj):
        activities_list = obj.get_activities_cl_short_name_list()
        return ', '.join([unicode(activity) for activity in activities_list])
    get_activities_cl_short_name.short_description = _("Activities")
    get_activities_cl_short_name.allow_tags = True

    def get_advanced_search_fields(self, request):
        advanced_search_fields = [
            ('contact',),
            ('contact__first_name__icontains:Nome contatto', 'contact__last_name__icontains:Cognome contatto',),
            ('contact__email__icontains:Email contatto', 'contact__phones__number__icontains:Telefono'),
            ('status', 'contact__privacy_acceptance_date__isnull: Privacy mancante',),
            ('contact__memberships__newspaper:Testata',),
            ('event', 'step',),
            ('has_partecipated:has partecipated',),
            ('event__name__icontains:Nome evento',),
            ('event__place__icontains:Luogo evento',),
            ('event__event_date_start__gte:Data inizio evento', 'event__event_date_start__lte:Data inizio evento'),
            ('event__event_date_end__gte:Data fine evento', 'event__event_date_end__lte:Data fine evento'),
            (
                'event__event_adhesion_deadline__gte:Data deadline adesione evento',
                'event__event_adhesion_deadline__lte:Data deadline adesione evento'
            ),
        ]
        if request and "event" in request.GET and request.GET['event'] == '1098':
            advanced_search_fields.append(('call_status: Esito telefonata', 'notes_from_phone_contact__text__icontains: Nota telefonata',))
            return advanced_search_fields
        return super(InviteAdmin, self).get_advanced_search_fields(request)

    def get_actions(self, request, *args, **kw):
        actions = super(InviteAdmin, self).get_actions(request, *args, **kw)
        try:
            if request and "event" in request.GET and request.GET['event'] == '1098':
                pass
            else:
                del actions['change_invite_call_status']
        except Exception, e:
            pass
        return actions


class BaseAgencyInviteAdmin(ExtendibleModelAdmin):
    save_on_top = True

    advanced_search_fields = (
        ('contact',),
        ('contact__first_name__icontains:Nome contatto', 'contact__last_name__icontains:Cognome contatto',),
        ('contact__email__icontains:Email contatto',),
        ('event',),
        ('event__name__icontains:Nome evento',),
    )

    def get_presences(self, obj):
        if obj.status in [
            settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
            settings.PUBLIEDITOR_EVENT_INVITE_POLL_DONE
        ]:
            return _('Confirmed')
        if obj.status in [
            settings.PUBLIEDITOR_EVENT_INVITE_PENDING,
            settings.PUBLIEDITOR_EVENT_INVITE_TO_SEND,
            settings.PUBLIEDITOR_EVENT_INVITE_SENT,
        ]:
            return _('To be confirmed')
        if obj.status == settings.PUBLIEDITOR_EVENT_INVITE_DECLINED:
            return _('Declined')
        if obj.status == settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED:
            return _('Delegated')
        return ''

    get_presences.short_description = _('Presences')
    get_presences.allow_tags = True

    def get_contact_last_name(self, obj):
        return obj.contact.last_name
    get_contact_last_name.short_description = _('Last name')
    get_contact_last_name.admin_order_field = 'contact__last_name'

    def get_contact_first_name(self, obj):
        return obj.contact.first_name
    get_contact_first_name.short_description = _('First name')
    get_contact_first_name.admin_order_field = 'contact__first_name'

    def get_cards(self, obj):
        return u", ".join([card.number for card in obj.contact.cards.all()])
    get_cards.short_description = "Frequent flayer"

    def get_contact_detail(self, obj):
        return """
            {title} {first_name} {last_name} <br/>
            <a href="mailto:{email}" target="_top">{email}</a> <br/>
            tel: {phone}
        """.format(
            title=obj.contact.get_title_display(),
            first_name=obj.contact.first_name,
            last_name=obj.contact.last_name,
            email=obj.contact.email,
            phone=obj.contact.default_phone,
        )
    get_contact_detail.short_description = _('contact')
    get_contact_detail.allow_tags = True

    def get_delegates(self, obj):
        delegates = ''
        for delegate in obj.delegated_invites.all():
            delegates += """
                {title} {first_name} {last_name} <br/>
                <a href="mailto:{email}" target="_top">{email}</a> <br/>
                tel: {phone}
            """.format(
                title=delegate.get_title_display(),
                first_name=delegate.first_name,
                last_name=delegate.last_name,
                email=delegate.email,
                phone=delegate.phone_number,
            )
        return delegates
    get_delegates.short_description = _('Delegates')
    get_delegates.allow_tags = True

    def event_link(self, obj):
        return obj.event.get_admin_change_url()
    event_link.allow_tags = True

    def get_intolerances(self, obj):
        return obj.contact.food_note


class TravelAgencyInviteAdmin(BaseAgencyInviteAdmin):
    list_display = (
        'get_edit_icon',
        'get_presences',
        'get_contact_newspaper_cl',
        'get_contact_last_name',
        'get_contact_first_name',
        'internal_note',
        'get_cards',
        'get_departure_details',
        'get_return_details',
    )
    fieldsets = (
        (_('main'), {
            'fields': (
                'contact', 'event', 'get_presences', 'get_contact_newspaper_cl', 'get_contact_last_name',
                'get_contact_first_name', 'internal_note', 'get_cards', 'get_departure_details',
                'get_return_details',
            )
        }),
    )
    readonly_fields = (
        'contact', 'event', 'get_presences', 'get_contact_newspaper_cl', 'get_contact_last_name',
        'get_contact_first_name', 'internal_note', 'get_cards', 'get_departure_details', 'get_return_details',
    )
    tabs = (
        (_('main'), {'items': [_('main'), _('notes'),  InviteTravelAgencyDocumentAjaxInline], 'active': True}),
    )
    actions = None

    def get_queryset(self, request):
        qs = super(BaseAgencyInviteAdmin, self).get_queryset(request)
        if not request.user.is_superuser:
            qs = qs.filter(event__travel_agencies=request.user)
        return qs

    def get_filterset_class(self, request):
        TkFilterset = super(BaseAgencyInviteAdmin, self).get_filterset_class(request)

        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.form.fields['contact'].widget = autocomplete_light.ChoiceWidget('ContactTravelAgencyAutocomplete')
                self.form.fields['event'].widget = autocomplete_light.ChoiceWidget('EventTravelAgencyAutocomplete')

        return TkFilters

    def get_changelist(self, request):
        return CustomAdvancedSearchChangeList


class LogisticAgencyInviteAdmin(ExtendibleModelAdmin):
    advanced_search_fields = (
        ('contact__first_name__icontains:Nome contatto', 'contact__last_name__icontains:Cognome contatto',),
        ('contact__email__icontains:Email contatto', 'contact__phones__number__icontains:Tel contatto',),
        ('event_place:Tappa preferita', 'assigned_place:Tappa assegnata',),
        ('favourite_vehicle_model:Veicolo preferita', 'assigned_vehicle:Veicolo assegnata',),
        ('cdu__isnull:CDU', 'call_status: Esito telefonata',)
    )

    list_display = (
        'get_edit_icon',
        'registration_date',
        'get_contact_first_name',
        'get_contact_last_name',
        'get_contact_email',
        'get_contact_mobile_phone',
        'reservation_code',
        'get_license',
        'event_place',
        'assigned_place',
        'favourite_vehicle_model',
        'assigned_vehicle',
        'get_home_address',
        'get_cdu_cl',
        'has_notes_from_phone_contact',
        'call_status'
    )

    list_display_csv = (
        'registration_date',
        'get_contact_first_name',
        'get_contact_last_name',
        'get_contact_email',
        'get_contact_mobile_phone',
        'reservation_code',
        'license_number',
        'license_types',
        'license_emitted_by',
        'license_issue_date',
        'license_expiration_date',
        'event_place',
        'assigned_place',
        'favourite_vehicle_model',
        'assigned_vehicle',
        'get_home_address',
        'has_cdu',
        'notes_text',
        'call_status'
    )

    def registration_date(self, obj):
        return obj.date_create.strftime("%d/%m/%Y %H:%M")
    registration_date.short_description = _('Registration Date')

    def license_number(self, obj):
        return obj.contact.license_number

    def license_types(self, obj):
        return obj.contact.get_license_types_list()

    def license_emitted_by(self, obj):
        return obj.contact.license_emitted_by

    def license_issue_date(self, obj):
        return obj.contact.license_issue_date

    def license_expiration_date(self, obj):
        return obj.contact.license_expiration_date

    def notes_text(self, obj):
        return u'  |||  '.join(obj.notes_from_phone_contact.all().values_list('text', flat=True))

    def reservation_code(self, obj):
        return obj.make_base36()
    reservation_code.short_description = _('Reservation Code')

    fieldsets = (
        (_('main'), {
            'fields': (
                'get_contact_first_name',
                'get_contact_last_name',
                'get_contact_email',
                'get_contact_mobile_phone',
                'get_license',
                'event_place',
                'assigned_place',
                'favourite_vehicle_model',
                'assigned_vehicle',
                'get_home_address',
                'get_cdu_cl',
            )
        }),
    )
    readonly_fields = (
        'get_contact_first_name',
        'get_contact_last_name',
        'get_contact_email',
        'get_contact_mobile_phone',
        'get_license',
        'event_place',
        'assigned_place',
        'favourite_vehicle_model',
        'assigned_vehicle',
        'get_home_address',
        'get_cdu_cl',
    )
    tabs = (
        (_('main'), {'items': [_('main'), _('notes'),], 'active': True}),
        (_('note from contact'), {'items': [NoteFromPhoneContactInlineReadOnly]}),
    )
    # actions = None

    def get_queryset(self, request):
        qs = super(LogisticAgencyInviteAdmin, self).get_queryset(request)
        if not request.user.is_superuser:
            qs = qs.filter(event__logistic_agencies=request.user)
        return qs

    def get_filterset_class(self, request):
        TkFilterset = super(LogisticAgencyInviteAdmin, self).get_filterset_class(request)

        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                # self.form.fields['contact'].widget = autocomplete_light.ChoiceWidget('ContactLogisticAgencyAutocomplete')
                # self.form.fields['event'].widget = autocomplete_light.ChoiceWidget('EventLogisticAgencyAutocomplete')
        return TkFilters

    def get_changelist(self, request):
        return CustomAdvancedSearchChangeList

    def has_notes_from_phone_contact(self, obj):
        return bool(obj.notes_from_phone_contact.all())
    has_notes_from_phone_contact.short_description = _('Note')
    has_notes_from_phone_contact.boolean = True


class HostessInviteAdmin(BaseAgencyInviteAdmin):
    list_display = (
        'get_edit_icon',
        'get_contact_first_name_cl',
        'get_contact_last_name_cl',
        'get_contact_email_cl',
        'get_contact_phone_cl',
        'get_license',
        'event_place',
        'assigned_place',
        'favourite_vehicle_model',
        'assigned_vehicle',
        'get_home_address',
        'get_cdu_cl',
        # 'get_notes_from_phone_contact'
    )
    fieldsets = (
        (_('main'), {
            'fields': (
                'get_contact_first_name_cl',
                'get_contact_last_name_cl',
                'get_contact_email_cl',
                'get_contact_phone_cl',
                'get_license',
                'event_place',
                'assigned_place',
                'favourite_vehicle_model',
                'assigned_vehicle',
                'get_home_address',
                'get_cdu_cl',
            )
        }),
    )
    readonly_fields = (
        'get_contact_first_name_cl',
        'get_contact_last_name_cl',
        'get_contact_email_cl',
        'get_contact_phone_cl',
        'get_license',
        'event_place',
        'assigned_place',
        'favourite_vehicle_model',
        'assigned_vehicle',
        'get_home_address',
        'get_cdu_cl',
    )
    tabs = (
        (_('main'), {'items': [_('main'), _('notes'),  InviteHostessDocumentAjaxInline], 'active': True}),
        (_('note from contact'), {'items': [NoteFromPhoneContactInlineReadOnly]}),
    )
    actions = None

    def get_queryset(self, request):
        qs = super(BaseAgencyInviteAdmin, self).get_queryset(request)
        if not request.user.is_superuser:
            qs = qs.filter(event__hostess=request.user)
        return qs

    def get_filterset_class(self, request):
        TkFilterset = super(BaseAgencyInviteAdmin, self).get_filterset_class(request)

        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.form.fields['contact'].widget = autocomplete_light.ChoiceWidget('ContactHostessAutocomplete')
                self.form.fields['event'].widget = autocomplete_light.ChoiceWidget('EventHostessAutocomplete')
        return TkFilters

    def get_changelist(self, request):
        return CustomAdvancedSearchChangeList


class FlightDetailAdmin(ExtendibleModelAdmin):
    form = FlightDetailAdminForm
    list_display = (
        'get_edit_icon',
        'event_travel_option',
        'flight', 'departure', 'arrival',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'event_travel_option',
                'direction',
                'flight',
                ('departure', 'arrival'),
                'form_flight_departure',
                'form_flight_arrival',
                'day_after',
            )
        }),
    )

    class Media:
        js = (
            "event/js/admin_event.js",
            "event/js/admin_flightdetail.js",
        )
        css = {
            "all": (
                "event/css/admin_event.css",
            )
        }

    def get_readonly_fields(self, request, obj=None):
        return self.readonly_fields + (
            'turn_time', 'day_after'
        )
        

class TrainDetailAdmin(ExtendibleModelAdmin):
    form = TrainDetailAdminForm
    list_display = (
        'get_edit_icon',
        'event_travel_option',
        'train', 'departure', 'arrival',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'event_travel_option',
                'direction',
                'train',
                ('departure', 'arrival'),
                'form_train_departure',
                'form_train_arrival',
                'day_after',
            )
        }),
    )

    class Media:
        js = (
            "event/js/admin_event.js",
            "event/js/admin_traindetail.js",
        )
        css = {
            "all": (
                "event/css/admin_event.css",
            )
        }

    def get_readonly_fields(self, request, obj=None):
        return self.readonly_fields + (
            'turn_time', 'day_after'
        )


class CustomTransportDetailAdmin(ExtendibleModelAdmin):
    form = CustomTransportDetailAdminForm
    list_display = (
        'get_edit_icon',
        'event_travel_option',
        'customtransport', 'departure', 'arrival',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'event_travel_option',
                'direction',
                'customtransport',
                ('departure', 'arrival'),
                ('form_custom_transport_departure', 'form_custom_transport_arrival'),
            )
        }),
    )


class CustomTransportAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'description',
        'departure', 'arrival',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'description',
                ('departure', 'arrival'),
                ('duration',),
            )
        }),
    )


class AirportAdmin(ExtendibleModelAdmin):
    search_fields = ("iata_code", "name", "place")
    list_display = (
        'get_edit_icon',
        'iata_code', 'name', 'place',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'iata_code', 'name', 'place',
            )
        }),
    )


class AirlineAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'iata_code', 'name',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': ('iata_code', 'name',)
        }),
    )


class StationAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'place', 'name',
        'get_delete_icon',
    )
    search_fields = ("name", "place")
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'place',
                'name',
            )
        }),
    )


class TrainTypeAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'code', 'name',
        'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'code', 'name',
            )
        }),
    )


class DocumentCategoryModelAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'name',
        'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'name',
            )
        }),
    )


########################
# ADMIN ANSWER CLASSES #
########################
class TransportAnswerAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'event_link', 'contact_link', 'invite_link', 'transport_detail',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': ('invite', 'transport_detail')
        }),
    )

    form = TransportAnswerAdminForm

    def event_link(self, obj):
        return obj.invite.event.get_admin_change_url()
    event_link.allow_tags = True

    def contact_link(self, obj):
        return obj.invite.contact.get_admin_change_url()
    contact_link.allow_tags = True

    def invite_link(self, obj):
        return obj.invite.get_admin_change_url()
    invite_link.allow_tags = True


class TrainAnswerAdmin(ExtendibleModelAdmin):
    pass


class FlightAnswerAdmin(ExtendibleModelAdmin):
    pass


class FreeAnswerAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'invite',
        'answer',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'invite',
                'answer',
            )
        }),
    )


class OwnAnswerAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'invite',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'invite',
            )
        }),
    )


class BaseAnswerAdmin(ExtendibleModelAdmin):
    advanced_search_fields = (
        ('question',),
        ('invite__event:Event', 'invite__contact:Contact',),
    )
    list_display = (
        'get_edit_icon',
        'get_delete_icon',
            'invite', 'question', 'answer',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'invite',
                'answer'
            )
        }),
    )

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return ('invite',)
        return super(BaseAnswerAdmin, self).get_readonly_fields(request, obj)

    def has_add_permission(self, request):
       return False

    def get_filterset_class(self, request):
        TkFilterset = super(BaseAnswerAdmin, self).get_filterset_class(request)
        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.form.fields['question'].widget = autocomplete_light.ChoiceWidget('QuestionAutocomplete')
                self.form.fields['invite__contact'].widget = autocomplete_light.ChoiceWidget('ContactAutocomplete')
                self.form.fields['invite__event'].widget = autocomplete_light.ChoiceWidget('EventAutocomplete')
        return TkFilters

    def get_event(self, instance):
        return u"{}".format(instance.invite.event.name)
    get_event.short_description = _("Event")
    get_event.admin_order_field = ('invite__event__name')

    def get_contact(self, instance):
        return u"{}".format(instance.invite.contact.__unicode__())
    get_contact.short_description = _("Contact")
    get_contact.admin_order_field = ('invite__contact__last_name')


class ActivityAnswerAdmin(BaseAnswerAdmin):
    pass


class PollAnswerAdmin(BaseAnswerAdmin):
    pass


class RegisterAnswerAdmin(BaseAnswerAdmin):
    pass


class DeclineAnswerAdmin(BaseAnswerAdmin):
    pass


class EventTemplateAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'name', 'file',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'name',
                'file'
            )
        }),
    )


class EventSummaryDocumentTemplateAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'name', 'language', 'template',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'name',
                'language',
                'template',
                ('header_image', 'body_image',)
            )
        }),
    )


class AreaCodeAdmin(LocalExtededModelAdmin):
    search_fields = ('area', 'code')
    list_display = ('get_edit_icon', 'area', 'code', 'get_delete_icon')
    fieldsets = (
        FIELDSETS_DEFAULT_COLLAPSED,
        (_('main'), {
            'fields': (
                'area', 'code',
            )
        })
    )


class EventImageModelAdmin(ExtendibleModelAdmin):
    search_fields = ('title', 'description')
    advanced_search_fields = (
        ('date_create:Creation Date', 'date_last_modify:Last modification date'),
        ('event_image:Event image',),
        ('photo_id:Photo id',),
        ('image_size:Image size',),
        ('title__icontains:Title',),
        ('description__icontains:Description',),
    )
    list_display = (
        'get_edit_icon',
        'event_image',
        'photo_id',
        'title',
        'description',
        'image_size',
        'width',
        'height',
        'weigth',
        'get_original_link',
        'get_thumb_link',
        'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'date_create', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (_('main'), {
            'fields': (
                'event_image',
                'title',
                'description',
                'photo_id',
                'image_size',
                ('width', 'height', 'weigth',),
            ),
        }),
    )
    save_on_top = True

    def get_original_link(self, instance):
        if instance.get_photo_url_dict()["orig"]:
            return "<a href='%s'>link</a>" % instance.get_photo_url_dict()["orig"]
    get_original_link.short_description = _("Original link")
    get_original_link.allow_tags = True

    def get_thumb_link(self, instance):
        if instance.get_photo_url_dict()["thumb"]:
            return "<a href='%s'>link</a>" % instance.get_photo_url_dict()["thumb"]
    get_thumb_link.short_description = _("Thumb link")
    get_thumb_link.allow_tags = True


class EventPlaceAdmin(ExtendibleModelAdmin):
    model = EventPlace
    form = EventPlaceAdminForm
    list_display = ('get_edit_icon', 'get_delete_icon', 'event',  'date', 'location', 'name', 'number', 'zip_code', 'locality', 'province', 'country')
    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (_('main'), {'fields': (
            'event', 'date', 'vat_number', 'business_name', 'location', 'name', 'number', 'zip_code', 'locality', 'province', 'country',
        )}),
        (_('hostess'), {'fields': (
            'hostess',
        )}),
        (_('vehicles'), {'fields': (
            'vehicles',
        )}),

    )
    filter_horizontal = ('hostess', 'vehicles')

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return ('event',)
        return super(EventPlaceAdmin, self).get_readonly_fields(request, obj)

    tabs = [
        (_('main'), {'items': [_('main')], 'active': True}),
        (_('hostess'), {'items': [_('hostess')]}),
        (_('vehicles'), {'items': [_('vehicles')]}),
        (_('logs'), {'items': [_('logs admin')]}),
    ]


class BrandAdmin(ExtendibleModelAdmin):
    model = Brand
    inlines = [VehicleModelsInlineAdmin]
    list_display = ('get_edit_icon', 'get_delete_icon', 'name', 'url')
    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (_('main'), {'fields': (
            'name',
            'url',
            'logo',
            'logo_small',
            'logo_big',
            'note',
        )}),
    )

    tabs = [
        (_('main'), {'items': [_('main'), VehicleModelsInlineAdmin], 'active': True}),
        (_('logs'), {'items': [_('logs admin')]}),
    ]


class VehicleModelAdmin(ExtendibleModelAdmin):
    inlines = [VehicleInlineAdmin]
    model = VehicleModel
    prepopulated_fields = {'slug': ('name',), }
    list_display = ('get_edit_icon', 'get_delete_icon', 'brand', 'name', 'descr')
    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (_('main'), {'fields': (
            'brand',
            'name',
            'descr',
            'note',
            'slug',
        )}),
    )

    tabs = [
        (_('main'), {'items': [_('main'), VehicleInlineAdmin], 'active': True}),
        (_('logs'), {'items': [_('logs admin')]}),
    ]


class VehicleAdmin(ExtendibleModelAdmin):
    model = Vehicle
    list_display = ('get_edit_icon', 'get_delete_icon', 'vehicle_model', 'licence_plate',)
    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (_('main'), {'fields': (
            'vehicle_model',
            'licence_plate',
            'vin',
            'color',
        )}),
    )

    tabs = [
        (_('main'), {'items': [_('main')], 'active': True}),
        (_('logs'), {'items': [_('logs admin')]}),
    ]


class CDUAdmin(ExtendibleModelAdmin):
    list_display = ('get_edit_icon', 'get_delete_icon', 'invite',)

    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (_('main'), {'fields': (
            'invite', 'cdu_file',
        )}),
    )

    # not remove this because there are more FK and admin crash
    readonly_fields = ('invite',)


class NoteFromPhoneContactForm(ConstrainedModelForm):
    hidden_fields = ('invite',)

    class Meta:
        model = NoteFromPhoneContact


class NoteFromPhoneContactAdmin(ExtendibleModelAdmin):
    model = NoteFromPhoneContact

    list_display = (
        'get_edit_icon', 'get_delete_icon', 'last_modifier', 'date_last_modify', 'creator', 'date_create',
        'text', 'call_result'
    )
    form = NoteFromPhoneContactForm
    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify', 'creator', 'last_modifier',),
            'classes': ('collapse',),
        }),
        (_('main'), {'fields': (
            'invite',
            'call_result',
            'text',
        )}),
    )
    # readonly_fields = ('invite',)

    tabs = [
        (_('main'), {'items': [_('main')], 'active': True}),
        (_('logs'), {'items': [_('logs admin')]}),
    ]


register_inline(ActivityQuestionAjaxInline)
register_inline(RegisterQuestionAjaxInline)
register_inline(DeclineQuestionAjaxInline)
register_inline(PollQuestionAjaxInline)
register_inline(FlightDetailAjaxInline)
register_inline(TrainDetailAjaxInline)
register_inline(CustomTransportDetailAjaxInline)
register_inline(GreetingsAjaxInline)
register_inline(EventDocumentAjaxInline)
register_inline(NewsletterDocumentAjaxInline)
register_inline(InviteDocumentAjaxInline)
register_inline(InviteTravelAgencyDocumentAjaxInline)
register_inline(InviteLogisticAgencyDocumentAjaxInline)
register_inline(InviteHostessDocumentAjaxInline)
register_inline(EventPlaceAjaxInline)
register_inline(EventTurnProxyAjaxInline)
register_inline(EventStartPlaceAjaxInline)
register_inline(EventTravelOptionAjaxInline)

admin.site.register(Event, EventAdmin)
admin.site.register(EventTravelAgencyProxy, EventTravelAgencyProxyAdmin)
admin.site.register(EventLogisticAgencyProxy, EventLogisticAgencyProxyAdmin)
admin.site.register(EventHostessProxy, EventHostessProxyAdmin)
admin.site.register(Flight, FlightAdmin)
admin.site.register(Train, TrainAdmin)
admin.site.register(TransportAnswer, TransportAnswerAdmin)
admin.site.register(Invite, InviteAdmin)
admin.site.register(TravelAgencyInvite, TravelAgencyInviteAdmin)
admin.site.register(LogisticAgencyInvite, LogisticAgencyInviteAdmin)
admin.site.register(HostessInvite, HostessInviteAdmin)
admin.site.register(EventTurnProxy, EventTurnProxyAdmin)
admin.site.register(EventStartPlace, EventStartPlaceAdmin)
admin.site.register(EventTravelOption, EventTravelOptionAdmin)
admin.site.register(Stage, StageAdmin)
admin.site.register(License, LicenseAdmin)
admin.site.register(Transport, TransportAdmin)
admin.site.register(FlightDetail, FlightDetailAdmin)
admin.site.register(Airport, AirportAdmin)
admin.site.register(Airline, AirlineAdmin)
admin.site.register(TrainDetail, TrainDetailAdmin)
admin.site.register(CustomTransportDetail, CustomTransportDetailAdmin)
admin.site.register(Station, StationAdmin)
admin.site.register(TrainType, TrainTypeAdmin)
admin.site.register(Signature, SignatureAdmin)
admin.site.register(EventQuestion, EventQuestionModelAdmin)
admin.site.register(RegisterEventQuestionProxy, RegisterEventQuestionProxyModelAdmin)
admin.site.register(ActivityEventQuestionProxy, ActivityEventQuestionProxyModelAdmin)
admin.site.register(DeclineEventQuestionProxy, DeclineEventQuestionProxyModelAdmin)
admin.site.register(PollEventQuestionProxy, PollEventQuestionProxyModelAdmin)
admin.site.register(EventDocument, EventDocumentModelAdmin)
admin.site.register(NewsletterDocument, NewsletterDocumentModelAdmin)
admin.site.register(InviteDocument, InviteDocumentModelAdmin)
admin.site.register(InviteDocumentTravelAgencyProxy, InviteDocumentTravelAgencyProxyAdmin)
admin.site.register(InviteDocumentLogisticAgencyProxy, InviteDocumentLogisticAgencyProxyAdmin)
admin.site.register(InviteDocumentHostessProxy, InviteDocumentHostessProxyAdmin)
admin.site.register(GreetingsTranslation, GreetingsTranslationModelAdmin)
admin.site.register(Document, DocumentModelAdmin)
admin.site.register(FreeAnswer, FreeAnswerAdmin)
admin.site.register(OwnAnswer, OwnAnswerAdmin)
admin.site.register(ActivityAnswer, ActivityAnswerAdmin)
admin.site.register(PollAnswer, PollAnswerAdmin)
admin.site.register(RegisterAnswer, RegisterAnswerAdmin)
admin.site.register(EventTemplate, EventTemplateAdmin)
admin.site.register(DeclineAnswer, DeclineAnswerAdmin)
admin.site.register(DocumentCategory, DocumentCategoryModelAdmin)
admin.site.register(EventSummaryDocumentTemplate, EventSummaryDocumentTemplateAdmin)
admin.site.register(DelegatedContact, DelegatedContactAdmin)
admin.site.register(AreaCode, AreaCodeAdmin)
admin.site.register(EventImage, EventImageModelAdmin)
admin.site.register(EventPlace, EventPlaceAdmin)
admin.site.register(Brand, BrandAdmin)
admin.site.register(VehicleModel, VehicleModelAdmin)
admin.site.register(Vehicle, VehicleAdmin)
admin.site.register(CDU, CDUAdmin)
admin.site.register(NoteFromPhoneContact, NoteFromPhoneContactAdmin)

#### EMAIL OBJECT
from jmb.mailer.admin import EmailObjectAdmin


class InviteEmailObjectAdmin(EmailObjectAdmin):
    advanced_search_fields = (
        ('status__in:Status', 'mailing_status__status:Mailing Status',),
        ('is_test',),
        ('date_create__gte:Creation Date', 'date_create__lte:Creation Date',),
        ('email_address__icontains:email', 'subject__icontains:Oggetto',),
        ('invite__contact:Contatto', 'invite__event:Event'),
        ('invite__contact__first_name:Nome contatto', 'invite__contact__last_name:Cognome contatto',),
        ('invite__event__name__icontains:Nome evento', 'invite__contact__groups__in:Gruppo contatto',),
    )
    list_display = (
        'status', 'get_mailing_status', 'is_test', 'event', 'contact', 'date_create', 'email_address', 'subject', 'smtp_server',
        'bounce_server', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv = (
        'get_status_display',
        'get_event_csv',
        'get_contact_csv',
        'date_create',
        'email_address',
        'subject',
    )

    def get_filterset_class(self, request):
        TkFilterset = super(InviteEmailObjectAdmin, self).get_filterset_class(request)
        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.form.fields['invite__contact'].widget = autocomplete_light.ChoiceWidget('ContactAutocomplete')
                self.form.fields['invite__event'].widget = autocomplete_light.ChoiceWidget('EventAutocomplete')
        return TkFilters

    def get_event_csv(self, instance):
        return instance.event
    get_event_csv.short_description = _("Event")

    def get_contact_csv(self, instance):
        return instance.contact
    get_contact_csv.short_description = _("Contact")

    def get_actions(self, request, *args, **kw):
        actions = super(EmailObjectAdmin, self).get_actions(request, *args, **kw)
        try:
            del actions['set_disactive']
            del actions['set_active']
            del actions['send_mail_test']
            del actions['make_ready_to_send']
            del actions['make_cancel_sending']
        except Exception, e:
            pass
        return actions

    def get_mailing_status(self, instance):
        from .utils.generic import get_choice_by_id
        qs_status = instance.mailing_status.all()
        status_id_list = qs_status.values_list('status', flat=True)
        status_name_list = [get_choice_by_id(choice_id, settings.MAILER_MAILINGSTATUS_STATUS_CHOICES).encode() for choice_id in status_id_list]
        choices_html = u'<br>'.join(status_name_list)
        return choices_html
    get_mailing_status.short_description = _("Mailing Status")
    get_mailing_status.allow_tags = True


class DocumentsEmailObjectAttachmentInline(admin.TabularInline):
    extra = 0
    model = Attachment
    fields = ("title", "file_attachment",)
    readonly_fields = ("title", "file_attachment",)
    delete = False

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class DocumentsEmailObjectStatusInline(admin.TabularInline):
    extra = 0
    model = MailingStatus
    fields = ("status", "rfc_status", "diagnostic_code")
    readonly_fields = ("status", "rfc_status", "diagnostic_code")
    delete = False

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class DocumentsEmailObjectAdmin(InviteEmailObjectAdmin):
    list_display = (
        'get_edit_icon',
        'status', 'is_test', 'event', 'contact', 'date_create', 'email_address', 'subject', 'get_mailing_status',
        'get_delete_icon'
    )
    readonly_fields = ("get_contact", "get_event", "get_mailing_status")
    inlines = [DocumentsEmailObjectAttachmentInline, DocumentsEmailObjectStatusInline]
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields': ('ordering',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                ('get_event', 'get_contact'),
                'status', 'email_address', 'subject', 'content'
            )
        }),
        (_('Configuration'), {
            'fields': (
                ('smtp_server', 'bounce_server',),
                ('header_sender', 'header_reply',),
                'slug',
            ),
            'classes': ('collapse',)
        }),
    )

    def get_event(self, instance):
        return u"{}".format(instance.invite.event.name)
    get_event.short_description = _("Event")

    def get_contact(self, instance):
        return u"{}".format(instance.invite.contact.__unicode__())
    get_contact.short_description = _("Contact")

    def get_mailing_status(self, instance):
        qs = instance.mailing_status.all().order_by('-date_create')
        return u"<br>".join(
            ["{} {}".format(ms.date_create.strftime("%d/%m/%Y %H:%M"), ms.get_status_display()) for ms in qs]
        )
    get_mailing_status.short_description = "Status"
    get_mailing_status.allow_tags = True

    def has_delete_permission(self, request, obj=None):
        return False


class TurnAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon',
        'code',
        'title',
        'event',
        'turn_start',
        'turn_end',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'code',
                'title',
                'event',
                'turn_start',
                'turn_end',
            )
        }),
    )


class MailingStatusProxyAdmin(MailingStatusAdmin):
    advanced_search_fields = (
        ('status__in', 'rfc_status__icontains', 'diagnostic_code__icontains'),
        ('date_create__gte', 'date_create__lte',),
        ('email_object__email_address__icontains:Email address', 'email_object__subject__icontains:Email subject',),
        ('status_link__link__title__icontains:Titolo link', 'status_link__link__url__icontains:Url link',),
        ('email_object__inviteemailobject__invite__event:Event', 'email_object__inviteemailobject__invite__contact:Contact'),
    )
    list_filter = ('status',)

    class Media:
        js = (
            'event/js/list_filter_toggle.js',
        )

    def get_filterset_class(self, request):
        TkFilterset = super(MailingStatusProxyAdmin, self).get_filterset_class(request)
        class TkFilters(TkFilterset):
            def __init__(self, *args, **kw):
                super(TkFilters, self).__init__(*args, **kw)
                self.form.fields['email_object__inviteemailobject__invite__contact'].widget = \
                    autocomplete_light.ChoiceWidget('ContactAutocomplete')
        return TkFilters


class PasswordRecoverySettingsAdmin(ExtendibleModelAdmin):
    list_display = ('site', 'email_from', 'subject', 'get_edit_icon', 'get_delete_icon')
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('site', 'email_from', 'subject', 'body', 'link_text',)
        }),
    )

    class Media:
        js = (
            "event/js/admin_password_recovery_settings.js",
        )


class ProfileDoneEmailSettingsAdmin(ExtendibleModelAdmin):
    list_display = ('site', 'email_from', 'subject', 'get_edit_icon', 'get_delete_icon')
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('site', 'email_from', 'subject', 'body', 'link_text',)
        }),
    )

    class Media:
        js = (
            "event/js/admin_password_recovery_settings.js",
        )


class ContactEditDiffAdmin(ExtendibleModelAdmin):
    list_display = ('contact', 'body', 'get_edit_icon', 'get_delete_icon')
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('contact', 'body')
        }),
    )

admin.site.register(Turn, TurnAdmin)
admin.site.register(InviteEmailObject, InviteEmailObjectAdmin)
admin.site.register(DocumentsEmailObject, DocumentsEmailObjectAdmin)
admin.site.register(MailingStatusProxy, MailingStatusProxyAdmin)
admin.site.register(PasswordRecoverySettings, PasswordRecoverySettingsAdmin)
admin.site.register(ProfileDoneEmailSettings, ProfileDoneEmailSettingsAdmin)


admin.site.register(ContactEditDiff, ContactEditDiffAdmin)


