# -*- coding: utf-8 -*-

from django.contrib import admin, messages
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe


########################################################################################################################
# CONTACT ACTIONS ######################################################################################################
########################################################################################################################
def contact_add_to_group(self, request, queryset):
    """
    Adds contacts to a group
    """
    from publieditor.event.tasks import ContactAddToGroupTask
    contact_group_id = int(request.POST.get("contact_group"))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    ContactAddToGroupTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        contact_group_id=contact_group_id,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_contact_changelist")+"?{}".format(request.GET.urlencode()))
contact_add_to_group.short_description = _('Add to group')


def contact_add_to_event_invited_list(self, request, queryset):
    """
    Adds contact to en event invited list.
    """
    from publieditor.event.tasks import ContactAddToEventInvitedListTask
    event_id = int(request.POST.get("event"))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    ContactAddToEventInvitedListTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        event_id=event_id,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_contact_changelist")+"?{}".format(request.GET.urlencode()))
contact_add_to_event_invited_list.short_description = _('Add to event invited list')


def contact_export_csv_async(modeladmin, request, queryset):
    """
    Exports contacts csv asynchrounously
    """
    from publieditor.event.tasks import ContactExportCsvAsync
    request_get = request.GET.copy()
    pks = request.POST.getlist('_selected_action')
    select_across = int(request.POST.get('select_across'))
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    ContactExportCsvAsync.delay_or_fail(
        app_label='contact', model=modeladmin.model,
        pks=pks, select_across=select_across, get=request_get
    )
    return redirect(reverse("admin:event_contact_changelist")+"?{}".format(request.GET.urlencode()))
contact_export_csv_async.short_description = _("Export csv async")


def contact_export_xls_async(modeladmin, request, queryset):
    """
    Exports contacts xls asynchrounously
    """
    from publieditor.event.tasks import ContactExportXlsAsync
    request_get = request.GET.copy()
    pks = request.POST.getlist('_selected_action')
    select_across = int(request.POST.get('select_across'))
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    ContactExportXlsAsync.delay_or_fail(
        app_label='contact', model=modeladmin.model,
        pks=pks, select_across=select_across, get=request_get
    )
    return redirect(reverse("admin:event_contact_changelist")+"?{}".format(request.GET.urlencode()))
contact_export_xls_async.short_description = _("Export xls async")


########################################################################################################################
# DELEGATED CONTACT ACTIONS ############################################################################################
########################################################################################################################
def delegated_contact_invite_send(self, request, queryset):
    """
    Send invites to delegated contacts
    """
    from publieditor.event.tasks import DelegatedContactInviteSendTask
    test = bool(request.POST.get("test", False))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    DelegatedContactInviteSendTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        test=test,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_delegatedcontact_changelist")+"?{}".format(request.GET.urlencode()))
delegated_contact_invite_send.short_description = _('Send')


def delegated_contact_invite_create(self, request, queryset):
    """
    Create invites for delegated contacts
    """
    from publieditor.event.tasks import DelegatedContactInviteCreateTask
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    DelegatedContactInviteCreateTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_delegatedcontact_changelist")+"?{}".format(request.GET.urlencode()))
delegated_contact_invite_create.short_description = _('Create invite')


########################################################################################################################
# CONTACT GROUP ACTIONS ################################################################################################
########################################################################################################################
def contact_group_add_to_event_invited_list(self, request, queryset):
    """
    Adds groups contacts to en event invited list.
    """
    from publieditor.event.tasks import ContactGroupAddToEventInvitedListTask
    event_id = int(request.POST.get("event"))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    ContactGroupAddToEventInvitedListTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        event_id=event_id,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_contactgroup_changelist")+"?{}".format(request.GET.urlencode()))
contact_group_add_to_event_invited_list.short_description = _('Add to event invited list')


########################################################################################################################
# MAILING LIST ACTIONS #################################################################################################
########################################################################################################################
def mailing_list_add_to_event_invited_list(self, request, queryset):
    """
    Adds mailing lists contacts to en event invited list.
    """
    from publieditor.event.tasks import MailingListAddToEventInvitedListTask
    event_id = int(request.POST.get("event"))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    MailingListAddToEventInvitedListTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        event_id=event_id,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_mailinglist_changelist")+"?{}".format(request.GET.urlencode()))
mailing_list_add_to_event_invited_list.short_description = _('Add to event invited list')


########################################################################################################################
# INVITE ACTIONS #######################################################################################################
########################################################################################################################
def invite_send(self, request, queryset):
    """
    Send invites
    """
    from publieditor.event.tasks import InviteSendTask
    test = bool(request.POST.get("test", False))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    InviteSendTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        test=test,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_invite_changelist")+"?{}".format(request.GET.urlencode()))
invite_send.short_description = _('Send')


def invite_resend(self, request, queryset):
    """
    Resend invites
    """
    from publieditor.event.tasks import InviteReSendTask
    test = bool(request.POST.get("test", False))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    InviteReSendTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        test=test,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_invite_changelist")+"?{}".format(request.GET.urlencode()))
invite_resend.short_description = _('Resend')


def invite_send_followup(self, request, queryset):
    """
    Send follow ups
    """
    from publieditor.event.tasks import InviteSendFollowUpTask
    test = bool(request.POST.get("test", False))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    InviteSendFollowUpTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        test=test,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_invite_changelist")+"?{}".format(request.GET.urlencode()))
invite_send_followup.short_description = _('Send followup')


def invite_send_documents(self, request, queryset):
    """
    Send documents
    """
    from publieditor.event.tasks import InviteSendDocumentsTask
    test = bool(request.POST.get("test", False))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    InviteSendDocumentsTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        test=test,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_invite_changelist")+"?{}".format(request.GET.urlencode()))
invite_send_documents.short_description = _('Send documents')


def change_invites_status(self, request, queryset):
    """
    Send documents
    """
    status = request.POST.get('status', None)
    if status:
        queryset.update(status=status)
        messages.info(request, _("Invites status updated"))
    return redirect(reverse("admin:event_invite_changelist")+"?{}".format(request.GET.urlencode()))
change_invites_status.short_description = _('Change status')


def change_invite_call_status(self, request, queryset):
    """
    Change last note_from_phone_contact call_result
    """
    call_status = request.POST.get('call_status', None)
    for invite in queryset:
        last_note_from_phone_contact = invite.notes_from_phone_contact.first()
        last_note_from_phone_contact.call_result = call_status
        last_note_from_phone_contact.save()
    return redirect(reverse("admin:event_invite_changelist") + "?{}".format(request.GET.urlencode()))
change_invite_call_status.short_description = _('Change call status')

########################################################################################################################
# EVENT ACTIONS ########################################################################################################
########################################################################################################################
def event_send_invites(self, request, queryset):
    """
    Send invites to event invited list
    """
    from publieditor.event.tasks import EventSendInvitesTask
    test = bool(request.POST.get("test", False))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    EventSendInvitesTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        test=test,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_event_changelist")+"?{}".format(request.GET.urlencode()))
event_send_invites.short_description = _('Send invites')


def event_resend_invites(self, request, queryset):
    """
    Send invites to event invited list
    """
    from publieditor.event.tasks import EventReSendInvitesTask
    test = bool(request.POST.get("test", False))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    EventReSendInvitesTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        test=test,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_event_changelist")+"?{}".format(request.GET.urlencode()))
event_resend_invites.short_description = _('Resend invites')


def event_send_followups(self, request, queryset):
    """
    Send invites to event invited list
    """
    from publieditor.event.tasks import EventSendFollowUpsTask
    test = bool(request.POST.get("test", False))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    EventSendFollowUpsTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        test=test,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_event_changelist")+"?{}".format(request.GET.urlencode()))
event_send_followups.short_description = _('Send followups')


def event_send_documents(self, request, queryset):
    """
    Send documents to event invited list
    """
    from publieditor.event.tasks import EventSendDocumentsTask
    test = bool(request.POST.get("test", False))
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    EventSendDocumentsTask.delay_or_fail(
        app_label='event',
        model=self.model,
        request_get=request_get,
        request_request=request_request,
        test=test,
        pks=pks,
        select_across=select_across,
    )
    return redirect(reverse("admin:event_event_changelist")+"?{}".format(request.GET.urlencode()))
event_send_documents.short_description = _('Send documents')
