# -*- coding: utf-8 -*-

from django.contrib.admin.widgets import AdminFileWidget
from django.utils.html import escape
from django.utils.safestring import mark_safe
from easy_thumbnails.files import get_thumbnailer


class PublicImageWidget(AdminFileWidget):
    """
    Image field widget used to display image dimensions to end users
    """
    template_with_initial = '%(initial_text)s<br />%(input)s'

    def render(self, name, value, attrs=None):
        output = []
        if value and getattr(value, "url", None):
            try:
                image_url = value.url
                options = {'size': (100, 100), 'crop': True}
                thumbnail_url = get_thumbnailer(value).get_thumbnail(options).url
                file_name = str(value)
                output.append(u'<a href="%s" target="_blank"><img src="%s" alt="%s"/></a>' % (
                    image_url, thumbnail_url, escape(file_name))
                )
            except Exception, e:
                pass
        output.append(super(AdminFileWidget, self).render(name, value, attrs))
        return mark_safe(u''.join(output))


class AdminImageWidget(AdminFileWidget):
    """
    Image field widget used to display image attrs in admin
    """
    def render(self, name, value, attrs=None):
        output = []
        if value and getattr(value, "url", None):
            try:
                image_url = value.url
                options = {'size': (100, 100), 'crop': True}
                thumbnail_url = get_thumbnailer(value).get_thumbnail(options).url
                file_name = str(value)
                output.append(u'<a href="%s" target="_blank"><img src="%s" alt="%s"/></a>' % (
                    image_url, thumbnail_url, escape(file_name))
                )
                file_size = "%0.1f KB" % (value.size/(1024.0))
                output.append("&nbsp;&nbsp;%s - (%sx%s)" % (file_size, value.width, value.height))
            except Exception, e:
                pass
        output.append(super(AdminFileWidget, self).render(name, value, attrs))
        return mark_safe(u''.join(output))


class AdminProportionImageWidget(AdminFileWidget):
    """
    Image field widget used to display image attrs in admin end a proportional resized image
    """
    template_with_initial = '%(initial_text)s<br />%(input)s %(clear_template)s'

    def render(self, name, value, attrs=None):
        output = []
        if value and getattr(value, "url", None):
            try:
                image_url = value.url
                option_width = value.width/5 if value.width/5 <= 300 else 300
                option_height = value.height/5 if value.width/5 <= 200 else 200
                options = {'size': (option_width, option_height)}
                thumbnail_url = get_thumbnailer(value).get_thumbnail(options).url
                file_name = str(value)
                output.append(u'<a href="%s" target="_blank"><img src="%s" alt="%s"/></a>' % (
                    image_url, thumbnail_url, escape(file_name))
                )
                file_size = "%0.1f KB" % (value.size/(1024.0))
                output.append("&nbsp;&nbsp;%s - (%sx%s)" % (file_size, value.width, value.height))
            except Exception, e:
                pass
        output.append(super(AdminFileWidget, self).render(name, value, attrs))
        return mark_safe(u''.join(output))

