# -*- coding: utf-8 -*-
"""
ajax module for publieditor.event
"""
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from dajax.core import Dajax
from dajaxice.decorators import dajaxice_register


@dajaxice_register
def on_flight_selected(request, *args, **kwargs):
    from publieditor.event.models import Flight
    dajax = Dajax()
    try:
        pk = kwargs['id_flight']
        dajax.add_css_class('#id_flight', 'red')
        if pk:
            pk = kwargs['id_flight'][0]
            flight = Flight.objects.get(pk=pk)
            departure = flight.departure.strftime("%H:%M")
            arrival = flight.arrival.strftime("%H:%M")
            dajax.assign("#id_form_flight_departure", 'value', departure)
            dajax.assign("#id_form_flight_arrival", 'value', arrival)
            if flight.day_after:
                dajax.add_css_class('#id_flight', 'dayafter')
            else:
                dajax.remove_css_class('#id_flight', 'dayafter')
        else:
            dajax.assign("#id_form_flight_departure", 'value', '')
            dajax.assign("#id_form_flight_arrival", 'value', '')
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_train_selected(request, *args, **kwargs):
    from publieditor.event.models import Train
    dajax = Dajax()
    try:
        pk = kwargs['id_train']
        dajax.add_css_class('#id_train', 'red')
        if pk:
            pk = kwargs['id_train'][0]
            train = Train.objects.get(pk=pk)
            departure = train.departure.strftime("%H:%M")
            arrival = train.arrival.strftime("%H:%M")
            dajax.assign("#id_form_train_departure", 'value', departure)
            dajax.assign("#id_form_train_arrival", 'value', arrival)
            if train.day_after:
                dajax.add_css_class('#id_train', 'dayafter')
            else:
                dajax.remove_css_class('#id_train', 'dayafter')
        else:
            dajax.assign("#id_form_train_departure", 'value', '')
            dajax.assign("#id_form_train_arrival", 'value', '')
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_enabled(request, *args, **kwargs):
    from publieditor.event.models import EventQuestion
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_eventquestion']
        eventquestion_id = element.split('_')[-1]
        eventquestion = EventQuestion.objects.get(id=eventquestion_id)
        eventquestion.enabled = not eventquestion.enabled
        eventquestion.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % ('yes' if eventquestion.enabled else 'no')
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_contact_status(request, *args, **kwargs):
    """
    Used when status icon is clicked on ContactAdmin change list
    """
    from publieditor.event.models import Contact
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_contact']
        contact_id = element.split('_')[-1]
        contact = Contact.objects.get(id=contact_id)
        contact.status = not contact.status
        contact.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % (
            'yes' if contact.status else 'no'
        )
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_contact_contactemailcc_default(request, *args, **kwargs):
    """
    Used when default icon is clicked on ContactAdmin emailcc ajaxinline
    """
    from publieditor.event.models import ContactEmailCc
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_contact_contactemailcc']
        contactemailcc_id = element.split('_')[-1]
        contactemailcc = ContactEmailCc.objects.get(id=contactemailcc_id)
        if contactemailcc.default:
            contactemailcc.default = False
        else:
            contact = contactemailcc.contact
            if not contact.cc_emails.exclude(id=contactemailcc.id).filter(default=True).count():
                contactemailcc.default = True
            else:
                dajax.alert(_('Contact already has a default email cc address'))
        contactemailcc.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % (
            'yes' if contactemailcc.default else 'no'
        )
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_contact_phone_default(request, *args, **kwargs):
    """
    Used when default icon is clicked on ContactAdmin phone ajaxinline
    """
    from publieditor.event.models import Phone
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_contact_phone']
        phone_id = element.split('_')[-1]
        phone = Phone.objects.get(id=phone_id)
        if phone.default:
            phone.default = False
        else:
            contact = phone.contact
            if not contact.phones.exclude(id=phone.id).filter(default=True).count():
                phone.default = True
            else:
                dajax.alert(_('Contact already has a default phone'))
        phone.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % (
            'yes' if phone.default else 'no'
        )
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_contact_contactaddress_default(request, *args, **kwargs):
    """
    Used when default icon is clicked on ContactAdmin address ajaxinline
    """
    from publieditor.event.models import ContactAddress
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_contact_contactaddress']
        contactaddress_id = element.split('_')[-1]
        contactaddress = ContactAddress.objects.get(id=contactaddress_id)
        if contactaddress.default:
            contactaddress.default = False
        else:
            contact = contactaddress.contact
            if not contact.addresses.exclude(id=contactaddress.id).filter(default=True).count():
                contactaddress.default = True
            else:
                dajax.alert(_('Contact already has a default address'))
        contactaddress.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % (
            'yes' if contactaddress.default else 'no'
        )
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_contact_membership_default(request, *args, **kwargs):
    """
    Used when default icon is clicked on ContactAdmin membership ajaxinline
    """
    from publieditor.event.models import Membership
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_contact_membership']
        membership_id = element.split('_')[-1]
        membership = Membership.objects.get(id=membership_id)
        if membership.default:
            membership.default = False
        else:
            contact = membership.contact
            if not contact.memberships.exclude(id=membership.id).filter(default=True).count():
                membership.default = True
            else:
                dajax.alert(_('Contact already has a default membership'))
        membership.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % (
            'yes' if membership.default else 'no'
        )
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_contact_card_default(request, *args, **kwargs):
    """
    Used when default icon is clicked on ContactAdmin card ajaxinline
    """
    from publieditor.event.models import Card
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_contact_card']
        card_id = element.split('_')[-1]
        card = Card.objects.get(id=card_id)
        if card.default:
            card.default = False
        else:
            contact = card.contact
            if not contact.cards.exclude(id=card.id).filter(default=True).count():
                card.default = True
            else:
                dajax.alert(_('Contact already has a default frequent flyer'))
        card.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % (
            'yes' if card.default else 'no'
        )
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_invite_event_filter_selected(request, *args, **kwargs):
    dajax = Dajax()
    try:
        event_id = kwargs['event_id']
        redirect_url = reverse('admin:event_invite_changelist')+"?event=%s" % event_id
        dajax.redirect(redirect_url)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_required(request, *args, **kwargs):
    from publieditor.event.models import EventQuestion
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_requiredquestion']
        eventquestion_id = element.split('_')[-1]
        eventquestion = EventQuestion.objects.get(id=eventquestion_id)
        eventquestion.required = not eventquestion.required
        eventquestion.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % ('yes' if eventquestion.required else 'no')
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_invite_no_show(request, *args, **kwargs):
    from publieditor.event.models import Invite
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_invite']
        invite_id = element.split('_')[-1]
        invite = Invite.objects.get(id=invite_id)
        invite.no_show = not invite.no_show
        invite.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % ('yes' if invite.no_show else 'no')
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_invite_managed(request, *args, **kwargs):
    from publieditor.event.models import Invite
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_invite']
        invite_id = element.split('_')[-1]
        invite = Invite.objects.get(id=invite_id)
        invite.managed = not invite.managed
        invite.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % ('yes' if invite.managed else 'no')
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_save_event_language_preference(request, *args, **kwargs):
    from publieditor.event.models import Invite, EventTranslationPreference, Event
    from jmb.i18n.models import Language
    dajax = Dajax()
    user = request.user
    language_code = kwargs.get('language_code', None)
    event_id = kwargs.get('event_id', None)
    invite_id = kwargs.get('invite_id', None)
    if language_code:
        language = Language.objects.get(code=language_code)
        if invite_id and invite_id.isdigit():
            invite = Invite.objects.get(id=invite_id)
            EventTranslationPreference.objects.create(invite=invite, event=invite.event, language=language)
        elif request.user and event_id and event_id.isdigit():
            event = Event.objects.get(id=event_id)
            EventTranslationPreference.objects.create(user=user, event=event, language=language)
        dajax.redirect(request.META.get('HTTP_REFERER'))
    return dajax.json()


@dajaxice_register
def on_click_document_show_online(request, *args, **kwargs):
    from publieditor.event.models import EventDocument
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_document']
        instance_id = element.split('_')[-1]
        instance = EventDocument.objects.get(id=instance_id)
        instance.show_online = not instance.show_online
        instance.save()
        image = u"""<img alt="False" src="/static/admin/img/icon-%s.gif">""" % (
            'yes' if instance.show_online else 'no'
        )
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_document_attach_in_mail(request, *args, **kwargs):
    from publieditor.event.models import EventDocument
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_document']
        instance_id = element.split('_')[-1]
        instance = EventDocument.objects.get(id=instance_id)
        instance.attach_in_mail = not instance.attach_in_mail
        instance.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % (
            'yes' if instance.attach_in_mail else 'no'
        )
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()


@dajaxice_register
def on_click_document_attach_in_invite(request, *args, **kwargs):
    from publieditor.event.models import EventDocument
    dajax = Dajax()
    try:
        element = kwargs['publieditor_event_document']
        instance_id = element.split('_')[-1]
        instance = EventDocument.objects.get(id=instance_id)
        instance.attach_in_invite = not instance.attach_in_invite
        instance.save()
        image = """<img alt="False" src="/static/admin/img/icon-%s.gif">""" % (
            'yes' if instance.attach_in_invite else 'no'
        )
        dajax.assign("#"+element, 'innerHTML', image)
    except Exception, e:
        print e
    return dajax.json()