# -*- coding: utf-8 -*-
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext as _

from jmb.core.db.models import DateModel, UserModel
from jmb.core.db.cmsmodels import StatusModel

from cms.models.pluginmodel import CMSPlugin


# Root class to inherit
class AbstractSettings(CMSPlugin, DateModel, UserModel,  StatusModel):
    invite_status_to_be_visible = models.IntegerField(
        choices=settings.PUBLIEDITOR_EVENT_INVITE_STATUS_CODES,
        null=True, blank=True,
        verbose_name=_('Invite status to be visible')
    )

    def __unicode__(self):
        return "%s %s" % (
            ('On - ' if self.status else 'Off - '),
            self.template_name
        )

    class Meta:
        app_label = "event"
        abstract = True


# PresentationSettings class
class PresentationSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_PRESENTATION_TEMPLATES,
        default="event/cms_plugins/presentation.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_PRESENTATION_CLASSES,
        default="publieditor.event.cms.forms.PresentationForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "presentation"


# ScheduleSettings class
class ScheduleSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_SCHEDULE_TEMPLATES,
        default="event/cms_plugins/schedule.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_SCHEDULE_CLASSES,
        default="publieditor.event.cms.forms.ScheduleForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "schedule"


# AdhesionSettings class
class AdhesionSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_ADHESION_TEMPLATES,
        default="event/cms_plugins/adhesion.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_ADHESION_CLASSES,
        default="publieditor.event.cms.forms.AdhesionForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "adhesion"


# RegisterSettings class, used when invite is accepted
class RegisterSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_REGISTER_TEMPLATES,
        default="event/cms_plugins/register.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_REGISTER_CLASSES,
        default="publieditor.event.cms.forms.RegisterForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "register"


# DelegateSettings class, used when invite is delegated
class DelegateSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_DELEGATE_TEMPLATES,
        default="event/cms_plugins/delegate.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_DELEGATE_CLASSES,
        default="publieditor.event.cms.forms.DelegateForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "delegate"


# DeclineSettings class, used when invite is declined
class DeclineSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_DECLINE_TEMPLATES,
        default="event/cms_plugins/decline.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_DECLINE_CLASSES,
        default="publieditor.event.cms.forms.DeclineForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "decline"


# TransportSettings class
class TransportSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_TRANSPORT_TEMPLATES,
        default="event/cms_plugins/transport.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_TRANSPORT_CLASSES,
        default="publieditor.event.cms.forms.TransportForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "transport"


# ActivitySettings class
class ActivitySettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_ACTIVITY_TEMPLATES,
        default="event/cms_plugins/activity.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_ACTIVITY_CLASSES,
        default="publieditor.event.cms.forms.ActivityForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "activity"


# SummarySettings class
class SummarySettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_SUMMARY_TEMPLATES,
        default="event/cms_plugins/summary.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_SUMMARY_CLASSES,
        default="publieditor.event.cms.forms.SummaryForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "summary"


# GreetingsSettings class
class DocumentsSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_DOCUMENTS_TEMPLATES,
        default="event/cms_plugins/documents.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_DOCUMENTS_CLASSES,
        default="publieditor.event.cms.forms.DocumentsForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "documents"


# GreetingsSettings class
class GreetingsSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_GREETINGS_TEMPLATES,
        default="event/cms_plugins/greetings.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_GREETINGS_CLASSES,
        default="publieditor.event.cms.forms.GreetingsForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "greetings"


# SurveySettings class
class PollSettings(AbstractSettings):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_POLL_TEMPLATES,
        default="event/cms_plugins/poll.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_EVENT_CMSPLUGIN_POLL_CLASSES,
        default="publieditor.event.cms.forms.PollForm",
    )

    @property
    def invite_step_to_be_visible(self):
        return "poll"


class HeaderImageSettings(AbstractSettings):
    width = models.CharField(
        _("width"),
        max_length=10,
        default="100%"
    )
    height = models.CharField(
        _("height"),
        max_length=10,
        default="100%"
    )

    def __unicode__(self):
        return "%s header image" % ('On - ' if self.status else 'Off - ')


class SignatureSettings(AbstractSettings):
    def __unicode__(self):
        return "%s signature" % ('On - ' if self.status else 'Off - ')


class TitleSettings(AbstractSettings):
    def __unicode__(self):
        return "%s title" % ('On - ' if self.status else 'Off - ')


class BreadcrumbSettings(AbstractSettings):
    def __unicode__(self):
        return "%s breadcrumbs" % ('On - ' if self.status else 'Off - ')
