# -*- coding: utf-8 -*-

from random import randint
from django.template import Context, Template
from django.template.loader import render_to_string
from crispy_forms.compatibility import text_type
from crispy_forms.bootstrap import Accordion as BaseAccordion
from crispy_forms.utils import render_field, TEMPLATE_PACK
from crispy_forms.layout import LayoutObject


class Accordion(BaseAccordion):
    """
    We need to overwrite base crispy_form Accordion.
    The base one always tries to open the first group with errors or first group.
    We instead want the chance to specify by hand which ones we need open.
    """
    def render(self, form, form_style, context, template_pack=TEMPLATE_PACK):
        content = ''

        if not self.css_id:
            self.css_id = "-".join(["accordion", text_type(randint(1000, 9999))])

        for group in self.fields:
            group.data_parent = self.css_id
            content += render_field(
                group, form, form_style, context, template_pack=template_pack
            )

        return render_to_string(
            self.template,
            Context({'accordion': self, 'content': content})
        )


class Formset(LayoutObject):
    """
    Layout object. It renders an entire formset, as though it were a Field.

    Example::

    Formset("attached_files_formset")
    """

    template = "%s/formset.html" % TEMPLATE_PACK

    def __init__(self, formset_name_in_context, helper_name_in_context, data_url_in_context, template=None):
        self.formset_name_in_context = formset_name_in_context
        self.helper_name_in_context = helper_name_in_context
        self.data_url_in_context = data_url_in_context

        # crispy_forms/layout.py:302 requires us to have a fields property
        self.fields = []

        # Overrides class variable with an instance level variable
        if template:
            self.template = template

    def render(self, form, form_style, context, template_pack=TEMPLATE_PACK):
        formset = context[self.formset_name_in_context]
        helper = context[self.helper_name_in_context]
        data_url = context[self.data_url_in_context]
        return render_to_string(self.template, Context({'wrapper': self, 'formset': formset, 'helper': helper, 'data_url': data_url}))
