# coding: utf-8
"""
Example usage
dj import_railway_stations -f .src/publieditor.event/publieditor/event/static/event/import/RSVP_FCA_IMPORT_AREA_CODES.xlsx
"""

import logging
import sys

from django.core.management.base import BaseCommand
from django.db import transaction

from optparse import make_option

from jmb.core.utils.data_import import XlsFile


class AreaCodeFile(XlsFile):

    fields_map = {
        'area': 'area',
        'code': 'code',
    }
    messages = dict()
    messages['obj_created'] = 0
    messages['obj_modified'] = 0
    messages['obj_error'] = 0

    @transaction.commit_manually
    def do(self, row, kw, j):

        from publieditor.event.models import AreaCode

        try:
            if j % 100 == 0:
                print "riga: %s" % j

            # Creazione stazione
            area = kw['area'].strip().lower().title()
            code = kw['code'].strip()
            area_code, created = AreaCode.objects.get_or_create(
                code=code,
                defaults={'area': area}
            )
            if created:
                self.messages['obj_created'] += 1
            else:
                self.messages['obj_modified'] += 1
            transaction.commit()
        except Exception, e:
            self.messages['obj_error'] += 1
            print 'Riga: %s - errore %s' % (j, e)
            transaction.rollback()


class Command(BaseCommand):
    """
    Get all messages from configured Host
    """
    help = "Import area codes FCA format"
    option_list = BaseCommand.option_list + (
        make_option('-f', '--file', dest='filename', help='Read file in argument', default=None),
    )
    output_transaction = True

    def handle(self, *args, **opts):
        if not opts.get('filename'):
            print "Manca il nome del file da importare (opt -f)"
            sys.exit(1)
        import_file = AreaCodeFile(opts.get('filename'), auto=True)
        import_file.read()
        print import_file.messages
