# coding: utf-8
"""
Example usage
dj import_contacts -f .src/publieditor.event/publieditor/event/static/event/import/RSVP_FCA_IMPORT_CONTACTS.xlsx
"""

import logging
import sys

from django.core.management.base import BaseCommand
from django.db import connection, transaction

from optparse import make_option

from jmb.core.utils.data_import import XlsFile,  CsvFile
from jmb.i18n.models import Language


logger = logging.getLogger('imports')


class NewspaperFile(XlsFile):
    fields_map = {
        'language': 'Country',
        'title': 'Sex',
        'first_name': 'Nome',
        'last_name': 'Cognome',
        'email': 'EMAIL',
        'internal_note': 'NoteGiornalista',
        'email_ccs': 'CC',
        'membership_newspaper_1': 'Testata1',
        'membership_role_1': 'RuoloTestata1',
        'membership_newspaper_2': 'Testata2',
        'membership_role_2': 'RuoloTestata2',
        'membership_newspaper_3': 'Testata3',
        'membership_role_3': 'RuoloTestata3',
        'membership_newspaper_4': 'Testata4',
        'membership_role_4': 'RuoloTestata4',
        'phone_number_1': 'TEL1',
        'phone_type_1': 'TEL1TIPO',
        'phone_default_1': 'TEL1PREDEF',
        'phone_note_1': 'TEL1NOTE',
        'phone_number_2': 'TEL2',
        'phone_type_2': 'TEL2TIPO',
        'phone_default_2': 'TEL2PREDEF',
        'phone_note_2': 'TEL2NOTE',
        'phone_number_3': 'TEL3',
        'phone_type_3': 'TEL3TIPO',
        'phone_default_3': 'TEL3PREDEF',
        'phone_note_3': 'TEL3NOTE',
        'groups': 'CATTUTTE',
    }
    messages = dict()
    messages['obj_created'] = 0
    messages['obj_modified'] = 0
    messages['obj_error'] = 0

    @transaction.commit_manually
    def do(self, row, kw, j):
        from publieditor.event.models import Contact, ContactEmailCc, Newspaper, Role, Membership, Phone, ContactGroup

        try:
            if j % 100 == 0:
                print "riga: %s" % j

            # Mappature campi
            role_fields = [
                ('membership_newspaper_1', 'membership_role_1'),
                ('membership_newspaper_2', 'membership_role_2'),
                ('membership_newspaper_3', 'membership_role_3'),
                ('membership_newspaper_4', 'membership_role_4'),
            ]
            phone_fields = [
                ('phone_number_1', 'phone_type_1', 'phone_default_1', 'phone_note_1',),
                ('phone_number_2', 'phone_type_2', 'phone_default_2', 'phone_note_2',),
                ('phone_number_3', 'phone_type_3', 'phone_default_3', 'phone_note_3',),
            ]
            phone_types_dict = {'Telefono': 'P', 'Cellulare': 'M'}
            language = Language.objects.filter(code__iexact='it')[0]
            # Creazione contatto
            email = kw['email'].strip().lower()
            first_name = kw['first_name'].strip()
            last_name = kw['last_name'].strip()
            internal_note = kw['internal_note'].strip()
            language = language
            title = kw['title'].strip().lower()
            contact, created = Contact.objects.get_or_create(
                email=email,
                defaults={
                    'first_name': first_name, 'last_name': last_name,
                    'language': language, 'title': title,
                    'internal_note': internal_note
                }
            )
            if created:
                self.messages['obj_created'] += 1
                # Creazione email CC
                for email in kw['email_ccs'].strip().split(","):
                    if email and email.strip() != "":
                        email_cc = ContactEmailCc(contact=contact, email=email.strip())
                        email_cc.save()
                # Creazione membership
                for newspaper_kw, role_kw in role_fields:
                    newspaper_name = kw[newspaper_kw].strip()
                    if newspaper_name and newspaper_name != '':
                        if Newspaper.objects.filter(name__iexact=newspaper_name).count():
                            newspaper = Newspaper.objects.filter(name__iexact=newspaper_name)[0]
                        else:
                            newspaper = Newspaper.objects.create(name=newspaper_name)
                        role = None
                        role_name = None if kw[role_kw].strip() == '' else kw[role_kw].strip()
                        if role_name:
                            if Role.objects.filter(name__iexact=role_name).count():
                                role = Role.objects.filter(name__iexact=role_name)[0]
                            else:
                                role = Role.objects.create(name=role_name)
                        membership = Membership(contact=contact, newspaper=newspaper, role=role)
                        membership.save()
                # Creazione telefoni
                for phone_number_kw, phone_type_kw, phone_default_kw, phone_note_kw in phone_fields:
                    phone_number = str(int(kw[phone_number_kw])) if isinstance(kw[phone_number_kw], (float, int)) else kw[phone_number_kw].strip()
                    if phone_number and phone_number != "":
                        phone_type = phone_types_dict[kw[phone_type_kw].strip()]
                        default = False if kw[phone_default_kw].strip() in ["FALSE", ""] else True
                        phone = Phone(
                            contact=contact,
                            number=phone_number,
                            phone_type=phone_type,
                            default=default,
                            note=kw[phone_note_kw].strip()
                        )
                        phone.save()
                # Creazione gruppi
                for group_name in kw['groups'].strip().split(","):
                    group_name = group_name.strip()
                    if group_name and group_name != "":
                        if ContactGroup.objects.filter(name__iexact=group_name).count():
                            contact_group = ContactGroup.objects.filter(name__iexact=group_name)[0]
                        else:
                            contact_group = ContactGroup.objects.create(name=group_name)
                        contact.groups.add(contact_group)
            else:
                self.messages['obj_modified'] += 1
            transaction.commit()
        except Exception, e:
            self.messages['obj_error'] += 1
            print 'Riga: %s - errore %s' % (j, e)
            transaction.rollback()


class Command(BaseCommand):
    """
    Get all messages from configured Host
    """
    help = "Import newspapers FCA format"

    option_list = BaseCommand.option_list + (
        make_option('-l', '--limit', type=int, dest='limit', help='Limit Messages',
                    default=0),
        make_option('-s', '--start', type=int, dest='start', help='Start from message n. START',
                    default=0),
        make_option('-f', '--file', dest='filename', help='Read file in argument', default=None),
    )

    output_transaction = True

    def handle(self, *args, **opts):
        limit = opts.get('limit')
        start = opts.get('start')
        if not opts.get('filename'):
            print "Manca il nome del file da importare (opt -f)"
            sys.exit(1)
        if not Language.objects.filter(code__iexact='it').count():
            print "Nessuna lingua italiana presente nel database (i18n)"
            sys.exit(1)
        import_file = NewspaperFile(opts.get('filename'), auto=True)
        import_file.read(start=start, limit=limit)
        print import_file.messages


