# coding: utf-8
"""
.. _aziende_attive:

Aziende attive
=============================

Command per utilizzo::

  dj aziende_attive

:-d: variabile Booleana per cancellare il messaggi una volta recuperati
:-l: variabile Numero per limitare il numero di messaggi recuperare
:-v: variabile Booleana per visualizzare i print
:-b, --backup: backup messages in 
"""

import csv
import sys

from optparse import make_option

from django.core.management.base import BaseCommand
from django.db import connection, transaction
from django.db.models import Q

from publieditor.event.models import(
    Contact, Phone, ContactAddress, ContactGroup, Membership, Organization, Role, Newspaper
)
from jmb.i18n.models import Language
from jmb.l10n.models import Country

class Command(BaseCommand):
    """Get all messages from configured Host
    """
    help = "Marca come attive le aziende del file xls?"

    option_list = BaseCommand.option_list + (
        make_option('-D', '--debug', action='store_true', default=False,
                    dest='debug_message', help='Debug '),
        make_option('-l', '--limit', type=int, dest='limit', help='Limit Messages',
                    default=0),
        make_option('-s', '--start', type=int, dest='start', help='Start from message n. START',
                    default=0),
        make_option('-f', '--file', dest='filename', help='Leggi i dati dal file FILE', default=None),
    )

    output_transaction = True

    def handle(self, *args, **opts):
        if not opts.get('filename'):
            print "Manca il nome del file da importare (opt -f)"
            sys.exit(1)

        # ELimino tutti i record nelle tabelle interessate
        for table in [Contact, Phone, ContactAddress, ContactGroup]:
            table.objects.all().delete()

        # Recupero la lingua italiano una sola volta
        try:
            default_language = Language.objects.get(name__iexact='italiano')
        except Language.DoesNotExist:
            print "Lingua italiana da utilizzare come default non impostata"
            return

        # Controllo ci sia almeno un country impostato
        try:
            Country.objects.all()[0]
        except Exception, e:
            print "Nessun country presente in database, impossibile effettuare import"
            return

        reader = csv.reader(open(opts.get('filename'), 'rb'), delimiter=';')
        row_num = 0
        for row in reader:
            if row_num == 0:
                header = row
            else:
                self.create_contact(header, row, default_language)
            row_num += 1

    def update_tables_sequence(self):
        for table in ['event_contact', 'event_phone', 'event_contactaddress', 'event_contactgroup']:
            self.set_sequence(table)

    def create_contact(self, header, row, default_language):
        col_num = 0
        contact_dict = dict()
        for value in row:
            contact_dict[header[col_num]] = value.strip()
            col_num += 1

        # Creazione contatto
        contact = Contact()
        #contact.id
        contact.first_name = contact_dict["first"].strip() if "first" in contact_dict else ""
        contact.last_name = contact_dict["last"].strip() if "last" in contact_dict else ""
        contact.title = "mr"
        if "title" in contact_dict:
            title = contact_dict["title"].strip()
            contact.title = "mrs" if title in ["Sig.ra", "Mrs.", "Mrs"] else "mr"
        contact.email = contact_dict["email"].strip() if "email" in contact_dict else ""
        contact.language = default_language
        if "status" in contact_dict:
            status = contact_dict["status"].strip()
            contact.status = 1 if status in ["online", "new"] else 0
        contact.save()

        # Inserimento telefoni
        for key in ["phone", "phone2"]:
            if key in contact_dict:
                phone_number = contact_dict[key].strip()[:50]
                if phone_number not in ["NULL", ""]:
                    if not contact.phones.filter(number=phone_number).count():
                        phone = Phone()
                        phone.contact = contact
                        phone.phone_type = 'P'
                        phone.number = phone_number
                        phone.save()

        # Inserimento indirizzi
        contact_address = ContactAddress()
        contact_address.contact = contact
        contact_address.name = contact_dict["address"].strip() if "address" in contact_dict else ""
        contact_address.locality = contact_dict["city"].strip() if "city" in contact_dict else ""
        contact_address.province = contact_dict["prov"].strip()[:2] if "prov" in contact_dict else ""
        contact_address.zip_code = contact_dict["cap"].strip() if "cap" in contact_dict else ""
        if "country" in contact_dict:
            country = contact_dict["country"].strip()
            try:
                contact.country = Country.objects.get(Q(name__iexact=country) | Q(printable_name__iexact=country))
            except Country.DoesNotExist:
                contact.country = Country.objects.all()[0]
        contact_address.save()

        # Inserimento in gruppi
        if "groups" in contact_dict:
            for group in contact_dict["groups"].split(","):
                if not ContactGroup.objects.filter(name=group).count():
                    contact_group = ContactGroup()
                    contact_group.name = group
                    contact_group.save()
                    contact.groups.add(contact_group)

        ######################################################################
        # Gestione membership (company_role)
        ######################################################################

        # Check organization
        # contact_organizations = []
        # if 'company' in contact_dict:
        #     companies = contact_dict['company'].split("/")
        #     for company in companies:
        #         organization_name = company.strip()
        #         try:
        #             organization = Organization.objects.get(name=organization_name)
        #         except Organization.DoesNotExist:
        #             organization = Organization()
        #             organization.name = organization_name
        #             organization.save()
        #         contact_organizations.append(organization)
        #
        # # Check role
        # contact_roles = []
        # if 'job' in contact_dict:
        #     roles = contact_dict['job'].split("/")
        #     for role in roles:
        #         role_name = role.strip()
        #         try:
        #             role = Role.objects.get(name=role_name)
        #         except Role.DoesNotExist:
        #             role = Role()
        #             role.name = role_name
        #             role.save()
        #         contact_roles.append(role)
        #
        # print len(contact_organizations) == len(contact_roles)

        # # Check newspaper
        # try:
        #     newspaper = Newspaper.objects.get(name="non definito")
        # except Newspaper.DoesNotExist:
        #     newspaper = Newspaper()
        #     newspaper.name = "non definito"
        #     newspaper.save()

        # # Inserimento membership
        # if company and role:
        #     membership = Membership()
        #     membership.contact = contact
        #     membership.organization = organization
        #     membership.role = role
        #     membership.newspaper = newspaper
        #     membership.save()

        print "Inserito contatto %s: %s %s" % (contact.id, contact.first_name, contact.last_name)

    def set_sequence(self, table):
        cursor = connection.cursor()
        select = "SELECT setval('%s_id_seq', (SELECT MAX(id) FROM %s));" % (table, table)
        cursor.execute(select)
        row = cursor.fetchone()

# Example usage: dj import_aziende_xslx_contatti -f /tmp/Contatti.xlsx
