# coding: utf-8
"""
Example usage
dj import_lifestyle_contacts -f .src/publieditor.event/publieditor/event/static/event/import/LISTA_LIFESTYLE_CONNEXIA.xls
"""

import logging
import sys

from django.core.management.base import BaseCommand
from django.db import connection, transaction

from optparse import make_option

from jmb.core.utils.data_import import XlsFile,  CsvFile
from jmb.i18n.models import Language


logger = logging.getLogger('imports')


class NewspaperFile(XlsFile):
    fields_map = {
        'is_import': 'ISIMPORT',
        'title': 'Title',
        'last_name': 'Cognome',
        'first_name': 'Nome',
        'newspaper': 'Testata',
        'redazione_phone_1': 'TelRedazione1',
        'redazione_phone_2': 'TelRedazione2',
        'direct_phone_1': 'Teldiretto1',
        'direct_phone_2': 'TelDiretto2',
        'email': 'Emaildiretta'
    }
    messages = dict()
    messages['obj_created'] = 0
    messages['obj_modified'] = 0
    messages['obj_error'] = 0

    @transaction.commit_manually
    def do(self, row, kw, j):
        from publieditor.event.models import Contact, Newspaper, Membership, Phone, ContactGroup

        try:
            if j % 100 == 0:
                print "riga: %s" % j

            language = Language.objects.filter(code__iexact='it')[0]
            contact_group, contact_group_created = ContactGroup.objects.get_or_create(name="Lifestyle")
            phone_type = 'P'

            # Creazione contatto
            if kw['is_import'].strip() != "":
                email = kw['email'].strip().lower()
                if email and email != "":
                    first_name = kw['first_name'].strip()
                    last_name = kw['last_name'].strip()
                    language = language
                    title = kw['title'].strip().lower()
                    contact, created = Contact.objects.get_or_create(
                        email=email,
                        defaults={
                            'first_name': first_name, 'last_name': last_name,
                            'language': language, 'title': title,
                        }
                    )
                    if created:
                        # Associazione gruppo
                        contact.groups.add(contact_group)
                        # Creazione membership
                        newspaper_name = kw['newspaper'].strip()
                        if newspaper_name and newspaper_name != '':
                            if Newspaper.objects.filter(name__iexact=newspaper_name).count():
                                newspaper = Newspaper.objects.filter(name__iexact=newspaper_name)[0]
                            else:
                                newspaper = Newspaper.objects.create(name=newspaper_name)
                            membership = Membership(contact=contact, newspaper=newspaper)
                            membership.save()
                        # Creazione telefoni
                        for field_name in ['redazione_phone_1', 'redazione_phone_2', 'direct_phone_1', 'direct_phone_2']:
                            phone_number = kw[field_name].strip()
                            if phone_number and phone_number != "":
                                phone = Phone(
                                    contact=contact,
                                    number=phone_number,
                                    phone_type=phone_type,
                                )
                                phone.note = "Redazione" if field_name.startswith("redazione") else "Diretto"
                                phone.save()
                        self.messages['obj_created'] += 1
                    else:
                        self.messages['obj_modified'] += 1
            transaction.commit()
        except Exception, e:
            self.messages['obj_error'] += 1
            print 'Riga: %s - errore %s' % (j, e)
            transaction.rollback()


class Command(BaseCommand):
    """
    Get all messages from configured Host
    """
    help = "Import newspapers FCA format"

    option_list = BaseCommand.option_list + (
        make_option('-l', '--limit', type=int, dest='limit', help='Limit Messages',
                    default=0),
        make_option('-s', '--start', type=int, dest='start', help='Start from message n. START',
                    default=0),
        make_option('-f', '--file', dest='filename', help='Read file in argument', default=None),
    )

    output_transaction = True

    def handle(self, *args, **opts):
        limit = opts.get('limit')
        start = opts.get('start')
        if not opts.get('filename'):
            print "Manca il nome del file da importare (opt -f)"
            sys.exit(1)
        if not Language.objects.filter(code__iexact='it').count():
            print "Nessuna lingua italiana presente nel database (i18n)"
            sys.exit(1)
        import_file = NewspaperFile(opts.get('filename'), auto=True)
        import_file.read(start=start, limit=limit)
        print import_file.messages


