# coding: utf-8
"""
Example usage
dj import_newspapers -f .src/publieditor.event/publieditor/event/static/event/import/RSVP_FCA_IMPORT_NEWSPAPERS.xlsx
"""

import logging
import sys

from django.core.management.base import BaseCommand
from django.db import connection, transaction

from optparse import make_option

from jmb.core.utils.data_import import XlsFile,  CsvFile


logger = logging.getLogger('imports')


class NewspaperFile(XlsFile):
    fields_map = {
        'name': 'TestataNome',
        'types': 'TestataTipo',
        'periodicity': 'TestataPeriodicit',
        'categories': 'TestataTipologia',
        'sectors': 'TestataSettore',
        'url': 'TestataURL',
    }
    messages = dict()
    messages['obj_created'] = 0
    messages['obj_modified'] = 0
    messages['obj_error'] = 0

    @transaction.commit_manually
    def do(self, row, kw, j):
        from publieditor.event.models import Newspaper, NewspaperType, NewspaperCategory, NewspaperSector

        try:
            if j % 100 == 0:
                print "riga: %s" % j

            name = kw['name'].strip()[0:100]
            periodicity = kw['periodicity'].strip().lower()
            url = kw['url'].strip()[0:100]
            relateds = {
                'categories': {'model': NewspaperCategory, 'field_name': 'categories'},
                'sectors': {'model': NewspaperSector, 'field_name': 'sectors'},
                'types': {'model': NewspaperType, 'field_name': 'types'},
            }
            # Header
            newspaper, newspaper_created = Newspaper.objects.get_or_create(name=name)
            newspaper.url = url
            newspaper.periodicity = periodicity
            newspaper.save()

            # Relateds (categories, sectors, types)
            for related_name, related_dict in relateds.iteritems():
                values = kw[related_name].strip().split(",")
                for val in values:
                    val = val.strip()
                    if val and val != "":
                        object, created = related_dict['model'].objects.get_or_create(name=val)
                        getattr(newspaper, related_dict['field_name']).add(object)

            transaction.commit()
            if newspaper_created:
                self.messages['obj_created'] += 1
                logger.info('Riga: %s - oggetto creato %s' % (j, newspaper))
            else:
                self.messages['obj_modified'] += 1
                logger.info('Riga: %s - oggetto modificato %s' % (j, newspaper))

        except Exception, e:
            self.messages['obj_error'] += 1
            print 'Riga: %s - errore %s' % (j, e)
            transaction.rollback()


class Command(BaseCommand):
    """
    Get all messages from configured Host
    """
    help = "Import newspapers FCA format"

    option_list = BaseCommand.option_list + (
        make_option('-l', '--limit', type=int, dest='limit', help='Limit Messages',
                    default=0),
        make_option('-s', '--start', type=int, dest='start', help='Start from message n. START',
                    default=0),
        make_option('-f', '--file', dest='filename', help='Read file in argument', default=None),
    )

    output_transaction = True

    def handle(self, *args, **opts):
        limit = opts.get('limit')
        start = opts.get('start')
        if not opts.get('filename'):
            print "Manca il nome del file da importare (opt -f)"
            sys.exit(1)
        import_file = NewspaperFile(opts.get('filename'), auto=True)
        import_file.read(start=start, limit=limit)
        print import_file.messages


