# coding: utf-8
"""
Example usage
dj import_railway_stations -f .src/publieditor.event/publieditor/event/static/event/import/RSVP_FCA_IMPORT_RAILWAY_STATIONS.xlsx
"""

import logging
import sys

from django.core.management.base import BaseCommand
from django.db import transaction

from optparse import make_option

from jmb.core.utils.data_import import XlsFile


logger = logging.getLogger('imports')


class StationFile(XlsFile):

    fields_map = {
        'name': 'Stazione',
        'place': 'Comune',
    }
    messages = dict()
    messages['obj_created'] = 0
    messages['obj_modified'] = 0
    messages['obj_error'] = 0

    @transaction.commit_manually
    def do(self, row, kw, j):
        from publieditor.event.models import Station

        try:
            if j % 100 == 0:
                print "riga: %s" % j

            # Creazione stazione
            name = kw['name'].strip().lower()
            place = kw['place'].strip()
            station, created = Station.objects.get_or_create(
                name=name,
                defaults={'place': place}
            )
            if created:
                self.messages['obj_created'] += 1
            else:
                self.messages['obj_modified'] += 1
            transaction.commit()
        except Exception, e:
            self.messages['obj_error'] += 1
            print 'Riga: %s - errore %s' % (j, e)
            transaction.rollback()


class Command(BaseCommand):
    """
    Get all messages from configured Host
    """
    help = "Import newspapers FCA format"

    option_list = BaseCommand.option_list + (
        make_option('-l', '--limit', type=int, dest='limit', help='Limit Messages',
                    default=0),
        make_option('-s', '--start', type=int, dest='start', help='Start from message n. START',
                    default=0),
        make_option('-f', '--file', dest='filename', help='Read file in argument', default=None),
    )

    output_transaction = True

    def handle(self, *args, **opts):
        limit = opts.get('limit')
        start = opts.get('start')
        if not opts.get('filename'):
            print "Manca il nome del file da importare (opt -f)"
            sys.exit(1)
        import_file = StationFile(opts.get('filename'), auto=True)
        import_file.read(start=start, limit=limit)
        print import_file.messages
