# coding: utf-8
"""
Example usage
dj import_rsvp -f .src/publieditor.event/publieditor/event/static/event/import/RSVP_IMPORT.xlsx
"""

import datetime, logging, sys, urllib2, xlrd
from optparse import make_option
from urlparse import urlparse

from django.conf import settings
from django.core.files import File
from django.core.files.temp import NamedTemporaryFile
from django.core.management.base import BaseCommand
from django.db import connection
from django.db.utils import IntegrityError
from django.template.defaultfilters import slugify

from jmb.i18n.models import Language
from jmb.address.models import AddressScope
from publieditor.event.models import (
    Contact, ContactEmailCc, ContactGroup, Membership, Role, Newspaper,
    ContactAddress, Phone, AreaCode,
    Event, EventTranslation, Invite, DelegatedContact, Card
)

logger = logging.getLogger('imports')


class RsvpFileImport():

    def __init__(self, file_path):
        workbook = xlrd.open_workbook(file_path)
        self.worksheet_contacts = workbook.sheet_by_name('users_new')
        self.worksheet_events = workbook.sheet_by_name('events_ok')
        self.worksheet_invites = workbook.sheet_by_name('invites_ok')
        self.worksheet_phones = workbook.sheet_by_name('phones')
        self.default_language = Language.objects.get(code__iexact='it')
        self.english_language = Language.objects.get(code__iexact='en')
        self.found_contacts_map = dict()
        self.found_events_map = dict()

    def bulk_import(self):
        self.import_contacts()
        self.import_phones()
        self.import_events()
        self.import_invites()
        self.set_sequence('event_contact')
        self.set_sequence('event_event')
        self.set_sequence('event_invite')

    def import_contacts(self):
        print "Importazione contatti"
        ws = self.worksheet_contacts
        keys = [ws.cell(0, col_index).value for col_index in xrange(ws.ncols)]
        for row_index in range(1, ws.nrows):
            smoking_map = {"yes": "smoking", "no": "nosmoking", "": "none"}
            food_map = {"vegan": "vegan", "vegetarian": "vegetarian", "kosher": "kosher", "": "none"}
            row_dict = {
                keys[col_index]: self.clean_value(ws.cell(row_index, col_index).value)
                for col_index in xrange(ws.ncols)
            }
            Contact.objects.filter(id=row_dict["id"]).delete()
            # Gestione duplicati
            if Contact.objects.filter(id=row_dict["id"]).count():
                new_id = self.get_new_id(ws, Contact)
                self.found_contacts_map[row_dict["id"]] = new_id
                row_dict["id"] = new_id
            print "Contatto: %s" % row_dict["id"]
            # Creazione
            if row_dict["id"]:
                contact = Contact(
                    id=row_dict["id"],
                    title=row_dict["title"].lower(),
                    language=self.english_language if row_dict["lang"] == "english" else self.default_language,
                    first_name=row_dict["first"],
                    last_name=row_dict["last"],
                    email=row_dict["email"],
                    status=1 if row_dict["status"].lower() == "online" else 0,
                    smoking=smoking_map[row_dict["smoking"].lower()],
                    food=food_map[row_dict["food"].lower()],
                    food_note=row_dict["note_alimentari"],
                    height=row_dict["height"],
                    weight=row_dict["weight"],
                    shoes_size=row_dict["shoes"],
                    sweater_size=row_dict["sweater"].lower(),
                )
                if row_dict["privacy_OK"]:
                    contact.privacy_acceptance_date = row_dict["privacy_OK"]
                contact.save()
                # Gestione gruppi
                for group in row_dict["groups_OK"].split(","):
                    if group.strip():
                        contact_group, contact_group_created = ContactGroup.objects.get_or_create(name=group.strip())
                    contact.groups.add(contact_group)
                # Gestione cc_emails
                for email in row_dict["cc"].split(","):
                    if email.strip():
                        ContactEmailCc.objects.create(contact=contact, email=email.strip())
                # Gestione frequent flyer
                for frequent_flyer in row_dict["Frequent_flyer"].split(";"):
                    if frequent_flyer.strip():
                        Card.objects.create(contact=contact, number=frequent_flyer.strip())
                # Gestione membership
                if row_dict["testata"]:
                    role, role_created = Role.objects.get_or_create(name=row_dict["job"].title())
                    if row_dict["testata"]:
                        newspaper, newspaper_created = Newspaper.objects.get_or_create(name=row_dict["testata"].title())
                        Membership.objects.get_or_create(contact=contact, newspaper=newspaper, defaults={'role': role})
                    for other_newspaper in row_dict["altre testate"].split(","):
                        if other_newspaper:
                            newspaper, newspaper_created = Newspaper.objects.get_or_create(name=other_newspaper.title())
                            Membership.objects.get_or_create(contact=contact, newspaper=newspaper)
                # Gestione address
                contact_address = ContactAddress.objects.create(
                    contact=contact,
                    address_type=self.get_address_type(row_dict["tipo via"]),
                    name=row_dict["indirizzo"].title(),
                    locality=row_dict["city"].title(),
                    zip_code=row_dict["cap"].upper(),
                    province=row_dict["prov"].title(),
                )
                scope, scope_created = AddressScope.objects.get_or_create(
                    name=row_dict["address_kind"].title()
                )
                contact_address.address_scopes.add(scope)
                # Eliminazione
                # contact.delete()

    def import_phones(self):
        print "Importazione telefoni"
        ws = self.worksheet_phones
        keys = [ws.cell(0, col_index).value for col_index in xrange(ws.ncols)]
        for row_index in range(1, ws.nrows):
            row_dict = {
                keys[col_index]: self.clean_value(ws.cell(row_index, col_index).value)
                for col_index in xrange(ws.ncols)
            }
            print "Telefono: %s" % row_index
            if row_dict["id"] in self.found_contacts_map:
                row_dict["id"] = self.found_contacts_map[row_dict["id"]]
            try:
                contact = Contact.objects.get(id=row_dict["id"])
                phone_type_map = {"Telefono": "P", "Cellulare": "M", "Fax": "F"}
                area_code, area_code_created = AreaCode.objects.get_or_create(
                    code=row_dict["phone_country_code"], defaults={'area': ''}
                )
                # Gestione contatto
                Phone.objects.create(
                    contact=contact,
                    area_code=area_code,
                    phone_type=phone_type_map[row_dict["phone_kind"].title()],
                    number=row_dict["phone"]
                )
            except Contact.DoesNotExist:
                print "Contatto con id %s non trovato" % row_dict["id"]

    def import_events(self):
        today = datetime.datetime.now()
        ws = self.worksheet_events
        keys = [ws.cell(0, col_index).value for col_index in xrange(ws.ncols)]
        # img_url = "/home/luca/Scrivania/LogoFCA_coverBase.jpg"
        img_url = 'http://www.monashores90.com/wp-content/uploads/2010/06/rsvp-card.jpg'
        img_temp = NamedTemporaryFile(delete=True)
        img_temp.write(urllib2.urlopen(img_url).read())
        img_temp.flush()
        events_date = datetime.datetime(1900, 1, 1)
        for row_index in range(1, ws.nrows):
            row_dict = {
                keys[col_index]: self.clean_value(ws.cell(row_index, col_index).value)
                for col_index in xrange(ws.ncols)
            }
            if row_dict["id"]:
                # Gestione slug se duplicato
                event_file_name = row_dict["NAME"]
                event_name = event_file_name
                while True:
                    cont = Event.objects.filter(name=event_name).count()
                    if cont:
                        event_name = "%s-%s" % (cont, event_file_name)
                    else:
                        break
                # Gestione duplicati
                if Event.objects.filter(id=row_dict["id"]).count():
                    new_id = self.get_new_id(ws, Event)
                    self.found_events_map[row_dict["id"]] = new_id
                    row_dict["id"] = new_id
                print "Evento: %s" % row_dict["id"]
                # Creazione
                if row_dict["id"]:
                    event = Event.objects.create(
                        id=row_dict["id"],
                        status=0,
                        name=event_name[:100],
                        slug=slugify(event_name[:100]),
                        default_language=self.default_language,
                        landing_page_template=settings.PUBLIEDITOR_EVENT_BLANK_TEMPLATE,
                        event_date_start=events_date,
                        event_date_end=events_date,
                        event_adhesion_deadline=events_date,
                    )
                    event_translation = EventTranslation.objects.create(
                        event=event,
                        language=self.default_language,
                        title=row_dict["title"][:100],
                        web_body="",
                        email_body="",
                        email_from_name="rsvp@publieditor.it",
                    )
                    # name = urlparse(img_url).path.split('/')[-1]
                    # event_translation.header_image.save(name, File(img_temp))

    def import_invites(self):
        ws = self.worksheet_invites
        status_map = {
            "accetto": settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
            "declino": settings.PUBLIEDITOR_EVENT_INVITE_DECLINED,
            "delego": settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED,
        }
        keys = [ws.cell(0, col_index).value for col_index in xrange(ws.ncols)]
        delegated_maps = dict()
        for row_index in range(1, ws.nrows):
            row_dict = {
                keys[col_index]: self.clean_value(ws.cell(row_index, col_index).value)
                for col_index in xrange(ws.ncols)
            }
            if row_dict["id"]:
                print "Invito: %s" % row_dict["id"]
                try:
                    if row_dict["user_id"] in self.found_contacts_map:
                        row_dict["user_id"] = self.found_contacts_map[row_dict["user_id"]]
                    if row_dict["event_id"] in self.found_events_map:
                        row_dict["event_id"] = self.found_events_map[row_dict["event_id"]]
                    event = Event.objects.get(id=row_dict["event_id"])
                    contact = Contact.objects.get(id=row_dict["user_id"])
                    # Gestione duplicati
                    if Invite.objects.filter(id=row_dict["id"]).count():
                        new_id = self.get_new_id(ws, Invite)
                        row_dict["id"] = new_id
                    # Creazione
                    invite = Invite.objects.create(
                        id=row_dict["id"],
                        event=event, contact=contact,
                        status=status_map[row_dict["answer"]] if row_dict["answer"] in status_map else None,
                    )
                    if row_dict["substitute_invite_id"]:
                        delegated_maps[row_dict["id"]] = row_dict["substitute_invite_id"]
                except Contact.DoesNotExist:
                    print "Contatto con id %s non trovato" % contact.id
                except Event.DoesNotExist:
                    print "Evento con id %s non trovato" % event.id
                except IntegrityError:
                    print "Invito già esistente per evento %s e contatto %s" % (event.id, contact.id)
        # Creazione contatti delegati
        for invite_id, delegated_invite_id in delegated_maps.iteritems():
            try:
                invite = Invite.objects.get(id=invite_id)
                delegated_invite = Invite.objects.get(id=delegated_invite_id)
                DelegatedContact.objects.create(
                    invited=delegated_invite.contact,
                    invite=invite,
                    title=delegated_invite.contact.title,
                    first_name=delegated_invite.contact.first_name,
                    last_name=delegated_invite.contact.last_name,
                    email=delegated_invite.contact.email,
                    managed=True
                )
            except Invite.DoesNotExist:
                print "Invito per creazione delega non trovato"

    def get_address_type(self, address_type):
        for (key, value) in settings.ADDRESS_ADDRESS_TYPE_CHOICES:
            if value.lower() == address_type.lower():
                return key
        return None

    def clean_value(self, cell_value):
        if isinstance(cell_value, float):
            cell_value = int(cell_value)
        if isinstance(cell_value, (str, unicode)):
            cell_value = cell_value.strip()
        return cell_value

    def get_new_id(self, ws, model, col_index=1):
        ids = [int(cell.value) for cell in ws.col_slice(0, col_index) if cell.value]
        new_id = max(ids)+1
        while True:
            if model.objects.filter(id=new_id).count():
                new_id += 1
            else:
                break
        return new_id

    def set_sequence(self, table):
        cursor = connection.cursor()
        select = "SELECT setval('%s_id_seq', (SELECT MAX(id) FROM %s));" % (table, table)
        cursor.execute(select)
        row = cursor.fetchone()


class Command(BaseCommand):
    """
    Get all messages from configured Host
    """
    help = "Import database from old RSVP database export"

    option_list = BaseCommand.option_list + (
        make_option('-f', '--file', dest='filename', help='Read file in argument', default=None),
    )

    output_transaction = True

    def handle(self, *args, **opts):
        if not opts.get('filename'):
            print "Manca il percorso del file da importare (opt -f)"
            sys.exit(1)
        if not Language.objects.filter(code__iexact='it').count():
            print "Nessuna lingua italiana presente nel database (i18n)"
            sys.exit(1)
        import_file = RsvpFileImport(opts.get('filename'))
        import_file.bulk_import()