# coding: utf-8
"""
Example usage
dj import_rsvp_sbk -f .src/publieditor.event/publieditor/event/static/event/import/RSVP_SBK_IMPORT.xlsx
"""

import datetime, logging, sys, xlrd
from optparse import make_option

from django.core.management.base import BaseCommand
from django.db import connection

from jmb.i18n.models import Language
from publieditor.event.models import Contact, ContactGroup

logger = logging.getLogger('imports')


class RsvpFileImport():

    def __init__(self, file_path):
        workbook = xlrd.open_workbook(file_path)
        self.worksheet = workbook.sheet_by_index(0)
        self.default_language = Language.objects.get(code__iexact='it')
        self.english_language = Language.objects.get(code__iexact='en')

    def import_contacts(self):
        print "Importazione contatti SBK"
        ws = self.worksheet
        keys = [ws.cell(0, col_index).value for col_index in xrange(ws.ncols)]
        for row_index in range(1, ws.nrows):
            row_dict = {
                keys[col_index]: self.clean_value(ws.cell(row_index, col_index).value)
                for col_index in xrange(ws.ncols)
            }
            language = self.english_language if row_dict["Lang."] == "en" else self.default_language
            found_contacts = Contact.objects.filter(email=row_dict['mail'], language=language)
            if not found_contacts:
                contact = Contact.objects.create(
                    email=row_dict['mail'],
                    language=language,
                    first_name=row_dict['Nome'],
                    last_name=row_dict['Cognome'],
                    status=1 if row_dict["status"].lower() == "online" else 0,
                    title=row_dict["title"].lower(),
                )
                contact_group, contact_group_created = ContactGroup.objects.get_or_create(name=row_dict["Group"])
                contact.groups.add(contact_group)
            else:
                for contact in found_contacts:
                    contact_group, contact_group_created = ContactGroup.objects.get_or_create(name=row_dict["Group"])
                    contact.groups.add(contact_group)

    def clean_value(self, cell_value):
        if isinstance(cell_value, float):
            cell_value = int(cell_value)
        if isinstance(cell_value, (str, unicode)):
            cell_value = cell_value.strip()
        return cell_value


class Command(BaseCommand):
    """
    Get all messages from configured Host
    """
    help = "Import sbk contacts from old SBK RSVP database export"

    option_list = BaseCommand.option_list + (
        make_option('-f', '--file', dest='filename', help='Read file in argument', default=None),
    )

    output_transaction = True

    def handle(self, *args, **opts):
        if not opts.get('filename'):
            print "Manca il percorso del file da importare (opt -f)"
            sys.exit(1)
        if not Language.objects.filter(code__iexact='it').count():
            print "Nessuna lingua italiana presente nel database (i18n)"
            sys.exit(1)
        import_file = RsvpFileImport(opts.get('filename'))
        import_file.import_contacts()