"""
Example usage
dj update_statistics -t 2
"""

import datetime
import logging
import sys

try:
    from logutils.colorize import ColorizingStreamHandler as StreamHandler
    ## FIXME: colora debug ma non warn!!!
except ImportError:
    from logging import StreamHandler

from django.core.management.base import BaseCommand
from django.db import transaction, IntegrityError

from optparse import make_option
from publieditor.event.utils import update_event_statistics
from publieditor.event.models import LastMailingStatusProcessed


class Command(BaseCommand):

    """Get all messages from configured Host
    """
    help = """Update statistics with the MailingStatus data."""

    option_list = BaseCommand.option_list + (
        # -c enables logging using store_true
        make_option('-c', '--console', action='store_true', default=False,
                    help='Debug - write logging to console'),
        # -q changes opt --verbose setting to const
        make_option("-q", "--quiet",
                    action="store_const", const=0, dest="verbose"),
        # limit is an integer
        make_option('-L', '--limit', dest='limit', type='int', help='Limit Messages', default=1000),
        make_option('-l', '--debug-level', default='info',
                    help='Set debug level (debug, info, warnings) for console', ),
        make_option('-t', '--time', dest='time',
                    help='''Duration execution in minutes''', default=0),
    )

    output_transaction = True

    def handle(self, *args, **options):
        level = getattr(logging,  options.get('debug_level').upper())
        if options.get('console'):
            console_handler = StreamHandler()
            console_handler.setLevel(level)
            update_event_statistics.logger.addHandler(console_handler)
            update_event_statistics.logger.setLevel(level)
        if options.get('time') < 0:
            print "You must insert the duration of executions in minutes"
        try:
            if options.get('noexe'):
                try:
                    with transaction.atomic():
                        update_event_statistics.update_statistics(options.get('time'))
                        raise IntegrityError
                except IntegrityError:
                    update_event_statistics.logger.info("{} End of simulation".format(
                        datetime.datetime.now().strftime("%d-%m-%Y %H:%M")
                    ))
                    last_st = LastMailingStatusProcessed.objects.get()
                    last_st.is_script_running = False
                    last_st.save()
                    sys.exit(1)
            else:
                try:
                    update_event_statistics.update_statistics(options.get('time'))
                    last_st = LastMailingStatusProcessed.objects.get()
                    last_st.is_script_running = False
                    last_st.save()
                    sys.exit(0)
                except Exception, e:
                    update_event_statistics.logger.critical("{} Error: {} ".format(
                        datetime.datetime.now().strftime("%d-%m-%Y %H:%M"),
                        e
                    ))
                    last_st = LastMailingStatusProcessed.objects.get()
                    last_st.is_script_running = False
                    last_st.save()
                    sys.exit(1)
        except ValueError:
            print "Data format incorrect. Data must be in the form dd-mm-yyyy"
            last_st = LastMailingStatusProcessed.objects.get()
            last_st.is_script_running = False
            last_st.save()
            sys.exit(1)


