# -*- coding: utf-8 -*-

from django.conf import settings
from django.db.models.signals import post_save
from django.dispatch import receiver
from jmb.mailer.models import EmailObject, MailingStatus


@receiver(post_save, sender=EmailObject)
def update_invite_on_email_sending(sender, **kwargs):
    """
    Updates invite status when invite email is sent
    """
    email_object = kwargs.get('instance')
    if hasattr(email_object, 'inviteemailobject'):
        if email_object.status == settings.MAILER_EMAIL_STATUS_SENT:
            invite = email_object.inviteemailobject.invite
            invite.update_on_cron_sending()


@receiver(post_save, sender=MailingStatus)
def send_email_on_invite_email_opening(sender, **kwargs):
    """
    Signal which send email to backoffice when an invite is opened.
    Signal is sent from MailingStatus post_save
    """
    from publieditor.event.models import InviteFirstOpening
    if settings.PUBLIEDITOR_EVENT_SEND_EMAIL_ON_INVITE_OPENING:
        mailing_status = kwargs.get('instance')
        email_object = mailing_status.email_object
        check_stats = [
            settings.MAILER_MAILINGSTATUS_STATUS_OPENED,
            settings.MAILER_MAILINGSTATUS_STATUS_OPENED_ON_SITE
        ]
        if hasattr(email_object, 'inviteemailobject') and mailing_status.status in check_stats:
            invite_email_object = email_object.inviteemailobject
            invite = invite_email_object.invite
            event = invite.event
            if not event.is_newsletter and not invite_email_object.is_test and not hasattr(invite, 'first_opening'):
                InviteFirstOpening.objects.create(invite=invite)
                invite.send_email_on_invite_email_opening(mailing_status)
