# -*- coding: utf-8 -*-

import csv
import os
import logging
import xlwt

import autocomplete_light
from django.conf import settings
from django.utils.translation import ugettext as _
from django.contrib import admin
from django.http import HttpRequest

from jmb.async.tasks import GenericJmbJobtasticTask
from jmb.async.tasks import JmbJobtasticTask
from jmb.core.admin.options import _export_data
from jmb.async.models import Task

logger = logging.getLogger('celery.task')
autocomplete_light.autodiscover() 
########################################################################################################################
# CONTACT TASKS ########################################################################################################
########################################################################################################################
class ContactAddToGroupTask(GenericJmbJobtasticTask):
    """
    This task is used to insert a queryset of contacts in a defined group.
    The group was choosen by user in admin via extra_action_form and we have it in kwargs['contact_group_id']
    """
    task_name = _("Contact:  Add to group")

    def set_report_msg_dict(self):
        from publieditor.event.models import ContactGroup
        contact_group_id = self.kwargs.get("contact_group_id")
        contact_group = ContactGroup.objects.get(id=contact_group_id)
        self.report_msg_dict = dict(
            success_msg_0=_("no contacts added to group %s") % contact_group.name,
            success_msg_1=_("1 new contact added to group %s") % contact_group.name,
            success_msg_n=_("%s new contacts added to group %s") % (
                self.success_count, contact_group.name,
            ),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, contact):
        from publieditor.event.models import ContactGroup
        contact_group_id = self.kwargs.get("contact_group_id")
        contact_group = ContactGroup.objects.get(id=contact_group_id)
        result, msg = contact.add_to_group(contact_group)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class ContactAddToEventInvitedListTask(GenericJmbJobtasticTask):
    """
    This task is used to insert a queryset of contacts in a event invited list.
    The event was choosen by user in admin via extra_action_form and we have it in kwargs['event_id']
    """
    task_name = _("Contact:  Add to event invited list")

    def set_report_msg_dict(self):
        from publieditor.event.models import Event
        event_id = self.kwargs.get("event_id")
        event = Event.objects.get(id=event_id)
        self.report_msg_dict = dict(
            success_msg_0=_("no contacts added to event %s invited list") % event.name,
            success_msg_1=_("1 new contact added to event %s invited list") % event.name,
            success_msg_n=_("%s new contacts added to event %s invited list") % (
                self.success_count, event.name,
            ),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, contact):
        from publieditor.event.models import Event
        event_id = self.kwargs.get("event_id")
        event = Event.objects.get(id=event_id)
        result, msg = contact.add_to_event_invited_list(event)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class ContactExportCsvAsync(JmbJobtasticTask):
    task_name = _("Contact: Async export")
    significant_kwargs = []
    herd_avoidance_timeout = 0
    cache_duration = -1

    def calculate_result(self, model=None, pks=None, select_across=None, get=None):
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        if select_across:
            request = HttpRequest()
            request.GET = get
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)
        rows = []
        for count, row in enumerate(_export_data(modeladmin, queryset=qs)):
            rows += [row]
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
        task_id = self.request.id
        output_filename = self.create_file(
            rows, output_filename=os.path.join(settings.MEDIA_ROOT, 'tasks', "%s.csv" % task_id)
        )
        task = Task.objects.get(task_id=task_id)
        msg = 'exported %s records' % count
        task.message = msg
        task.url = "tasks/%s.csv" % task_id
        task.save()
        return {'message': msg, 'url': output_filename}

    @staticmethod
    def create_file(rows, output_filename=None):
        encoded_rows = []
        for row in rows:
            encoded_rows.append([value.encode("utf-8") for value in row])
        with open(output_filename, 'wb') as writer:
            out = csv.writer(writer, delimiter=';', quotechar='"', quoting=csv.QUOTE_ALL)
            out.writerows(encoded_rows)
        return output_filename


class ContactExportXlsAsync(JmbJobtasticTask):
    task_name = _("Contact: Async export")
    significant_kwargs = []
    herd_avoidance_timeout = 0
    cache_duration = -1

    def calculate_result(self, model=None, pks=None, select_across=None, get=None):
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        if select_across:
            request = HttpRequest()
            request.GET = get
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)
        rows = []
        for count, row in enumerate(_export_data(modeladmin, queryset=qs)):
            rows += [row]
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
        task_id = self.request.id
        output_filename = self.create_file(
            rows, output_filename=os.path.join(settings.MEDIA_ROOT, 'tasks', "%s.xls" % task_id)
        )
        task = Task.objects.get(task_id=task_id)
        msg = 'exported %s records' % count
        task.message = msg
        task.url = "tasks/%s.xls" % task_id
        task.save()
        return {'message': msg, 'url': output_filename}

    @staticmethod
    def create_file(rows, output_filename):
        wb = xlwt.Workbook()
        ws = wb.add_sheet("Contacts export")
        for row_num, row in enumerate(rows):
            for col_num, value in enumerate(row):
                ws.write(row_num, col_num, value)
        wb.save(output_filename)
        return output_filename


########################################################################################################################
# DELEGATED CONTACT TASKS ##############################################################################################
########################################################################################################################
class DelegatedContactInviteSendTask(GenericJmbJobtasticTask):
    """
    This task is used to send an invite to a delegated contact
    """
    task_name = _("Delegated Contact: Send invite")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no invites sent"),
            success_msg_1=_("1 invite sent"),
            success_msg_n=_("%s invites sent"),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, delegated_contact):
        from publieditor.event.models import Invite
        test = self.kwargs.get("test", False)
        invite, invite_created = Invite.objects.get_or_create(
            contact=delegated_contact.invited, event=delegated_contact.invite.event,
            defaults={'status': settings.PUBLIEDITOR_EVENT_INVITE_PENDING}
        )
        result, msg = invite.send(test)
        if result:
            self.success_count += 1
            delegated_contact.managed = True
            delegated_contact.save()
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class DelegatedContactInviteCreateTask(GenericJmbJobtasticTask):
    """
    This task is used to create an invite for a delegated contact
    """
    task_name = _("Delegated Contact: Send invite")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no invites created"),
            success_msg_1=_("1 invite created"),
            success_msg_n=_("%s invites created"),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, delegated_contact):
        from publieditor.event.models import Invite
        invite, invite_created = Invite.objects.get_or_create(
            contact=delegated_contact.invited, event=delegated_contact.invite.event,
            defaults={'status': settings.PUBLIEDITOR_EVENT_INVITE_PENDING}
        )
        if invite_created:
            self.success_count += 1
        else:
            self.error_count += 1


# class ContactSendInviteTask(GenericJmbJobtasticTask):
#
#     task_name = _("Contact: Send invite")
#
#     def set_report_msg_dict(self):
#         self.report_msg_dict = dict(
#             success_msg_0=_("no invites sent"),
#             success_msg_1=_("1 invite sent"),
#             success_msg_n=_("%s invites sent") % self.success_count,
#             error_msg_1=_("1 error found"),
#             error_msg_n=_("%s error found") % self.error_count,
#         )
#
#     def do_action(self, contact):
#         event_id = self.kwargs.get("event_id")
#         test = self.kwargs.get("test", False)
#         event = Event.objects.get(id=event_id)
#         result, msg = contact.send_invite(event, test)
#         if result:
#             self.success_count += 1
#         else:
#             self.error_msg.append(msg)
#             self.error_count += 1
#
# class SendContactDocumentsTask(GenericJmbJobtasticTask):
#     task_name = _("Send documents")
#     significant_kwargs = []
#     herd_avoidance_timeout = 60
#     cache_duration = 0
#
#     def calculate_result(
#             self, model=None, pks=None, select_across=None,
#             get=None, req=None, event=None, test=None):
#
#         logger.info('Sending documents xtastic')
#         admin.autodiscover()
#         modeladmin = admin.site._registry[model]
#         request = HttpRequest()
#         request.GET = get
#         request.REQUEST = req
#         if select_across:
#             qs = modeladmin.queryset(request)
#             n_of_steps = qs.count()
#         else:
#             qs = modeladmin.model.objects.filter(id__in=pks)
#             n_of_steps = len(pks)
#         ##########################################################################
#         contacts = qs.filter(status=1)
#         count = 0
#         for contact in contacts:
#             default_email = contact.get_default_email()
#             if default_email != "":
#                 send_documents_email(event, contact, request)
#                 count += 1
#         ##########################################################################
#         task_id = self.request.id
#         task = Task.objects.get(task_id=task_id)
#         msg = 'Sent documents to %s/%s contacts' % (count, n_of_steps)
#         task.message = msg
#         task.save()
#         return {'message': msg}


########################################################################################################################
# CONTACT GRPOUP ACTIONS ###############################################################################################
########################################################################################################################
class ContactGroupAddToEventInvitedListTask(GenericJmbJobtasticTask):
    """
    This task is used to insert a queryset of groups in a event invited list.
    The event was choosen by user in admin via extra_action_form and we have it in kwargs['event_id']
    """
    task_name = _("Contact group:  Add to event invited list")

    def set_report_msg_dict(self):
        from publieditor.event.models import Event
        event_id = self.kwargs.get("event_id")
        event = Event.objects.get(id=event_id)
        self.report_msg_dict = dict(
            success_msg_0=_("no groups added to event %s invited list") % event.name,
            success_msg_1=_("1 new group added to event %s invited list") % event.name,
            success_msg_n=_("%s new groups added to event %s invited list") % (
                self.success_count, event.name,
            ),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, group):
        from publieditor.event.models import Event
        event_id = self.kwargs.get("event_id")
        event = Event.objects.get(id=event_id)
        result, msg = group.add_to_event_invited_list(event)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


########################################################################################################################
# MAILING LIST TASKS ###################################################################################################
########################################################################################################################
class MailingListAddToEventInvitedListTask(GenericJmbJobtasticTask):
    """
    This task is used to insert a queryset of mailing lists in a event invited list.
    The event was choosen by user in admin via extra_action_form and we have it in kwargs['event_id']
    """
    task_name = _("Mailing list:  Add to event invited list")

    def set_report_msg_dict(self):
        from publieditor.event.models import Event
        event_id = self.kwargs.get("event_id")
        event = Event.objects.get(id=event_id)
        self.report_msg_dict = dict(
            success_msg_0=_("no mailing list added to event %s invited list") % event.name,
            success_msg_1=_("1 new mailing list added to event %s invited list") % event.name,
            success_msg_n=_("%s new mailing lists added to event %s invited list") % (
                self.success_count, event.name,
            ),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, mailing_list):
        from publieditor.event.models import Event
        event_id = self.kwargs.get("event_id")
        event = Event.objects.get(id=event_id)
        result, msg = mailing_list.add_to_event_invited_list(event)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


########################################################################################################################
# INVITE TASKS #########################################################################################################
########################################################################################################################
class InviteSendTask(GenericJmbJobtasticTask):
    """
    Task for send invites to invited list
    """
    task_name = _("Invite:  Send")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no invites sent"),
            success_msg_1=_("1 invite sent"),
            success_msg_n=_("%s invites sent" % self.success_count),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, invite):
        test = self.kwargs.get("test", False)
        result, msg = invite.send(test)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class InviteReSendTask(GenericJmbJobtasticTask):
    """
    Task for send invites to invited list
    """
    task_name = _("Invite:  Resend")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no invites resent"),
            success_msg_1=_("1 invite resent"),
            success_msg_n=_("%s invites resent" % self.success_count),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, invite):
        test = self.kwargs.get("test", False)
        result, msg = invite.resend(test)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class InviteSendFollowUpTask(GenericJmbJobtasticTask):
    """
    Task for send invites to invited list
    """
    task_name = _("Invite:  Send follow up")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no follow ups sent"),
            success_msg_1=_("1 follow up sent"),
            success_msg_n=_("%s follow up sent" % self.success_count),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, invite):
        test = self.kwargs.get("test", False)
        result, msg = invite.send_followup(test)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class InviteSendDocumentsTask(GenericJmbJobtasticTask):
    """
    Task for send invites to invited list
    """
    task_name = _("Invite:  Send documents")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no documents email sent"),
            success_msg_1=_("1 documents email sent"),
            success_msg_n=_("%s documents emails sent" % self.success_count),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, invite):
        test = self.kwargs.get("test", False)
        result, msg = invite.send_documents(test)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


########################################################################################################################
# EVENT TASKS #########################################################################################################
########################################################################################################################
class EventSendInvitesTask(GenericJmbJobtasticTask):
    """
    Task for send invites to event invited list
    """
    task_name = _("Event:  Send invites")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no invites sent"),
            success_msg_1=_("invites sent for 1 event"),
            success_msg_n=_("invites sent for %s event" % self.success_count),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, event):
        test = self.kwargs.get("test", False)
        result, msg = event.send_invites(test)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class EventReSendInvitesTask(GenericJmbJobtasticTask):
    """
    Task for resend invites to event recipients
    """
    task_name = _("Event:  Resend invite")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no invites resent"),
            success_msg_1=_("invites resent for 1 event"),
            success_msg_n=_("invites resent for %s event" % self.success_count),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, event):
        test = self.kwargs.get("test", False)
        result, msg = event.resend_invites(test)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class EventSendFollowUpsTask(GenericJmbJobtasticTask):
    """
    Task for send follow ups to events recipients
    """
    task_name = _("Event: Send followups")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no follow ups sent"),
            success_msg_1=_("follow ups sent for 1 event"),
            success_msg_n=_("follow ups sent for %s event" % self.success_count),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, event):
        test = self.kwargs.get("test", False)
        result, msg = event.send_followups(test)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class EventSendDocumentsTask(GenericJmbJobtasticTask):
    """
    Task for send documents to events recipients
    """
    task_name = _("Event: Send documents")

    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("no document emails sent"),
            success_msg_1=_("document emails sent for 1 event"),
            success_msg_n=_("document emails sent for %s event" % self.success_count),
            error_msg_1=_("1 error found"),
            error_msg_n=_("%s error found" % self.error_count)
        )

    def do_action(self, event):
        test = self.kwargs.get("test", False)
        result, msg = event.send_documents(test)
        if result:
            self.success_count += 1
        else:
            self.error_msg.append(msg)
            self.error_count += 1


class EventUpdateStatisticsAsyncTask(JmbJobtasticTask):
    """
        This task is used update event statistics
    """
    task_name = _("Event: update statistics")
    significant_kwargs = []
    herd_avoidance_timeout = 0
    cache_duration = -1

    def calculate_result(self, model=None, pks=None, select_across=None, get=None):
        # from django.core.management import call_command
        # call_command('update_event_statistics')
        from publieditor.event.utils.update_event_statistics import update_statistics
        update_statistics()
        task_id = self.request.id
        task = Task.objects.get(task_id=task_id)
        msg = 'Updated Statistics'
        task.message = msg
        task.save()
        return {'message': msg}


