# -*- coding: utf-8 -*-

from django import template
from publieditor.event.models import TransportDetail
from django.utils.translation import ugettext_lazy as _
from django.contrib.admin.templatetags.admin_list import (
    result_headers, result_hidden_fields, items_for_result, ResultList
)
from django.conf import settings

register = template.Library()

@register.filter(name='get_route')
def get_route(values):
    return TransportDetail.objects.filter(
        id__in=values.split("_")
    ).latest('departure').route

@register.filter(name='get_direction_date')
def get_direction_date(values, direction):
    detail = TransportDetail.objects.filter(
        id__in=values.split("_"),
        direction=direction,
    ).latest('departure')
    return detail.departure if direction == "d" else detail.arrival

@register.filter(name='get_directions')
def get_directions(values):
    transport_details = TransportDetail.objects.filter(
        id__in=values.split("_")
    ).all()
    directions = []
    for direction in [detail.direction for detail in transport_details]:
        if direction not in directions:
            directions.append(direction)
    return directions

@register.filter(name='get_details_by_direction')
def get_details_by_direction(values, direction):
    return TransportDetail.objects.filter(
        id__in=values.split("_"),
        direction=direction,
    ).all()

@register.filter(name='get_details_by_direction_count')
def get_details_by_direction_count(values, direction):
    return TransportDetail.objects.filter(
        id__in=values.split("_"),
        direction=direction,
    ).all().count()

@register.filter(name='get_display')
def get_display(direction):
    return _("departure") if direction == "d" else _("return")


def extended_results(cl):
    for res in cl.result_list:
        app_dict = dict()
        invite = ResultList(None, items_for_result(cl, res, None))
        transport_details = {}
        delegated_to = []
        own_answ = []
        free_answ = []
        activity_answers = []
        register_answers = []
        decline_answers = []

        if res.status == settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED:
            transport_details = res.get_transport_details_dict()
            own_answ = list(res.own_answers.all())
            free_answ = list(res.free_answers.all())
            activity_answers = list(res.activity_answers.all())
            register_answers = list(res.register_answers.all())

        if res.status == settings.PUBLIEDITOR_EVENT_INVITE_DECLINED:
            decline_answers = list(res.decline_answers.all())

        if res.status == settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED:
            if len(res.delegated_invites.all()):
                try:
                    del_contact = res.delegated_invites.all()[0].invited
                    delegated_to = [
                        del_contact.first_name,
                        del_contact.last_name,
                        del_contact.email
                    ]
                except Exception as e:
                    delegated_to = []

        app_dict = {
            'invite': invite,
            'transport_details': transport_details,
            'own_answers': own_answ,
            'free_answers': free_answ,
            'activity_answers': activity_answers,
            'register_answers': register_answers,
            'decline_answers': decline_answers,
            'delegated_to': delegated_to
        }
        print transport_details
        yield app_dict


@register.inclusion_tag("admin/event/event/event_invite_change_list_results.html")
def extended_result_list(cl):
    """
    Displays the headers and data list together
    """
    headers = list(result_headers(cl))
    num_sorted_fields = 0
    for h in headers:
        if h['sortable'] and h['sorted']:
            num_sorted_fields += 1
    return {'cl': cl,
            'result_hidden_fields': list(result_hidden_fields(cl)),
            'result_headers': headers,
            'num_sorted_fields': num_sorted_fields,
            'results': list(extended_results(cl))}


# Usage : {{ dict|key:key_name }}
@register.filter(name='key')
def key(d, key_name):
    return d[key_name]
# key = register.filter('key', key)


@register.assignment_tag
def enable_qrcode():
    return settings.PUBLIEDITOR_EVENT_ENABLE_QRCODE
