# -*- coding: utf-8 -*-

from django.conf.urls import url, patterns
from django.contrib.auth.views import logout
from django.views.generic import TemplateView
from django.contrib.auth.decorators import login_required

from publieditor.event.views import LoginView
from publieditor.event.class_views import (
    InviteDispatchView, AcceptFromEmailView, DeclineFromEmailView, DelegateFromEmailView,
    SetSessionByInviteFormView, ForwardingFromEmailView)
from publieditor.event.views import (
    RegisterView, ProfileView, ChangePasswordView, ResetPasswordView, ResetPasswordRequestView, LoginRedirectView,
    ProfileFormAjaxSubmit, ProfileAddressesFormsetAjaxSubmit, ProfilePhonesFormsetAjaxSubmit,
    ProfileEmailsFormsetAjaxSubmit, ProfileMembershipsFormsetAjaxSubmit, ProfileCardsFormsetAjaxSubmit,
)

urlpatterns = patterns(
    'publieditor.event.views',
    url(
        r'^login/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/(?P<next_page>.*)/$',
        LoginRedirectView.as_view(),
        name='login_redirect'
    ),
    url(r'^login/$', LoginView.as_view(), name='login'),
    url(r'^logout/$', logout, {'next_page': '/event/login'}, name='logout'),
    url(
        r'^profile/$',
        login_required(ProfileView.as_view()),
        name='profile'
    ),
    url(
        r'^profile-done/$',
        login_required(TemplateView.as_view(template_name="event/user/profile_done.html")),
        name='profile_done'
    ),
    url(
        r'^privacy-policy/$',
        TemplateView.as_view(template_name="event/user/privacy_policy.html"),
        name='privacy_policy'
    ),
    url(
        r'^company-data/$',
        TemplateView.as_view(template_name="event/user/company_data.html"),
        name='company_data'
    ),
    url(
        r'^cookie-policy/$',
        TemplateView.as_view(template_name="event/user/cookie_policy.html"),
        name='cookie_policy'
    ),
    url(
        r'^profile-form-submit/$',
        login_required(ProfileFormAjaxSubmit.as_view()),
        name='profile_form_submit'
    ),
    url(
        r'^profile-addresses-formset-submit/$',
        login_required(ProfileAddressesFormsetAjaxSubmit.as_view()),
        name='profile_addresses_formset_submit'
    ),
    url(
        r'^profile-phones-formset-submit/$',
        login_required(ProfilePhonesFormsetAjaxSubmit.as_view()),
        name='profile_phones_formset_submit'
    ),
    url(
        r'^profile-emails-formset-submit/$',
        login_required(ProfileEmailsFormsetAjaxSubmit.as_view()),
        name='profile_emails_formset_submit'
    ),
    url(
        r'^profile-memberships-formset-submit/$',
        login_required(ProfileMembershipsFormsetAjaxSubmit.as_view()),
        name='profile_memberships_formset_submit'
    ),
    url(
        r'^profile-cards-formset-submit/$',
        login_required(ProfileCardsFormsetAjaxSubmit.as_view()),
        name='profile_cards_formset_submit'
    ),
    url(
        r'^change_password/$',
        login_required(ChangePasswordView.as_view()),
        name='change_password'
    ),
    url(
        r'^accept_from_email/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)-(?P<invite_id>.+)/$',
        AcceptFromEmailView.as_view(),
        name='accept_from_email'
    ),
    url(
        r'^decline_from_email/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)-(?P<invite_id>.+)/$',
        DeclineFromEmailView.as_view(),
        name='decline_from_email'
    ),
    url(
        r'^delegate_from_email/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)-(?P<invite_id>.+)/$',
        DelegateFromEmailView.as_view(),
        name='delegate_from_email'
    ),
    url(
        r'^forwarding_from_email/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)-(?P<invite_id>.+)/$',
        ForwardingFromEmailView.as_view(),
        name='forwarding_from_email'
    ),
    url(
        r'^registration/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)-(?P<invite_id>.+)/$',
        RegisterView.as_view(),
        name='register_and_redirect'
    ),
    url(
        r'^registration/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
        RegisterView.as_view(),
        name='register'
    ),
    url(
        r'^reset_password_request/$',
        ResetPasswordRequestView.as_view(),
        name='reset_password_request',
    ),
    url(
        r'^reset_password/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
        ResetPasswordView.as_view(),
        name='reset_password',
    ),
    url(
        r'^get_invite_ics/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
        'event_ics_export',
        name='publieditor_event_ics_export'
    ),
    url(
        r'^get_invite_pdf/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
        'get_invite_pdf',
        name='publieditor_event_get_invite_pdf'
    ),
    url(
        r'^poll/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
        'poll',
        name='publieditor_event_poll'
    ),
    url(
        r'^get-pass/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
        'get_invite_pass',
        name='publieditor_event_get_invite_pass'
    ),
    url(
        r'^read-pass/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
        'read_invite_pass',
        name='publieditor_event_read_invite_pass'
    ),
    url(
        r'^set_session_by_invite_form/(?P<invite_id>\d+)/$',
        SetSessionByInviteFormView.as_view(),
        name='set_session_by_invite_form'
    ),
    url(
        r'^(?P<step>[-\w]+)/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
        'set_session_by_invite',
        name='publieditor_event_set_session_by_invite'
    ),
    url(
        r'^my-invite/$',
        InviteDispatchView.as_view(),
        name='publieditor_event_my_invite'
    ),
    url(
        r'^my-invite/(?P<step>[-\w]+)/$',
        InviteDispatchView.as_view(),
        name='publieditor_event_my_invite'
    ),
    url(
        r'^session-expired/$',
        'session_expired',
        name='publieditor_event_session_expired'
    ),
    url(
        r'^deadline-expired/(?P<invite_id>\d+)/$',
        'event_deadline_expired',
        name='publieditor_event_deadline_expired'
    ),
    url(
        r'^test/(?P<contact_id>\d+)/$',
        'set_test_contact',
        name='publieditor_event_contact_set_test'
    ),
    url(
        r'^turn/(?P<turn_id>\d+)/$',
        'set_turn_status',
        name='publieditor_event_turn_set_status'
    ),
)
