# -*- coding: utf-8 -*-

import datetime

from django.conf import settings

from datetime import timedelta
from email.mime.text import MIMEText
from HTMLParser import HTMLParser

from django.core.exceptions import ObjectDoesNotExist
from django.core.files import File
from icalendar import Calendar as iCalendar
from icalendar import Event as iEvent
from icalendar import Alarm as iAlarm

from django.contrib.sites.models import Site

from jmb.core.utils.appypod import AppyPodTemplate

from publieditor.event.models import Invite, Contact
# from publieditor.event.utils.mailer import render_template_from_string


class HTMLStripper(HTMLParser):

    def __init__(self):
        self.reset()
        self.fed = []

    def handle_data(self, d):
        self.fed.append(d)

    def get_data(self):
        val = ''.join(self.fed)
        val = val.replace("\n", "").replace("\t", "")
        return val


def get_ics_mime_part(invite):
    cal = invite.get_ics_calendar()
    part = MIMEText(cal.to_ical(), 'calendar')
    part.add_header('Filename','%s.ics'%invite.event.slug)
    part.add_header('Content-Disposition','attachment; filename=%s.ics'%invite.event.slug)
    return part


def forward_invite(main_invite, fw_email='', fw_first_name='', fw_last_name='', fw_phone='', test=False):
    defaults_contact = {
        'first_name': fw_first_name,
        'last_name': fw_last_name,
        'language_id': 2,
    }
    contact, created = Contact.objects.get_or_create(
        email=fw_email, defaults=defaults_contact,
    )
    defaults_invite = {
        'step': main_invite.event.start_step,
        'status': settings.PUBLIEDITOR_EVENT_INVITE_TO_SEND
    }
    invite, created = Invite.objects.get_or_create(
        contact=contact, event=main_invite.event, forwarded_from=main_invite,
        defaults=defaults_invite,
    )
    # invite.send(test=test)
    return invite, created


def create_cdu_pdf(invite):
    """
    from publieditor.event import models
    from publieditor.event.util import create_cdu_pdf
    inv_ben = models.Invite.objects.get(contact__email='bcampanale@thux.it')
    create_cdu_pdf(inv_ben)
    """
    from publieditor.event.models import CDU

    contact = invite.contact
    cdu_context = {
        'place_locality': invite.assigned_place.locality,
        'full_name': u'{} {}'.format(contact.first_name, contact.last_name),
        'locality': u'{} ({})'.format(contact.default_address.locality, contact.default_address.province),
        'address': u'{} ,{}'.format(contact.default_address.name, contact.default_address.number),
        'license_number': unicode(contact.license_number),
        'license_emitted_by': unicode(contact.license_emitted_by),
        'license_issue_date': unicode(contact.license_issue_date),
        'license_expiration_date': unicode(contact.license_expiration_date),
        'now_time': unicode(datetime.datetime.now().strftime('%H:%M')),
        'vehicle_model_name': unicode(invite.assigned_vehicle.vehicle_model.name),
        'vehicle_licence_plate': unicode(invite.assigned_vehicle.licence_plate),
        'vehicle_vin': unicode(invite.assigned_vehicle.vin),
        'vehicle_color': unicode(invite.assigned_vehicle.color),
        'today_string': unicode(datetime.datetime.today().strftime('%d/%m/%Y')),
        'event_place': invite.assigned_place,
    }
    template_file = settings.PUBLIEDITOR_EVENT_CDU_TEMPLATES[invite.event.landing_page_template]
    output = AppyPodTemplate(template_file, cdu_context)

    file_name = '/tmp/{}.pdf'.format(str(invite.id))
    output.save_as(file_name)

    try:
        cdu = CDU.objects.get(invite=invite)
    except ObjectDoesNotExist:
        cdu = CDU(
            invite=invite
        )

    cdu.cdu_file.save(file_name, File(open(file_name, 'r')))
    return file_name
