# coding: utf-8

import uuid
from urllib2 import urlopen
from django.conf import settings
import os


# def get_photo_appy(relative_path):
#     """
#     This method is used to save on file system an image from web url.
#     We need to use local paths in appy so we have to download the image from
#     the web and save in on local storage
#     """
#     file_name, file_extension = relative_path.split("/")[-1].split(".")
#     img_url = "{}://{}{}".format(
#         settings.SITE_PROTOCOL, settings.SITE_DOMAIN, relative_path
#     )
#     urlf = urlopen(img_url)
#     file_name = "/tmp/{}.{}".format(uuid.uuid4(), file_extension)
#     with open(file_name, 'w') as f:
#         f.write(urlf.read())
#     return file_name


def get_photo_appy(relative_path):
    """
    This method is used to copy an image from the media directory to the local
    filesystem with a unique file name. This is useful for handling images
    where local paths are required.
    """
    try:
        # Construct the absolute path to the image in the media directory
        #source_file_path = os.path.join(settings.MEDIA_ROOT, relative_path)
        source_file_path = relative_path

        # Ensure the file exists
        if not os.path.exists(relative_path):
            raise IOError("The file {} does not exist.".format(source_file_path))

        # Extract the file extension
        _, file_extension = os.path.splitext(relative_path)

        # Generate a unique file name
        destination_file_path = "/tmp/{}{}".format(uuid.uuid4(), file_extension)

        # Copy the file to the destination path
        with open(source_file_path, 'rb') as source_file:
            with open(destination_file_path, 'wb') as destination_file:
                destination_file.write(source_file.read())

        return destination_file_path

    except IOError as e:
        # Log the exception or handle it as needed
        print("An error occurred: {}".format(e))
        return None


import requests


def convert_to_pdf(
    files,
    output_filename=None,
    output_format_orig='pdf',
    merge=False,
    preprocessor=None,
    timeout=settings.POD_CONVERTER_TIMEOUT,
):
    """Convert input to pdf. If many files are handled one single merged file is returned

    :arg file: filename or open file handler (can be a list/tuple)
    :arg output_filename: name of the output file (optional)
    :arg merge: (boolean). Suggest that we need to generate an output that is the merge
       of input files (that need to be ``.pdf``)
    :arg timeout: timeout for the connection to the server (default settings.POD_CONVERTER_TIMEOUT)
    :arg preprocessor: (str). Call a preprocessor before merge or convert. Must be already defined and mounted on
        gotenberg instance.
    """
    if not isinstance(files, (list, tuple)):
        files = [files]

    files_dict = {}
    for j, file in enumerate(files):
        if isinstance(file, str):
            files_dict['file{0:02}'.format(j)] = open(file, 'rb')
        else:
            files_dict['file{0:02}'.format(j)] = file

    if merge:
        pod_converter_url = settings.POD_CONVERTER + settings.POD_ENDPOINT_MERGE
    else:
        pod_converter_url = pod_converter_url = '{}{}?output_format={}'.format(settings.POD_CONVERTER, settings.POD_ENDPOINT_CONVERT, output_format_orig)

    data = {"format": output_format_orig}
    if preprocessor:
        data["preprocessor"] = preprocessor

    rh = requests.post(pod_converter_url, files=files_dict, timeout=timeout, data=data)

    if output_filename:
        with open(output_filename, 'wb') as f:
            f.write(rh.content)
    return rh