# coding: utf-8

from django.utils.translation import ugettext as ugettext
from django.template.defaultfilters import date as _date


def stript_value(instance, key, rel_map):
    if "single_ch" in rel_map and key in rel_map["single_ch"]:
        call_method = "get_{}_display".format(key)
        return getattr(instance, call_method)()
    elif "single_fk" in rel_map and key in rel_map["single_fk"]:
        try:
            return getattr(getattr(instance, key), rel_map["single_fk"][key])
        except Exception, e:
            return ""
    return getattr(instance, key)


def check_differences(old_instance, instance, changed_fields, related_map={}, m2m_fields=[], old_m2m_values={}):
    diffs = dict()
    for field in changed_fields:
        if field not in m2m_fields:
            if old_instance:
                diffs[field] = dict(
                    old_value=stript_value(old_instance, field, related_map),
                    new_value=stript_value(instance, field, related_map),
                )
            else:
                diffs[field] = dict(
                    old_value=None,
                    new_value=stript_value(instance, field, related_map)
                )
    if m2m_fields:
        for m2m_field in m2m_fields:
            if m2m_field in changed_fields and m2m_field in old_m2m_values:
                old_values = old_m2m_values[m2m_field]
                new_values = ",".join(sorted([related.__unicode__() for related in getattr(instance, m2m_field).all()]))
                if old_values != new_values:
                    diffs[m2m_field] = dict(
                        old_value=old_values,
                        new_value=new_values,
                    )
    return diffs


def field_verbose_name(field, model):
    return model._meta.get_field(field).verbose_name


def get_transport_detail(transport_detail):
    if hasattr(transport_detail, 'flightdetail'):
        return "{} {} {} {} {} {} ({}) - {} {} {} ({})".format(
            _date(transport_detail.departure, "D d/m/Y").title(),
            ugettext("Flight"),
            transport_detail.flightdetail.flight.airline.iata_code,
            transport_detail.flightdetail.flight.number,
            transport_detail.flightdetail.flight.departure.strftime("%H:%M"),
            transport_detail.flightdetail.flight.dep_airport.place,
            transport_detail.flightdetail.flight.dep_airport.iata_code,
            transport_detail.flightdetail.flight.arrival.strftime("%H:%M"),
            "+1" if transport_detail.flightdetail.day_after() else "",
            transport_detail.flightdetail.flight.arr_airport.place,
            transport_detail.flightdetail.flight.arr_airport.iata_code,
        )
    elif hasattr(transport_detail, 'traindetail'):
        return "{} {} {} {} {} ({}) - {} {} {} ({})".format(
            _date(transport_detail.departure, "D d/m/Y").title(),
            ugettext("Train"),
            transport_detail.traindetail.train.code,
            transport_detail.traindetail.train.departure.strftime("%H:%M"),
            transport_detail.traindetail.train.dep_station.place,
            transport_detail.traindetail.train.dep_station.name,
            transport_detail.traindetail.train.arrival.strftime("%H:%M"),
            "+1" if transport_detail.traindetail.day_after() else "",
            transport_detail.traindetail.train.arr_station.place,
            transport_detail.traindetail.train.arr_station.name,
        )
    return u""


def get_choice_by_id(choice_id, choices_list):
    for settings_choice_id, choice_value in choices_list:
        if choice_id == settings_choice_id:
            return choice_value
    return None
