#-*- coding: utf-8 -*-

import os
from unidecode import unidecode
from urllib2 import urlopen
from urlparse import urlparse
from django.conf import settings
from django.contrib import messages
from django.contrib.sites.models import get_current_site
from django.core.mail import EmailMultiAlternatives
from django.core.files import File
from django.core.files.temp import NamedTemporaryFile
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q
from django.template import loader
from django.template.base import Context
from django.template.loader import get_template, render_to_string
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse

from jmb.mailer.models import EmailObjectCC, Attachment
from jmb.mailer.utils.email_object_constructor import EmailObjectConstructor

from publieditor.event.models import Invite, InviteEmailObject, FollowUpEmailObject, DocumentsEmailObject
from publieditor.event.util import get_ics_mime_part


def get_queryset(obj, request, queryset):
    if request.POST.get('select_across') == '1':
        # se sono stati selezionati tutti i contatti (anche delle pagine successive)
        # quindi occorre ricostruire il queryset
        queryset = obj.queryset(request)
    return queryset


def render_template_from_string(context_dict, body):
    if 'invite' in context_dict:
        if hasattr(context_dict['invite'], "get_landingpage"):
            #context_dict['invite'].get_landingpage = mark_safe(context_dict['invite'].get_landingpage())
            context_dict['invite'].get_landingpage = " > Aderisci online qui sotto"
    context = Context(context_dict)
    template = loader.get_template_from_string(body)
    ret = template.render(context)
    return ret if not "<code></code>" in ret else ret.replace("<code></code>", "")


def print_messages(request, event, sent_in_test, create, not_create, test):
    # PRINT MESSAGES
    if create == 0 and not test:
        messages.error(request, _('%s not create') % event)
    if sent_in_test == 0 and test:
        messages.error(request, _('%s not sent in test') % event)
    if create > 0 and not test:
        messages.info(request, _('{0} succesfully create to {1} contacts').format(
            event, create)
        )
    if sent_in_test > 0 and test:
        messages.info(request, _('{0} succesfully sent in test to {1} contacts').format(
            event, sent_in_test)
        )
    if not_create > 0:
        messages.error(request, _('{0}: {1} in error').format(event, not_create))
    return


def get_description(translations, invite):
    event_translation = translations.get(
        language_id=invite.contact.language.id,
    )
    description = render_template_from_string(
        dict(contact=invite.contact, invite=invite),
        event_translation.body
    )
    return description


def get_invited_description(translations, invite, invitant_contact):
    event_translation = translations.get(
        language_id=invite.contact.language.id,
    )
    description = render_template_from_string(
        dict(contact=invite.contact, invite=invite, invitant_contact=invitant_contact),
        event_translation.invited_body
    )
    return description


def send_documents_email(event, contact, request, send_to=None):
    """
    Send invite documents to contact
    """
    try:
        html_template = get_template(settings.PUBLIEDITOR_EVENT_DOCUMENTS_EMAIL_TEMPLATE)
        text_template = settings.PUBLIEDITOR_EVENT_DOCUMENTS_EMAIL_TEMPLATE.split('.')[0] + '.txt'
        invite = Invite.objects.get(event=event, contact=contact)
        current_site = get_current_site(request)
        site_name = current_site.name
        title = event.translations.get(language=contact.language).title
        ctx_dict = Context({
            'site_name': site_name,
            'contact': contact,
            'event': invite.event,
            'title': title,
        })
        subject = "Reminder partecipazione evento %s" % title
        from_email = "info@publieditor.net"
        to = [contact.email]
        text_message = render_to_string(text_template, ctx_dict)
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(subject, text_message, from_email, to)
        msg.attach_alternative(html_message, "text/html")
        for event_document in event.event_documents.all():
            file_url = settings.PROJECT_PATH + event_document.document.document.url
            msg.attach_file(file_url)
        if event.enable_ics:
            msg.attach(get_ics_mime_part(invite))
        msg.send()
    except Invite.DoesNotExist:
        print "Invito non esistente"
    except Exception, e:
        print e


class InviteEmailObjectConstructor(EmailObjectConstructor):

    class_ = InviteEmailObject

    def get_context(self):
        invite = self.origin
        event = invite.event
        contact = invite.contact
        privacy_acceptance_date = getattr(contact, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD)
        translation = invite.get_step_translation("translations")
        url_landing_page = invite.get_url_landingpage() if not invite.event.is_privacy else \
            invite.get_privacy_landingpage()
        url_accept_from_email = invite.get_accept_from_email_url()
        url_decline_from_email = invite.get_decline_from_email_url()
        url_delegate_from_email = invite.get_delegate_from_email_url()
        url_forwarding_from_email = invite.get_forwarding_from_email_url()
        adhesion_in_email = invite.event.adhesion_in_email

        confirm_token = '{}-{}'.format(invite.make_base36(), invite.make_token())
        manual_adhesion_url = reverse('event:set_session_by_invite_form', kwargs={'invite_id': invite.pk})
        button_text = "Aderisci online" if not invite.event.is_privacy else "Registrati"
        body = render_template_from_string(
            dict(
                contact=contact,
                invite=invite,
                confirm_token=confirm_token,
            ),
            translation.email_body
        )
        return dict(
            invite=invite,
            event=event,
            translation=translation,
            external_images=translation.get_bmw_external_images(),
            body=body,
            header_image=translation.header_image,
            footer_image=translation.footer_image,
            left_signature=translation.left_signature,
            right_signature=translation.right_signature,
            site_protocol=settings.SITE_PROTOCOL,
            site_domain=settings.SITE_DOMAIN,
            url_landing_page=url_landing_page,
            adhesion_in_email=adhesion_in_email,
            url_accept_from_email=url_accept_from_email,
            url_decline_from_email=url_decline_from_email,
            url_delegate_from_email=url_delegate_from_email,
            url_forwarding_from_email=url_forwarding_from_email,
            button_text=button_text,
            static_url=settings.STATIC_URL,
            is_follow_up=False,
            privacy_acceptance_date=privacy_acceptance_date,
            contact_base36=invite.contact.make_base36(),
            contact_token=invite.contact.token,
            confirm_token=confirm_token,
            manual_adhesion_url=manual_adhesion_url,
        )

    def set_additional_values(self, email_object, **kwargs):
        invite = self.origin
        translation = invite.get_step_translation("translations")
        email_object.invite = invite
        email_object.header_sender = invite.get_invite_email_from_header()
        email_object.header_reply = translation.header_reply if translation.header_reply else None
        email_object.is_test = self.is_test

    def build_title_content(self):
        invite = self.origin
        translation = invite.get_step_translation("translations")
        return translation.title

    def save_ccs(self, email_object):
        invite = self.origin
        for email_cc in invite.contact.cc_emails.all():
            email_address = email_cc.contact_cc.email if email_cc.contact_cc else email_cc.email
            email_object_cc = EmailObjectCC(email_object=email_object, email=email_address)
            email_object_cc.save()

    def save_attachments(self, email_object):
        invite = self.origin
        event = invite.event
        contact = invite.contact
        newsletter_documents = event.newsletter_documents.all()
        newsletter_documents = newsletter_documents.filter(
            Q(use_always=True) | Q(language=contact.language)
        ).distinct()
        for newsletter_document in newsletter_documents:
            file_ext = newsletter_document.document.name.split("/")[-1].split(".")[-1]
            attachment_title = newsletter_document.title
            title = unidecode(u"{}.{}".format(attachment_title, file_ext))
            Attachment.objects.create(
                email_object=email_object,
                title=title,
                file_attachment=newsletter_document.document
            )


class InviteFollowupEmailObjectConstructor(EmailObjectConstructor):

    class_ = FollowUpEmailObject

    def get_template(self, template=None):
        event = self.origin.event
        return settings.PUBLIEDITOR_EVENT_FOLLOWUP_EMAIL_TEMPLATES[event.landing_page_template]

    def get_context(self):
        invite = self.origin
        event = invite.event
        contact = invite.contact
        presentation_translation = invite.get_step_translation("translations")
        followup_translation = invite.get_step_translation("followup_translations")
        try:
            follow_up_translation = invite.event.followup_translations.get(
                language__code=invite.contact.get_language_code()
            )
            left_signature = follow_up_translation.left_signature
            right_signature = follow_up_translation.right_signature
        except:
            left_signature = None
            right_signature = None
        body = render_template_from_string(
            dict(contact=contact, invite=invite),
            followup_translation.body
        )
        return dict(
            invite=invite,
            event=event,
            translation=followup_translation,
            body=body,
            header_image=presentation_translation.header_image,
            footer_image=presentation_translation.footer_image,
            left_signature=left_signature,
            right_signature=right_signature,
            static_url=settings.STATIC_URL,
            site_protocol=settings.SITE_PROTOCOL,
            site_domain=settings.SITE_DOMAIN,
            url_landing_page=invite.get_url_pollpage(),
            is_follow_up=True,
        )

    def set_additional_values(self, email_object, **kwargs):
        invite = self.origin
        translation = invite.get_step_translation("translations")
        email_object.invite = invite
        email_object.header_sender = invite.get_invite_email_from_header()
        email_object.header_reply = translation.header_reply if translation.header_reply else None
        email_object.is_test = self.is_test

    def build_title_content(self):
        invite = self.origin
        followup_translation = invite.get_step_translation("followup_translations")
        presentation_translation = invite.get_step_translation("translations")
        if followup_translation.title:
            return followup_translation.title
        return presentation_translation.title

    def save_ccs(self, email_object):
        invite = self.origin
        for email_cc in invite.contact.cc_emails.all():
            email_address = email_cc.contact_cc.email if email_cc.contact_cc else email_cc.email
            email_object_cc = EmailObjectCC(email_object=email_object, email=email_address)
            email_object_cc.save()


class InviteDocumentsEmailObjectConstructor(EmailObjectConstructor):

    class_ = DocumentsEmailObject

    def get_template(self, template=None):
        event = self.origin.event
        return settings.PUBLIEDITOR_EVENT_DOCUMENTS_EMAIL_TEMPLATES[event.landing_page_template]

    def get_context(self):
        invite = self.origin
        event = invite.event
        contact = invite.contact
        presentation_translation = invite.get_step_translation("translations")
        documents_translation = invite.get_step_translation("document_email_translations")
        body = render_template_from_string(
            dict(contact=contact, invite=invite),
            documents_translation.body
        )
        return dict(
            invite=invite,
            event=event,
            translation=documents_translation,
            body=body,
            header_image=presentation_translation.header_image,
            footer_image=presentation_translation.footer_image,
            left_signature=documents_translation.left_signature,
            right_signature=documents_translation.right_signature,
            static_url=settings.STATIC_URL,
            site_protocol=settings.SITE_PROTOCOL,
            site_domain=settings.SITE_DOMAIN,
            url_landing_page=invite.get_url_pollpage(),
            is_documents=True,
        )

    def set_additional_values(self, email_object, **kwargs):
        invite = self.origin
        translation = invite.get_step_translation("translations")
        email_object.invite = invite
        email_object.header_sender = invite.get_invite_email_from_header()
        email_object.header_reply = translation.header_reply if translation.header_reply else None
        email_object.is_test = self.is_test

    def build_title_content(self):
        invite = self.origin
        documents_translation = invite.get_step_translation("document_email_translations")
        presentation_translation = invite.get_step_translation("translations")
        if documents_translation.title:
            return documents_translation.title
        return presentation_translation.title

    def save_ccs(self, email_object):
        invite = self.origin
        for email_cc in invite.contact.cc_emails.all():
            email_address = email_cc.contact_cc.email if email_cc.contact_cc else email_cc.email
            email_object_cc = EmailObjectCC(email_object=email_object, email=email_address)
            email_object_cc.save()

    def save_attachments(self, email_object):
        invite = self.origin
        event = invite.event

        # Event documents are being attached
        for event_document in event.event_documents.all():
            file_ext = event_document.document.document.name.split("/")[-1].split(".")[-1]
            title = unidecode(u"{0}.{1}".format(event_document.document.name, file_ext))
            Attachment.objects.create(
                email_object=email_object,
                title=title,
                file_attachment=event_document.document.document
            )

        # Invite documents are being attached
        for invite_document in invite.documents.all():
            file_ext = invite_document.document.name.split("/")[-1].split(".")[-1]
            title = unidecode(u"{0}.{1}".format(invite_document.name, file_ext))
            Attachment.objects.create(
                email_object=email_object,
                title=title,
                file_attachment=invite_document.document
            )

        # If event has the personalized program enabled it is attached
        if event.enable_personalized_program:
            file_name = unidecode(u"RSVP-Programma-{0}-{1}".format(invite.contact.last_name, event.id))
            relative_file_url = reverse(
                'event:publieditor_event_get_invite_pdf',
                kwargs={'uidb36': invite.make_base36(), 'token': invite.make_token()}
            )
            self.create_http_attachment(email_object, relative_file_url, file_name, "pdf")

        # If event has the ics it is attached
        if event.enable_ics:
            file_name = unidecode(u"RSVP-ics-{0}-{1}".format(invite.contact.last_name, event.id))
            relative_file_url = reverse(
                'event:publieditor_event_ics_export',
                kwargs={'uidb36': invite.make_base36(), 'token': invite.make_token()}
            )
            self.create_http_attachment(email_object, relative_file_url, file_name, "ics")

        # If the project as the qr code enabled it is attached
        if settings.PUBLIEDITOR_EVENT_ENABLE_QRCODE:
            file_name = unidecode(u"RSVP-pass-{0}-{1}".format(invite.contact.last_name, event.id))
            relative_file_url = reverse(
                'event:publieditor_event_get_invite_pass',
                kwargs={'uidb36': invite.make_base36(), 'token': invite.make_token()}
            )
            self.create_http_attachment(email_object, relative_file_url, file_name, "pdf")



    @staticmethod
    def create_http_attachment(email_object, relative_file_url, name, ext):
        title = unidecode(u"{0}.{1}".format(name, ext))
        absolute_file_url = "{0}://{1}{2}".format(
            settings.SITE_PROTOCOL,
            settings.SITE_DOMAIN,
            relative_file_url,
        )
        img_temp = NamedTemporaryFile(delete=True)
        img_temp.write(urlopen(absolute_file_url).read())
        img_temp.flush()
        attachment = Attachment(email_object=email_object, title=title)
        attachment.save()
        attachment.file_attachment.save(name, File(img_temp))