# -*- coding: utf-8 -*-

import base64
import os
import vobject
import urllib
from django.conf import settings


def vcard_string(contact):
    v_card = vobject.vCard()
    # File name
    param = v_card.add('n')
    param.value = vobject.vcard.Name(
        family=contact.last_name, given=contact.first_name
    )
    # Contact name
    param = v_card.add('fn')
    param.value = "{} {}".format(contact.first_name, contact.last_name)
    # Contact email
    param = v_card.add('email')
    param.value = contact.email
    # Birthday
    if contact.birthday:
        param = v_card.add( 'bday' )
        yearString  = str(contact.birthday.year)
        monthString = str(contact.birthday.month)
        dayString   = str(contact.birthday.day)
        if len(monthString) == 1:
            monthString = '0{}'.format(monthString)
        if len(dayString) == 1:
            dayString = '0{}'.format(dayString)
        param.value = "{}-{}-{}".format(yearString, monthString, dayString)
    # Contact photo
    if contact.photo:
        photo_url = "{}://{}/{}".format(
            settings.SITE_PROTOCOL, settings.SITE_DOMAIN,
            contact.photo.url.lstrip("/")
        )
        f = urllib.urlopen(photo_url)
        data = f.read()
        f.close()
        param = v_card.add('photo')
        param.type_param = os.path.splitext(photo_url)[1][1:]
        param.encoding_param = 'b'
        param.value = base64.encodestring(data)
    # Contact phones
    for contact_phone in contact.phones.all().order_by('ordering'):
        param = v_card.add('tel')
        param.type_param = contact_phone.get_phone_type_display()
        param.value = contact_phone.number
    # Contact addresses
    for address in contact.addresses.all():
        param = v_card.add('adr')
        name = address.name if address.name else ''
        locality = address.locality if address.locality else ''
        country = address.country.name if address.country else ''
        zip_code = address.zip_code if address.zip_code else ''
        province = address.province if address.province else ''
        param.type_param = address.get_vcard_address_type_display()
        param.value = vobject.vcard.Address(
            street=name,
            city=locality,
            country=country,
            code=zip_code,
            region=province,
        )
    # Contact memberships
    for membership in contact.memberships.all():
        param = v_card.add('org')
        param.value[0] = membership.newspaper.name
        if membership.role:
            param.value.append(membership.role.name)
    # Contact sociale media
    for social_media in contact.social_media_links.all():
        param = v_card.add('X-SOCIALPROFILE')
        param.type_param = social_media.get_social_display()
        param.value = social_media.full_link
    return v_card.serialize()