# -*- coding: utf-8 -*-
from __future__ import absolute_import
import base64
import os
import datetime
import HTMLParser
import html2text
from datetime import timedelta
from collections import OrderedDict
from ckeditor.fields import RichTextField as HTMLFieldTinyMCE
from django.core.files import File
from icalendar import Calendar as iCalendar
from icalendar import Event as iEvent
from icalendar import Alarm as iAlarm
from django.conf import settings
from django.template.defaultfilters import date as _date
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.core.mail import EmailMultiAlternatives
from django.db import models
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugettext
from django.utils.http import int_to_base36
from django.utils.encoding import force_unicode
from django.template import loader, Template
from django.template.base import Context
from django.template.loader import get_template, render_to_string
from django.utils.safestring import mark_safe
from easy_thumbnails.files import get_thumbnailer
from colorful.fields import RGBColorField
from imagekit.models import ImageSpecField
from imagekit.processors import ResizeToFill
from jmb.address.utils import get_default_country
from jmb.l10n.models import Country
from south.modelsinspector import add_introspection_rules

from jmb.address.models import Address
from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.core.utils.functions import zerofill
from jmb.i18n.models import Language
from jmb.core.utils.token import TokenGenerator
from jmb.core.utils.ooutils import OOUtils
from jmb.mailer.models import EmailObject, MailingStatus, Link
from jmb.mailer.utils.email_object_sender import EmailObjectSender
from publieditor.survey.models import Question, Answer
from publieditor.event.utils.generic import field_verbose_name, get_transport_detail
from .validators import validate_newsletter_document

#
# class EmailObjectSender():
#     pass

add_introspection_rules([], ["^colorful.fields.RGBColorField"])

User = get_user_model()

########################################################################################################################
# MAILING LIST #########################################################################################################
########################################################################################################################
class MailingList(StatusModel, UserModel, DateModel, OrderedModel):
    """
    This model was used in the past when contact were grouped by mailing list.
    Leaving it defined in our models allows us to reuse it in futurre project which requires it.
    """
    name = models.CharField(
        _("Name"),
        max_length=255
    )
    slug = models.SlugField(
        _("Slug"),
        unique=True,
        max_length=100,
        default=datetime.datetime.now
    )
    description = models.TextField(
        _("Description"),
        blank=True, null=True
    )

    def __unicode__(self):
        return u"{}".format(self.name)

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('Mailing list')
        verbose_name_plural = _('Mailing lists')

    def add_to_event_invited_list(self, event):
        """
        Add mailing list contacts to an event invited list
        :arg event: Event instance which contact must be added
        """
        if not self.status:
            return False, ugettext("Mailing list %s is inactive" % self.name)

        for contact in self.contacts.filter(status=1):
            invited, msg = contact.add_to_event_invited_list(event)

        return True, ugettext("Mailing list %s invited to event %s" % (self.name, event.name))


#######################################################################################################################
# CONTACTS ############################################################################################################
#######################################################################################################################
class Organization(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Contacts can be assigned to an organization defined in this model
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
    )

    class Meta(object):
        verbose_name = _('organization')
        verbose_name_plural = _('organizations')
        ordering = ('name',)

    def __unicode__(self):
        return u"{}".format(self.name)


class License(DateModel, UserModel):
    """
    Contacts can own different license types which are defined in this model.
    """
    name = models.CharField(
        max_length=5,
        verbose_name=_('name'),
    )

    class Meta(object):
        verbose_name = _('license')
        verbose_name_plural = _('licenses')
        ordering = ('name',)

    def __unicode__(self):
        return u"{}".format(self.name)


class Contact(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Main contact model.
    All common contact field are used in this model, most of them are unrequired so we can choose
    which of them must be enabled in the current project.
    For example in FCA we didn't use any of ski related fields.
    """

    def get_photo_storage_path(self, filename):
        """
        Used to select photo field upload_to folder based on current user and filename.
        :param filename: Filename of the contact photo
        :return: Path to the correct upload folder where image will be stored.
        """
        photo_path = ""
        if self.last_name:
            photo_path = self.last_name[0:1].upper()
        filename = force_unicode(filename)
        path = os.path.join(settings.MEDIA_URL, settings.PUBLIEDITOR_EVENT_CONTACT_PHOTO, photo_path, filename)
        return path[1:]

    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        null=True, blank=True,
        on_delete=models.SET_NULL,
    )
    photo = models.ImageField(
        _('photo'),
        null=True, blank=True,
        max_length=255,
        upload_to=get_photo_storage_path
    )
    title = models.CharField(
        max_length=10,
        verbose_name=_("title"),
        choices=settings.PUBLIEDITOR_CONTACT_TITLE_CHOICES
    )
    first_name = models.CharField(
        max_length=100,
        verbose_name=_('first name'),
        blank=True, null=True,
    )
    last_name = models.CharField(
        max_length=100,
        verbose_name=_('last name'),
        blank=True, null=True,
    )
    company = models.CharField(
        max_length=100,
        verbose_name=_('company'),
        blank=True, null=True,
    )
    birth_place = models.CharField(
        max_length=100,
        verbose_name=_('birthplace'),
        blank=True, null=True,
    )
    birthday = models.DateField(
        blank=True, null=True,
        verbose_name=_('birthday'),
    )
    ssn = models.CharField(
        blank=True, null=True,
        max_length=32,
        verbose_name=_('ssn'),
    )
    email = models.EmailField(
        verbose_name=_('email'),
        unique=True,
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
        related_name="contact_contacts"
    )
    groups = models.ManyToManyField(
        'ContactGroup',
        related_name='contacts',
        verbose_name=_('groups'),
        null=True, blank=True
    )
    agent = models.ForeignKey(
        'Agent',
        blank=True, null=True,
        related_name='contacts',
        verbose_name=_('agent')
    )
    is_test_contact = models.BooleanField(
        default=False,
        verbose_name=_('test'),
    )
    is_lead = models.BooleanField(
        default=False,
        verbose_name=_('is lead'),
        help_text=_('Contact that is not a Journalist')
    )
    privacy_acceptance_date = models.DateTimeField(
        verbose_name=_('privacy acceptance date'),
        blank=True, null=True
    )
    fca_privacy_acceptance_date = models.DateTimeField(
        verbose_name=_('privacy acceptance date'),
        blank=True, null=True
    )
    smoking = models.CharField(
        max_length=25,
        verbose_name=_("smoking"),
        choices=settings.PUBLIEDITOR_CONTACT_SMOKING_CHOICES,
        blank=True, null=True,
    )
    food = models.CharField(
        max_length=25,
        verbose_name=_("food"),
        choices=settings.PUBLIEDITOR_CONTACT_FOOD_CHOICES,
        blank=True, null=True,
    )
    shirt_size = models.CharField(
        max_length=10,
        verbose_name=_("shirt size"),
        choices=settings.PUBLIEDITOR_CONTACT_SHIRT_CHOICES,
        blank=True, null=True,
    )
    height = models.CharField(
        max_length=100,
        verbose_name=_("height"),
        blank=True, null=True,
    )
    weight = models.CharField(
        max_length=100,
        verbose_name=_("weight"),
        blank=True, null=True,
    )
    sweater_size = models.CharField(
        max_length=10,
        verbose_name=_("sweater size"),
        choices=settings.PUBLIEDITOR_CONTACT_SWEATER_CHOICES,
        blank=True, null=True,
    )
    shoes_size = models.CharField(
        max_length=10,
        verbose_name=_("shoes size"),
        blank=True, null=True,
    )
    ski = models.CharField(
        max_length=25,
        choices=settings.PUBLIEDITOR_EVENT_CONTACT_SKI_CHOICES,
        verbose_name=_("ski"),
        blank=True, null=True,
    )
    ski_level = models.CharField(
        max_length=25,
        choices=settings.PUBLIEDITOR_EVENT_CONTACT_SKI_LEVEL_CHOICES,
        verbose_name=_("ski level"),
        blank=True, null=True,
    )
    ski_equipment = models.CharField(
        max_length=10,
        verbose_name=_("ski equipment"),
        choices=settings.PUBLIEDITOR_CONTACT_SKI_EQUIPMENT_CHOICES,
        blank=True, null=True,
    )
    ski_boots_size = models.CharField(
        max_length=10,
        verbose_name=_("ski boots size"),
        blank=True, null=True,
    )
    id_card_number = models.CharField(
        max_length=25,
        verbose_name=_("id card number"),
        blank=True, null=True,
    )
    id_card_issue_date = models.DateField(
        verbose_name=_("id card issue date"),
        blank=True, null=True,
    )
    id_card_expiration_date = models.DateField(
        verbose_name=_("id card expiration date"),
        blank=True, null=True,
    )
    id_card_scan = models.ImageField(
        verbose_name=_("id card scan"),
        upload_to='id_card_scans/%Y/%m',
        null=True, blank=True
    )
    passport_number = models.CharField(
        max_length=25,
        verbose_name=_("passport number"),
        blank=True, null=True,
    )
    passport_issue_date = models.DateField(
        verbose_name=_("passport issue date"),
        blank=True, null=True,
    )
    passport_expiration_date = models.DateField(
        verbose_name=_("passport expiration date"),
        blank=True, null=True,
    )
    passport_scan = models.ImageField(
        verbose_name=_("passport scan"),
        upload_to='passport_scans/%Y/%m',
        null=True, blank=True
    )
    license_number = models.CharField(
        max_length=25,
        verbose_name=_("license number"),
        blank=True, null=True,
    )
    license_issue_date = models.DateField(
        verbose_name=_("license issue date"),
        blank=True, null=True,
    )
    license_expiration_date = models.DateField(
        verbose_name=_("license expiration date"),
        blank=True, null=True,
    )
    license_emitted_by = models.CharField(
        max_length=100,
        verbose_name=_("license emitted by"),
        blank=True, null=True,
    )
    license_types = models.ManyToManyField(
        'License',
        related_name='contacts',
        verbose_name=_('license types'),
        null=True, blank=True
    )
    license_scan = models.ImageField(
        verbose_name=_("license scan"),
        upload_to='license_scans/%Y/%m',
        null=True, blank=True
    )
    press_card_number = models.CharField(
        max_length=10,
        verbose_name=_("press card number"),
        blank=True, null=True,
    )
    elenco = models.CharField(
        max_length=50,
        verbose_name=_("elenco"),
        choices=settings.PUBLIEDITOR_CONTACT_ELENCO_CHOICES,
        blank=True, null=True,
    )
    food_note = models.TextField(
        verbose_name=_("food notes"),
        blank=True, null=True,
    )
    note = models.TextField(
        verbose_name=_("note"),
        blank=True, null=True,
    )
    internal_note = models.TextField(
        verbose_name=_("internal note"),
        blank=True, null=True,
    )

    @property
    def rendered_privacy_acceptance_date(self):
        """
        Correct contact privacy field is the one selected in PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD setting
        We use that setting to select the correct contact privacy acceptance date
        :return: Readable contact privacy acceptance date
        """
        privacy_acceptance_date = getattr(self, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD)
        if privacy_acceptance_date:
            return privacy_acceptance_date.strftime("%Y-%m-%d")
        return ""

    def get_link_status(self):
        """
        Used in admin to show an icon representing the current contact user.
        Via dajaxice the backend users can immediately change the status clicking on the changelist icon
        :return: Html snippet containing the clickable image representing the contact status
        """
        return """
            <a id='publieditor_event_contact_status_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.status else 'no'))
    get_link_status.short_description = _('Status')
    get_link_status.allow_tags = True

    def save(self, *args, **kwargs):
        """
        We choosed to overwrite save method in order to correct contact fist/last name
        based on the format setted in PUBLIEDITOR_EVENT_CONTACT_NAME_STYLE setting
        :return: the current updated contact
        """
        cns = settings.PUBLIEDITOR_EVENT_CONTACT_NAME_STYLE
        if cns == "title":
            self.last_name = self.last_name.title() if self.last_name else ''
            self.first_name = self.first_name.title() if self.first_name else ''
        elif cns == "lower":
            self.last_name = self.last_name.lower() if self.last_name else ''
            self.first_name = self.first_name.lower() if self.first_name else ''
        elif cns == "upper":
            self.last_name = self.last_name.upper() if self.last_name else ''
            self.first_name = self.first_name.upper() if self.first_name else ''
        super(Contact, self).save(*args, **kwargs)

    def get_web_title(self):
        """
        This method is used in event presentation/follow up body to print the correct contact title
        I had to use Italian translation in order to understand the correct contact gender.
        """
        translations = {
            "mr": _("Caro"),
            "ms": _("Cara"),
            "dear": _("Spett."),
        }
        return translations[self.title]

    def get_company(self):
        """
        :return: Contact company name if defined else empty string
        """
        return self.company or ""
    get_company.short_description = _("Company")

    def get_language_code(self):
        """
        :return: Contact language code if defined else empty string
        """
        if self.language:
            return self.language.code
        return ""
    get_language_code.short_description = _("Language")

    def admin_thumbnail(self):
        """
        Creates a thumbnail for contact photo image to preview it in admin changelist
        :return: Html snippet containg the photo preview, if errors occur returns an empty string
        """
        if self.photo:
            try:
                options = {'size': (35, 35), 'crop': True}
                thumb_url = get_thumbnailer(self.photo).get_thumbnail(options).url
                return u'<img src="%s" />' % thumb_url
            except:
                pass
        return ""
    admin_thumbnail.short_description = _('Photo')
    admin_thumbnail.allow_tags = True

    def get_privacy_status(self):
        """
        Used to understand if privacy was accepted by the contact based
        on the PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD setting.
        :return: Html snippet containing the icon representing contact privacy status
        """
        field = settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD
        has_privacy = True if getattr(self, field) else False
        return u"<img alt='Privacy' src='/static/admin/img/icon-{}.gif'>".format('yes' if has_privacy else 'no')
    get_privacy_status.short_description = _('Privacy')
    get_privacy_status.allow_tags = True
    get_privacy_status.admin_order_field = settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD

    def groups_display(self):
        """
        Used in admin to list all user groups
        :return: Readable contact groups list separated by comma
        """
        groups = self.groups.all().values_list("name", flat=True).distinct()
        return u", ".join(groups)
    groups_display.short_description = _("Groups")

    def newspapers_display(self):
        """
        Used in admin to list all contact newspapers taken by its memberships
        :return: Readable contact newspapaers list separated by comma
        """
        newspapers = self.memberships.all().values_list("newspaper__name", flat=True).distinct()
        return u", ".join(newspapers)
    newspapers_display.short_description = _("Newspapers")
    newspapers_display.admin_order_field = 'memberships__newspaper__name'

    def get_admin_change_url(self):
        """
        Used in admin to get a clickable contact full name referencing to its change form
        :return: Html snippet with contact full name link to correct change form
        """
        return u'<a href="{}">{}<br/>({})</a>'.format(
            reverse('admin:event_contact_change', args=[self.id]), self.full_name, self.get_default_email(),
        )

    def get_default_email(self):
        """
        :return: Contact default email
        """
        return self.email

    def get_default_email_cc(self):
        """
        Contact default email is calculated taking the default one if defined, if undefined first email is taken
        :return: Contact default email
        """
        cc_emails = list(self.cc_emails.all())
        for mail in cc_emails:
            if mail.default:
                return mail
        if len(cc_emails) > 0:
            return cc_emails[0]
        return None

    def get_all_email_cc(self):
        """
        :return: List containing all contact emails
        """
        return list(self.cc_emails.all())

    def get_home_address(self):
        try:
            return self.addresses.filter(address_scopes__id=5)[0]
        except IndexError:
            return ''

    def get_mobile_phone(self):
        try:
            return self.phones.filter(phone_type='M')[0]
        except IndexError:
            return ''

    @property
    def default_phone(self):
        """
        :return: Default phone if defined, if undefined first one is returned
        """
        if self.phones.count():
            default_phone = self.phones.filter(default=True)
            if default_phone:
                return default_phone[0]
            return self.phones.all()[0]
        return None

    @property
    def default_address(self):
        """
        :return: Default address if defined, if undefined first one is returned
        """
        if self.addresses.all().count():
            if self.addresses.filter(default=True).count():
                return self.addresses.get(default=True)
            return self.addresses.all()[0]
        return None

    @property
    def default_card(self):
        """
        :return: Default card if defined, if undefined first one is returned
        """
        if self.cards.all().count():
            if self.cards.filter(default=True).count():
                return self.cards.get(default=True)
            return self.cards.all()[0]
        return None

    @property
    def get_default_membership(self):
        """
        :return: Default membership if defined, if undefined first one is returned
        """
        if self.memberships.all().count():
            if self.memberships.filter(default=True).count():
                return self.memberships.get(default=True)
            return self.memberships.all()[0]
        return None

    @property
    def full_name(self):
        """
        :return: Contact full name which is calculated based on its first name, last name and company
        """
        if self.first_name or self.last_name:
            return "%s %s" % (self.first_name, self.last_name)
        if self.company and self.company.strip():
            return self.company
        else:
            return self.email

    @property
    def can_create_message(self):
        """
        Checks if contact can receive emails (commonly newsletters or invites)
        This method might be no longer used, however it could be convenient in future projects.
        :return: A tuple composed by the ability of the contact to receive emails and a message text
        """
        return True, "OK"

    @property
    def invite_name(self):
        """
        :return: Company name if defined else first name and last name
        """
        # if isinstance(self.company, (str, unicode)) and self.company != "":
        #     return self.company
        return u"{} {}".format(self.last_name, self.first_name)

    def add_to_group(self, contact_group):
        """
        Add contact to a selected group
        We first check if contact can be added to the group checking both status and only if not already associated
        :arg contact_group: ContactGroup instance which contact must be added
        :return: Tuple containing a boolean and a string representing the add action results
        """
        if not self.status:
            return False, ugettext("Contact %s is inactive" % self.full_name)
        if not contact_group.status:
            return False, ugettext("Group %s is inactive" % contact_group.name)
        if contact_group in self.groups.all():
            return False, ugettext("Contact %s already in group %s" % (self.full_name, contact_group.name))

        self.groups.add(contact_group)
        return True, ugettext("Contact %s added to group %s" % (self.full_name, contact_group.name))

    def add_to_mailing_list(self, mailing_list):
        """
        Add contact to a selected mailing list
        We first check if contact can be added to the ml checking both status and only if not already associated
        :arg mailing_list: MailingList instance which contact must be added
        :return: Tuple containing a boolean and a string representing the add action results
        """
        if not self.status:
            return False, ugettext("Contact %s is inactive" % self.full_name)
        if not mailing_list.status:
            return False, ugettext("Mailing list %s is inactive" % mailing_list.name)
        if mailing_list in self.mailing_lists.all():
            return False, ugettext("Contact %s already in mailing list %s" % (self.full_name, mailing_list.name))

        self.mailing_lists.add(mailing_list)
        return True, ugettext("Contact %s added to mailing list %s" % (self.full_name, mailing_list.name))

    def add_to_event_invited_list(self, event):
        """
        Add contact to an event invited list
        Before performing the action we need to checking some conditions about contact and event
        The add actions is actually performed by creating an invite with event/contact foreign keys,
        first step is taken by event settings, while initial status in alway pending because we need to send
        the invite to really change its status.
        :arg event: Event instance which contact must be added
        :return: Tuple containing a boolean and a string representing the add action results
        """
        if not self.status:
            return False, ugettext("Contact %s is inactive" % self.full_name)
        if not event.status:
            return False, ugettext("Event %s is inactive" % event.name)
        if self.invites.filter(event=event).count():
            return False, ugettext("Contact %s is already in event %s invited list" % (self.full_name, event.name))
        if event.is_expired():
            return False, ugettext("Event %s is expired" % event.name)
        if event.is_over():
            return False, ugettext("Event %s is over" % event.name)
        if event.is_deadline_passed():
            return False, ugettext("Event %s deadline is passed" % event.name)

        invite = Invite(
            contact=self, event=event, step=event.start_step,
            status=settings.PUBLIEDITOR_EVENT_INVITE_PENDING,
        )
        invite.save()
        return True, ugettext("Contact %s added to event %s invited list" % (self.full_name, event.name))

    def accept(self, invite):
        """
        I think this method is no longer used, it was created in really first commits where the app was just a draft.
        However it should be set invite status to accepted, doing this code in this model is really no sense.
        Please check before deleting
        """
        from publieditor.event.models import Invite

        event = invite.event
        if event.is_expired():
            return False, _('Event %s expired' % event.name)
        if event.status == 0:
            return False, _('Event %s disabled' % event.name)
        if not self.invites.filter(id=invite.id):
            return False, _('Invalid invite')
        if self.status == 0:
            return False, _('Contact %s disabled' % self.full_name)

        invite.status = Invite.ACCEPTED
        invite.save()
        return True, invite

    def delegate(self, invite, new_contact):
        """
        As accept method I think this method is no longer used.
        Performing this action in contact model is no sense in my opinion.
        Please check before deleting
        """
        from publieditor.event.models import Invite

        event = invite.event
        if event.is_expired():
            return False, _('Event %s expired' % event.name)
        if event.status == 0:
            return False, _('Event %s disabled' % event.name)
        if not self.invites.filter(id=invite.id):
            return False, _('Invalid invite')
        if self == new_contact:
            return False, _('Contacts must be different')
        if new_contact.invites.filter(event=event):
            return False, _('New Contact already invited')
        if new_contact.status == 0:
            return False, _('New Contact disabled')

        invite.status = Invite.DELEGATED
        invite.save()

        new_invite = Invite()
        new_invite.event = invite.event
        new_invite.contact = new_contact
        new_invite.save()
        return True, new_invite

    def refuse(self, invite):
        """
        As accept/delegate method I think this method is no longer used.
        Performing this action in contact model is no sense in my opinion.
        Please check before deleting
        """
        from publieditor.event.models import Invite

        event = invite.event
        if event.is_expired():
            return False, _('Event %s expired' % event.name)
        if event.status == 0:
            return False, _('Event %s disabled' % event.name)
        if not self.invites.filter(id=invite.id):
            return False, _('Invalid invite')
        if self.status == 0:
            return False, _('Contact disabled')

        invite.status = Invite.REFUSED
        invite.save()
        return True, invite

    def get_events(self):
        """
        :return: Queryset of Event instances containing all contact events
        """
        from publieditor.event.models import Event
        return Event.objects.filter(invites__contact=self)

    def get_birth_info(self):
        if self.birth_place and self.birthday:
            return u'{}, {}'.format(self.birth_place, self.birthday.strftime('%d/%m/%Y'))
        elif self.birth_place:
            return self.birth_place
        elif self.birthday:
            return self.birthday.strftime('%d/%m/%Y')
        else:
            return ''


    @property
    def get_register_answers(self):
        """
        :return: Queryset of RegisterAnswer instances containing all contact ones
        """
        return RegisterAnswer.objects.filter(invite__contact=self)

    @property
    def get_list_register_answers(self):
        """
        :return: List of all register answers
        """
        return [unicode(answer) for answer in self.get_register_answers]

    @property
    def get_string_register_answers(self):
        """
        :return: String of all contact register answers separated by comma
        """
        return u", ".join(self.get_list_register_answers)

    @property
    def get_transport_answers(self):
        """
        :return: Queryset of TransportAnswer instances containing all contact ones
        """
        return TransportAnswer.objects.filter(invite__contact=self)

    @property
    def get_list_transport_answers(self):
        """
        :return: List of all contact transport answers
        """
        return [unicode(answer) for answer in self.get_transport_answers]

    @property
    def get_string_transport_answers(self):
        """
        :return: String of all contact transport answers separated by comma
        """
        return u", ".join(self.get_list_transport_answers)

    @property
    def get_activity_answers(self):
        """
        :return: Queryset of ActivityAnswer instances containing all contact ones
        """
        return ActivityAnswer.objects.filter(invite__contact=self)

    @property
    def get_list_activity_answers(self):
        """
        :return: List of all contact activity answers
        """
        return [unicode(answer) for answer in self.get_activity_answers]

    @property
    def get_string_activity_answers(self):
        """
        :return: String of all contact activity answers separated by comma
        """
        return u", ".join(self.get_list_activity_answers)

    def set_register_answer(self, invite, register):
        """
        I think this method is no longer used.
        However it should save a new register answer for a given invite.
        Please check before deleting
        """
        from publieditor.survey.models import Answer

        event = invite.event
        if event.is_expired():
            return False, _('Event expired')
        if event.status == 0:
            return False, _('Event disabled')
        if not self.invites.filter(id=invite.id):
            return False, _('Invalid invite')
        if self.status == 0:
            return False, _('Contact disabled')
        if not event.registers.filter(register_id=register.id):
            return False, _('Register is not a valid question')
        if self.get_register_answers().filter(invites=invite, register=register):
            return False, _('Answer already exist')

        register_answer = Answer()
        register_answer.invite = invite
        register_answer.register = register
        register_answer.save()
        return True, register_answer

    @property
    def get_phones(self):
        """
        :return: Queryset of all contact enabled phones
        """
        return self.phones.filter(status=1)

    @property
    def get_list_phones(self):
        """
        :return: List of all contact enabled phones
        """
        return [unicode(phone) for phone in self.get_phones]

    @property
    def get_string_phones(self):
        """
        :return: Readable string of all contact enabled phones separated by comma
        """
        return u", ".join(self.get_list_phones)

    @property
    def get_memberships(self):
        """
        :return: Queryset of all contact memberships
        """
        return self.memberships.all()

    @property
    def get_list_memberships(self):
        """
        :return: List of all contact memberships
        """
        return [unicode(membership) for membership in self.get_memberships]

    @property
    def get_string_memberships(self):
        """
        :return: Readable string of all contact memberships
        """
        return u", ".join(self.get_list_memberships)

    def toggle_test(self):
        """
        Changes the contact test status saving it as the opposite of the current one
        """
        self.is_test_contact = not self.is_test_contact
        self.save()

    def make_base36(self):
        """
        :return: Base36 of contact id
        """
        return int_to_base36(self.id)

    @property
    def code_for_token(self):
        """
        :return: Token code calculated on contact id and email
        """
        return u"{}-{}".format(zerofill(self.id, 20), self.email)

    @property
    def token(self):
        """
        :return: Contact token culculated on code_for_token instance property result
        """
        token_generator = TokenGenerator()
        return token_generator.make_token(self.code_for_token)

    def check_token(self, token, duration=None):
        """
        This method is used to check if a given token can be accepted.
        The date check status is currently commented out but could be useful in future projects
        :param token: Given token
        :param duration: Used check token by an expiration date
        :return: Boolean representing check result
        """
        token_generator = TokenGenerator()
        if not token_generator.check_token(self.code_for_token, token):
            return False
        # if duration and datetime.datetime.now() > self.date_create + datetime.timedelta(seconds=duration):
        #     return False
        return True

    def get_license_types_list(self):
        return ', '.join(self.license_types.all().values_list('name', flat=True))

    class Meta(object):
        verbose_name = _('contact')
        verbose_name_plural = _('contacts')
        ordering = ('-date_last_modify',)

    def __unicode__(self):
        return u"{} <{}>".format(self.full_name, self.email)


class Agent(DateModel, UserModel, StatusModel, OrderedModel):
    """
    This model is not really used, we thought we had to use it to assign somebody
    as contact or group manager.
    Please delete all dependencies if deleting it out.
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )

    class Meta(object):
        verbose_name = _('agent')
        verbose_name_plural = _('agents')
        ordering = ('ordering', 'name',)

    def __unicode__(self):
        return u"{}".format(self.name)


class ContactGroup(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Groups used to categorize contacts.
    A standard django M2M was built to store associations, remember we don't have
    our common audit model fields (as date_create) on it.
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )
    agent = models.ForeignKey(
        Agent,
        blank=True, null=True,
        related_name='groups',
        verbose_name=_('agent')
    )

    class Meta(object):
        verbose_name = _('group')
        verbose_name_plural = _('groups')
        ordering = ('ordering', 'name',)

    def __unicode__(self):
        return u"{}".format(self.name)

    def add_to_event_invited_list(self, event):
        """
        Adds all group contact to an event invited list
        :arg event: Event for which invites must be created
        """
        if not self.status:
            return False, ugettext("Group %s is inactive" % self.name)

        for contact in self.contacts.filter(status=1):
            invited, msg = contact.add_to_event_invited_list(event)

        return True, ugettext("Group %s invited to event %s" % (self.name, event.name))


class Role(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Used to store the role contact has for a newspaper defined in a Membership instance.
    """
    name = models.CharField(
        max_length=100,
        unique=True,
        verbose_name=_('role'),
    )

    class Meta(object):
        verbose_name = _('role')
        verbose_name_plural = _('roles')
        ordering = ('ordering', 'name',)

    def __unicode__(self):
        return u"{}".format(self.name)


class NewspaperCategory(DateModel, UserModel):
    """
    Used to associate a newspaper to a category
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )

    class Meta(object):
        verbose_name = _('newspaper category')
        verbose_name_plural = _('newspaper categories')
        ordering = ('name',)

    def __unicode__(self):
        return u"{}".format(self.name)


class NewspaperType(DateModel, UserModel):
    """
    Used to associate a newspaper to a type
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )

    class Meta(object):
        verbose_name = _('newspaper type')
        verbose_name_plural = _('newspaper types')
        ordering = ('name',)

    def __unicode__(self):
        return u"{}".format(self.name)


class NewspaperSector(DateModel, UserModel):
    """
    Used to associate a newspaper to a sector
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )

    class Meta(object):
        verbose_name = _('newspaper sector')
        verbose_name_plural = _('newspaper sectors')
        ordering = ('name',)

    def __unicode__(self):
        return u"{}".format(self.name)


class Newspaper(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Used to associate a contact to a newspaper with a Membership instance
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )
    types = models.ManyToManyField(
        NewspaperType,
        related_name='newspapers',
        verbose_name=_('type'),
        blank=True, null=True,
    )
    periodicity = models.CharField(
        max_length=25,
        choices=settings.PUBLIEDITOR_EVENT_NEWSPAPER_PERIODICITY_CHOICES,
        verbose_name=_('periodicity'),
        blank=True, null=True,
    )
    categories = models.ManyToManyField(
        NewspaperCategory,
        related_name='newspapers',
        verbose_name=_('categories'),
        blank=True, null=True,
    )
    sectors = models.ManyToManyField(
        NewspaperSector,
        related_name='newspapers',
        verbose_name=_('sectors'),
        blank=True, null=True,
    )
    url = models.URLField(
        verbose_name=_('url'),
        blank=True, null=True,
    )

    def get_link_url(self):
        """
        :return: Html snippet with link to newspaper url
        """
        if self.url:
            return u"<a href={} target='_blank'>LINK</a>".format(self.url)
        return ""
    get_link_url.short_description = _("URL")
    get_link_url.allow_tags = True

    class Meta(object):
        verbose_name = _('newspaper')
        verbose_name_plural = _('newspapers')
        ordering = ('ordering', 'name',)

    def __unicode__(self):
        return u"{}".format(self.name)


class Card(DateModel, UserModel, OrderedModel):
    """
    This method is used to store contact cards.
    In reality it is used to store contact frequent flyer cards and not generic ones
    """
    contact = models.ForeignKey(
        Contact,
        related_name='cards',
        verbose_name=_('contact'),
    )
    airline = models.ForeignKey(
        "Airline",
        blank=True, null=True,
        related_name='cards',
        verbose_name=_('airline'),
    )
    number = models.CharField(
        max_length=50,
        verbose_name=_("number"),
    )
    expiration_date = models.DateField(
        verbose_name=_("expiration date"),
        blank=True, null=True,
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('default'),
    )

    def get_link_default(self):
        """
        :return: Html snippet with clickable icon representing card status which can be changed
            in changelist via dajaxice
        """
        return """
            <a id='publieditor_event_contact_card_default_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.default else 'no'))
    get_link_default.short_description = _('Default')
    get_link_default.allow_tags = True

    class Meta(object):
        verbose_name = _('frequent flyer card')
        verbose_name_plural = _('frequent flyer cards')
        ordering = ('ordering',)

    def __unicode__(self):
        return u"{} - {}".format(self.airline, self.number)


class ContactSocialMedia(DateModel, UserModel):
    """
    Used to store conacts social media links (facebook, google etc.)
    """
    contact = models.ForeignKey(
        Contact,
        related_name='social_media_links',
        verbose_name=_('contact'),
    )
    social = models.CharField(
        max_length=100,
        choices=settings.CONTACT_SOCIAL_MEDIA_LINK_CHOICES,
        verbose_name=_('social media'),
    )
    link = models.CharField(
        verbose_name=_("link"),
        max_length=250,
    )

    @property
    def full_link(self):
        """
        :return: Correct external link base on social type
        """
        if self.social == "facebook":
            return "https://www.facebook.com/{}".format(self.link)
        elif self.social == "twitter":
            return "https://twitter.com/{}".format(self.link)
        elif self.social == "linkedin":
            return "https://it.linkedin.com/in/{}".format(self.link)
        elif self.social == "instagram":
            return "https://instagram.com/{}".format(self.link)
        return self.link

    def link_open(self):
        """
        :return: Href html snippet of contact social media correct external url
        """
        return "<a href='{}' target='_blank'>X</a>".format(self.full_link)
    link_open.short_description = _('Open')
    link_open.allow_tags = True

    class Meta(object):
        verbose_name = _('social media link')
        verbose_name_plural = _('social media links')

    def __unicode__(self):
        return u"{} ({})".format(self.social, self.link)


class Membership(DateModel, UserModel):
    """
    Used to store associations by contacts and newspapers.
    Role field enable us to define different roles for the same newspaper on a contact.
    """
    newspaper = models.ForeignKey(
        Newspaper,
        related_name='memberships',
        verbose_name=_('newspaper'),
    )
    contact = models.ForeignKey(
        Contact,
        related_name='memberships',
        verbose_name=_('contact'),
    )
    role = models.ForeignKey(
        Role,
        related_name='memberships',
        verbose_name=_("role"),
        blank=True, null=True,
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('default'),
        help_text=_('At least one default role must be active')
    )

    def get_link_default(self):
        """
        :return: Dajaxice clickable href html snippet representing membership status
        """
        return """
            <a id='publieditor_event_contact_memberships_default_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.default else 'no'))
    get_link_default.short_description = _('Default')
    get_link_default.allow_tags = True

    class Meta(object):
        verbose_name = _('company role')
        verbose_name_plural = _('company roles')
        ordering = ('date_create',)
        unique_together = ('newspaper', 'contact', 'role')

    def __unicode__(self):
        if self.role:
            return u"{} ({})".format(self.newspaper.name, self.role.name)
        return u"{}".format(self.newspaper.name)

    def clean(self):
        """
        :return: an exception trying to save a contact default membership if already defined
        """
        if self.default is True and self.contact.memberships.filter(default=True).exclude(id=self.id).count():
            raise ValidationError(_("Contact default membership already found"))


class ContactEmailCc(DateModel, UserModel):
    """
    Used to store all contact cc emails which will also receives his/her emails
    """
    contact = models.ForeignKey(
        Contact,
        related_name='cc_emails',
        verbose_name=_('contact'),
    )
    email = models.EmailField(
        verbose_name=_('email'),
        null=True, blank=True,
    )
    contact_cc = models.ForeignKey(
        Contact,
        null=True, blank=True,
        related_name='cc_emails_for',
        verbose_name=_('contact cc'),
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('default'),
    )
    note = models.TextField(
        null=True, blank=True,
        verbose_name=_('note'),
    )

    def clean(self):
        """
        :return: An exception only when trying to save a default record if it is already present
        """
        if self.default is True and self.contact.cc_emails.filter(default=True).exclude(id=self.id).count():
            raise ValidationError(_("Contact default email cc already found"))

    def get_link_default(self):
        """
        :return: Dajaxice clickable html snippet representing current default status
        """
        return """
            <a id='publieditor_event_contact_contactemailcc_default_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.default else 'no'))
    get_link_default.short_description = _('Default')
    get_link_default.allow_tags = True

    def email_details(self):
        """
        :return: Correct email address checking which of the fk or the EmailField were used
        """
        if self.contact_cc:
            return self.contact_cc.email
        return self.email
    email_details.short_description = _("Email")

    class Meta(object):
        verbose_name = _('email cc')
        verbose_name_plural = _('emails cc')

    def __unicode__(self):
        return u"{}".format(self.email)


class ContactEditDiff(DateModel, UserModel):
    """
    Model used to store in a textual way which data changed about the contact when he interacts with its profile
    forms.
    This textual result will be sent via email to desired email addresses.
    These forms are currently just first registration one (views.ProfileView),
    and register step form (class_views.RegisterUpdateView)
    This is not an optimal approach but I had an other quick alternative when working on that.
    """
    contact = models.ForeignKey(
        Contact,
        related_name='edits_diff',
        verbose_name=_('contact'),
    )
    body = models.TextField(
        null=True, blank=True,
        verbose_name=_('body'),
        default="",
    )

    class Meta(object):
        verbose_name = _('contact edit differences')
        verbose_name_plural = _('contacts edit differences')

    def __unicode__(self):
        return u"{}: {}".format(self.id, self.contact)

    def save_contact_edits(self, differences):
        """
        Starts creating the record with initial differences if present
        """
        msg = u"Il contatto {} è stato modificato alle {} del {} con le seguenti modifiche:".format(
            self.contact.full_name, self.date_create.strftime("%H:%M"), self.date_create.strftime("%d/%m/%Y"),
        )
        if differences:
            for difference_field, difference_values in differences.iteritems():
                msg += u"\n{}: {} -> {}".format(
                    field_verbose_name(difference_field, Contact),
                    difference_values["old_value"], difference_values["new_value"],
                )
        self.body = msg
        self.save()

    def save_phone_edits(self, new_instance, differences, is_created=False, is_deleted=False):
        """
        Continue updating the record saving differences about phone related data
        :param new_instance: new record instance
        :param differences: differences beetween old record and new one
        :param is_created: represents the record creation flag
        :param is_deleted: represents the record deletion flag
        """
        if differences:
            if is_created:
                differences.update({'id': dict(new_value=new_instance.id)})
                msg = u"\n\nUn nuovo telefono è stato creato con i seguenti valori:"
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, Phone),
                        difference_values["new_value"]
                    )
            elif is_deleted:
                msg = u"\n\nIl telefono coi seguenti valori è stato cancellato:"
                for difference_field, difference_value in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, Phone),
                        difference_value
                    )
            else:
                msg = u"\n\nIl telefono con id {} ha subito le seguenti modifiche:".format(new_instance.id)
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {} -> {}".format(
                        field_verbose_name(difference_field, Phone),
                        difference_values["old_value"], difference_values["new_value"]
                    )
            self.body += msg
            self.save()

    def save_membership_edits(self, new_instance, differences, is_created=False, is_deleted=False):
        """
        Continue updating the record saving differences about memberships related data
        :param new_instance: new record instance
        :param differences: differences beetween old record and new one
        :param is_created: represents the record creation flag
        :param is_deleted: represents the record deletion flag
        """
        if differences:
            if is_created:
                differences.update({'id': dict(new_value=new_instance.id)})
                msg = u"\n\nUna nuova testata di appartenenza è stata aggiunta con i seguenti valori:"
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, Membership),
                        difference_values["new_value"]
                    )
            elif is_deleted:
                msg = u"\n\nLa testata di appartenenza coi seguenti valori è stata cancellata:"
                for difference_field, difference_value in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, Membership),
                        difference_value
                    )
            else:
                msg = u"\n\nLa testata di appartenenza con id {} ha subito le seguenti modifiche:".format(new_instance.id)
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {} -> {}".format(
                        field_verbose_name(difference_field, Membership),
                        difference_values["old_value"], difference_values["new_value"]
                    )
            self.body += msg
            self.save()

    def save_email_cc_edits(self, new_instance, differences, is_created=False, is_deleted=False):
        """
        Continue updating the record saving differences about email_cc related data
        :param new_instance: new record instance
        :param differences: differences beetween old record and new one
        :param is_created: represents the record creation flag
        :param is_deleted: represents the record deletion flag
        """
        if differences:
            if is_created:
                differences.update({'id': dict(new_value=new_instance.id)})
                msg = u"\n\nUna nuova email cc è stata aggiunta con i seguenti valori:"
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, ContactEmailCc),
                        difference_values["new_value"]
                    )
            elif is_deleted:
                msg = u"\n\nL'email cc coi seguenti valori è stata cancellata:"
                for difference_field, difference_value in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, ContactEmailCc),
                        difference_value
                    )
            else:
                msg = u"\n\nL'email cc con id {} ha subito le seguenti modifiche:".format(new_instance.id)
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {} -> {}".format(
                        field_verbose_name(difference_field, ContactEmailCc),
                        difference_values["old_value"], difference_values["new_value"]
                    )
            self.body += msg
            self.save()

    def save_address_edits(self, new_instance, differences, is_created=False, is_deleted=False):
        """
        Continue updating the record saving differences about addresses related data
        :param new_instance: new record instance
        :param differences: differences beetween old record and new one
        :param is_created: represents the record creation flag
        :param is_deleted: represents the record deletion flag
        """
        if differences:
            if is_created:
                differences.update({'id': dict(new_value=new_instance.id)})
                msg = u"\n\nUn nuovo indirizzo è stato aggiunto con i seguenti valori:"
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, ContactAddress),
                        difference_values["new_value"],
                    )
            elif is_deleted:
                msg = u"\n\nL'indirizzo coi seguenti valori è stato cancellato:"
                for difference_field, difference_value in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, ContactAddress),
                        difference_value,
                    )
            else:
                msg = u"\n\nL'indirizzo con id {} ha subito le seguenti modifiche:".format(new_instance.id)
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {} -> {}".format(
                        field_verbose_name(difference_field, ContactAddress),
                        difference_values["old_value"], difference_values["new_value"]
                    )
            self.body += msg
            self.save()

    def save_card_edits(self, new_instance, differences, is_created=False, is_deleted=False):
        """
        Continue updating the record saving differences about cards related data
        :param new_instance: new record instance
        :param differences: differences beetween old record and new one
        :param is_created: represents the record creation flag
        :param is_deleted: represents the record deletion flag
        """
        if differences:
            if is_created:
                differences.update({'id': dict(new_value=new_instance.id)})
                msg = u"\n\nUna nuova frequent flyer è stata aggiunta con i seguenti valori:"
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, Card),
                        difference_values["new_value"]
                    )
            elif is_deleted:
                msg = u"\n\nLa frequent flyer coi seguenti valori è stata cancellata:"
                for difference_field, difference_value in differences.iteritems():
                    msg += u"\n{}: {}".format(
                        field_verbose_name(difference_field, Card),
                        difference_value
                    )
            else:
                msg = u"\n\nLa frequent flyer con id {} ha subito le seguenti modifiche:".format(new_instance.id)
                for difference_field, difference_values in differences.iteritems():
                    msg += u"\n{}: {} -> {}".format(
                        field_verbose_name(difference_field, Card),
                        difference_values["old_value"], difference_values["new_value"]
                    )
            self.body += msg
            self.save()

    def send(self):
        """
        Send emails with textual differences result to PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO and
        PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN recipients
        """
        subject = "Modifiche contatto {}".format(self.contact)
        from_email = settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM
        to = [settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO]
        msg = EmailMultiAlternatives(
            subject, self.body, from_email, to,
            bcc=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN,
        )
        msg.send()


class Phone(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Used to store contact phones
    """
    contact = models.ForeignKey(
        Contact,
        related_name='phones',
        verbose_name=_('contact'),
    )
    area_code = models.ForeignKey(
        'AreaCode',
        blank=True, null=True,
        related_name='phones',
        verbose_name=_('area code'),
        default=lambda: AreaCode.objects.get(code="+39") if AreaCode.objects.filter(code="+39").count() else None
    )
    phone_type = models.CharField(
        max_length=2,
        choices=settings.CONTACT_PHONE_PHONE_TYPE_CHOICES,
        default="P",
        verbose_name=_('type'),
    )
    number = models.CharField(
        max_length=50,
        verbose_name=_('number'),
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('default'),
    )
    note = models.TextField(
        null=True, blank=True,
        verbose_name=_('note'),
    )

    @property
    def full_number(self):
        """
        :return: Full number based on area_code and number
        """
        if self.area_code:
            return u"{}{}".format(self.area_code.code, self.number)
        return u"{}".format(self.number)

    def get_link_default(self):
        """
        :return: Dajaxice clickable icon representing default flag
        """
        return """
            <a id='publieditor_event_contact_phone_default_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.default else 'no'))
    get_link_default.short_description = _('Default')
    get_link_default.allow_tags = True

    def save(self, *args, **kwargs):
        """
        Set default to False if saving/updating a new default one for current contact
        """
        if self.default is True:
            phones = self.contact.phones.filter(default=True)
            if self.id:
                phones = phones.exclude(id=self.id)
            for phone in phones:
                phone.default = False
                phone.save()
        super(Phone, self).save(*args, **kwargs)

    class Meta(object):
        verbose_name = _('phone')
        verbose_name_plural = _('phones')
        ordering = ('ordering', 'number', 'phone_type',)

    def __unicode__(self):
        return u"{} {}".format(self.get_phone_type_display(), self.number)


class ContactAddress(Address):
    """
    Used to store all contact addresses
    """
    contact = models.ForeignKey(
        Contact,
        related_name='addresses',
        verbose_name=_('contact'),
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('default'),
    )
    note = models.TextField(
        null=True, blank=True,
        verbose_name=_('note'),
    )

    def get_link_default(self):
        """
        :return: Dajaxice clickable icon representing recaord default flag
        """
        return """
            <a id='publieditor_event_contact_contactaddress_default_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.default else 'no'))
    get_link_default.short_description = _('Default')
    get_link_default.allow_tags = True

    class Meta(object):
        verbose_name = _('contact address')
        verbose_name_plural = _('contact addresses')
        ordering = ('ordering', 'name', 'number',)

    def get_vcard_address_type_display(self):
        address_type = self.get_address_type_display()
        return address_type if address_type else u''

    def __unicode__(self):
        address_str = u""
        # address_type
        if self.address_type and self.address_type != "":
            address_str += u"{0} ".format(self.get_address_type_display())
        # name
        if self.name and self.name != "":
            address_str += u"{0} ".format(self.name)
        # number
        if self.number and self.number != "":
            address_str += u"{0} ".format(self.number)
        # locality
        if self.locality and self.locality != "":
            address_str += u"{0} ".format(self.locality)
        # zip_code
        if self.zip_code and self.zip_code != "":
            address_str += u"{0} ".format(self.zip_code)
        # province
        if self.province and self.province != "":
            address_str += u"({0}) ".format(self.province)
        # country
        if address_str and self.country and self.country.name != "":
            address_str += u"{0}".format(self.country.name)
        return u"{0}".format(address_str)

    def clean(self):
        """
        :return: Exception when trying to save a default record if current already has a default one
        """
        if self.default is True and self.contact.addresses.filter(default=True).exclude(id=self.id).count():
            raise ValidationError(_("Contact default address already found"))


##############################################################################
# General
##############################################################################
class Event(DateModel, UserModel, StatusModel, OrderedModel, SEOModel):
    """
    Events main model
    It stores a lot of informations about event configuration and online rendering and user interaction about that
    """
    is_newsletter = models.BooleanField(
        default=False,
        verbose_name=_("Newsletter"),
    )
    is_privacy = models.BooleanField(
        default=False,
        verbose_name=_("Privacy"),
    )
    adhesion_in_email = models.BooleanField(
        default=False,
        verbose_name=_("adhesion in email"),
        help_text=_('Set this to have Accept/Decline/Delegate directly in the invite email')
    )
    name = models.CharField(
        max_length=100,
        unique=True,
        verbose_name=_('name'),
    )
    slug = models.SlugField(
        max_length=100,
        unique=True,
        verbose_name=_('slug'),
    )
    place = models.CharField(
        max_length=255,
        verbose_name=_('place'),
    )
    event_date_start = models.DateTimeField(
        verbose_name=_('start date'),
    )
    event_date_end = models.DateTimeField(
        verbose_name=_('end date')
    )
    event_adhesion_deadline = models.DateTimeField(
        verbose_name=_('adhesion deadline')
    )
    landing_page_template = models.CharField(
        max_length=250,
        verbose_name=_("landing page template"),
        choices=settings.PUBLIEDITOR_EVENT_LANDING_PAGE_TEMPLATE_CHOICES
    )
    default_language = models.ForeignKey(
        Language,
        related_name="events",
        verbose_name=_("default language")
    )
    enable_own = models.BooleanField(
        default=True,
        blank=True,
        verbose_name=_("enable own"),
    )
    enable_free = models.BooleanField(
        default=True,
        blank=True,
        verbose_name=_("enable free"),
    )
    enable_delegate = models.BooleanField(
        default=True,
        blank=True,
        verbose_name=_("enable delegate"),
    )
    enable_personalized_program = models.BooleanField(
        default=True,
        blank=True,
        verbose_name=_("enable personalized program download"),
    )
    enable_ics = models.BooleanField(
        default=True,
        blank=True,
        verbose_name=_("enable ics download"),
    )
    enable_forwarding = models.BooleanField(
        default=False,
        blank=True,
        verbose_name=_("enable forwarding"),
    )
    max_limit_forwarders = models.IntegerField(
        null=True, blank=True,
        default=10,
        verbose_name=_("max limit forwarders"),
    )
    documents = models.ManyToManyField(
        'EventDocument',
        related_name="events",
    )
    start_step = models.CharField(
        max_length=20,
        choices=settings.PUBLIEDITOR_EVENT_INVITE_FIRST_STEP,
        default='presentation',
        verbose_name=_('invite start step'),
    )
    ask_phone = models.BooleanField(
        default=False,
        verbose_name=_("ask phone"),
    )
    phone_question_type = models.CharField(
        max_length=20,
        blank=True, null=True,
        choices=settings.PUBLIEDITOR_EVENT_PHONE_QUESTION_TYPE,
        verbose_name=_('phone question type'),
    )
    is_phone_required = models.BooleanField(
        default=False,
        verbose_name=_("phone required"),
    )
    ask_address = models.BooleanField(
        default=False,
        verbose_name=_("ask address"),
    )
    address_question_type = models.CharField(
        max_length=20,
        blank=True, null=True,
        choices=settings.PUBLIEDITOR_EVENT_ADDRESS_QUESTION_TYPE,
        verbose_name=_('address question type'),
    )
    is_address_required = models.BooleanField(
        default=False,
        verbose_name=_("address required"),
    )
    ask_id_card = models.BooleanField(
        default=False,
        verbose_name=_("ask id card"),
    )
    is_id_card_required = models.BooleanField(
        default=False,
        verbose_name=_("id card required"),
    )
    ask_passport = models.BooleanField(
        default=False,
        verbose_name=_("ask passport"),
    )
    is_passport_required = models.BooleanField(
        default=False,
        verbose_name=_("passport required"),
    )
    is_id_card_or_passport_required = models.BooleanField(
        default=False,
        verbose_name=_("id card or passport required"),
    )
    ask_license = models.BooleanField(
        default=False,
        verbose_name=_("ask license"),
    )
    is_license_required = models.BooleanField(
        default=False,
        verbose_name=_("license required"),
    )
    ask_press_card_number = models.BooleanField(
        default=False,
        verbose_name=_("ask press card number"),
    )
    is_press_card_number_required = models.BooleanField(
        default=False,
        verbose_name=_("press card number required"),
    )
    ask_frequent_flyer = models.BooleanField(
        default=False,
        verbose_name=_("ask frequent flyer card"),
    )
    frequent_flyer_question_type = models.CharField(
        max_length=20,
        blank=True, null=True,
        choices=settings.PUBLIEDITOR_EVENT_FREQUENT_FLYER_QUESTION_TYPE,
        verbose_name=_('frequent flyer question type'),
    )
    is_frequent_flyer_required = models.BooleanField(
        default=False,
        verbose_name=_("frequent flyer required"),
    )
    ask_food_hotel_preferences = models.BooleanField(
        default=False,
        verbose_name=_("ask hotel/food preferences"),
    )
    is_food_hotel_preferences_required = models.BooleanField(
        default=False,
        verbose_name=_("hotel/food preferences required"),
    )
    ask_clothing = models.BooleanField(
        default=False,
        verbose_name=_("ask clothing"),
    )
    is_clothing_required = models.BooleanField(
        default=False,
        verbose_name=_("clothing required"),
    )
    ask_body_measurements = models.BooleanField(
        default=False,
        verbose_name=_("ask body measurements"),
    )
    is_body_measurements_required = models.BooleanField(
        default=False,
        verbose_name=_("body measurements"),
    )
    ask_ski = models.BooleanField(
        default=False,
        verbose_name=_("ask ski"),
    )
    is_ski_required = models.BooleanField(
        default=False,
        verbose_name=_("ski required"),
    )
    show_breadcrumbs = models.BooleanField(
        default=True,
        verbose_name=_("show breadcrumbs"),
    )
    breadcrumbs_position = models.CharField(
        max_length=20,
        choices=settings.PUBLIEDITOR_EVENT_EVENT_BREADCRMUBS_POSITION_CHOICES,
        default='top',
        verbose_name=_('breadcrumbs position'),
    )
    show_presentation = models.BooleanField(
        default=True,
        verbose_name=_("show presentation"),
    )
    show_schedule = models.BooleanField(
        default=True,
        verbose_name=_("show schedule"),
    )
    show_adhesion = models.BooleanField(
        default=True,
        verbose_name=_("show adhesion"),
    )
    show_register = models.BooleanField(
        default=True,
        verbose_name=_("show register"),
    )
    show_transport = models.BooleanField(
        default=True,
        verbose_name=_("show transport"),
    )
    show_activity = models.BooleanField(
        default=True,
        verbose_name=_("show activity"),
    )
    show_summary = models.BooleanField(
        default=True,
        verbose_name=_("show summary"),
    )
    show_decline = models.BooleanField(
        default=True,
        verbose_name=_("show decline"),
    )
    show_delegate = models.BooleanField(
        default=True,
        verbose_name=_("show delegate"),
    )
    show_forwarding = models.BooleanField(
        default=False,
        verbose_name=_("show forwarding"),
    )
    show_documents = models.BooleanField(
        default=True,
        verbose_name=_("show documents"),
    )
    show_greetings = models.BooleanField(
        default=True,
        verbose_name=_("show greetings"),
    )
    ask_poll_on_followup = models.BooleanField(
        default=False,
        verbose_name=_("insert poll url"),
    )
    header_color = RGBColorField(
        _("Header color"),
    )
    invites_send_from = models.DateTimeField(
        verbose_name=_('invites send from'),
        blank=True, null=True
    )
    invites_send_to = models.DateTimeField(
        verbose_name=_('invites send to'),
        blank=True, null=True
    )
    travel_agencies = models.ManyToManyField(
        'auth.User',
        related_name='travel_agency_events',
        verbose_name=_('travel agencies'),
        null=True, blank=True
    )
    logistic_agencies = models.ManyToManyField(
        'auth.User',
        related_name='logistic_agency_events',
        verbose_name=_('logistic agencies'),
        null=True, blank=True
    )
    hostess = models.ManyToManyField(
        'auth.User',
        related_name='hostess_events',
        verbose_name=_('hostess'),
        null=True, blank=True
    )
    ask_birthday = models.BooleanField(
        default=False,
        verbose_name=_("ask birthday"),
    )
    is_birthday_required = models.BooleanField(
        default=False,
        verbose_name=_("birthday required"),
    )
    ask_birth_place = models.BooleanField(
        default=False,
        verbose_name=_("ask birth place"),
    )
    is_birth_place_required = models.BooleanField(
        default=False,
        verbose_name=_("birth place required"),
    )
    ask_ssn = models.BooleanField(
        default=False,
        verbose_name=_("ask ssn"),
    )
    is_ssn_required = models.BooleanField(
        default=False,
        verbose_name=_("ssn required"),
    )
    has_manual_adhesion = models.BooleanField(
        default=False,
        verbose_name=_("has manual adhesion"),
        help_text=_('Set this if invited contacts receive truncated link in email')
    )
    vehicle_models = models.ManyToManyField(
        'VehicleModel',
        through='EventVehicleModel',
        verbose_name=_('vehicle models'),
        help_text=_('vehicle models at the event'),
    )

    class Meta(object):
        verbose_name = _('event')
        verbose_name_plural = _('events')
        ordering = ('-event_date_start',)

    def __unicode__(self):
        return u"{}".format(self.name)

    def get_step_translation(self, type, contact=None, user=None, is_preview=False):
        """
        :return: correct required step translation base on language preferences
        """
        if is_preview and user and self.translation_preferences.filter(user=user).count():
            language = self.translation_preferences.filter(
                user=user
            ).order_by('-date_create')[0].language
        elif contact and self.translation_preferences.filter(invite__contact=contact).count():
            language = self.translation_preferences.filter(
                invite__contact=contact
            ).order_by('-date_create')[0].language
        elif contact:
            language = contact.language
        try:
            translation = getattr(self, type).get(status=1, language=language)
        except (ObjectDoesNotExist, UnboundLocalError):
            if getattr(self, type).filter(status=1, language=self.default_language).count() == 1:
                translation = getattr(self, type).get(status=1, language=self.default_language)
            elif getattr(self, type).all().count():
                translation = getattr(self, type).all()[0]
            else:
                translation = None
        return translation

    @property
    def first_schedule_translation(self):
        """
        :return: firs schedule translation
        """
        return self.schedule_translations.all()[0]

    def get_date_start_cl(self):
        """
        :return: Event date start for changelist
        """
        return self.event_date_start.strftime("%Y-%m-%d (%a)")
    get_date_start_cl.short_description = _("Date start")

    def get_date_end_cl(self):
        """
        :return: Event end start for changelist
        """
        return self.event_date_end.strftime("%Y-%m-%d (%a)")
    get_date_end_cl.short_description = _("Date end")

    def get_adhesion_deadline_cl(self):
        """
        :return: Event adhesion deadline start for changelist
        """
        return self.event_adhesion_deadline.strftime("%Y-%m-%d (%a)")
    get_adhesion_deadline_cl.short_description = _("Adhesion before")

    def get_preview_links(self):
        """
        :return: Href html snippeto for event web previews
        """
        return u"""<a href="%s">WEB</a>""" % reverse(
            'admin:event_web_preview', kwargs={"event_id": self.id},
        )
    get_preview_links.short_description = _("Previews")
    get_preview_links.allow_tags = True

    @property
    def has_transport_details(self):
        """
        :return: True if event has transport details else False
        """
        return bool(self.transport_details.all().count())

    @property
    def has_activity_questions(self):
        """
        :return: True if event has activity questions else False
        """
        return bool(self.questions.filter(question__typology="A").all().count())

    def has_registration_required_field(self):
        return any([
            self.is_phone_required,
            self.is_address_required,
            self.is_id_card_required,
            self.is_passport_required,
            self.is_license_required,
            self.is_press_card_number_required,
            self.is_frequent_flyer_required,
            self.is_food_hotel_preferences_required,
            self.is_clothing_required,
            self.is_body_measurements_required,
            self.is_ski_required,
            self.is_birthday_required,
            self.is_birth_place_required,
            self.is_ssn_required,
        ])

    def get_admin_change_url(self):
        """
        :return: Href html snippet for event change form
        """
        return u'<a href="%s">%s</a>' % (
            reverse('admin:event_event_change', args=[self.id]), self.__unicode__()
        )

    def get_admin_invites_change_list_url(self):
        """
        :return: url of invetes change list. ES: /admin/event/invite/?event=903
        """
        return reverse('admin:event_invite_changelist') + '?event={}'.format(self.id)

    def send_invites(self, test=False):
        """
        Sends invites to event recipients
        :return: Boolean and msg about action results
        """
        for invite in self.invites.all():
            invited, msg = invite.send(test)
        return True, ugettext("Invites sent for event %s" % self.name)

    def resend_invites(self, test=False):
        """
        Resend invites to event recipients
        :return: Boolean and msg about action results
        """
        for invite in self.invites.all():
            invited, msg = invite.resend(test)
        return True, ugettext("Invites resent for event %s" % self.name)

    def send_followups(self, test=False):
        """
        Send followups to event recipients
        :return: Boolean and msg about action results
        """
        for invite in self.invites.all():
            invited, msg = invite.send_followup(test)
        return True, ugettext("Follow ups sent for event %s" % self.name)

    def send_documents(self, test=False):
        """
        Send documents to event recipients
        :return: Boolean and msg about action results
        """
        for invite in self.invites.all():
            invited, msg = invite.send_documents(test)
        return True, ugettext("Documents sent for event %s" % self.name)

    def is_expired(self):
        """
        :return: True if event is expired else False
        """
        now = datetime.datetime.now()
        return self.event_date_start <= now

    def is_over(self):
        """
        :return: True if event date end is expired else False
        """
        now = datetime.datetime.now()
        return self.event_date_end <= now

    def is_deadline_passed(self):
        """
        :return: True if event deadline is passed else False
        """
        now = datetime.datetime.now()
        return self.event_adhesion_deadline <= now

    def get_string_registers(self):
        """
        I think this method is no longer used as self.registers is not defined in models.
        Please check before deleting.
        """
        return ', '.join([unicode(reg) for reg in self.registers.all()])

    def get_string_transports(self):
        """
        :return: String of event trasports separated by comma
        """
        return ', '.join([unicode(transp) for transp in self.transports.all()])

    def get_string_activities(self):
        """
        :return: String of event activities separated by comma
        """
        return ', '.join([unicode(act) for act in self.activities.all()])

    def get_transport_available_groups(self):
        """
        I think this is no longer used.
        Please check before deleting.
        """
        event_transport_groups = []
        transports = self.transports.all().distinct('group')
        for transport in transports:
            if transport.group:
                event_transport_groups.append((transport.group, transport.get_group_display()))
            else:
                event_transport_groups.append((None, None))
        return event_transport_groups

    def get_flight_turns(self):
        """
        :return: Available turns based on event flight details
        """
        return Turn.objects.filter(flight_details__event_id=self.id).distinct()

    def get_flight_stages(self):
        """
        :return: Available stages base on event flight details
        """
        return Stage.objects.filter(flights__flight_details__event_id=self.id).distinct()

    def get_transports(self):
        """
        :return: Event TransportDetail queryset
        """
        transports = TransportDetail.objects.filter(
            event_id=self.id,
        ).order_by('departure')
        return transports

    def get_transport_detail_aggregation_keys(self):
        """
        I think this is no longer used, in other case it should be refactored.
        Please check before delting.
        """
        aggregation_keys_dict = dict()
        for detail in TransportDetail.objects.filter(event=self).values_list('turn', 'start_place',  'id'):
            key = "%s:%s" % (detail[0], detail[1])
            if key not in aggregation_keys_dict:
                aggregation_keys_dict[key] = "%s" % detail[2]
            else:
                aggregation_keys_dict[key] += "_%s" % detail[2]
        return aggregation_keys_dict

    def get_email_mailing_status(self):
        """
        :return: MailingStatus queryset filtered on current event
        """
        newsletter_type = ContentType.objects.get_for_model(self)
        email_mailing_status = MailingStatus.objects.filter(
            email_object__content_type__pk=newsletter_type.id,
            email_object__object_id=self.id
        )
        return email_mailing_status

    def is_presentation_multilingual(self):
        """
        :return: True if event has both italian and english translations else False
        """
        has_italian_translation = bool(self.translations.filter(status=1, language__code__iexact='it').count())
        has_english_translation = bool(self.translations.filter(status=1, language__code__iexact='en').count())
        if has_italian_translation and has_english_translation:
            return True
        return False

    def get_home_page_template_name(self):
        return settings.PUBLIEDITOR_EVENT_HOME_PAGE_TEMPLATES[self.landing_page_template]

    def clean(self):
        """
        :return: Exception if event dates are not consistent
        """
        if self.event_date_end and self.event_date_end and self.event_adhesion_deadline:
            if self.event_date_start > self.event_date_end:
                raise ValidationError(_("Event start date can't be greater than end one"))
            if self.event_adhesion_deadline > self.event_date_start:
                raise ValidationError(_("Event adhesion deadline date can't be greater than start one"))

    def save(self, *args, **kwargs):
        """
        Saves a correct eventy slug to preserve db consistency. It should be refactored to support errors handling
        where second slugify result is already present in db.
        """
        if not self.id:
            slug = slugify(self.name)
            while True:
                records_found = Event.objects.filter(slug=slug).count()
                if records_found:
                    slug = slugify("%s-%s" % (slug, records_found))
                else:
                    break
            self.slug = slug
        super(Event, self).save(*args, **kwargs)


class EventTravelAgencyProxy(Event):
    """
    Simple Event Proxy to add travel agencies functionalities
    """
    class Meta:
        verbose_name = _('travel agency event')
        verbose_name_plural = _('travel agency events')
        proxy = True


class EventLogisticAgencyProxy(Event):
    """
    Simple Event proxy to add logistic agencies functionalities
    """
    class Meta:
        verbose_name = _('logistic agency event')
        verbose_name_plural = _('logistic agency events')
        proxy = True


class EventHostessProxy(Event):
    """
    Simple Event proxy to add hostess functionalities
    """
    class Meta:
        verbose_name = _('hostess event')
        verbose_name_plural = _('hostess events')
        proxy = True


class Invite(DateModel, UserModel, OrderedModel):
    """
    Invite main model.
    This fundamentally involves relation between event and contact, a unique_together exists to preserve uniqueness
    """
    status = models.IntegerField(
        choices=settings.PUBLIEDITOR_EVENT_INVITE_STATUS_CODES,
        null=True, blank=True,
        verbose_name=_('status'),
        default=settings.PUBLIEDITOR_EVENT_INVITE_PENDING,
    )
    event = models.ForeignKey(
        Event,
        related_name='invites',
        verbose_name=_('event')
    )
    contact = models.ForeignKey(
        Contact,
        related_name='invites',
        verbose_name=_('contact')
    )
    step = models.CharField(
        max_length=20,
        default='presentation',
        choices=settings.PUBLIEDITOR_EVENT_INVITE_STEP_CHOICES,
        verbose_name=_('step')
    )
    forwarded_from = models.ForeignKey(
        "self",
        verbose_name=_('forwarded from'),
        related_name='forwarded_invitations',
        blank=True, null=True,
        default=None,
    )
    has_partecipated = models.BooleanField(
        default=False,
        verbose_name=_('has partecipated'),
    )
    no_show = models.BooleanField(
        default=False,
        verbose_name=_('no show'),
    )
    sent = models.BooleanField(
        default=False,
        verbose_name=_('sent'),
    )
    managed = models.BooleanField(
        default=False,
        verbose_name=_('managed'),
    )
    note = models.TextField(
        null=True, blank=True,
        verbose_name=_('note'),
    )
    internal_note = models.TextField(
        verbose_name=_("internal note"),
        blank=True, null=True,
    )
    turn_preference = models.ForeignKey(
        "Turn",
        related_name='preference_invites',
        verbose_name=_('turn preference'),
        blank=True, null=True,
    )
    favourite_vehicle_model = models.ForeignKey(
        'VehicleModel',
        blank=True, null=True,
        related_name='invites',
        verbose_name=_('favourite vehicle model'),
        help_text=_('field auto filled in invited registration'),
    )
    assigned_vehicle = models.ForeignKey(
        'Vehicle',
        blank=True, null=True,
        related_name='invites',
        verbose_name=_('assigned vehicle'),
        help_text=_('field auto filled from hostess during event'),
    )
    assigned_place = models.ForeignKey(
        'EventPlace',
        blank=True, null=True,
        related_name='invites_assigned',
        verbose_name=_('assigned place'),
        help_text=_('field auto filled from hostess during event'),
    )
    event_place = models.ForeignKey(
        'EventPlace',
        blank=True, null=True,
        related_name='invites',
        verbose_name=_('favourite event places'),
        help_text=_('field auto filled in invited registration'),
    )
    call_status = models.CharField(
        choices=settings.PUBLIEDITOR_EVENT_PHONE_CONTACT_RESULT,
        max_length=10,
        default='tc',
        verbose_name=_('call status'),
    )

    class Meta(object):
        verbose_name = _('invite')
        verbose_name_plural = _('invites')
        unique_together = ('event', 'contact', 'event_place')

    def __unicode__(self):
        return u"%s: %s %s - %s" % (
            self.event.name,
            self.contact.first_name,
            self.contact.last_name,
            self.get_status_display()
        )

    @property
    def is_delegated(self):
        """
        Checks if invite was created by a delegation
        """
        return bool(DelegatedContact.objects.filter(invite__event=self.event, invited=self.contact).count())

    @property
    def is_forwarded(self):
        return bool(self.forwarded_from)

    def get_delegated_text(self):
        """
        :return: text to inform contact that he has been delegate by an other contact
        """
        if self.is_delegated:
            delegated_contact = DelegatedContact.objects.filter(invite__event=self.event, invited=self.contact)[0]
            text = "%s %s " % (
                delegated_contact.invite.contact.first_name,
                delegated_contact.invite.contact.last_name,
            )
            if delegated_contact.invite.contact.get_default_membership:
                text += "di %s " % delegated_contact.invite.contact.get_default_membership.newspaper
            text += "ci ha informato che prenderai il Suo posto all'evento in oggetto."
            return text
        return ""

    def get_forwarded_text(self):
        if self.is_forwarded:
            from django.template import Context, Template

            context = Context({
                'invite': self,
                'contact': self.contact,
            })

            text = ''
            try:
                body = self.event.email_forwarded_translations.get(
                    language=self.contact.language,
                ).body
                text = Template(body).render(context)
            except:
                pass
            return text
        return ""

    def get_summary_transport_detail(self, is_html=True):
        """
        :param is_html: Boolean which inofmre what kind of result we expect
        :return: invite summary detail text
        """
        invites = Invite.objects.filter(pk=self.pk).annotate(
            own_answer_count=models.Count('own_answers'),
            free_answer_count=models.Count('free_answers'),
            travel_option_answer_count=models.Count('travel_option_answers')
        ).first()

        if invites.own_answer_count:
            return _("I'll come by myself")
        elif invites.free_answer_count:
            free_answer = self.free_answers.first()
            return free_answer.answer
        elif invites.travel_option_answer_count:
            invite_travel_option = self.travel_option_answers.first()
            if is_html:
                return invite_travel_option.event_travel_option.get_render_transport_group()
            else:
                return invite_travel_option.event_travel_option.get_plain_transport_group()
        return ""

    def get_event_travel_option_preference(self):
        """
        :return: invite first travel option
        """
        if self.travel_option_answers.all():
            return self.travel_option_answers.all()[0]
        return None

    def get_event_cl(self):
        """
        :return: Event text to be displayed in changelist
        """
        return u"{}".format(self.event)
    get_event_cl.short_description = _("Event")

    def get_cdu_cl(self):
        """
        :return: Href html snippet about contact
        """
        if hasattr(self, 'cdu'):
            return u"""
            <a href="{cdu_url}" download>
                <img alt='CDU' src='/static/img/pdf_icon.png' style='height: 20px;'>
            </a>
            """.format(cdu_url=self.cdu.cdu_file.url)
        else:
            return ''
    get_cdu_cl.short_description = _("CDU")
    get_cdu_cl.allow_tags = True
    get_cdu_cl.admin_order_field = 'cdu'

    def has_cdu(self):
        if hasattr(self, 'cdu'):
            return True
        else:
            return False
    has_cdu.short_description = _('has cdu')

    def get_contact_data_cl(self):
        """
        :return: Href html snippet about contact
        """
        email = "<br>&nbsp;&nbsp;<a href='mailto:%s'>%s</a>" % (self.contact.email, self.contact.email)
        email_cc_html = ""
        for email_obj in self.contact.get_all_email_cc():
            email_obj_email = email_obj.contact_cc.email if email_obj.contact_cc else email_obj.email
            email_cc_html += "<br>&nbsp;&nbsp;cc: <a href='mailto:%s'>%s</a>" % (
                email_obj_email,
                email_obj_email,
            )
        return u"<a href='%s'>%s</a> %s %s" % (
            reverse('admin:event_contact_change', args=[self.contact.id]),
            self.contact.invite_name,
            email,
            email_cc_html
        )
    get_contact_data_cl.short_description = _("Contact")
    get_contact_data_cl.allow_tags = True
    get_contact_data_cl.admin_order_field = 'contact__last_name'

    def get_contact_privacy_status_cl(self):
        """
        :return: True if invite contact has given privecy else False
        """
        field = settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD
        has_privacy = bool(getattr(self.contact, field))
        return u"<img alt='Privacy' src='/static/admin/img/icon-{}.gif'>".format('yes' if has_privacy else 'no')
    get_contact_privacy_status_cl.short_description = _('Privacy')
    get_contact_privacy_status_cl.allow_tags = True
    get_contact_privacy_status_cl.admin_order_field = 'contact__{}'.format(
        settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD
    )

    def get_contact_first_name_cl(self):
        """
        :return: Href html snippet with contact first name
        """
        return u"<a href='%s'>%s</a>" % (
            reverse('admin:event_contact_change', args=[self.contact.id]),
            self.contact.first_name
        )
    get_contact_first_name_cl.short_description = _("First name")
    get_contact_first_name_cl.allow_tags = True
    get_contact_first_name_cl.admin_order_field = 'contact__first_name'

    def get_contact_last_name_cl(self):
        """
        :return: Href html snippet with last first name
        """
        return u"<a href='%s'>%s</a>" % (
            reverse('admin:event_contact_change', args=[self.contact.id]),
            self.contact.last_name
        )
    get_contact_last_name_cl.short_description = _("Last name")
    get_contact_last_name_cl.allow_tags = True
    get_contact_last_name_cl.admin_order_field = 'contact__last_name'

    def get_contact_email_cl(self):
        """
        :return: Href html snippet with contact email and email_cc data
        """
        email_cc_html = ""
        for email_obj in self.contact.get_all_email_cc():
            email_cc_html += "<br>&nbsp;&nbsp;cc: <a href='mailto:%s'>%s</a>" % (
                email_obj.email,
                email_obj.email
            )
        return "<a href='mailto:%s'>%s</a>%s" % (
            self.contact.email, self.contact.email, email_cc_html
        )
    get_contact_email_cl.short_description = _("Email")
    get_contact_email_cl.allow_tags = True
    get_contact_email_cl.admin_order_field = 'contact__email'

    def get_contact_first_name(self):
        return self.contact.first_name
    get_contact_first_name.short_description = _("First name")
    get_contact_first_name.allow_tags = True
    get_contact_first_name.admin_order_field = 'contact__first_name'

    def get_contact_last_name(self):
        return self.contact.last_name
    get_contact_last_name.short_description = _("Last name")
    get_contact_last_name.allow_tags = True
    get_contact_last_name.admin_order_field = 'contact__last_name'

    def get_contact_email(self):
        return self.contact.get_default_email()
    get_contact_email.short_description = _("Email")
    get_contact_email.allow_tags = True
    get_contact_email.admin_order_field = 'contact__email'

    def get_contact_mobile_phone(self):
        if self.contact.get_mobile_phone:
            try:
                return self.contact.get_mobile_phone().number
            except AttributeError:
                return ''
        else:
            return ''
    get_contact_mobile_phone.short_description = _("Mobile")
    get_contact_mobile_phone.allow_tags = True

    def get_contact_newspaper_cl(self):
        """
        :return: list of contact newspapers separated by comma
        """
        contact = self.contact
        newspapers = contact.memberships.all().values_list("newspaper__name", flat=True).distinct()
        return ", ".join(newspapers)
    get_contact_newspaper_cl.short_description = _("Newspapers")
    get_contact_newspaper_cl.admin_order_field = 'contact__memberships__newspaper__name'

    def get_contact_phone_cl(self):
        """
        :return: Html snippet about contact phones data
        """
        all_phones = ""
        for phone in self.contact.phones.all().order_by('ordering'):
            phone_number = "%s" % phone.number.replace(" ", "")
            if phone.default:
                phone_number = "<b>%s</b>" % phone_number
            if phone.phone_type == "P":
                ph_img = "<span><img src='/static/event/img/phone.png' alt='%s'>%s</span>" % (
                    phone.get_phone_type_display(), phone_number
                )
            elif phone.phone_type == "M":
                ph_img = "<span><img src='/static/event/img/cellphone.png' alt='%s'>%s</span>" % (
                    phone.get_phone_type_display(), phone_number
                )
            elif phone.phone_type == "F":
                ph_img = "<span><img src='/static/event/img/cellphone.png' alt='%s'>%s</span>" % (
                    phone.get_phone_type_display(), phone_number
                )
            else:
                ph_img = None
            all_phones += "%s<br>" % ph_img if ph_img else "<br>"
        return all_phones
    get_contact_phone_cl.short_description = _("Phone")
    get_contact_phone_cl.allow_tags = True

    def get_send_status_and_count(self):
        """
        :return: html snippet about invite sendings count and related icon
        """
        sent_count = str(self.email_objects.filter(
            status__in=[settings.MAILER_EMAIL_STATUS_SENT, settings.MAILER_EMAIL_STATUS_SENT_TEST]
        ).count())
        if self.sent:
            return u"""
                <span><img src='/static/admin/img/icon-yes.gif' alt='Inviata' style='vertical-align:middle'></span> (%s)
            """ % sent_count
        else:
            return u"""
                <span><img src='/static/admin/img/icon-no.gif' alt='Non inviata' style='vertical-align:middle'></span> (%s)
            """ % sent_count
    get_send_status_and_count.short_description = _("sents")
    get_send_status_and_count.allow_tags = True

    def get_answers_cl(self):
        """
        :return: Html snippet about invite answers
        """
        if self.status in [settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED, settings.PUBLIEDITOR_EVENT_INVITE_POLL_DONE]:
            return u"<strong>{}</strong>".format(ugettext("Accepted"))
        elif self.status == settings.PUBLIEDITOR_EVENT_INVITE_DECLINED:
            return u"<strong>{}</strong>".format(ugettext("Declined"))
        elif self.status == settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED and self.delegated_invites.all().count():
            delegated_invite = self.delegated_invites.all()[0]
            return u"<strong>{}: </strong>{} {} ({})".format(
                ugettext("Delegated"),
                delegated_invite.first_name,
                delegated_invite.last_name,
                delegated_invite.email,
            )
        return u""
    get_answers_cl.allow_tags = True
    get_answers_cl.admin_order_field = 'status'
    get_answers_cl.short_description = _("Answer")

    def get_note_cl(self):
        """
        :return: Html snippet about invite notes
        """
        notes = ''
        if self.note:
            notes += u"""<span title="{title}\n\n{content}"><img alt="free note" src="/static/event/img/note_free.png"/></span>""".format(
                title=ugettext('Note:'),
                content=self.note,
            )
        if self.internal_note:
            if self.note:
                notes += u'<br>'
            notes += u"<span title='{title}\n\n{content}'><img alt='free note' src='/static/event/img/note_private.png'/></span>".format(
                title=ugettext('Internal Note:'),
                content=self.internal_note,
            )
        if self.notes_from_phone_contact.all().count():
            notes += u'<span title="{title}\n\n{content}"><img alt="phone note" src="/static/event/img/phone.png"/> ({tot})</span>'.format(
                title=_('Last Phone Note:'),
                content=self.notes_from_phone_contact.all().order_by('-date_create')[0].text,
                tot=self.notes_from_phone_contact.all().count()
            )
        return notes
    get_note_cl.short_description = _("Note")
    get_note_cl.allow_tags = True

    def get_turn_cl(self):
        """
        :return: Invite turn preference
        """
        if self.turn_preference:
            return self.turn_preference.get_code_display()
        return ""
    get_turn_cl.admin_order_field = 'turn_preference__code'
    get_turn_cl.short_description = _("Turn")

    def get_transport_cl(self):
        """
        :return: Invite transport answer base on free/own/available choices selection
        """

        invite = Invite.objects.get(pk=self.pk)

        if invite.own_answers.exists():
            return unicode(_("Mezzi propri"))
        # elif invites.free_answer_count:
        #     free_answer = self.free_answers.first()
        #     if free_answer.answer.strip() != "":
        #         answer_detail = u"<span title='{}'><img src='/static/event/img/note.png' />Altro</span>".format(
        #             free_answer.answer
        #         )
        #     return answer_detail
        # Check for free transport answer
        free_answers = list(invite.free_answers.all())  # Fetch all and process in Python to avoid multiple DB hits
        if free_answers and free_answers[0].answer.strip() != "":
            answer_detail = u"<span title='{}'><img src='/static/event/img/note.png' />Altro</span>".format(
                free_answers[0].answer
            )
            return answer_detail

        # Check for travel option answer
        travel_option_answers = list(invite.travel_option_answers.all())  # Same fetching strategy as free_answers
        if travel_option_answers:
            travel_option_answer = travel_option_answers[0]
            return u"<span title='{}'><img src='/static/event/img/note.png' />&nbsp;{}</span> da: {}".format(
                html2text.html2text(self.get_summary_transport_detail() or ""),
                travel_option_answer.event_travel_option.name,
                travel_option_answer.event_travel_option.start_place.name
            )

        # elif invites.travel_option_answer_count:
        #     travel_option_answer = self.travel_option_answers.first()
        #     return u"<span title='{}'><img src='/static/event/img/note.png' />&nbsp;{}</span> da: {}".format(
        #         html2text.html2text(self.get_summary_transport_detail() or ""),
        #         travel_option_answer.event_travel_option.name,
        #         travel_option_answer.event_travel_option.start_place.name
        #     )
        # return ""
    get_transport_cl.short_description = _("Transport")
    get_transport_cl.allow_tags = True

    def get_documents_cl(self):
        """
        :return: Span html snippet about invite documents
        """
        documents = self.documents.all()
        if documents:
            titles = ", ".join(documents.values_list('name', flat=True))
            count = documents.count()
            return u"<span title='{}'><img src='/static/event/img/note.png' />&nbsp;<b>{}</b></span>".format(titles, count)
        return ""
    get_documents_cl.short_description = _("Documents")
    get_documents_cl.allow_tags = True

    def get_activities_cl(self):
        """
        :return: Html snippet about invite selected activities
        """
        activity_answers = []
        for activity_answer in self.activity_answers.all():
            question = activity_answer.question.question
            answer = activity_answer.answer
            text = "%s: %s" % (question, answer)
            activity_answers.append(text)
        return "<br>".join(activity_answers)
    get_activities_cl.short_description = _("Activities")
    get_activities_cl.allow_tags = True

    def get_activities_cl_short_name_list(self):
        questions = self.event.questions.all().order_by('ordering')

        activity_list = []

        # Get all answers related to this invite and relevant questions at once
        all_answers = ActivityAnswer.objects.filter(invite_id=self.id)
        answers_map = {answer.question_id: answer for answer in all_answers}

        for question in questions:
            activity = question.get_short_name() + ': '
            if question.id in answers_map:
                activity += answers_map[question.id].answer
                activity_list.append(activity)
            # try:
            #     # Fetch the related activity answer for the current invite using filter
            #     answer = question.question.activityanswer_set.filter(invite_id=self.id)[:1].get()
            #     activity += answer.answer
            #     activity_list.append(activity)
            # except ActivityAnswer.DoesNotExist:
            #     pass
        return activity_list
    get_activities_cl_short_name_list.short_description = _("Activities")
    get_activities_cl_short_name_list.allow_tags = True

    def get_web_view_cl(self):
        """
        :return: Html snippet used to display a link for the invite landing page preview
        """
        return """<a href=/event/%s/%s-%s/ target="_blank">LINK</a>""" % (
            self.step,
            self.make_base36(),
            self.make_token()
        )
    get_web_view_cl.short_description = _("Web")
    get_web_view_cl.allow_tags = True

    def get_email_view_cl(self):
        """
        :return: Html snippet used to display a link for the invite email preview
        """
        return """<a href=/event/%s/%s-%s/ target="_blank">LINK</a>""" % (
            self.step,
            self.make_base36(),
            self.make_token()
        )
    get_email_view_cl.short_description = _("Email")
    get_email_view_cl.allow_tags = True

    def get_txt_view_cl(self):
        """
        :return: Html snippet used to fisplay a link for the invite text preview
        """
        return """<a href=/event/%s/%s-%s/ target="_blank">LINK</a>""" % (
            self.step,
            self.make_base36(),
            self.make_token()
        )
    get_txt_view_cl.short_description = _("Text")
    get_txt_view_cl.allow_tags = True

    def get_status_cl(self):
        """
        :return: Invite status display
        """
        return "%s" % self.get_status_display()
    get_status_cl.short_description = _("Status")
    get_status_cl.admin_order_field = 'status'

    def get_step_cl(self):
        """
        :return: Invite step display
        """
        return "%s" % self.get_step_display()
    get_step_cl.short_description = _("Step")
    get_step_cl.admin_order_field = 'step'

    def get_count_sent_invite_cl(self):
        """
        :return: Count about invite sendings
        """
        return self.email_objects.filter(status=settings.MAILER_EMAIL_STATUS_SENT).count()
    get_count_sent_invite_cl.short_description = _('sents count')

    def get_count_sent_in_test_invite_cl(self):
        """
        :return: Count abount invite test sendings
        """
        return self.email_objects.filter(status=settings.MAILER_EMAIL_STATUS_SENT_TEST).count()
    get_count_sent_in_test_invite_cl.short_description = _('sents in test count')

    def get_link_managed_cl(self):
        """
        :return: Dajaxice clickable href about invite managed flag
        """
        return """
            <a id='publieditor_event_invite_managed_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.managed else 'no'))
    get_link_managed_cl.short_description = _('Managed')
    get_link_managed_cl.allow_tags = True

    def get_link_no_show_cl(self):
        """
        :return: Dajaxice clickable href about invite no_show flag
        """
        return """
            <a id='publieditor_event_invite_no_show_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.no_show else 'no'))
    get_link_no_show_cl.short_description = _('No show')
    get_link_no_show_cl.allow_tags = True

    def get_link_has_partecipated_cl(self):
        """
        :return: Html img representing invite has_partecipated flag
        """
        return u"<img alt='False' src='/static/admin/img/icon-%s.gif'>" % (
            'yes' if self.has_partecipated else 'no'
        )
    get_link_has_partecipated_cl.short_description = _('partecipated')
    get_link_has_partecipated_cl.allow_tags = True

    def get_home_address(self):
        return self.contact.get_home_address()
    get_home_address.short_description = _('Birth info')

    def get_license(self):
        template_string = '''
{% load i18n %}
{% trans 'license number' %}: <strong>{{ instance.contact.license_number }} </strong> <br>
{% trans 'license typologies list' %}: <strong>{{ instance.contact.get_license_types_list }} </strong> <br>
{% trans 'emitted by' %}: <strong>{{ instance.contact.license_emitted_by }} </strong> <br>
{% trans 'issue date' %}: <strong>{{ instance.contact.license_issue_date }} </strong> <br>
{% trans 'expiration date' %}: <strong>{{ instance.contact.license_expiration_date }} </strong>
'''
        template = Template(template_string)
        context = Context({"instance": self})
        return template.render(context)
    get_license.short_description = _('License')
    get_license.allow_tags = True

    def update_on_cron_sending(self):
        """
        Updates invite status when a sending is performed by a crona ction
        """
        self.sent = True
        if self.status == settings.PUBLIEDITOR_EVENT_INVITE_TO_SEND:
            self.status = settings.PUBLIEDITOR_EVENT_INVITE_SENT
        self.save()

    @property
    def steps(self):
        """
        Property representing available steps configured on event
        """
        steps = []
        event = self.event
        fields_map = [
            {'field': 'show_presentation', 'step': 'presentation', 'status': None},
            {'field': 'show_schedule', 'step': 'schedule', 'status': None},
            {'field': 'show_adhesion', 'step': 'adhesion', 'status': None},
            {'field': 'show_forwarding', 'step': 'forward_to', 'status': None},
            {'field': 'show_register', 'step': 'register', 'status': settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED},
            {'field': 'show_transport', 'step': 'transport', 'status': settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED},
            {'field': 'show_activity', 'step': 'activity', 'status': settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED},
            {'field': 'show_summary', 'step': 'summary', 'status': settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED},
            {'field': 'show_documents', 'step': 'documents', 'status': settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED},
            {'field': 'show_decline', 'step': 'decline', 'status': settings.PUBLIEDITOR_EVENT_INVITE_DECLINED},
            {'field': 'show_delegate', 'step': 'delegate', 'status': settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED},
            {'field': 'show_greetings', 'step': 'greetings', 'status': None},
        ]
        for field_map in fields_map:
            if getattr(event, field_map['field']):
                steps.append(dict(step=field_map['step'], enabled_status=field_map['status']))
        return steps

    @property
    def current_step(self):
        """
        :return: invite current step
        """
        for counter, step in enumerate(self.steps):
            if self.step == step['step']:
                return self.steps[counter]['step']
        return None

    @property
    def next_step(self):
        """
        :return: invite next step
        """
        for counter, step in enumerate(self.steps):
            if self.step == step['step'] and counter <= len(self.steps):
                next_steps = self.steps[counter+1:]
                for next_step in next_steps:
                    if next_step['enabled_status'] in [None, self.status]:
                        return next_step['step']
        return None

    @property
    def previous_step(self):
        """
        :return: invite previous step
        """
        for counter, step in enumerate(self.steps):
            if self.step == step['step'] and counter:
                prev_steps = self.steps[:counter][::-1]
                for prev_step in prev_steps:
                    if prev_step['enabled_status'] in [None, self.status]:
                        return prev_step['step']
        return None

    def update_step(self, step):
        """
        Updates invite step
        """
        self.step = step
        self.save()

    def get_step_translation(self, type):
        """
        Get the desired event step translation
        """
        translation = self.event.get_step_translation(type, self.contact)
        return translation

    def get_presentation(self):
        """
        :return: Event translation step
        """
        return self.get_step_translation("translations")

    def get_invite_email_from_header(self):
        """
        :return: Used by mailer to set the email sender display name
        """
        translation = self.get_presentation()
        return "%s <%s>" % (translation.email_from_name, translation.email_from)

    def get_rendered_presentation_body(self):
        """
        :return: Body of presentation step, used in email
        """
        translation = self.get_presentation()
        context_dict = dict(contact=self.contact, invite=self)
        body = translation.email_body
        if 'invite' in context_dict and hasattr(context_dict['invite'], "get_landingpage"):
            context_dict['invite'].get_landingpage = " > Aderisci online qui sotto"
        context = Context(context_dict)
        template = loader.get_template_from_string(body)
        ret = template.render(context)
        return ret if "<code></code>" not in ret else ret.replace("<code></code>", "")

    def get_rendered_presentation_web_body(self):
        """
        :return: Body of presentation step, used in landing page
        """
        translation = self.get_presentation()
        context_dict = dict(contact=self.contact, invite=self)
        body = translation.web_body
        if 'invite' in context_dict and hasattr(context_dict['invite'], "get_landingpage"):
            context_dict['invite'].get_landingpage = " > Aderisci online qui sotto"
        context = Context(context_dict)
        template = loader.get_template_from_string(body)
        ret = template.render(context)
        return ret if "<code></code>" not in ret else ret.replace("<code></code>", "")

    def get_schedule(self):
        """
        :return: Invite event schedule translation
        """
        return self.get_step_translation("schedule_translations")

    def get_activity_introduction(self):
        """
        :return: Invite event schedule translation
        """
        return self.get_step_translation("activities_introduction_translations")

    def get_adhesion(self):
        """
        :return: Invite event schedule translation
        """
        return self.get_step_translation("adhesion_translations")

    def get_rendered_privacy(self):
        """
        :return: Message text used on related bootstrp tooltip widget in register step form
        """
        privacy_acceptance_date = getattr(self.contact, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD)
        if privacy_acceptance_date and settings.PUBLIEDITOR_EVENT_CUSTOM_ACCEPTED_PRIVACY_MESSAGE:
            return "%s %s" % (
                "Privacy accettata in data %s. <br/>" % privacy_acceptance_date.strftime("%d/%m/%Y"),
                settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED_PRIVACY
            )
        privacy = self.get_step_translation("privacy_translations")
        context = dict(invite=self, contact=self.contact, event=self.event)
        body = privacy.body
        context = Context(context)
        template = loader.get_template_from_string(body)
        ret = template.render(context)
        return ret if "<code></code>" not in ret else ret.replace("<code></code>", "")

    def get_greetings(self):
        """
        :return: Invite event greetings translation
        """
        try:
            translation = self.event.greetings_translations.get(
                status=1, language=self.contact.language, invite_status=self.status
            )
        except ObjectDoesNotExist:
            translation = self.event.greetings_translations.get(
                status=1, language=self.event.default_language, invite_status=self.status
            )
        return translation

    def get_rendered_greetings_web_body(self):
        """
        :return: Body of greetings step, used in landing page
        """
        translation = self.get_greetings()
        context_dict = dict(contact=self.contact, invite=self)
        body = translation.body
        # if 'invite' in context_dict and hasattr(context_dict['invite'], "get_landingpage"):
        #     context_dict['invite'].get_landingpage = " > Aderisci online qui sotto"
        context = Context(context_dict)
        template = loader.get_template_from_string(body)
        ret = template.render(context)
        return ret if "<code></code>" not in ret else ret.replace("<code></code>", "")

    def memberships_inline(self):
        """
        It doesn't make sense, I think it culd be delete.
        Please check before deleting
        """
        memberships = ", ".join(["Publieditor"])
        return memberships
    memberships_inline.short_description = _("Memberships")

    def delete_all_answers(self, step=None):
        """
        Deletes all invite answers, used when the contact changes decision about its invite
        (ex. from accepted to declined)
        """
        self.register_answers.all().delete()
        self.transport_answers.all().delete()
        self.activity_answers.all().delete()
        self.decline_answers.all().delete()
        self.delegated_invites.all().delete()
        self.travel_option_answers.all().delete()
        self.turn_preference = None
        self.save()

    def delete_answers_from_register(self):
        """
        Deletes just decline and delegated answers when the contact decides
        to accept an already declined/delegated invite
        """
        self.decline_answers.all().delete()
        self.delegated_invites.all().delete()

    def get_admin_change_url(self):
        """
        :return: Href with link to invite change form
        """
        return '<a href="%s">%s</a>' % (
            reverse('admin:event_invite_change', args=[self.id]), self.__unicode__()
        )

    def send(self, test=False):
        """
        Send current invite, before sending a list of checks are performed when sending is not a test one
        :return: Boolean and message explaining action results
        """
        from publieditor.event.utils.mailer import InviteEmailObjectConstructor

        # Checks send conditions
        if test:
            if not self.contact.is_test_contact:
                return False, ugettext("Contact %s is not a test one" % self.contact.full_name)
        else:
            if self.status != settings.PUBLIEDITOR_EVENT_INVITE_PENDING:
                return False, ugettext("Invite for contact is not in pending status %s" % self.contact.full_name)
            if self.event.is_expired():
                return False, ugettext("Event %s is expired" % self.event.name)
            if self.event.is_over():
                return False, ugettext("Event %s is over" % self.event.name)
            if self.event.is_deadline_passed():
                return False, ugettext("Event %s deadline is passed" % self.event.name)
            if not self.contact.status:
                return False, ugettext("Contact %s is inactive" % self.contact.full_name)
            if not self.event.status:
                return False, ugettext("Event %s is inactive" % self.event.name)

        # Email creation
        email_constructor = InviteEmailObjectConstructor(self.contact, origin=self, is_test=test)
        mail_created, email_object, msg = email_constructor.create()
        if mail_created:
            if self.status == settings.PUBLIEDITOR_EVENT_INVITE_PENDING:
                self.status = settings.PUBLIEDITOR_EVENT_INVITE_TO_SEND
                self.save()
            if test:
                email_object_sender = EmailObjectSender(
                    email_object,
                    is_test=True,
                    from_address=self.get_invite_email_from_header(),
                )
                email_object_sender.send()

                if self.status == settings.PUBLIEDITOR_EVENT_INVITE_TO_SEND:
                    self.status = settings.PUBLIEDITOR_EVENT_INVITE_SENT
                    self.save()

                return True, ugettext("Invite sent")

            return True, ugettext("Invite email ready to send")
        return False, ugettext("Invite not sent")

    def resend(self, test=False):
        """
        Resend invites, a list of checks are performed before sending if not a test one,
        to enable sending invite must not be altered from its initial state
        :return:  Boolean and msg explaining actions results
        """
        from publieditor.event.utils.mailer import InviteEmailObjectConstructor

        # Checks resend conditions
        if test:
            if not self.contact.is_test_contact:
                return False, ugettext("Contact %s is not a test one" % self.contact.full_name)
        else:
            # if self.status != settings.PUBLIEDITOR_EVENT_INVITE_SENT:
            #     return False, _("Invite for contact %s is not in sent status" % self.contact.full_name)
            if self.event.is_expired():
                return False, ugettext("Event %s is expired" % self.event.name)
            if self.event.is_over():
                return False, ugettext("Event %s is over" % self.event.name)
            if self.event.is_deadline_passed():
                return False, ugettext("Event %s deadline is passed" % self.event.name)
            if not self.contact.status:
                return False, ugettext("Contact %s is inactive" % self.contact.full_name)
            if not self.event.status:
                return False, ugettext("Event %s is inactive" % self.event.name)

        # Email creation
        email_constructor = InviteEmailObjectConstructor(self.contact, origin=self, is_test=test)
        mail_created, email_object, msg = email_constructor.create()
        if mail_created:
            if test:
                email_object_sender = EmailObjectSender(
                    email_object,
                    is_test=True,
                    from_address=self.get_invite_email_from_header(),
                )
                email_object_sender.send()
                return True, ugettext("Invite resent")
            return True, ugettext("Invite email ready to resend")
        return False, ugettext("Invite not resent")

    def send_followup(self, test=False):
        """
        Sends invite folow ups, a list of check are performed before sending if not a test one.
        The invite must be accepted in order to receive follow up email.
        :return: Boolean and message explaining action results
        """
        from publieditor.event.utils.mailer import InviteFollowupEmailObjectConstructor

        # Checks send followup conditions
        if test:
            if not self.contact.is_test_contact:
                return False, ugettext("Contact %s is not a test one" % self.contact.full_name)
        else:
            if self.status != settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED:
                return False, ugettext("Invite is not in accepted status")
            if not self.contact.status:
                return False, ugettext("Contact is %s disabled" % self.contact.full_name)
            if not self.event.status:
                return False, ugettext("Event is %s disabled" % self.event.name)
            if not self.event.status:
                return False, ugettext("Event %s is inactive" % self.event.name)
        if not self.get_step_translation("followup_translations"):
            return False, ugettext("Follouwp translation not found")

        # Email creation
        email_constructor = InviteFollowupEmailObjectConstructor(self.contact, origin=self, is_test=test)
        mail_created, email_object, msg = email_constructor.create()
        if mail_created:
            if test:
                email_object_sender = EmailObjectSender(
                    email_object,
                    is_test=True,
                    from_address=self.get_invite_email_from_header(),
                )
                email_object_sender.send()
                return True, ugettext("Follow up send")
            return True, ugettext("Follow up ready to send")
        return False, ugettext("Follow up not sent")

    def send_documents(self, test=False):
        """
        Sends invite document email, a list of checks are performed before sending if not a test one.
        Invite must be accepted in order to receive document email (Eg they usually use it to attach flight tickets)
        :return: Boolean and msg explaining action results
        """
        from publieditor.event.utils.mailer import InviteDocumentsEmailObjectConstructor

        # Checks send followup conditions
        if test:
            if not self.contact.is_test_contact:
                return False, ugettext("Contact %s is not a test one" % self.contact.full_name)
        else:
            if self.status != settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED:
                return False, ugettext("Invite is not in accepted status")
            if not self.contact.status:
                return False, ugettext("Contact is %s disabled" % self.contact.full_name)
            if not self.event.status:
                return False, ugettext("Event is %s disabled" % self.event.name)
            if not self.event.status:
                return False, ugettext("Event %s is inactive" % self.event.name)
            if self.event.is_over():
                return False, ugettext("Event %s is over" % self.event.name)
        if not self.get_step_translation("document_email_translations"):
            return False, ugettext("Document translation not found")

        # Email creation
        email_constructor = InviteDocumentsEmailObjectConstructor(self.contact, origin=self, is_test=test)
        mail_created, email_object, msg = email_constructor.create()
        if mail_created:
            if test:
                email_object_sender = EmailObjectSender(
                    email_object,
                    is_test=True,
                    from_address=self.get_invite_email_from_header(),
                )
                email_object_sender.send()
                return True, ugettext("Documents email sent")
            return True, ugettext("Documents email ready to send")
        return False, ugettext("Documents email not sent")

    def get_code(self):
        """
        :return: Invite id
        """
        return "%s" % self.id

    def make_token(self):
        """
        :return: Token generator calculated on invite id
        """
        token_generator = TokenGenerator()
        return token_generator.make_token(self.get_code())

    def make_base36(self):
        """
        :return: Base36 of invite id
        """
        return int_to_base36(self.id)

    def check_token(self, token):
        """
        :return: True if token is acceptable esle False
        """
        token_generator = TokenGenerator()
        return token_generator.check_token(self.get_code(), token)

    def get_url_landingpage(self):
        """
        Please change it using django reverse funxtionality when refactoring
        :return: Url to invite landing page
        """
        return "/event/%s/%s-%s/" % (self.step, self.make_base36(), self.make_token())

    def get_absolute_url_landing_page(self):
        return u'{}/{}'.format(settings.SITE_DOMAIN, self.get_url_landingpage())

    def get_accept_from_email_url(self):
        """
        Accept url based on contact language
        """
        from django.utils import translation
        translation.activate(self.contact.language.code)

        return reverse("event:accept_from_email", kwargs={
            'uidb36': self.contact.make_base36(),
            'token': self.contact.token,
            'invite_id': self.id,
        })

    def get_decline_from_email_url(self):
        """
        Decline url based on contact language
        """
        from django.utils import translation
        translation.activate(self.contact.language.code)

        return reverse("event:decline_from_email", kwargs={
            'uidb36': self.contact.make_base36(),
            'token': self.contact.token,
            'invite_id': self.id,
        })

    def get_delegate_from_email_url(self):
        """
        Decline url based on contact language
        """
        from django.utils import translation
        translation.activate(self.contact.language.code)

        return reverse("event:delegate_from_email", kwargs={
            'uidb36': self.contact.make_base36(),
            'token': self.contact.token,
            'invite_id': self.id,
        })

    def get_forwarding_from_email_url(self):
        """
        Decline url based on contact language
        """
        from django.utils import translation
        translation.activate(self.contact.language.code)

        return reverse("event:forwarding_from_email", kwargs={
            'uidb36': self.contact.make_base36(),
            'token': self.contact.token,
            'invite_id': self.id,
        })

    def get_privacy_landingpage(self):
        """
        :return: Url to privacy landing page
        """
        return reverse("event:register", kwargs={
            'uidb36': self.contact.make_base36(),
            'token': self.contact.token
        })

    def get_register_and_redirect_url(self):
        """
        :return: url to register and redirect view, used when registration is performed starting from an invite
        """
        return reverse("event:register_and_redirect", kwargs={
            'uidb36': self.contact.make_base36(),
            'token': self.contact.token,
            'invite_id': self.id
        })

    def get_absolute_url(self):
        """
        :return: Url to read invite pass view, used to read invite qr code
        """
        return reverse("event:publieditor_event_read_invite_pass", args=(self.make_base36(), self.make_token()))

    def get_link_webpage_cl(self):
        """
        :return: Href link to invite access from admin changelist
        """
        if not self.event.is_privacy:
            return u"""<a href=/event/{}/{}-{}/ target="_blank">LINK</a>""".format(
                self.step,
                self.make_base36(),
                self.make_token()
            )
        return u""
    get_link_webpage_cl.short_description = _("Web page")
    get_link_webpage_cl.allow_tags = True

    def get_link_set_delegated(self):
        """
        :return: Href link to invite delegate vie by admin
        """
        if self.status not in [
            settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
            settings.PUBLIEDITOR_EVENT_INVITE_DECLINED,
            settings.PUBLIEDITOR_EVENT_INVITE_POLL_DONE,
        ]:
            return "<a href='{0}'>D</a>".format(
                reverse('admin:invite_set_delegated', kwargs={'invite_id': self.id})
            )
        return ""
    get_link_set_delegated.short_description = _("Del.")
    get_link_set_delegated.allow_tags = True

    def get_link_pdf_cl(self):
        """
        :return: Href link to invite pdf generation from admin
        """
        url = reverse("event:publieditor_event_get_invite_pdf", args=(self.make_base36(), self.make_token()))
        return u"""<a href=%s target="_blank">PDF</a>""" % url
    get_link_pdf_cl.short_description = _("PDF")
    get_link_pdf_cl.allow_tags = True

    def get_url_pollpage(self):
        """
        :return: Href link to invite poll page
        """
        return "/event/poll/%s-%s/" % (self.make_base36(), self.make_token())

    def get_pollpage(self):
        """
        :return: Href link to invite poll page from admin
        """
        return """<a href=/event/poll/%s-%s/ target="_blank">POLL</a>""" % (
            self.make_base36(),
            self.make_token()
        )
    get_pollpage.short_description = _("Poll page")
    get_pollpage.allow_tags = True

    def get_default_template_path(self):
        """
        :return: Get event invite template based on event setting
        """
        return settings.PUBLIEDITOR_EVENT_INVITE_TEMPLATES[self.event.landing_page_template]

    def get_default_title(self):
        """
        :return: Default translation title
        """
        translations = self.event.translations.filter(
            language=self.event.default_language,
        )
        if translations:
            return translations[0].title
        return self.event.translations.all()[0].title

    def get_transport_dict(self):
        """
        Used to collect transport answers
        """
        transport_dict = dict()
        for transport_answer in TransportAnswer.objects.filter(invite=self):
            transport = transport_answer.transport_detail
            if transport.turn not in transport_dict:
                transport_dict[transport.turn] = OrderedDict()
            if transport.start_place not in transport_dict[transport.turn]:
                transport_dict[transport.turn][transport.start_place] = OrderedDict()
            if transport.direction not in transport_dict[transport.turn][transport.start_place]:
                transport_dict[transport.turn][transport.start_place][transport.direction] = []
            transport_dict[transport.turn][transport.start_place][transport.direction].append(transport)
        return transport_dict

    def get_report(self, file_type='pdf'):
        """
        I think this is no longer used.
        Please check before deleting
        """
        oo = OOUtils()
        file_path = oo.save_pdf2(
            obj=self,
            template_file='appy/event_summary.odt',
            save_in=None,
            extra_context=dict(invite=self, contact=self.contact, event=self.event),
        )
        return file_path

    def get_ics_calendar(self):
        """
        :return: Invite ical to be used by the contact
        """
        from html2text import html2text
        event = self.event
        translation = self.get_presentation()
        cal = iCalendar()
        cal.add('method', 'PUBLISH')
        cal.add('version', '2.0')
        # Impostazioni generali event
        date_start = self.turn_preference.turn_start if self.turn_preference else event.event_date_start
        date_end = self.turn_preference.turn_end if self.turn_preference else event.event_date_end
        ical_event = iEvent()
        ical_event.add('dtstart', date_start)
        ical_event.add('dtend', date_end)
        ical_event.add('dtstamp', datetime.datetime.now())
        ical_event.add(
            'url',
            "{}://{}/{}{}".format(settings.SITE_PROTOCOL, settings.SITE_DOMAIN, self.contact.language.code, self.get_url_landingpage())
        )
        ical_event.add('summary', translation.title)
        ical_event.add('location', event.place)
        description = html2text(self.get_schedule().body).replace("  ", " ")
        description = description.replace("\n\n|\n\n", " | ")
        for i in range(3):
            description = description.replace("\n \n", "\n")
        if self.get_summary_transport_detail(False):
            description += u"\n{}: {}".format(
                _("Transports"),
                self.get_summary_transport_detail(False)
            )
        ical_event.add('description', description)
        ical_event.add('uid', '%d.event.events.%s' % (self.id, settings.SITE_DOMAIN))
        cal.add_component(ical_event)
        # Impostazioni alarm
        ical_alarm = iAlarm()
        ical_alarm.add('description', 'Reminder')
        ical_alarm.add('action', 'DISPLAY')
        ical_alarm.add('repeat', 2)
        ical_alarm.add('trigger', date_start-timedelta(hours=1))
        ical_alarm.add('duration', timedelta(minutes=15))
        cal.add_component(ical_alarm)
        return cal

    def get_ics_calendar_url(self):
        from django.utils import translation
        translation.activate(self.contact.language.code)

        token_generator = TokenGenerator()
        return u'{}{}'.format(
            settings.SITE_DOMAIN,
            reverse(
                'event:publieditor_event_ics_export',
                kwargs={
                    'uidb36': int_to_base36(self.id),
                    'token': token_generator.make_token(self.id),
                }
            )
        )

    def get_grouped_answers(self):
        """
        :return: Grouped answers by direction
        """
        grouped_invite_answer = dict()
        for event_question in self.event.questions.filter(question__typology__in=["A", "R"]).order_by('ordering'):
            answer = None
            if event_question.question.category not in grouped_invite_answer:
                grouped_invite_answer[event_question.question.category] = []
            if event_question.question.typology == "R":
                try:
                    answer = self.register_answers.get(question=event_question.question)
                except RegisterAnswer.DoesNotExist:
                    continue
            elif event_question.question.typology == "A":
                try:
                    answer = self.activity_answers.get(question=event_question.question)
                except ActivityAnswer.DoesNotExist:
                    continue
            if answer:
                grouped_invite_answer[event_question.question.category].append(answer)
        return grouped_invite_answer

    def get_transport_details_dict(self):
        """
        :return: Dict representing available transport details by direction and type
        """
        # Transport Answer
        transport_answers = self.transport_answers.all()
        time_format = "%H:%M"
        date_format = "%A %e %B %Y"
        departure_details = []
        return_details = []
        departure_count = 1
        return_count = 1
        departure_date = ''
        arrival_date = ''
        turn = ''
        start_place = ''
        for detail in transport_answers:
            app = dict()
            if detail.transport_detail.category == "flightdetail":
                app['departure_place'] = detail.transport_detail.flightdetail.flight.dep_airport
                app['arrival_place'] = detail.transport_detail.flightdetail.flight.arr_airport
                app['number'] = detail.transport_detail.flightdetail.flight.number
                app['departure'] = detail.transport_detail.flightdetail.departure.strftime(time_format)
                app['arrival'] = detail.transport_detail.flightdetail.arrival.strftime(time_format)
                if not departure_date:
                    departure_date = detail.transport_detail.flightdetail.departure.strftime(date_format)
                if not arrival_date:
                    arrival_date = detail.transport_detail.flightdetail.arrival.strftime(date_format)
            if detail.transport_detail.category == "traindetail":
                app['departure_place'] = detail.transport_detail.traindetail.train.dep_station
                app['arrival_place'] = detail.transport_detail.traindetail.train.arr_station
                app['number'] = detail.transport_detail.traindetail.flight.number
                app['departure'] = detail.transport_detail.traindetail.departure.strftime(time_format)
                app['arrival'] = detail.transport_detail.traindetail.arrival.strftime(time_format)
                if not departure_date:
                    departure_date = detail.transport_detail.traindetail.departure.strftime(date_format)
                if not arrival_date:
                    arrival_date = detail.transport_detail.traindetail.arrival.strftime(date_format)
            if detail.transport_detail.direction == "d":
                app['seq'] = departure_count
                departure_count += 1
                departure_details.append(app)
            if detail.transport_detail.direction == "r":
                app['seq'] = return_count
                return_count += 1
                return_details.append(app)
            turn = detail.transport_detail.turn
            start_place = detail.transport_detail.start_place
        transport_detail_dict = {
            "departure_details": departure_details,
            "return_details": return_details,
            "turn": turn,
            "start_place": start_place,
        }
        return transport_detail_dict

    def send_on_accept_email(self):
        """
        :return: Send email to backoffice when contact accepts an invite
        """
        text_template_path = "event/emails/accept_invite.txt"
        html_template_path = "event/emails/accept_invite.html"
        subject = "Invito accettato"
        html_template = get_template(html_template_path)
        ctx_dict = Context({'invite': self, 'contact': self.contact, 'event': self.event})
        from_email = settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM
        to = [settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO]
        text_message = render_to_string(text_template_path, ctx_dict)
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(
            subject, text_message, from_email, to,
            bcc=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN,
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_on_change_decision_email(self, status):
        """
        :return: Sends email to backoffice when contact changes its decision about the invite
        """
        text_template_path = "event/emails/change_decision.txt"
        html_template_path = "event/emails/change_decision.html"
        subject = "Decisione partecipazione invito modificata"
        html_template = get_template(html_template_path)
        ctx_dict = Context({
            'invite': self, 'contact': self.contact, 'event': self.event,
            'status': status,
        })
        from_email = settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM
        to = [settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO]
        text_message = render_to_string(text_template_path, ctx_dict)
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(
            subject, text_message, from_email, to,
            bcc=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN,
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_on_confirm_email(self):
        """
        :return: Sends email to backoffice when contact confirm an event,
            this is done when contact completes summary step
        """
        text_template_path = "event/emails/confirm_invite.txt"
        html_template_path = "event/emails/confirm_invite.html"
        subject = "Invito confermato"
        html_template = get_template(html_template_path)
        ctx_dict = Context({
            'invite': self, 'contact': self.contact, 'event': self.event
        })
        from_email = settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM
        to = [settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO]
        text_message = render_to_string(text_template_path, ctx_dict)
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(
            subject, text_message, from_email, to,
            bcc=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN,
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_on_decline_email(self):
        """
        :return: Sends email to backoffice when contact declines invite
        """
        text_template_path = "event/emails/decline_invite.txt"
        html_template_path = "event/emails/decline_invite.html"
        subject = "Invito declinato"
        html_template = get_template(html_template_path)
        ctx_dict = Context({
            'invite': self, 'contact': self.contact, 'event': self.event,
            'answers': self.decline_answers.all()
        })
        from_email = settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM
        to = [settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO]
        text_message = render_to_string(text_template_path, ctx_dict)
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(
            subject, text_message, from_email, to,
            bcc=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN,
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_on_delegate_email(self, contact_differs=False):
        """
        :return: Sends email to backoffice when contact delegates the invite
        """
        text_template_path = "event/emails/delegate_invite.txt"
        html_template_path = "event/emails/delegate_invite.html"
        event = self.event
        contact = self.contact
        delegated_contact = self.delegated_invites.all()[0]
        existing_contact = None
        if Contact.objects.filter(email__iexact=delegated_contact.email).count():
            existing_contact = Contact.objects.filter(email__iexact=delegated_contact.email)[0]
        subject = "Invito delegato"
        html_template = get_template(html_template_path)
        ctx_dict = Context({
            'invite': self,
            'contact': contact,
            'event': event,
            'delegated_contact': delegated_contact,
            'existing_contact': existing_contact,
            'contact_differs': contact_differs,
            'contact_url': "%s://%s/admin/event/contact/%s" % (
                settings.SITE_PROTOCOL,
                settings.SITE_DOMAIN,
                contact.id
            ),
            'delegated_contact_url': "%s://%s/admin/event/delegatedcontact/?invite__event=%s&invited=%s" % (
                settings.SITE_PROTOCOL,
                settings.SITE_DOMAIN,
                delegated_contact.invite.event.id,
                delegated_contact.invited.id,
            )
        })
        from_email = settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM
        to = [settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO]
        text_message = render_to_string(text_template_path, ctx_dict)
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(
            subject, text_message, from_email, to,
            bcc=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN,
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_forward_approval(self):
        """
        :return: Sends email to backoffice when contact delegates the invite
        """
        text_template_path = "event/emails/forward_approval.txt"
        html_template_path = "event/emails/forward_approval.html"
        subject = "Ci sono inviti da approvare per l'evento {}".format(self.event)
        html_template = get_template(html_template_path)
        ctx_dict = Context({
            'invite_forwarded': self,
            'invites_change_list_url': '{protocol}://{site_domain}{url}'.format(
                protocol=settings.SITE_PROTOCOL,
                site_domain=settings.SITE_DOMAIN,
                url=self.event.get_admin_invites_change_list_url()
            )
        })
        from_email = settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM
        to = [settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO]
        text_message = render_to_string(text_template_path, ctx_dict)
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(
            subject, text_message, from_email, to,
            bcc=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN,
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_email_on_invite_email_opening(self, mailing_status):
        """
        :return: Sends email to backoffice when invite email was opend by the contact
        """
        text_template_path = "event/emails/invite_email_opening.txt"
        html_template_path = "event/emails/invite_email_opening.html"
        subject = "Email invito aperta"
        html_template = get_template(html_template_path)
        ctx_dict = Context({
            'invite': self, 'contact': self.contact, 'event': self.event, 'mailing_status': mailing_status,
        })
        from_email = settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM
        to = [settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO]
        text_message = render_to_string(text_template_path, ctx_dict)
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(
            subject, text_message, from_email, to,
            bcc=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN,
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_qrcode_email(self):
        """
        :return: Sends email to contact containing qr code
        """
        template = "event/emails/invite_qrcode.html"
        subject = "Qr code personale per partecipazione ad evento %s" % self.event.name
        html_template = get_template(template)
        ctx_dict = Context({'invite': self})
        from_email = settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM
        to = [settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO]
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(
            subject, "", from_email, to,
            bcc=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_TO_CCN,
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_summary_email(self, contact=None):
        contact = contact if contact else self.contact
        to = [contact.email]
        cc = [email.email_details() for email in contact.cc_emails.all()]

        base_url = "{}://{}".format(
            settings.EVENT_INVITE_TRIUMPH_SITE_PROTOCOL, settings.EVENT_INVITE_TRIUMPH_SITE_DOMAIN
        )
        context = Context({
            'invite': self,
            'base_url': base_url,
        })

        subject_template = Template('Grazie {{ invite.contact.first_name }} per esserti registrato')
        subject = subject_template.render(context)

        html_message = render_to_string('event/emails/triumph_summary.html', context)
        text_message = html_message
        msg = EmailMultiAlternatives(
            subject=subject, body=text_message, from_email=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM_TRIUMPH,
            to=to, cc=cc,
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_confirmation_email(self, contact=None):
        contact = contact if contact else self.contact
        to = [contact.email]
        cc = [email.email_details() for email in contact.cc_emails.all()]

        base_url = "{}://{}".format(settings.EVENT_INVITE_TRIUMPH_SITE_PROTOCOL, settings.EVENT_INVITE_TRIUMPH_SITE_DOMAIN)
        context = Context({
            'invite': self,
            'base_url': base_url,
            'url_landingpage': self.get_url_landingpage(),
        })

        subject_template = Template('Conferma la tua email per procedere con la registrazione')
        subject = subject_template.render(context)

        html_message = render_to_string('event/emails/triumph_confirm_email.html', context)
        text_message = html_message
        msg = EmailMultiAlternatives(
            subject=subject, body=text_message,
            from_email=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM_TRIUMPH, to=to, cc=cc
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_cdu_email(self, file_name, contact=None, delete_attachment_from_tmp=False):

        contact = contact if contact else self.contact

        to = [contact.email]
        cc = [email.email_details() for email in contact.cc_emails.all()]

        base_url = "{}://{}".format(settings.EVENT_INVITE_TRIUMPH_SITE_PROTOCOL, settings.EVENT_INVITE_TRIUMPH_SITE_DOMAIN)
        context = Context({
            'invite': self,
            'base_url': base_url,
        })

        subject_template = Template("Triumph tour 2018, documento di comodato d'uso")
        subject = subject_template.render(context)

        html_message = render_to_string('event/emails/triumph_cdu_email.html', context)
        text_message = html_message
        msg = EmailMultiAlternatives(
            subject=subject, body=text_message,
            from_email=settings.PUBLIEDITOR_EVENT_INVITE_ACTION_EMAIL_FROM_TRIUMPH, to=to, cc=cc
        )

        attachment = open(file_name, 'rb')
        msg.attach('comodato_uso.pdf', attachment.read(), 'application/pdf')
        msg.attach_alternative(html_message, "text/html")
        msg.send()
        if delete_attachment_from_tmp:
           os.remove(file_name)

    def generate_cdu_file(self):
        from publieditor.event.util import create_cdu_pdf
        return create_cdu_pdf(invite=self)

    def get_all_answers(self, is_html=True):
        """
        :return: List of dict representing invite answers, used in pdf generation
        """
        contact = self.contact
        event = self.event
        tables = []

        # Dati anagrafici
        rows = [{'title': ugettext("Full name"), 'value': "%s < %s >" % (contact.full_name, contact.email)}]
        table_dati_anagrafici = {'title': ugettext("Dati anagrafici"), 'rows': rows}
        tables.append(table_dati_anagrafici)

        # Telefono
        if event.ask_phone:
            phones_queryset = self.contact.phones.all()
            if event.phone_question_type == "default":
                contact_default_phone = self.contact.default_phone
                if contact_default_phone:
                    phones_queryset = phones_queryset.filter(id=contact_default_phone.id)
                else:
                    phones_queryset = phones_queryset.none()
            rows = []
            for phone in phones_queryset:
                if phone.area_code:
                    phone_display = "%s %s" % (phone.area_code.code, phone.number)
                else:
                    phone_display = "%s" % phone.number

                rows.append({'title': phone.get_phone_type_display(), 'value': phone_display})
            if rows:
                table_phone = {'title': ugettext("Phones"), 'rows': rows}
                tables.append(table_phone)

        # Indirizzo
        if event.ask_address:
            addresses_queryset = self.contact.addresses.all()
            if event.address_question_type == "default":
                contact_default_address = self.contact.default_address
                if contact_default_address:
                    addresses_queryset = addresses_queryset.filter(id=contact_default_address.id)
                else:
                    addresses_queryset = addresses_queryset.none()
            rows = []
            for address in addresses_queryset:
                title = " - ".join([address_scope.name for address_scope in address.address_scopes.all()])
                value = "%s %s %s - %s %s (%s) - %s" % (
                    address.get_address_type_display(),
                    address.name, address.number,
                    address.locality,  address.zip_code,
                    address.province, address.country
                )
                rows.append({'title': title, 'value': value})
            if rows:
                table_phone = {'title': ugettext("Addresses"), 'rows': rows}
                tables.append(table_phone)

        # Carta identità
        if event.ask_id_card:
            rows = []
            if contact.id_card_number:
                rows.append({'title': ugettext("Number"), 'value': contact.id_card_number})
            if contact.id_card_issue_date:
                rows.append({'title': ugettext("Issue date"), 'value': _date(contact.id_card_issue_date, "d/m/Y")})
            if contact.id_card_expiration_date:
                rows.append({'title': ugettext("Expiration date"), 'value': _date(contact.id_card_expiration_date, "d/m/Y")})

            if contact.id_card_scan and is_html:
                rows.append({'title': ugettext("Scan"), 'value': u"<img height=100 src='%s'>" % (
                    "%s://%s%s" % (settings.SITE_PROTOCOL, settings.SITE_DOMAIN, contact.id_card_scan.url),
                )})
            if rows:
                table_id_card = {'title': ugettext("Documents: Id card"), 'rows': rows}
                tables.append(table_id_card)

        # Passaporto
        if event.ask_passport:
            rows = []
            if contact.passport_number:
                rows.append({'title': ugettext("Number"), 'value': contact.passport_number})
            if contact.passport_issue_date:
                rows.append({'title': ugettext("Issue date"), 'value': _date(contact.passport_issue_date, "d/m/Y")})
            if contact.passport_expiration_date:
                rows.append({'title': ugettext("Expiration date"), 'value': _date(contact.passport_expiration_date, "d/m/Y")})
            if contact.passport_scan and is_html:
                rows.append({'title': ugettext("Scan"), 'value': u"<img height=100 src='%s'>" % (
                    "%s://%s%s" % (settings.SITE_PROTOCOL, settings.SITE_DOMAIN, contact.passport_scan.url),
                )})
            if rows:
                table_passport = {'title': ugettext("Documents: Passport"), 'rows': rows}
                tables.append(table_passport)

        # Patente
        if event.ask_license:
            rows = []
            if contact.license_number:
                rows.append({'title': ugettext("Number"), 'value': contact.license_number})
            if contact.license_issue_date:
                rows.append({'title': ugettext("Issue date"), 'value': _date(contact.license_issue_date, "d/m/Y")})
            if contact.license_expiration_date:
                rows.append({'title': ugettext("Expiration date"), 'value': _date(contact.license_expiration_date, "d/m/Y")})
            if contact.license_emitted_by:
                rows.append({'title': ugettext("Emitted by"), 'value': contact.license_emitted_by})
            if contact.license_types:
                value = " - ".join([license_type.name for license_type in contact.license_types.all()])
                rows.append({'title': ugettext("Types"), 'value': value})
            if contact.license_scan and is_html:
                rows.append({'title': ugettext("Scan"), 'value': u"<img height=100 src='%s'>" % (
                    "%s://%s%s" % (settings.SITE_PROTOCOL, settings.SITE_DOMAIN, contact.license_scan.url),
                )})
            if rows:
                table_license = {'title': ugettext("Documents: License"), 'rows': rows}
                tables.append(table_license)

        # Tessera giornalisti
        if event.ask_press_card_number:
            rows = []
            if contact.elenco:
                rows.append({'title': ugettext("Elenco"), 'value': contact.get_elenco_display()})
            if contact.press_card_number:
                rows.append({'title': ugettext("Number"), 'value': contact.press_card_number})
            if rows:
                table_press_card = {'title': ugettext("Documents: Professional card"), 'rows': rows}
                tables.append(table_press_card)

        # Frequent flyer
        if event.ask_frequent_flyer:
            cards_queryset = self.contact.cards.all()
            if event.frequent_flyer_question_type == "default":
                contact_default_card = self.contact.default_card
                if contact_default_card:
                    cards_queryset = cards_queryset.filter(id=contact_default_card.id)
                else:
                    cards_queryset = cards_queryset.none()
            rows = []
            for card in cards_queryset:
                app = "%s" % card.number
                if card.expiration_date:
                    app += " (%s %s)" % (ugettext("Exp."), _date(card.expiration_date, "d/m/Y"))
                rows.append({'title': card.airline, 'value': app})
            if rows:
                table_phone = {'title': ugettext("Frequent flyer cards"), 'rows': rows}
                tables.append(table_phone)

        # Abbigliamento
        if event.ask_clothing:
            rows = []
            if contact.shirt_size:
                rows.append({'title': ugettext("Shirt"), 'value': contact.get_shirt_size_display()})
            if contact.sweater_size:
                rows.append({'title': ugettext("Sweater"), 'value': contact.get_sweater_size_display()})
            if contact.shoes_size:
                rows.append({'title': ugettext("Shoes"), 'value': contact.shoes_size})
            if rows:
                table_clothing = {'title': ugettext("Clothing: sizes"), 'rows': rows}
                tables.append(table_clothing)

        # Misure corporee
        if event.ask_body_measurements:
            rows = []
            if contact.height:
                rows.append({'title': ugettext("Height"), 'value': contact.height})
            if contact.weight:
                rows.append({'title': ugettext("Weight"), 'value': contact.weight})
            if rows:
                table_body_measurements = {'title': ugettext("Body measurements"), 'rows': rows}
                tables.append(table_body_measurements)

        # Sci
        if event.ask_ski:
            rows = []
            if contact.ski:
                rows.append({'title': ugettext("Ski"), 'value': contact.get_ski_display()})
            if contact.ski_level:
                rows.append({'title': ugettext("Ski level"), 'value': contact.get_ski_level_display()})
            if contact.ski_equipment:
                rows.append({'title': ugettext("Ski equipment"), 'value': contact.ski_equipment})
            if contact.ski_boots_size:
                rows.append({'title': ugettext("Ski boots size"), 'value': contact.ski_boots_size})
            if rows:
                table_ski = {'title': ugettext("Ski"), 'rows': rows}
                tables.append(table_ski)

        # Misure corporee
        if event.ask_food_hotel_preferences:
            rows = []
            if contact.smoking:
                rows.append({'title': ugettext("Smoking"), 'value': contact.get_smoking_display()})
            if contact.food:
                rows.append({'title': ugettext("Food"), 'value': contact.get_food_display()})
            if contact.food_note:
                rows.append({'title': ugettext("Food note"), 'value': contact.food_note})
            if rows:
                table_body_measurements = {'title': ugettext("Accommodation and food preferences"), 'rows': rows}
                tables.append(table_body_measurements)

        # Trasporto
        if event.show_transport:
            rows = []
            if self.event.turns.all().count() > 1:
                rows.append({'title': ugettext("Turn"), 'value': self.turn_preference})
            rows.append({'title': ugettext("Travel option"), 'value': self.get_summary_transport_detail(is_html)})
            table_transport = {'title': ugettext("Transport"), 'rows': rows}
            tables.append(table_transport)

        # Attività
        if event.show_activity:
            rows = []
            for activity_answer in self.activity_answers.all():
                rows.append({'title': activity_answer.question.question, 'value': activity_answer.answer})
            if rows:
                table_activity = {'title': ugettext("Activities"), 'rows': rows}
                tables.append(table_activity)

        # Note invito
        if self.note:
            rows = []
            rows.append({'title': "", 'value': self.note})
            table_note = {'title': ugettext("General note"), 'rows': rows}
            tables.append(table_note)

        return tables

    def get_departure_details(self):
        if self.own_answers.all().count():
            return ugettext("I'll come by myself")
        elif self.free_answers.all().count():
            free_answer = self.free_answers.all()[0]
            return free_answer.answer
        elif self.travel_option_answers.all().count():
            transports = []
            travel_option_answer = self.travel_option_answers.all()[0]
            event_travel_option = travel_option_answer.event_travel_option
            departure_details = event_travel_option.get_transport_details_dict().get('d', [])
            for departure_detail in departure_details:
                transports.append(get_transport_detail(departure_detail))
            return u"<br>".join(transports)
        return u""
    get_departure_details.short_description = _('Departures details')

    def get_return_details(self):
        if self.travel_option_answers.all().count():
            transports = []
            travel_option_answer = self.travel_option_answers.all()[0]
            event_travel_option = travel_option_answer.event_travel_option
            return_details = event_travel_option.get_transport_details_dict().get('r', [])
            for return_detail in return_details:
                transports.append(get_transport_detail(return_detail))
            return u"<br>".join(transports)
        return u""
    get_return_details.short_description = _('Returns details')

    def get_max_forwards(self):
        if self.event.enable_forwarding:
            return int(self.event.max_limit_forwarders)
        return 0

    def get_forwards(self):
        return self.forwards.filter(sender=self.contact)

    def get_forwards_count(self):
        return self.get_forwards().count()

    def get_sent_forwards(self):
        return self.get_forwards().filter(is_sent=True)

    def get_sent_forwards_count(self):
        return self.get_sent_forwards().count()

    def get_unsent_forwards(self):
        return self.get_forwards().filter(is_sent=False)

    def get_unsent_forwards_count(self):
        return self.get_forwards().count()

    def get_cdu_file_path(self):
        """
        TODO: Valutare la soluzuine per salvare il cdu
        """
        base_path = settings.MEDIA_ROOT + '/' + settings.PUBLIEDITOR_EVENT_CDU_PATH
        if not os.path.exists(base_path):
            os.makedirs(base_path)
        return settings.MEDIA_ROOT + '/' + settings.PUBLIEDITOR_EVENT_CDU_PATH + self.make_token() + '.pdf'

    def get_cdu_file_media_path(self):
        """
        TODO: Valutare la soluzuine per salvare il cdu
        """
        return settings.MEDIA_URL + settings.PUBLIEDITOR_EVENT_CDU_PATH + self.make_token() + '.pdf'


class NoteFromPhoneContact(DateModel, UserModel, OrderedModel):
    """
    Model used to store logs notes retrieved from publieditor back office
    """
    invite = models.ForeignKey(
        Invite,
        related_name='notes_from_phone_contact',
        verbose_name=_('note from phone contact'),
    )
    text = models.TextField(
        verbose_name=_('text')
    )
    call_result = models.CharField(
        choices=settings.PUBLIEDITOR_EVENT_PHONE_CONTACT_RESULT,
        max_length=10,
        default='tc',
        verbose_name=_('call result'),
    )

    class Meta(object):
        verbose_name = _('Note From Phone Contact')
        verbose_name_plural = _('Notes From Phone Contact')
        ordering = ('-id',)

    def __unicode__(self):
        return u"Note of {}".format(self.invite.contact)

    def save(self, *args, **kwargs):
        super(NoteFromPhoneContact, self).save(*args, **kwargs)
        last_note_from_call = self.invite.notes_from_phone_contact.first()
        if last_note_from_call.call_result != self.invite.call_status:
            self.invite.call_status = self.call_result
            self.invite.save()

    def delete(self, using=None):
        super(NoteFromPhoneContact, self).delete()
        last_note_from_call = self.invite.notes_from_phone_contact.first()
        if last_note_from_call:
            if last_note_from_call.call_result != self.invite.call_status:
                self.invite.call_status = last_note_from_call.call_result
                self.invite.save()
        else:
            self.invite.call_status = 'tc'
            self.invite.save()


class InviteEmailObject(EmailObject):
    """
    InviteEmailObject: we need to subclass EmailObject to store the related invite email
    """
    invite = models.ForeignKey(
        Invite,
        verbose_name=_("Invite"),
        related_name='email_objects',
    )
    is_test = models.BooleanField(
        default=False,
        verbose_name=_('is test'),
    )

    def get_custom_context(self):
        return dict(
            url_landing_page=self.invite.get_url_landingpage(),
        )

    @property
    def can_send(self):
        """
        :return: True if invite can be sent else False
        """
        event = self.invite.event
        now = datetime.datetime.now()
        can_send = True
        if event.invites_send_from and event.invites_send_to:
            can_send = bool(event.invites_send_from <= now <= event.invites_send_to)
        elif event.invites_send_from and not event.invites_send_to:
            can_send = bool(event.invites_send_from <= now)
        elif not event.invites_send_from and event.invites_send_to:
            can_send = bool(now <= event.invites_send_to)
        return can_send

    @property
    def contact(self):
        """
        :return: email invite contact
        """
        return self.invite.contact.full_name

    @property
    def event(self):
        """
        :return: email invite event
        """
        return self.invite.event.name

    def save(self, *args, **kw):
        """
        Updates type field to add search functionality in generic EmailObject
        """
        self.type = "inviteemailobject"
        super(InviteEmailObject, self).save(*args, **kw)

    class Meta:
        verbose_name = _('invite email')
        verbose_name_plural = _('invite email')


class FollowUpEmailObject(EmailObject):
    """
    FollowUpEmailObject: we need to subclass EmailObject to store the related follow up email
    """
    invite = models.ForeignKey(
        Invite,
        verbose_name=_("Invite"),
        related_name='follow_up_email_objects',
    )
    is_test = models.BooleanField(
        default=False,
        verbose_name=_('is test'),
    )

    def get_custom_context(self):
        return dict(
            url_landing_page=self.invite.get_url_pollpage(),
        )

    def save(self, *args, **kw):
        """
        Updates type field to add search functionality in generic EmailObject
        """
        self.type = "followupemailobject"
        super(FollowUpEmailObject, self).save(*args, **kw)

    @property
    def contact(self):
        """
        :return: email invite contact
        """
        return self.invite.contact.full_name

    @property
    def event(self):
        """
        :return: email invite event
        """
        return self.invite.event.name


class DocumentsEmailObject(EmailObject):
    """
    DocumentsEmailObject: we need to subclass EmailObject to store the related email object and invite
    """
    invite = models.ForeignKey(
        Invite,
        verbose_name=_("Invite"),
        related_name='documents_email_objects',
    )
    is_test = models.BooleanField(
        default=False,
        verbose_name=_('is test'),
    )

    def get_custom_context(self):
        return dict(
            url_landing_page=self.invite.get_url_pollpage(),
        )

    def save(self, *args, **kw):
        """
        Updates type field to add search functionality in generic EmailObject
        """
        self.type = "documentsemailobject"
        super(DocumentsEmailObject, self).save(*args, **kw)

    @property
    def contact(self):
        """
        :return: email invite contact
        """
        return self.invite.contact.full_name

    @property
    def event(self):
        """
        :return: email invite event
        """
        return self.invite.event.name


class InviteProxy(Invite):
    """
    Simple invite proxy, I don't really know what this is used for
    """
    class Meta:
        verbose_name = _('invite')
        verbose_name_plural = _('invites')
        proxy = True


class AcceptedInviteManager(models.Manager):
    """
    Simple manager to return only accepted invites
    """
    def get_query_set(self):
        return super(AcceptedInviteManager, self).get_query_set().filter(
            status=settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED
        )


class ConfirmedInviteManager(models.Manager):
    """
    Simple manager to return only confirmed invites
    """
    def get_query_set(self):
        return super(ConfirmedInviteManager, self).get_query_set().filter(
            status__in=[
                settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
                settings.PUBLIEDITOR_EVENT_INVITE_POLL_DONE
            ],
        )


class TravelAgencyInvite(Invite):
    """
    Simple proxy to be used when logged user is a travel agency
    """
    objects = ConfirmedInviteManager()

    class Meta:
        proxy = True
        verbose_name = _('travel agency invite')
        verbose_name_plural = _('travel agency invites')

    def get_contact_data_cl(self):
        """
        Overwrites method instead than standard Invite one, we don't want clickable to change form here
        :return: Contact data
        """
        email = "<a href='mailto:%s'>%s</a>" % (self.contact.email, self.contact.email)
        email_cc_html = ""
        for email_obj in self.contact.get_all_email_cc():
            email_obj_email = email_obj.contact_cc.email if email_obj.contact_cc else email_obj.email
            email_cc_html += "<br>&nbsp;&nbsp;cc: <a href='mailto:%s'>%s</a>" % (
                email_obj_email,
                email_obj_email,
            )
        return u"{} {} {}".format(
            self.contact.invite_name,
            email,
            email_cc_html
        )
    get_contact_data_cl.short_description = _("Contact")
    get_contact_data_cl.allow_tags = True
    get_contact_data_cl.admin_order_field = 'contact__last_name'

    def get_documents_cl(self):
        """
        Filters invite documents taking only ones create from a travel agency
        :return: Span html snippet with invite documents
        """
        documents = self.documents.filter(created_from_travel_agency=True)
        if documents:
            titles = ", ".join(documents.values_list('name', flat=True))
            count = documents.count()
            return u"<span title='{}'><img src='/static/event/img/note.png' />&nbsp;<b>{}</b></span>".format(titles, count)
        return ""
    get_documents_cl.short_description = _("Documents")
    get_documents_cl.allow_tags = True


class LogisticAgencyInvite(Invite):
    """
    Simple proxy to be used when logged user is a logistic agency
    """
    # objects = AcceptedInviteManager()

    class Meta:
        proxy = True
        verbose_name = _('logistic agency invite')
        verbose_name_plural = _('logistic agency invites')

    def get_contact_data_cl(self):
        """
        Overwrites method instead than standard Invite one, we don't want clickable to change form here
        :return: Contact data
        """
        email = "<a href='mailto:%s'>%s</a>" % (self.contact.email, self.contact.email)
        email_cc_html = ""
        for email_obj in self.contact.get_all_email_cc():
            email_obj_email = email_obj.contact_cc.email if email_obj.contact_cc else email_obj.email
            email_cc_html += "<br>&nbsp;&nbsp;cc: <a href='mailto:%s'>%s</a>" % (
                email_obj_email,
                email_obj_email,
            )
        return u"{} {} {}".format(
            self.contact.invite_name,
            email,
            email_cc_html
        )
    get_contact_data_cl.short_description = _("Contact")
    get_contact_data_cl.allow_tags = True
    get_contact_data_cl.admin_order_field = 'contact__last_name'

    def get_documents_cl(self):
        """
        Filters invite documents taking only ones created from a logistic agency
        :return: Span html snippet with invite documents
        """
        documents = self.documents.filter(created_from_logistic_agency=True)
        if documents:
            titles = ", ".join(documents.values_list('name', flat=True))
            count = documents.count()
            return u"<span title='{}'><img src='/static/event/img/note.png' />&nbsp;<b>{}</b></span>".format(titles, count)
        return ""
    get_documents_cl.short_description = _("Documents")
    get_documents_cl.allow_tags = True


class HostessInvite(Invite):
    """
    Simple proxy to be used when logged user is a hostess
    """
    # objects = AcceptedInviteManager()

    class Meta:
        proxy = True
        verbose_name = _('hostess invite')
        verbose_name_plural = _('hostess invites')

    def get_contact_data_cl(self):
        """
        Overwrites method instead than standard Invite one, we don't want clickable to change form here
        :return: Contact data
        """
        email = "<a href='mailto:%s'>%s</a>" % (self.contact.email, self.contact.email)
        email_cc_html = ""
        for email_obj in self.contact.get_all_email_cc():
            email_obj_email = email_obj.contact_cc.email if email_obj.contact_cc else email_obj.email
            email_cc_html += "<br>&nbsp;&nbsp;cc: <a href='mailto:%s'>%s</a>" % (
                email_obj_email,
                email_obj_email,
            )
        return u"{} {} {}".format(
            self.contact.invite_name,
            email,
            email_cc_html
        )
    get_contact_data_cl.short_description = _("Contact")
    get_contact_data_cl.allow_tags = True
    get_contact_data_cl.admin_order_field = 'contact__last_name'

    def get_documents_cl(self):
        """
        Filters invite documents taking only ones created from a hostess
        :return: Span html snippet with invite documents
        """
        documents = self.documents.filter(created_from_hostess=True)
        if documents:
            titles = ", ".join(documents.values_list('name', flat=True))
            count = documents.count()
            return u"<span title='{}'><img src='/static/event/img/note.png' />&nbsp;<b>{}</b></span>".format(titles, count)
        return ""
    get_documents_cl.short_description = _("Documents")
    get_documents_cl.allow_tags = True


##############################################################################
# Documents management
##############################################################################
class DocumentCategory(DateModel, UserModel):
    """
    Simple model to add a category to documents
    """
    name = models.CharField(
        max_length=100,
        unique=True,
        verbose_name=_('name'),
    )

    def __unicode__(self):
        return u"%s" % self.name


class Document(DateModel, UserModel):
    """
    Document model used to add generic documents to event with a M2M on EventDocument model
    """
    category = models.ForeignKey(
        DocumentCategory,
        verbose_name=_('category'),
    )
    name = models.CharField(
        max_length=100,
        unique=True,
        verbose_name=_('name'),
    )
    document = models.FileField(
        upload_to='Documents/%Y/%m',
        verbose_name=_('document'),
    )

    class Meta(object):
        verbose_name = _('document')
        verbose_name_plural = _('documents')
        ordering = ('category', 'name')

    def __unicode__(self):
        return u"{}".format(self.name)


class EventDocument(DateModel, UserModel, StatusModel, OrderedModel):
    """
    M2M table to associate event and document. These are used in document email and documents step.
    We define which documents can be sent with attach_in_mail setting
    """
    event = models.ForeignKey(
        Event,
        related_name="event_documents",
        verbose_name=_("event"),
    )
    document = models.ForeignKey(
        Document,
        related_name="event_documents",
        verbose_name=_("document"),
    )
    attach_in_mail = models.BooleanField(
        default=True,
        verbose_name=_('attach in documents email'),
    )

    class Meta(object):
        verbose_name = _('event document')
        verbose_name_plural = _('event documents')

    def __unicode__(self):
        return u"{}".format(self.document.name)


class NewsletterDocument(DateModel, UserModel):
    """
    Model to store event newsletter documents.
    It is only used when invite email is sent.
    """
    event = models.ForeignKey(
        Event,
        related_name="newsletter_documents",
        verbose_name=_("event"),
    )
    document = models.FileField(
        upload_to='Documents/%Y/%m',
        verbose_name=_('document'),
        validators=[validate_newsletter_document],
        help_text=_('File must be under 2MB, allowed formats are: xls, xlsx, pdf, doc, docx, jpeg, ics, zip'),
    )
    title = models.CharField(
        max_length=50,
        verbose_name=_('title'),
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    use_always = models.BooleanField(
        default=True,
        verbose_name=_('use always')
    )

    class Meta(object):
        verbose_name = _('newsletter document')
        verbose_name_plural = _('newsletter documents')

    def __unicode__(self):
        return u"{}".format(self.title)


##############################################################################
# Translations management
##############################################################################
class Signature(DateModel, UserModel):
    """
    Model used to store left and right signature of an event
    """
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
    )
    title = models.CharField(
        max_length=100,
        verbose_name=_('title'),
    )
    company = models.CharField(
        max_length=100,
        verbose_name=_('company'),
        blank=True, null=True,
    )
    signature = models.ImageField(
        upload_to=settings.PUBLIEDITOR_EVENT_SIGNATURE_IMAGE_UPLOAD_TO
    )
    signature_online = ImageSpecField(
        source='signature',
        processors=[ResizeToFill(200, 100)],
        format='JPEG',
        options={'quality': 80}
    )
    email = models.EmailField(
        blank=True, null=True,
        verbose_name=_('email'),
    )
    note = models.CharField(
        max_length=100,
        verbose_name=_('note'),
        blank=True, null=True,
    )

    class Meta(object):
        verbose_name = _('signature')
        verbose_name_plural = _('signatures')
        ordering = ('name',)

    def __unicode__(self):
        if self.note:
            return u"{} - {} ({})".format(self.name, self.title, self.note)
        return u"{} - {}".format(self.name, self.title)


class EventSummaryDocumentTemplate(UserModel, DateModel):
    """
    Model used to store which template use to create event summary pdf.
    I think this is no longer used, PUBLIEDITOR_EVENT_PERSONALIZED_PROGRAM_TEMPLATES setting is used instead.
    """
    name = models.CharField(
        _("Name"),
        unique=True,
        max_length=255,
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    template = models.FileField(
        upload_to=settings.PUBLIEDITOR_EVENT_DOCUMENT_TEMPLATE_PATH,
        verbose_name=_("Template file")
    )
    header_image = models.ImageField(
        upload_to=settings.PUBLIEDITOR_EVENT_DOCUMENT_IMAGE_PATH,
        verbose_name=_("header image")
    )
    body_image = models.ImageField(
        upload_to=settings.PUBLIEDITOR_EVENT_DOCUMENT_IMAGE_PATH,
        verbose_name=_("Body image")
    )

    class Meta:
        ordering = ('name',)
        verbose_name = _("event summary document template")
        verbose_name_plural = _("event summary document templates")

    def __unicode__(self):
        return u"{0}".format(self.name)


class EventTemplate(UserModel, DateModel, OrderedModel):
    """
    I think this model was used in the past when event email template could be freely chose by the user.
    PUBLIEDITOR_EVENT_DOCUMENTS_EMAIL_TEMPLATES setting is currently used.
    Please verify before deleting.
    """
    name = models.CharField(
        _("Name"),
        unique=True,
        max_length=255,
    )
    file = models.FileField(
        upload_to=settings.PUBLIEDITOR_EVENT_TEMPLATE_PATH,
        verbose_name=_("Template")
    )

    class Meta:
        ordering = ('ordering', 'name')
        verbose_name = _("Template")
        verbose_name_plural = _("Templates")

    def __unicode__(self):
        return u"{}".format(self.name)


class EventTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event translations used in presentation step and invite email
    """
    event = models.ForeignKey(
        Event,
        related_name="translations",
        verbose_name=_("event")
    )
    header_image = models.ImageField(
        verbose_name=_('header image'),
        upload_to=settings.PUBLIEDITOR_EVENT_HEADER_IMAGE_UPLOAD_TO,
    )
    header_image_online = ImageSpecField(
        source='header_image',
        processors=[ResizeToFill(1200, 400)],
        format='JPEG',
        options={'quality': 80}
    )
    header_image_pdf = ImageSpecField(
        source='header_image',
        processors=[ResizeToFill(800, 270)],
        format='JPEG',
        options={'quality': 80}
    )
    footer_image = models.ImageField(
        null=True, blank=True,
        verbose_name=_('footer image'),
        upload_to=settings.PUBLIEDITOR_EVENT_FOOTER_IMAGE_UPLOAD_TO,
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    title = models.CharField(
        max_length=100,
        verbose_name=_('title'),
    )
    web_body = HTMLFieldTinyMCE(
        verbose_name=_('web body'),
        help_text=_("invite web body"),
    )
    email_body = HTMLFieldTinyMCE(
        verbose_name=_('email body'),
        help_text=_(
            """invite email body, to include image remember to <b>use absolute path</b> like
               http://rsvp.publieditor.it/ and <b>include img in a paragraph</b>
            """
        )
    )
    email_from = models.EmailField(
        verbose_name=_('email from'),
        default=settings.PUBLIEDITOR_EVENT_TRANSLATION_DEFAULT_EMAIL,
    )
    email_from_name = models.CharField(
        verbose_name=_('email from name'),
        max_length=100,
    )
    header_reply = models.EmailField(
        _('Reply to'),
        null=True, blank=True,
        max_length=255,
    )
    note = models.TextField(
        null=True, blank=True,
        verbose_name=_('note'),
    )
    css = models.TextField(
        null=True, blank=True,
        verbose_name=_('css'),
    )
    left_signature = models.ForeignKey(
        Signature,
        related_name="translations_left",
        null=True, blank=True,
        verbose_name=_('left signature'),
    )
    right_signature = models.ForeignKey(
        Signature,
        related_name="translations_right",
        null=True, blank=True,
        verbose_name=_('right signature'),
    )
    bmw_external_images = models.TextField(
        null=True, blank=True,
        verbose_name=_('bmw external images')
    )
    disclaimer_text = HTMLFieldTinyMCE(
        blank=True, null=True,
        verbose_name=_('disclaimer text'),
    )
    disclaimer_image = models.ImageField(
        blank=True, null=True,
        upload_to=settings.PUBLIEDITOR_EVENT_DISCLAIMER_IMAGE_UPLOAD_TO,
        verbose_name=_('disclaimer image'),
    )
    disclaimer_in_personalized_program = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in personalized program')
    )
    disclaimer_in_email = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in email')
    )
    disclaimer_in_presentation_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in presentation step')
    )
    disclaimer_in_schedule_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in schedule step')
    )
    disclaimer_in_adhesion_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in adhesion step')
    )
    disclaimer_in_register_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in register step')
    )
    disclaimer_in_transport_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in transport step')
    )
    disclaimer_in_activity_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in activity step')
    )
    disclaimer_in_summary_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in summary step')
    )
    disclaimer_in_decline_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in decline step')
    )
    disclaimer_in_delegate_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in delegate step')
    )
    disclaimer_in_documents_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in documents step')
    )
    disclaimer_in_greetings_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in greetings step')
    )
    disclaimer_in_poll_step = models.BooleanField(
        default=False,
        verbose_name=_('disclaimer in poll step')
    )

    def get_bmw_external_images(self):
        """
        :return: String of bmw external images separeted by comma
        """
        if self.bmw_external_images and self.bmw_external_images.strip() != "":
            return self.bmw_external_images.split(",")
        return None

    def display_header_image(self):
        """
        :return: Translation header image with size, width and height
        """
        try:
            if self.header_image:
                file_size = "%0.1f KB" % (self.header_image.size/(1024.0))
                file_attrs = "%s - (%sx%s)" % (file_size, self.header_image.width, self.header_image.height)
                return u'<img src="%s" />%s' % (self.header_image.url, file_attrs)
        except Exception, ex:
            return ""
    display_header_image.short_description = _('Image')
    display_header_image.allow_tags = True

    class Meta(object):
        verbose_name = _('event translation')
        verbose_name_plural = _('event translations')
        ordering = ('-event__event_date_start', 'event__name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"{}: {}".format(self.language, self.event.name)

    def save(self, *args, **kw):
        """
        Body is parsed before saving to preserve db html conistency
        """
        html_parser = HTMLParser.HTMLParser()
        self.web_body = html_parser.unescape(self.web_body)
        self.email_body = html_parser.unescape(self.email_body)
        super(EventTranslation, self).save(*args, **kw)


class AdhesionTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event adhesion translations
    """
    event = models.ForeignKey(
        Event,
        related_name="adhesion_translations",
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
        help_text=_("default text is loaded when changed")
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text=_("adhesion web body")
    )

    class Meta(object):
        verbose_name = _('adhesion translation')
        verbose_name_plural = _('adhesion translations')
        ordering = ('name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"{}: {}".format(self.name, self.language,)


class EmailForwardedTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event email forwarded translations
    """
    event = models.ForeignKey(
        Event,
        related_name="email_forwarded_translations",
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
        help_text=_("default text is loaded when changed")
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text=_("text returned by {{ invite.get_forwarded_text }}, you can use {{ contact }} / {{ invite }} / {{ invite.forwarded_from }} / {{ invite.forwarded_from.contact }}")
    )

    class Meta(object):
        verbose_name = _('email forwarded translation')
        verbose_name_plural = _('email forwarded translations')
        ordering = ('name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"{}: {}".format(self.name, self.language,)


class ScheduleTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event schedule translations
    """
    event = models.ForeignKey(
        Event,
        related_name="schedule_translations"
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text=_(
            """schedule web body, to include image remember to <b>use absolute path</b> like
               http://rsvp.publieditor.it/ and <b>include img in a paragraph</b>
            """
        )
    )

    class Meta(object):
        verbose_name = _('schedule translation')
        verbose_name_plural = _('schedule translations')
        ordering = ('event__name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"{}: {}".format(self.language, self.event)


class ActivityIntroductionTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event activities introduction in language
    """
    event = models.ForeignKey(
        Event,
        related_name="activities_introduction_translations"
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    title = models.CharField(
        max_length=512,
        blank=True, null=True,
        verbose_name=_('title'),
        help_text=_('if not set it will not be displayed')
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text=_(
            """activities introduction web body, to include image remember to <b>use absolute path</b> like
               http://rsvp.publieditor.it/ and <b>include img in a paragraph</b>
            """
        )
    )

    class Meta(object):
        verbose_name = _('activity introduction translation')
        verbose_name_plural = _('activity introduction translations')
        ordering = ('event__name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"{}: {}".format(self.language, self.event)


class FollowUpTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event follow up translations
    """
    event = models.ForeignKey(
        Event,
        related_name="followup_translations"
    )
    title = models.CharField(
        max_length=100,
        verbose_name=_('title'),
        null=True, blank=True,
        help_text=_("If blank presentation title will be used")
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text=_("followup web body")
    )
    left_signature = models.ForeignKey(
        'Signature',
        related_name="followup_translations_as_left",
        null=True, blank=True,
        verbose_name=_('left signature'),
    )
    right_signature = models.ForeignKey(
        'Signature',
        related_name="followup_translations_as_right",
        null=True, blank=True,
        verbose_name=_('right signature'),
    )

    class Meta(object):
        verbose_name = _('follow up translation')
        verbose_name_plural = _('follow up translations')
        ordering = ('event__name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"{}".format(self.title)

    def save(self, *args, **kw):
        """
        Body is unescaped before saving to preserve db consistency
        """
        html_parser = HTMLParser.HTMLParser()
        self.body = html_parser.unescape(self.body)
        super(FollowUpTranslation, self).save(*args, **kw)


class EventDocumentEmailTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event document email translations
    """
    event = models.ForeignKey(
        Event,
        related_name="document_email_translations"
    )
    title = models.CharField(
        max_length=100,
        verbose_name=_('title'),
        null=True, blank=True,
        help_text=_("If blank presentation title will be used")
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text=_("followup web body")
    )
    left_signature = models.ForeignKey(
        'Signature',
        related_name="invite_left",
        null=True, blank=True,
        verbose_name=_('left signature'),
    )
    right_signature = models.ForeignKey(
        'Signature',
        related_name="invite_right",
        null=True, blank=True,
        verbose_name=_('right signature'),
    )

    class Meta(object):
        verbose_name = _('document email translation')
        verbose_name_plural = _('document emails translations')
        ordering = ('event__name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"{}".format(self.title)

    def save(self, *args, **kw):
        """
        Body is unescaped before saving to preserve db consistency
        """
        html_parser = HTMLParser.HTMLParser()
        self.body = html_parser.unescape(self.body)
        super(EventDocumentEmailTranslation, self).save(*args, **kw)


class TransportTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event transport translations
    """
    transport = models.ForeignKey(
        'Transport',
        related_name="translations"
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    description = models.TextField(
        null=True, blank=True,
        verbose_name=_('description')
    )

    class Meta(object):
        verbose_name = _('transport translation')
        verbose_name_plural = _('transport translations')
        ordering = ('transport__name', 'language')
        unique_together = ('transport', 'language')

    def __unicode__(self):
        return u"{}: {}".format(self.language, self.transport.name)


class PrivacyTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event privacy translations
    """
    event = models.ForeignKey(
        Event,
        related_name="privacy_translations",
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
        help_text=_("default text is loaded when changed"),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text=_("privacy web body")
    )

    class Meta(object):
        verbose_name = _('privacy translation')
        verbose_name_plural = _('privacy translations')
        ordering = ('event__name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"{}: {}".format(self.language, self.event)


class GreetingsTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Model used to store event greetings translations
    """
    event = models.ForeignKey(
        Event,
        related_name="greetings_translations"
    )
    invite_status = models.IntegerField(
        choices=settings.PUBLIEDITOR_EVENT_INVITE_STATUS_CODES,
        verbose_name=_('Invite status')
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
    )

    class Meta(object):
        verbose_name = _('greetings translation')
        verbose_name_plural = _('greetings translations')
        ordering = ('event__name', 'language')
        unique_together = ('event', 'invite_status', 'language')

    def __unicode__(self):
        return u"%s: %s" % (
            self.language,
            self.event
        )

    def get_body(self):
        return self.body
    get_body.allow_tags = True


##############################################################################
# Transport management #######################################################
##############################################################################
class EventTravelOption(DateModel, UserModel, OrderedModel):
    """
    Model used to store event travel option based on turn and start place
    """
    event = models.ForeignKey(
        "Event",
        related_name="travel_options",
        verbose_name=_("event"),
    )
    turn = models.ForeignKey(
        "Turn",
        related_name="travel_options",
        verbose_name=_("turn"),
    )
    start_place = models.ForeignKey(
        "EventStartPlace",
        related_name="travel_options",
        verbose_name=_("start place"),
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
    )

    def get_transport_details_dict(self):
        """
        :return: Returns flight and trains grouped by direction
        """
        transport_details_dict = {"d": [], "r": []}
        for direction_key in ["d", "r"]:
            transport_details = self.transport_details.filter(direction=direction_key).order_by("departure")
            transport_details_dates = sorted(set([detail.departure.date() for detail in transport_details]))
            for transport_detail_date in transport_details_dates:
                transport_detail_date_details = sorted(
                    transport_details.filter(
                        departure__year=transport_detail_date.year,
                        departure__month=transport_detail_date.month,
                        departure__day=transport_detail_date.day,
                    ),
                    key=lambda dt: dt.child.child_detail.departure,
                )
                transport_details_dict[direction_key].extend(transport_detail_date_details)
        return transport_details_dict

    def get_render_transport_group(self):
        """
        :return: Html string with its transport details
        """
        transport_details_dict = self.get_transport_details_dict()
        html_str = u""
        direction_labels = {"d": _("Departure"), "r": _("Return")}
        for transport_direction in ["d", "r"]:
            direction_transport_details = transport_details_dict[transport_direction]
            if direction_transport_details:
                # Print direction details
                html_str += " %s " % direction_labels[transport_direction].title()
                html_str += "<ol>"
                for direction_transport_detail in direction_transport_details:
                    # Print detail in transport is a flight
                    if direction_transport_detail.category == "flightdetail":
                        html_str += "<li>"
                        html_str += "%s <strong>%s %s%s %s</strong> %s (%s) - <strong>%s %s</strong> %s (%s)" % (
                            _date(direction_transport_detail.departure, "D d/m/Y").title(),
                            _("Flight"),
                            direction_transport_detail.flightdetail.flight.airline.iata_code,
                            direction_transport_detail.flightdetail.flight.number,
                            direction_transport_detail.flightdetail.flight.departure.strftime("%H:%M"),
                            direction_transport_detail.flightdetail.flight.dep_airport.place,
                            direction_transport_detail.flightdetail.flight.dep_airport.iata_code,
                            direction_transport_detail.flightdetail.flight.arrival.strftime("%H:%M"),
                            "+1" if direction_transport_detail.flightdetail.day_after() else "",
                            direction_transport_detail.flightdetail.flight.arr_airport.place,
                            direction_transport_detail.flightdetail.flight.arr_airport.iata_code,
                        )
                        html_str += "</li>"
                    # Print detail in transport is a train
                    elif direction_transport_detail.category == "traindetail":
                        html_str += "<li>"
                        html_str += "%s <strong>%s %s %s</strong> %s (%s) - <strong>%s %s</strong> %s (%s)" % (
                            _date(direction_transport_detail.departure, "D d/m/Y").title(),
                            _("Train"),
                            direction_transport_detail.traindetail.train.code,
                            direction_transport_detail.traindetail.train.departure.strftime("%H:%M"),
                            direction_transport_detail.traindetail.train.dep_station.place,
                            direction_transport_detail.traindetail.train.dep_station.name,
                            direction_transport_detail.traindetail.train.arrival.strftime("%H:%M"),
                            "+1" if direction_transport_detail.traindetail.day_after() else "",
                            direction_transport_detail.traindetail.train.arr_station.place,
                            direction_transport_detail.traindetail.train.arr_station.name,
                        )
                        html_str += "</li>"
                    elif direction_transport_detail.category == "customtransportdetail":
                        html_str += "<li>"
                        html_str += "%s <strong>%s %s</strong> %s - <strong>%s</strong> %s" % (
                            _date(direction_transport_detail.departure, "D d/m/Y").title(),
                            direction_transport_detail.customtransportdetail.customtransport.description,
                            direction_transport_detail.customtransportdetail.customtransport.departure.strftime("%H:%M"),
                            direction_transport_detail.customtransportdetail.customtransport.departure_place,
                            direction_transport_detail.customtransportdetail.customtransport.arrival.strftime("%H:%M"),
                            direction_transport_detail.customtransportdetail.customtransport.arrival_place,
                        )
                        html_str += "</li>"
                html_str += "</ol>"
        return html_str
    get_render_transport_group.short_description = _("Details")
    get_render_transport_group.allow_tags = True

    def get_plain_transport_group(self):
        """
        :return: Textual transport details
        """
        transport_details_dict = self.get_transport_details_dict()
        html_str = ""
        direction_labels = {"d": _("Departure"), "r": _("Return")}
        for transport_direction in ["d", "r"]:
            direction_transport_details = transport_details_dict[transport_direction]
            if direction_transport_details:
                # Print direction details
                html_str += " %s " % direction_labels[transport_direction].title()
                html_str += "\n"
                for direction_transport_detail in direction_transport_details:

                    # Print detail in transport is a flight
                    if direction_transport_detail.category == "flightdetail":
                        html_str += "- %s (%s) - %s (%s)\n" % (
                            _date(direction_transport_detail.departure, "D d/m/Y"),
                            direction_transport_detail.flightdetail.flight.departure.strftime("%H:%M"),
                            _date(direction_transport_detail.arrival, "D d/m/Y"),
                            direction_transport_detail.flightdetail.flight.arrival.strftime("%H:%M"),
                        )
                        html_str += "  Volo %s%s: " % (
                            direction_transport_detail.flightdetail.flight.airline.iata_code,
                            direction_transport_detail.flightdetail.flight.number,
                        )
                        html_str += "%s (%s) - %s (%s)" % (
                            direction_transport_detail.flightdetail.flight.dep_airport.place,
                            direction_transport_detail.flightdetail.flight.dep_airport.iata_code,
                            direction_transport_detail.flightdetail.flight.arr_airport.place,
                            direction_transport_detail.flightdetail.flight.arr_airport.iata_code,
                        )

                    # Print detail in transport is a train
                    elif direction_transport_detail.category == "traindetail":
                        html_str += "%s (%s) - %s (%s)\n" % (
                            _date(direction_transport_detail.departure, "D d/m/Y"),
                            direction_transport_detail.traindetail.train.departure.strftime("%H:%M"),
                            _date(direction_transport_detail.arrival, "D d/m/Y"),
                            direction_transport_detail.traindetail.train.arrival.strftime("%H:%M"),
                        )
                        html_str += "Treno %s%s: " % (
                            direction_transport_detail.traindetail.train.train_type.code,
                            direction_transport_detail.traindetail.train.number,
                        )
                        html_str += "%s (%s) - %s (%s)" % (
                            direction_transport_detail.traindetail.train.dep_station.place,
                            direction_transport_detail.traindetail.train.dep_station.name,
                            direction_transport_detail.traindetail.train.arr_station.place,
                            direction_transport_detail.traindetail.train.arr_station.name,
                        )
                    html_str += "\n\n"
        html_str = html_str.rstrip("\n\n")
        return html_str
    get_plain_transport_group.short_description = _("Details")
    get_plain_transport_group.allow_tags = True

    def __unicode__(self):
        return u"{}".format(self.name)

    class Meta(object):
        verbose_name = _('travel option')
        verbose_name_plural = _('travel options')
        ordering = ('event', 'ordering',)


class InviteDecisionAnswer(DateModel):
    """
    Used to save the decision the contact choosed for an invite
    """
    invite = models.ForeignKey(
        "Invite",
        related_name="decision_answers",
        verbose_name=_("invite"),
    )
    status = models.IntegerField(
        choices=settings.PUBLIEDITOR_EVENT_INVITE_STATUS_CODES,
        verbose_name=_('status'),
    )

    def __unicode__(self):
        return u"%s - %s %s" % (
            self.invite.event.name, self.invite.contact.full_name, self.invite.status
        )

    class Meta(object):
        verbose_name = _('invite decision answer')
        verbose_name_plural = _('invite decision answers')
        ordering = ('-date_create',)


class InviteTravelOptionAnswer(DateModel, UserModel):
    """
    Stores travel options selected on an invite
    """
    invite = models.ForeignKey(
        "Invite",
        related_name="travel_option_answers",
        verbose_name=_("invite"),
    )
    event_travel_option = models.ForeignKey(
        "EventTravelOption",
        related_name="invite_answers",
        verbose_name=_("event travel option"),
    )

    def __unicode__(self):
        return u"%s - %s %s" % (
            self.event_travel_option.name, self.invite.contact.full_name, self.invite.event.name
        )

    class Meta(object):
        verbose_name = _('invite travel option answer')
        verbose_name_plural = _('invite travel option answers')


class Turn(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Event evailable turns with a start, an end and a maximum number of participants
    """
    code = models.CharField(
        max_length=1,
        choices=settings.PUBLIEDITOR_EVENT_TURN_CODE_CHOICES,
    )
    title = models.CharField(
        max_length=100,
        verbose_name=_('title'),
    )
    event = models.ForeignKey(
        Event,
        related_name="turns",
        verbose_name=_("event"),
    )
    turn_start = models.DateTimeField(
        verbose_name=_('start'),
    )
    turn_end = models.DateTimeField(
        verbose_name=_('end')
    )
    maximum_participants = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('maximum participants')
    )

    @property
    def available_seats(self):
        """
        :return: Available number of seats
        """
        if not self.maximum_participants:
            return None
        busy_seats = self.event.invites.filter(
            turn_preference=self,
            status=settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
        ).count()
        available_seats = self.maximum_participants - busy_seats
        return available_seats

    def get_available_seats(self):
        """
        :return: Available number of seats
        """
        return self.available_seats
    get_available_seats.short_description = _("Available seats")

    def turn_time(self):
        """
        :return: Readable turn time
        """
        print_format = "%d/%m/%Y (%H:%M)"
        return "%s - %s" % (
            self.turn_start.strftime(print_format),
            self.turn_end.strftime(print_format),
        )
    turn_time.short_description = "%s %s" % (
        _('turn'), _('time')
    )

    class Meta(object):
        verbose_name = _('turn')
        verbose_name_plural = _('turns')
        ordering = ('ordering', 'event__name', 'turn_start', 'turn_end', 'code', 'title')
        unique_together = ('code', 'event')

    def __unicode__(self):
        return u"%s (%s - %s)" % (
            self.title,
            self.turn_start.strftime("%d/%m/%Y %H:%M"),
            self.turn_end.strftime("%d/%m/%Y %H:%M"),
        )

    def extended_unicode(self):
        """
        :return: Readable full partecipants string
        """
        if self.maximum_participants:
            return "%s - %s %s" % (
                self.__unicode__(),
                self.available_seats,
                _("available seats")
            )
        return self.__unicode__()

    def toggle_status(self):
        """
        Changes turn status
        """
        self.status = not self.status
        self.save()

    def clean(self):
        """
        Cleans model before saving, dates are checked
        """
        # Data inizio turno deve essere nel range di date dell'evento
        if self.turn_start and not self.event.event_date_start <= self.turn_start <= self.event.event_date_end:
            raise ValidationError(_("Turn start date must be contained in event start/end range"))
        # Data fine turno deve essere nel range di date dell'evento
        if self.turn_end and not self.event.event_date_start <= self.turn_end <= self.event.event_date_end:
            raise ValidationError(_("Turn end date must be contained in event start/end range"))


class EventTurnProxy(Turn):
    """
    Simple prxy for event turn
    """
    class Meta:
        verbose_name = _('event turn')
        verbose_name_plural = _('event turns')
        proxy = True


class EventStartPlace(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Event start place, used to aggregate transports
    """
    event = models.ForeignKey(
        Event,
        related_name="start_places",
        verbose_name=_("start place"),
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
    )

    class Meta(object):
        verbose_name = _('start place')
        verbose_name_plural = _('start places')
        ordering = ('ordering',)

    def __unicode__(self):
        return u"{}".format(self.name)


class Stage(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Stage model, it was used to add intermediate steps about trasports.
    I think this is no longer used
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )

    class Meta(object):
        verbose_name = _('stage')
        verbose_name_plural = _('stages')
        ordering = ('ordering',)

    def __unicode__(self):
        return u"{}".format(self.name)


class Transport(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Transport generic model, used especially for sublcassing
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name')
    )

    class Meta(object):
        verbose_name = _('transport')
        verbose_name_plural = _('transports')

    @property
    def category(self):
        """
        :return: transport type string
        """
        if hasattr(self, 'flight'):
            return 'flight'
        elif hasattr(self, 'train'):
            return 'train'
        elif hasattr(self, 'own'):
            return 'own'
        elif hasattr(self, 'free'):
            return 'free'
        return None

    def __unicode__(self):
        if hasattr(self, 'flight'):
            return unicode(self.flight)
        elif hasattr(self, 'train'):
            return unicode(self.train)
        elif hasattr(self, 'own'):
            return unicode(self.own)
        elif hasattr(self, 'free'):
            return unicode(self.free)
        return None


class TransportDetail(DateModel, UserModel):
    """
    TransportDetail model, used to create transports for an option and a direction
    """
    event_travel_option = models.ForeignKey(
        EventTravelOption,
        related_name='transport_details',
        verbose_name=_('event travel option'),
    )
    direction = models.CharField(
        max_length=1,
        choices=settings.PUBLIEDITOR_EVENT_TRANSPORT_DIRECTIONS,
        verbose_name=_('direction')
    )
    departure = models.DateTimeField(
        verbose_name=_('departure'),
    )
    arrival = models.DateTimeField(
        verbose_name=_('arrival'),
    )

    @property
    def category(self):
        if hasattr(self, 'flightdetail'):
            return 'flightdetail'
        elif hasattr(self, 'traindetail'):
            return 'traindetail'
        elif hasattr(self, 'customtransportdetail'):
            return 'customtransportdetail'
        return None

    @property
    def child_detail_map(self):
        return dict(flightdetail="flight", traindetail="train", customtransportdetail="customtransport")

    @property
    def child(self):
        if self.category:
            return getattr(self, self.category)
        return None

    @property
    def child_detail(self):
        if self.category and self.child:
            return getattr(self.child, self.child_detail_map[self.category])
        return None

    @property
    def formatted_departure(self):
        return u"{} {}".format(
            _date(self.departure, "D d/m/Y").title(), self.child_detail.departure.strftime("%H:%M")
        )

    @property
    def formatted_arrival(self):
        return u"{} {}".format(
            _date(self.arrival, "D d/m/Y").title(), self.child_detail.arrival.strftime("%H:%M")
        )

    def get_formatted_departure_date(self):
        return _date(self.departure, "D d/m/Y").title()
    get_formatted_departure_date.short_description = _("Departure")

    def get_formatted_arrival_date(self):
        return _date(self.arrival, "D d/m/Y").title()
    get_formatted_arrival_date.short_description = _("Arrival")

    def __unicode__(self):
        return u"{} da {}. {}: {},  arrivo: {}".format(
            self.get_direction_display(), self.event_travel_option.start_place,
            _("Departure"), self.formatted_departure,
            _("Arrival"), self.formatted_arrival,
        )

    class Meta(object):
        verbose_name = _('transport detail')
        verbose_name_plural = _('transport details')
        ordering = ('departure', 'arrival')


class FlightDetail(TransportDetail):
    """
    Transport detail model specific to fliught ones
    """
    flight = models.ForeignKey(
        'Flight',
        related_name='flight_details',
        verbose_name=_('flight'),
    )

    def flight_departure(self):
        return self.flight.departure
    flight_departure.short_description = "%s %s" % (_('departure'), _('time'))

    def flight_arrival(self):
        return self.flight.arrival
    flight_arrival.short_description = "%s %s" % (_('arrival'), _('time'))

    def get_formatted_departure_date(self):
        return "%s (%s)" % (
            _date(self.departure, "D d/m/Y").title(),
            self.flight.departure.strftime("%H:%M"),
        )
    get_formatted_departure_date.short_description = _("Departure")

    def get_formatted_arrival_date(self):
        return "%s (%s)" % (
            _date(self.arrival, "D d/m/Y").title(),
            self.flight.arrival.strftime("%H:%M"),
        )
    get_formatted_arrival_date.short_description = _("Arrival")

    @property
    def turn_time(self):
        return self.get_turn_time()

    def get_turn_time(self):
        return self.event_travel_option.turn.turn_time()
    get_turn_time.short_description = "%s %s" % (
        _('turn'), _('time')
    )

    def day_after(self):
        return self.flight.day_after if self.flight else False
    day_after.short_description = _('day after')
    day_after.boolean = True

    def __unicode__(self):
        return u"flight: %s %s %s %s" % (
            self.direction, self.event_travel_option.turn, self.event_travel_option.start_place, self.flight
        )

    class Meta(object):
        verbose_name = _('flight detail')
        verbose_name_plural = _('flight details')
        ordering = ('departure', 'arrival')


class Flight(Transport):
    """
    Generic flight model, used when a fligh must be selected for a travel option
    """
    number = models.CharField(
        max_length=10,
        verbose_name=_('number'),
    )
    code = models.CharField(
        max_length=10,
        verbose_name=_('code'),
        blank=True, null=True,
    )
    dep_airport = models.ForeignKey(
        'Airport',
        related_name='departure_flights',
        verbose_name=_('departure airport'),
    )
    arr_airport = models.ForeignKey(
        'Airport',
        related_name='arrival_flights',
        verbose_name=_('arrival airport'),
    )
    airline = models.ForeignKey(
        'Airline',
        related_name='flights',
        verbose_name=_('airline'),
    )
    departure = models.TimeField(
        verbose_name=_('departure'),
    )
    arrival = models.TimeField(
        verbose_name=_('arrival'),
    )
    duration = models.TimeField(
        blank=True, null=True,
        verbose_name=_('duration'),
    )
    day_after = models.BooleanField(
        default=False,
        verbose_name=_('day after'),
    )

    def get_code(self):
        code = "%s%s" % (self.airline.iata_code, self.number)
        return code
    get_code.short_description = _("Code")

    def save(self, *args, **kwargs):
        self.code = "%s%s" % (self.airline.iata_code, self.number)
        super(Flight, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"%s: %s (%s) - %s (%s)" % (
            self.code,
            self.dep_airport.name,
            self.dep_airport.iata_code,
            self.arr_airport.name,
            self.arr_airport.iata_code,
        )

    class Meta(object):
        verbose_name = _('flight')
        verbose_name_plural = _('flights')
        ordering = ('departure',)
        unique_together = (('number', 'airline'),)


class Airline(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Airline model (Eg. Alitalia, Easyject etc...)
    """
    iata_code = models.CharField(
        max_length=3,
        verbose_name=_('IATA code'),
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )

    def __unicode__(self):
        return u"%s (%s)" % (self.name, self.iata_code)

    class Meta(object):
        verbose_name = _('airline')
        verbose_name_plural = _('airlines')
        ordering = ('ordering', 'name',)


class Airport(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Airport model
    """
    iata_code = models.CharField(
        max_length=3,
        verbose_name=_('IATA code'),
        unique=True,
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )
    place = models.CharField(
        max_length=100,
        verbose_name=_('place'),
        blank=True, null=True,
    )

    def __unicode__(self):
        return u"%s %s - %s" % (
            self.place, self.name, self.iata_code,
        )

    class Meta(object):
        verbose_name = _('airport')
        verbose_name_plural = _('airports')
        ordering = ('ordering', 'name',)


class TrainDetail(TransportDetail):
    """
    Transport detail model specific for train ones.
    """
    train = models.ForeignKey(
        'Train',
        related_name='train_details',
        verbose_name=_('train'),
    )

    def get_formatted_departure_date(self):
        return "%s (%s)" % (
            _date(self.departure, "D d/m/Y").title(),
            self.train.departure.strftime("%H:%M"),
        )
    get_formatted_departure_date.short_description = _("Departure")

    def get_formatted_arrival_date(self):
        return "%s (%s)" % (
            _date(self.arrival, "D d/m/Y").title(),
            self.train.arrival.strftime("%H:%M"),
        )
    get_formatted_arrival_date.short_description = _("Arrival")

    def train_departure(self):
        return self.train.departure
    train_departure.short_description = "%s %s" % (_('departure'), _('time'))

    def train_arrival(self):
        return self.train.arrival
    train_arrival.short_description = "%s %s" % (_('arrival'), _('time'))

    def turn_time(self):
        return self.event_travel_option.turn.turn_time()
    turn_time.short_description = "%s %s" % (_('turn'), _('time'))

    def day_after(self):
        return self.train.day_after if self.train else False
    day_after.short_description = _('day after')
    day_after.boolean = True

    def __unicode__(self):
        return u"train: %s %s %s %s" % (
            self.direction, self.event_travel_option.turn, self.event_travel_option.start_place, self.train
        )

    class Meta(object):
        verbose_name = _('train detail')
        verbose_name_plural = _('train details')
        ordering = ('departure', 'arrival')


class Train(Transport):
    """
    Train generic model, used when a travel option is being created for an event
    """
    number = models.CharField(
        max_length=10,
        verbose_name=_('number'),
    )
    code = models.CharField(
        max_length=10,
        verbose_name=_('code'),
        blank=True, null=True,
    )
    dep_station = models.ForeignKey(
        'Station',
        related_name='departure_railways',
        verbose_name=_('departure station'),
    )
    arr_station = models.ForeignKey(
        'Station',
        related_name='arrival_railways',
        verbose_name=_('arrival station'),
    )
    train_type = models.ForeignKey(
        'TrainType',
        related_name='railways',
        verbose_name=_('train type'),
    )
    departure = models.TimeField(
        verbose_name=_('departure'),
    )
    arrival = models.TimeField(
        verbose_name=_('arrival'),
    )
    duration = models.TimeField(
        blank=True, null=True,
        verbose_name=_('duration'),
    )
    day_after = models.BooleanField(
        default=False,
        verbose_name=_('day after'),
    )

    def save(self, *args, **kwargs):
        self.code = "%s%s" % (self.train_type.code, self.number)
        super(Train, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"%s: %s - %s" % (
            self.code,
            self.dep_station.name,
            self.arr_station.name,
        )

    class Meta(object):
        verbose_name = _('train')
        verbose_name_plural = _('trains')
        ordering = ('departure',)


class TrainType(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Train type generic model (Ec. Fraccia rossa, Eurocity etc...)
    """
    code = models.CharField(
        max_length=5,
        verbose_name=_('code'),
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )

    def __unicode__(self):
        return u"%s" % self.name

    class Meta(object):
        verbose_name = _('train type')
        verbose_name_plural = _('train types')
        ordering = ('ordering', 'name',)


class Station(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Train station model
    """
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
        unique=True,
    )
    place = models.CharField(
        max_length=100,
        verbose_name=_('place'),
    )

    def __unicode__(self):
        return u"%s" % self.name

    class Meta(object):
        verbose_name = _('railway station')
        verbose_name_plural = _('railway stations')
        ordering = ('ordering', 'name',)


class CustomTransportDetail(TransportDetail):
    """
    Custom Transport detail model with specific test: for example "with blablacar"
    """
    customtransport = models.ForeignKey(
        'CustomTransport',
        related_name='custom_transport_details',
        verbose_name=_('customtransport'),
    )

    def __unicode__(self):
        return u"train: %s %s %s %s" % (
            self.direction, self.event_travel_option.turn, self.event_travel_option.start_place, self.customtransport
        )

    class Meta(object):
        verbose_name = _('Custom transport detail')
        verbose_name_plural = _('Custom transport details')
        ordering = ('departure', 'arrival')


class CustomTransport(Transport):
    """
    CustomTransport generic model, used when a travel option is being created for an event
    """
    description = models.CharField(
        max_length=256,
        verbose_name=_('description'),
    )
    departure = models.TimeField(
        verbose_name=_('departure'),
    )
    arrival = models.TimeField(
        verbose_name=_('arrival'),
    )
    departure_place = models.CharField(
        max_length=256,
        verbose_name=_('departure place')
    )
    arrival_place = models.CharField(
        max_length=256,
        verbose_name=_('arrival place')
    )
    duration = models.TimeField(
        blank=True, null=True,
        verbose_name=_('duration'),
    )
    day_after = models.BooleanField(
        default=False,
        verbose_name=_('day after'),
    )

    def __unicode__(self):
        return u'{}'.format(self.description)

    class Meta(object):
        verbose_name = _('CustomTransport')
        verbose_name_plural = _('CustomTransports')
        ordering = ('departure',)


##############################################################################
# Transport answer management
##############################################################################
class TransportAnswer(DateModel, UserModel):
    """
    Model used to store transport detail (which is an aggregation on transports) is selected for an invite
    """
    transport_detail = models.ForeignKey(
        'TransportDetail',
        related_name='transport_answers',
        verbose_name=_('transport detail')
    )
    invite = models.ForeignKey(
        Invite,
        related_name='transport_answers',
        verbose_name=_('invite')
    )

    def __unicode__(self):
        return u"%s - %s: %s" % (
            self.invite.event.name, self.invite.contact.full_name, self.transport_detail
        )

    class Meta(object):
        verbose_name = _('transport answer')
        verbose_name_plural = _('transport answers')


##############################################################################
# Event answer management
##############################################################################
class EventQuestion(DateModel, UserModel, OrderedModel):
    """
    Event questions, this model really stores all event answers, proxies are used to filter them by step
    """
    event = models.ForeignKey(
        'Event', related_name="questions",
        verbose_name=_('event')
    )
    question = models.ForeignKey(
        Question, related_name="event_questions",
        verbose_name=_('question')
    )
    enabled = models.BooleanField(
        verbose_name=_("enabled"),
        default=True,
    )
    required = models.BooleanField(
        verbose_name=_("required"),
        default=False,
    )

    class Meta(object):
        verbose_name = _('event question')
        verbose_name_plural = _('event questions')
        ordering = ('question__ordering', 'question__question')

    def __unicode__(self):
        return u"%s %s" % (self.event.name, self.question)

    def get_category(self):
        return self.question.category
    get_category.short_description = _("category")

    def get_choices(self):
        return self.question.get_choices()
    get_choices.short_description = _("choices")

    def get_question_qtype(self):
        return self.question.get_qtype_display()
    get_question_qtype.short_description = _("qtype")

    def get_short_name(self):
        return 'Q_{}'.format(self.ordering)
    get_short_name.short_description = _('Short name')

    def get_link_enabled(self):
        return """
            <a id='publieditor_event_eventquestion_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.enabled else 'no'))
    get_link_enabled.short_description = _('Set')
    get_link_enabled.allow_tags = True

    def get_link_required(self):
        return """
            <a id='publieditor_event_requiredquestion_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.required else 'no'))
    get_link_required.short_description = _('Required')
    get_link_required.allow_tags = True


# REGISTER QUESTION MANAGER
class RegisterEventQuestionManager(models.Manager):
    def get_queryset(self):
        return super(RegisterEventQuestionManager, self).get_queryset().filter(
            question__typology='R'
        )


class RegisterEventQuestionProxy(EventQuestion):
    """
    Event questions proxy used to filter on register ones
    """

    objects = RegisterEventQuestionManager()

    class Meta:
        verbose_name = _('register event question')
        verbose_name_plural = _('register event questions')
        proxy = True


# ACTIVITY QUESTION MANAGER
class ActivityEventQuestionManager(models.Manager):
    def get_queryset(self):
        return super(ActivityEventQuestionManager, self).get_queryset().filter(
            question__typology='A'
        )


class ActivityEventQuestionProxy(EventQuestion):
    """
    Event questions proxy used to filter on activity ones
    """

    objects = ActivityEventQuestionManager()

    class Meta:
        verbose_name = _('activity event question')
        verbose_name_plural = _('activity event questions')
        proxy = True


# DECLINE QUESTION MANAGER
class DeclineEventQuestionManager(models.Manager):
    def get_queryset(self):
        return super(DeclineEventQuestionManager, self).get_queryset().filter(
            question__typology='D'
        )


class DeclineEventQuestionProxy(EventQuestion):
    """
    Event questions proxy used to filter on decline ones
    """

    objects = DeclineEventQuestionManager()

    class Meta:
        verbose_name = _('decline event question')
        verbose_name_plural = _('decline event questions')
        proxy = True


# DECLINE QUESTION MANAGER
class PollEventQuestionManager(models.Manager):
    def get_queryset(self):
        return super(PollEventQuestionManager, self).get_queryset().filter(
            question__typology='P'
        )


class PollEventQuestionProxy(EventQuestion):
    """
    Event questions proxy used to filter on poll ones
    """

    objects = PollEventQuestionManager()

    class Meta:
        verbose_name = _('poll event question')
        verbose_name_plural = _('poll event questions')
        proxy = True


class RegisterAnswer(Answer):
    """
    Answer for an invite about register step
    """
    invite = models.ForeignKey(
        Invite,
        related_name='register_answers',
        verbose_name=_('invite')
    )

    def __unicode__(self):
        return u"%s - %s: %s? %s" % (
            self.invite.event.name, self.invite.contact.full_name, self.question, self.answer
        )

    class Meta(object):
        verbose_name = _('register answer')
        verbose_name_plural = _('register answers')


class ActivityAnswer(Answer, OrderedModel):
    """
    Answer for an invite about activity step
    """
    invite = models.ForeignKey(
        Invite,
        related_name='activity_answers',
        verbose_name=_('invite')
    )

    def __unicode__(self):
        return u"%s %s" % (self.question.question, self.answer)

    class Meta(object):
        verbose_name = _('activity answer')
        verbose_name_plural = _('activity answers')
        ordering = ('ordering',)


class PollAnswer(Answer):
    """
    Answer for an invite about poll step
    """
    invite = models.ForeignKey(
        Invite,
        related_name='poll_answers',
        verbose_name=_('invite')
    )

    @property
    def ordering(self):
        event = self.invite.event
        return event.questions.get(question=self.question).ordering

    def __unicode__(self):
        return u"{} {}".format(self.question.question, self.answer)

    class Meta(object):
        verbose_name = _('poll answer')
        verbose_name_plural = _('poll answers')


class FreeAnswer(DateModel, UserModel):
    """
    Answer for an invite about free transport question
    """
    invite = models.ForeignKey(
        Invite,
        related_name='free_answers',
        verbose_name=_('invite')
    )
    answer = models.TextField(
        verbose_name=_('answer text')
    )

    def __unicode__(self):
        return u"%s %s" % (self.invite.event.name, self.answer)

    class Meta(object):
        verbose_name = _('free answer')
        verbose_name_plural = _('free answers')


class DeclineAnswer(Answer, OrderedModel):
    """
    Answer for an invite about decline step
    """
    invite = models.ForeignKey(
        Invite,
        related_name='decline_answers',
        verbose_name=_('invite')
    )

    def __unicode__(self):
        return u"%s %s" % (self.question.question, self.answer)

    class Meta(object):
        verbose_name = _('decline answer')
        verbose_name_plural = _('decline answers')
        ordering = ('ordering',)


class OwnAnswer(DateModel, UserModel):
    """
    Answer for an invite about register own transport detail.
    If record exists it means contact selected own transports option
    """
    invite = models.ForeignKey(
        Invite,
        related_name='own_answers',
        verbose_name=_('invite')
    )

    class Meta(object):
        verbose_name = _('own answer')
        verbose_name_plural = _('own answers')


class DelegatedContact(DateModel, UserModel):
    """
    Links an invite to its delegator.
    Records exists when an invite was delegated so a new one was created,
    this table is involved in keeping invites linked each other
    """
    invited = models.ForeignKey(
        Contact,
        related_name='delegated_invites',
        verbose_name=_('contact')
    )
    invite = models.ForeignKey(
        Invite,
        related_name='delegated_invites',
        verbose_name=_('invite')
    )
    title = models.CharField(
        max_length=10,
        verbose_name=_("title"),
        choices=settings.PUBLIEDITOR_CONTACT_TITLE_CHOICES
    )
    first_name = models.CharField(
        max_length=100,
        verbose_name=_('first name'),
    )
    last_name = models.CharField(
        max_length=100,
        verbose_name=_('last name'),
    )
    email = models.EmailField(
        verbose_name=_('email'),
    )
    phone_number = models.CharField(
        max_length=50,
        verbose_name=_('phone number'),
    )
    managed = models.BooleanField(
        default=False,
        verbose_name=_('managed'),
    )

    def get_event_name(self):
        return self.invite.event.__unicode__()
    get_event_name.short_description = _("Event")

    def get_delegator_link(self):
        return "<a href='%s'>%s</a>" % (
            reverse('admin:event_invite_change', args=[self.invite.id]),
            self.invite.contact.full_name,
        )
    get_delegator_link.short_description = _("Delegator")
    get_delegator_link.allow_tags = True

    def get_delegator_full_name(self):
        return self.invite.contact.full_name
    get_delegator_full_name.short_description = _("Delegator")

    def get_delegated_full_name(self):
        return self.invited.full_name
    get_delegated_full_name.short_description = _("Delegated")

    class Meta(object):
        verbose_name = _('delegated contact')
        verbose_name_plural = _('delegated contacts')
        ordering = ('invite__event__event_date_start', 'invite__event__event_date_end',)

    def __unicode__(self):
        return u"%s: %s -> %s" % (
            self.get_event_name(),
            self.get_delegator_full_name(),
            self.get_delegated_full_name(),
        )


class Forward(DateModel, UserModel):
    invite = models.ForeignKey(
        Invite,
        verbose_name=_('Invite'),
        related_name='forwards'
    )
    sender = models.ForeignKey(
        Contact,
        verbose_name=_('Sender'),
        related_name='forwards_as_sender'
    )
    receiver = models.ForeignKey(
        Contact,
        verbose_name=_('Receiver'),
        related_name='forwards_as_receiver'
    )
    is_sent = models.BooleanField(
        verbose_name=_('is sent'),
        default=False,
    )

    class Meta(object):
        verbose_name = _('forward')
        verbose_name_plural = _('forwards')

    def __unicode__(self):
        return u"%s: %s -> %s (%s)" % (
            self.invite.event.__unicode__(),
            self.sender.full_name,
            self.receiver.full_name,
            'sent' if self.is_sent else 'unsent',
        )


class EventInviteListProxy(Invite):
    """
    Proxy model, I think it's no longer used.
    Please check before deleting
    """
    class Meta:
        proxy = True
        verbose_name = _("event invites")


class EventMailingListProxy(Invite):
    """
    Proxy model, I think it's no longer used.
    Please check before deleting
    """
    def delete(self):
        if (
            not self.status or
            self.status not in settings.PUBLIEDITOR_EVENT_INVITE_CONTACT_ACTIONS
        ):
            super(EventMailingListProxy, self).delete()
            return True
        return False

    class Meta:
        proxy = True
        verbose_name = _("event mailing list")


class AreaCode(UserModel, DateModel):
    """
    Model used to associate a phone to an area code, for example +39 for italian phones
    """
    area = models.CharField(
        max_length=100,
        verbose_name=_('area'),
    )
    code = models.CharField(
        max_length=25,
        verbose_name=_("code"),
        unique=True,
    )

    def __unicode__(self):
        return u"%s - %s" % (self.code, self.area)

    class Meta(object):
        verbose_name = _('area code')
        verbose_name_plural = _('area codes')
        ordering = ('area',)

    def save(self, *args, **kw):
        self.code = self.code.replace(" ", "")
        super(AreaCode, self).save(*args, **kw)


class EventImage(DateModel, OrderedModel):
    """
    Event image, I think this model is no longer used.
    Please check before deleting.
    """
    event_image = models.ForeignKey(
        Event,
        related_name='event_images',
        verbose_name=_("event image")
    )
    photo_id = models.CharField(
        max_length=255,
        verbose_name=_('photo id')
    )
    image_size = models.PositiveIntegerField(
        db_index=True,
        verbose_name=_('image size')
    )
    title = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('title')
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_('description')
    )
    width = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('width')
    )
    height = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('height')
    )
    weigth = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('weigth')
    )

    class Meta:
        verbose_name = _('image')
        verbose_name_plural = _('images')
        ordering = ('ordering', 'id',)
        index_together = (("ordering", "id"),)
        unique_together = (("photo_id", "event_image"),)

    def get_photo_dict(self):
        """
        return a dictionary of relative paths to get photos from media server
        Alta : https://mediapool.bmwgroup.com/mediapool/econtroller/pressclub/publicq?actEvent=high&dokNo=XXXXXXXXX
        Thumb : https://mediapool.bmwgroup.com/mediapool/econtroller/pressclub/publicq?actEvent=zoom&dokNo=XXXXXXXXX&imageSize=YYY
        """
        photo_dict = dict(
            orig='%s' % self.photo_id,
            thumb='%s&imageSize=%s' % (self.photo_id, self.image_size),
        )
        return photo_dict

    def get_photo_url_dict(self):
        """return a dictionary of absolute urls to get photos from media server"""
        photo_url_dict = dict(
            orig=settings.PUBLIEDITOR_EVENT_IMAGE_BASE_URL +
                 settings.PUBLIEDITOR_EVENT_IMAGE_URL_PREFIX_ORIGINAL +
                 self.get_photo_dict()['orig'] +
                 settings.PUBLIEDITOR_EVENT_IMAGE_URL_SUFFIX_ORIGINAL,
            thumb=settings.PUBLIEDITOR_EVENT_IMAGE_BASE_URL +
                  settings.PUBLIEDITOR_EVENT_IMAGE_URL_PREFIX_THUMB +
                  self.get_photo_dict()['thumb'] +
                  settings.PUBLIEDITOR_EVENT_IMAGE_URL_SUFFIX_THUMB
        )
        return photo_url_dict


class EventTranslationPreference(DateModel):
    """
    Stores language preference about an invite or event.
    It is actually used to get the correct translation
    """
    user = models.ForeignKey(
        'auth.User',
        verbose_name=_("User"),
        related_name="event_translation_preferences",
        blank=True, null=True,
    )
    invite = models.ForeignKey(
        'event.Invite',
        verbose_name=_("Invite"),
        related_name="translation_preferences",
        blank=True, null=True,
    )
    event = models.ForeignKey(
        'event.Event',
        verbose_name=_("Event"),
        related_name="translation_preferences",
        db_index=True,
    )
    language = models.ForeignKey(
        'i18n.Language',
        verbose_name=_("Language"),
        related_name="event_translation_preferences",
    )

    class Meta:
        verbose_name = _('Event translation preference')
        verbose_name_plural = _('Event translation preferences')
        ordering = ('-date_create',)

    def __unicode__(self):
        if self.user:
            return _(
                "Event language preference for user %s and event %s is %s" %
                (self.user.username, self.event.name, self.language.name)
            )
        elif self.invite:
            return _(
                "Event language preference for contact %s and event %s is %s" %
                (self.invite.contact.full_name, self.event.name, self.language.name)
            )
        else:
            return _(
                "Event language preference for event %s is %s" %
                (self.event.name, self.language.name)
            )


class InviteFirstOpening(DateModel, UserModel):
    """
    Stores the first opening details about an invite email
    """
    invite = models.OneToOneField(
        Invite,
        related_name='first_opening',
        verbose_name=_('invite'),
    )

    class Meta:
        verbose_name = _('Invite first opening')
        verbose_name_plural = _('Invites first opening')
        ordering = ('-date_create',)

    def __unicode__(self):
        return self.invite.__unicode__()


class InviteDocument(DateModel, UserModel):
    """
    Used to attach documents to an invite
    """
    invite = models.ForeignKey(
        Invite,
        related_name="documents",
        verbose_name=_("invite"),
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
    )
    document = models.FileField(
        upload_to='invite-documents/%Y/%m',
        verbose_name=_('document'),
    )
    created_from_travel_agency = models.BooleanField(
        default=False,
        verbose_name=_("Created from travel agency"),
    )
    created_from_logistic_agency = models.BooleanField(
        default=False,
        verbose_name=_("Created from logistic agency"),
    )
    created_from_hostess = models.BooleanField(
        default=False,
        verbose_name=_("Created from hostess"),
    )

    class Meta(object):
        verbose_name = _('invite document')
        verbose_name_plural = _('invite documents')
        ordering = ('invite', 'name')

    def __unicode__(self):
        return u"{}".format(self.name)


class InviteDocumentTravelAgencyProxy(InviteDocument):
    """
    Used to show invite documents to a travel agency
    """
    class Meta:
        verbose_name = _('travel agency invite document proxy')
        verbose_name_plural = _('travel agency invite documents proxy')
        proxy = True


class InviteDocumentLogisticAgencyProxy(InviteDocument):
    """
    Used to show invite documents to a logistic agency
    """
    class Meta:
        verbose_name = _('logistic agency invite document proxy')
        verbose_name_plural = _('logistic agency invite documents proxy')
        proxy = True


class InviteDocumentHostessProxy(InviteDocument):
    """
    Used to show invite documents to a hostess
    """
    class Meta:
        verbose_name = _('hostess invite document proxy')
        verbose_name_plural = _('hostess invite documents proxy')
        proxy = True


class MailingStatusProxy(MailingStatus):
    """
    Simple proxy about MailingStatus mailer model
    """
    class Meta:
        verbose_name = _('Mailing status')
        verbose_name_plural = _('Mailing status')
        proxy = True


class PasswordRecoverySettings(DateModel, UserModel):
    """
    Global model used to configure email used for contact password changing
    """
    site = models.OneToOneField(
        Site,
        related_name="event_password_recovery_settings"
    )
    email_from = models.EmailField(
        verbose_name=_('email from'),
        default=settings.PUBLIEDITOR_EVENT_TRANSLATION_DEFAULT_EMAIL,
    )
    subject = models.CharField(
        max_length=255,
        verbose_name=_('subject'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text=_('Use {{ link }} to include the password recovery link'),
    )
    link_text = models.CharField(
        max_length=100,
        verbose_name=_("link_text"),
        help_text=_('body {{ link }} will be replaced with this text'),
    )

    class Meta(object):
        verbose_name = _('Password recovery settings')
        verbose_name_plural = _('Password recovery settings')

    def send_email(self, contact):
        """
        Email configured in admin is sent to contact and its cc emails
        :param contact: who will receive the email
        """
        to = [contact.email]
        cc = [email.email_details() for email in contact.cc_emails.all()]
        subject = ''.join(self.subject.splitlines())
        html_message = Template(self.body)
        url = "{}://{}{}".format(
            settings.SITE_PROTOCOL,
            settings.SITE_DOMAIN,
            reverse("event:reset_password", kwargs={'uidb36': contact.make_base36(), 'token': contact.token})
        )
        context_dictionary = contact.__dict__
        context_dictionary['link'] = mark_safe(u"<a href='{}'>{}</a>".format(url, self.link_text))
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        text_message = html_message
        msg = EmailMultiAlternatives(
            subject=subject, body=text_message, from_email=self.email_from, to=to, cc=cc
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()


class ProfileDoneEmailSettings(DateModel, UserModel):
    """
    Global setting used to configure the email which is sent when a contact completes its profile
    """
    site = models.OneToOneField(
        Site,
        related_name="event_profile_done_email_settings"
    )
    email_from = models.EmailField(
        verbose_name=_('email from'),
        default=settings.PUBLIEDITOR_EVENT_TRANSLATION_DEFAULT_EMAIL,
    )
    subject = models.CharField(
        max_length=255,
        verbose_name=_('subject'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text=_('Use {{ link }} to include the login link'),
    )
    link_text = models.CharField(
        max_length=100,
        verbose_name=_("link_text"),
        help_text=_('body {{ link }} will be replaced with this text'),
    )

    class Meta(object):
        verbose_name = _('Profile done email settings')
        verbose_name_plural = _('Profile done emails settings')

    def send_email(self, contact):
        """
        Email configured in admin is sent to contact and its cc emails
        :param contact: who will receive the email
        """
        to = [contact.email]
        cc = [email.email_details() for email in contact.cc_emails.all()]
        subject = ''.join(self.subject.splitlines())
        html_message = Template(self.body)
        url = "{}://{}{}".format(
            settings.SITE_PROTOCOL,
            settings.SITE_DOMAIN,
            reverse("event:login")
        )
        context_dictionary = contact.__dict__
        context_dictionary['link'] = mark_safe(u"<a href='{}'>{}</a>".format(url, self.link_text))
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        text_message = html_message
        msg = EmailMultiAlternatives(
            subject=subject, body=text_message, from_email=self.email_from, to=to, cc=cc
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()


####################################################################################
# EVENTS STATISTICS ################################################################
####################################################################################
class LastMailingStatusProcessed(DateModel):
    """
    Last processed mailer.mailingstatus ID
    """
    last_processed_id = models.IntegerField(
        verbose_name=_('last mailingstatus id'),
        default=0
    )
    is_script_running = models.BooleanField(
        default=False,
        verbose_name=_('update script running')
    )


class EventStatusDateStatistic(DateModel):
    """
    Event statistics by:
     - event id
     - mailer.mailingstatus status
     - inviteemailobject date_create
    """
    event = models.ForeignKey(
        Event,
        related_name="event_status_data_email_statistic",
        verbose_name=_("event status data email statistic"),
    )
    status = models.IntegerField(
        verbose_name=_('status'),
        choices=settings.MAILER_MAILINGSTATUS_STATUS_CHOICES
    )
    date_invite = models.DateTimeField(
        verbose_name=_('dateinvite'),
    )
    unique_email_address = models.IntegerField(
        _('Unique email address'),
        default=0
    )
    total = models.IntegerField(
        _('Total'),
        default=0
    )

    class Meta:
        ordering = ['event', 'status', 'date_invite']
        verbose_name = _('Event Status Date Statistic')
        verbose_name_plural = _('Event Status Date Statistics')
        unique_together = ('event', 'status', 'date_invite')


class EventStatusStatistic(DateModel):
    """
    Event statistics by:
     - event id
     - mailer.mailingstatus status
    """
    event = models.ForeignKey(
        Event,
        related_name="event_statistic",
        verbose_name=_("event Statistic"),
    )
    status = models.IntegerField(
        verbose_name=_('status'),
        choices=settings.MAILER_MAILINGSTATUS_STATUS_CHOICES
    )
    total = models.IntegerField(
        verbose_name=_('total'),
        default=0
    )
    unique_email_address = models.IntegerField(
        verbose_name=_('unique'),
        default=0
    )

    class Meta:
        ordering = ['event', 'status']
        verbose_name = _('Event Status Statistic')
        verbose_name_plural = _('Event Status Statistics')
        unique_together = ('event', 'status')


class EventLinkDateStatistic(DateModel):
    """
    Statistic data for links
    """
    event = models.ForeignKey(
        Event,
        related_name="event_link_date_statistic",
        verbose_name=_("event link Statistic"),
    )
    link_title = models.CharField(
        _('Title'),
        max_length=255
    )
    link_url = models.CharField(
        _('URL'),
        max_length=255
    )
    date_invite = models.DateTimeField(
        verbose_name=_('dateinvite'),
    )
    total = models.IntegerField(
        verbose_name=_('total'),
        default=0
    )
    unique_email_address = models.IntegerField(
        verbose_name=_('unique'),
        default=0
    )

    class Meta:
        ordering = ['event', 'link_title', 'link_url', 'date_invite']
        verbose_name = _('Event Link Date Statistic')
        verbose_name_plural = _('Event Link Date Statistics')
        unique_together = ('event', 'date_invite', 'link_title', 'link_url')


class EventLinkStatistic(DateModel):
    """
    Statistic data for links
    """
    event = models.ForeignKey(
        Event,
        related_name="event_link_statistic",
        verbose_name=_("event link Statistic"),
    )
    link_title = models.CharField(
        _('Title'),
        max_length=255
    )
    link_url = models.CharField(
        _('URL'),
        max_length=255
    )
    total = models.IntegerField(
        verbose_name=_('total'),
        default=0
    )
    unique_email_address = models.IntegerField(
        verbose_name=_('unique'),
        default=0
    )

    class Meta:
        ordering = ['event', 'link_title', 'link_url']
        verbose_name = _('Event Link Statistic')
        verbose_name_plural = _('Event Link Statistics')
        unique_together = ('event', 'link_title', 'link_url')


class EventDissuasionMessageTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    event = models.ForeignKey(
        Event,
        related_name="dissuasion_message_translations"
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    deadline_expired_message = HTMLFieldTinyMCE(
        verbose_name=_('deadline expired message'),
        help_text=_("""Insert here the message to dissuade the user in case of an expired deadline.
        You can use follow context like {{ event.[field] }}:
        <br>{{ event.event_date_start }} to print start timestamp
        <br>{{ event.event_date_end }} to print end timestamp
        <br>{{ event.event_adhesion_deadline }} to print deadline timestamp
        <br>{{ event.name }} to print event name
        <br>{{ event.place }} to print event place
        """)
    )

    class Meta(object):
        verbose_name = _('Event Dissuasion Message Translation')
        verbose_name_plural = _('Event Dissuasion Messages Translation')
        ordering = ('event__name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"{event} ({language})".format(
            event=self.event,
            language=self.language,
        )

    def save(self, *args, **kw):
        """
        deadline_expired_message is unescaped before saving to preserve db consistency
        """
        html_parser = HTMLParser.HTMLParser()
        self.deadline_expired_message = html_parser.unescape(self.deadline_expired_message)
        super(EventDissuasionMessageTranslation, self).save(*args, **kw)


class EventPlace(DateModel, UserModel, StatusModel, OrderedModel):
    event = models.ForeignKey(
        Event,
        related_name="places",
        verbose_name=_('event')
    )
    location = models.CharField(
        max_length=255,
        verbose_name=_('location'),
        help_text=_('location name (EX. Hilton Hotel)')
    )
    name = models.CharField(
        blank=True, null=True,
        max_length=255,
        db_index=True,
        verbose_name=_("name"),
    )
    number = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("number"),
        db_column="anumber"
    )
    zip_code = models.CharField(
        max_length=20,
        db_index=True,
        verbose_name=_("zip code"),
        blank=True, null=True,
    )
    locality = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_("locality"),
        blank=True, null=True,
    )
    province = models.CharField(
        max_length=3,
        db_index=True,
        verbose_name=_("province"),
        blank=True, null=True,
    )
    country = models.ForeignKey(
        Country,
        blank=True, null=True,
        db_index=True,
        default=get_default_country,
        verbose_name=_("country"),
    )
    date = models.DateField(
        verbose_name=_("date"),
        help_text=_('You have to create one place for each day'),
    )
    vat_number = models.CharField(
        max_length=127,
        verbose_name=_("vat number"),
        blank=True, null=True,
    )
    business_name = models.CharField(
        max_length=127,
        verbose_name=_('business name'),
        blank=True, null=True,
    )
    hostess = models.ManyToManyField(
        User,
        blank=True,
        verbose_name=_('hostess'),
        related_name='event_place_hostesses',
        limit_choices_to={'groups__name': settings.PUBLIEDITOR_AUTH_GROUPS_NAME['hostess']}
    )
    vehicles = models.ManyToManyField(
        'Vehicle',
        blank=True,
        verbose_name=_('vehicles'),
        related_name='event_place_vehicles',
    )

    def get_date(self):
        return self.date.strftime('%Y-%m-%d')

    class Meta:
        verbose_name = _("event place")
        verbose_name_plural = _("event places")
        ordering = ('date', 'locality', 'business_name')

    def __unicode__(self):
        # [Data GG/MM/AAAA] | [Località] | [Location]
        return u"%(date)s | %(locality)s | %(location)s " % {
            'date': self.date.strftime('%d/%m/%Y'),
            'location': self.location,
            'locality': self.locality, 'province': self.province,
        }


class Brand(DateModel, UserModel, StatusModel, OrderedModel):

    name = models.CharField(
        max_length=255,
        verbose_name=_('brand name'),
    )
    url = models.CharField(
        max_length=16,
        blank=True, null=True,
        verbose_name=_('url'),
    )
    logo = models.ImageField(
        verbose_name=_('Logo'),
        upload_to="logo",
        null=True, blank=True
    )
    logo_small = models.ImageField(
        verbose_name=_('Logo small'),
        upload_to="logo_small",
        null=True, blank=True
    )
    logo_big = models.ImageField(
        verbose_name=_('Logo big'),
        upload_to="logo_big",
        null=True, blank=True
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_('note'),
    )

    class Meta(object):
        verbose_name = _('Brand')
        verbose_name_plural = _('Brands')

    def __unicode__(self):
        return u"{}".format(self.name)


class VehicleModel(DateModel, UserModel, StatusModel, OrderedModel):
    brand = models.ForeignKey(
        Brand,
        related_name="vehicle_models",
        verbose_name=_('brand'),
    )
    name = models.CharField(
        max_length=64,
        db_index=True,
        verbose_name=_('name'),
    )
    descr = models.TextField(
        blank=True, null=True,
        verbose_name=_('description'),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_('note'),
    )
    slug = models.SlugField(
        max_length=100,
        db_index=True,
        unique=True,
        verbose_name=_("slug"),
    )
    year = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('year'),
    )

    class Meta:
        verbose_name = _('VehicleModel')
        verbose_name_plural = _('Vehicle Models')

    def __unicode__(self):
        if self.year:
            return u"%s (%s)" % (self.name, self.year)
        else:
            return u"%s" % (self.name)


class EventVehicleModel(StatusModel, OrderedModel):
    event = models.ForeignKey(
        Event,
        related_name='vehicle_models_event_list',
        verbose_name=_('event'),
    )
    vehicle_model = models.ForeignKey(
        VehicleModel,
        related_name='vehicle_models_event_list',
        verbose_name=_('vehicle model'),
    )

    class Meta:
        verbose_name = _("event vehicle model")
        verbose_name_plural = _("events vehicle models")
        unique_together = (("event", "vehicle_model"),)

    def __unicode__(self):
        return u'%(event)s %(vehicle_model)s' % {'event': self.event, 'vehicle_model': self.vehicle_model}


class Vehicle(DateModel, UserModel, StatusModel, OrderedModel):
    vehicle_model = models.ForeignKey(
        VehicleModel,
        related_name='vehicles',
        verbose_name=_('vehicle model'),
    )
    licence_plate = models.CharField(
        max_length=32,
        verbose_name=_('license plate'),
    )
    vin = models.CharField(
        max_length=255,
        verbose_name=_('vin'),
    )
    color = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('color'),
    )

    class Meta:
        verbose_name = _('Vehicle')
        verbose_name_plural = _('Vehicles')

    def __unicode__(self):
        return u"%s - %s" % (self.vehicle_model.name, self.licence_plate)


class CDU(DateModel, UserModel, StatusModel, OrderedModel):
    def get_cdu_storage_path(self, filename):
        path = os.path.join(
            settings.MEDIA_URL, settings.PUBLIEDITOR_EVENT_CDU_PATH, str(self.invite.event.pk), str(self.invite.pk) + '.pdf'
        )
        return path[1:]

    invite = models.OneToOneField(
        Invite,
        related_name='cdu',
        verbose_name=_('invite'),
    )
    cdu_file = models.FileField(
        upload_to=get_cdu_storage_path,
        verbose_name=_('cdu file'),
    )

    class Meta:
        verbose_name = _('CDU')
        verbose_name_plural = _('CDUs')

    def __unicode__(self):
        return u"CDU {contact_full_name} - {assigned_vehicle}".format(
            contact_full_name=self.invite.contact.full_name,
            assigned_vehicle=self.invite.assigned_vehicle
        )


####################################################################################
# SIGNALS ##########################################################################
####################################################################################
from publieditor.event.signals import *



def test_time():
    import time
    import requests
    url = 'https://rsvp.publieditor.it/admin/event/invite/?event=1947'

    headers = {
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
        'accept-language': 'it-IT,it;q=0.9,en-US;q=0.8,en;q=0.7,ru;q=0.6,de;q=0.5,es;q=0.4,fr;q=0.3,ja;q=0.2',
        'cache-control': 'no-cache',
        'cookie': 'csrftoken=8c4PeFoIbqrFOR1ONHcavnNszlhBELE1; sessionid=vncb1xjxmzskukwsbaw5v292026ibajw',
        'pragma': 'no-cache',
        'sec-ch-ua': '"Google Chrome";v="123", "Not:A-Brand";v="8", "Chromium";v="123"',
        'sec-ch-ua-mobile': '?1',
        'sec-ch-ua-platform': '"Android"',
        'sec-fetch-dest': 'document',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-site': 'none',
        'sec-fetch-user': '?1',
        'upgrade-insecure-requests': '1',
        'user-agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Mobile Safari/537.36'
    }
    start_time = time.time()  # Start time
    response = requests.get(url, headers=headers)
    end_time = time.time()  # End time

    response_time = end_time - start_time
    return response_time